// $ANTLR 3.4 com/senseidb/bql/parsers/BQL.g 2012-08-05 09:59:01

package com.senseidb.bql.parsers;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;
import org.json.JSONObject;
import org.json.JSONArray;
import org.json.JSONException;
import java.text.ParseException;
import java.text.SimpleDateFormat;


import org.antlr.runtime.*;
import java.util.Stack;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;

import org.antlr.runtime.tree.*;


/*******************************************************************************************

BNF Grammar for BQL
===================

<statement> ::= ( <select_stmt> | <describe_stmt> ) [';']

<select_stmt> ::= SELECT <select_list> [<from_clause>] [<where_clause>] [<given_clause>]
                  [<additional_clauses>]

<describe_stmt> ::= ( DESC | DESCRIBE ) [<index_name>]

<select_list> ::= '*' | <column_name_list>

<column_name_list> ::= <column_name> ( ',' <column_name> )*

<from_clause> ::= FROM <index_name>

<index_name> ::= <identifier> | <quoted_string>

<where_clause> ::= WHERE <search_expr>

<search_expr> ::= <term_expr> ( OR <term_expr> )*

<term_expr> ::= <facet_expr> ( AND <facet_expr> )*

<facet_expr> ::= <predicate> 
               | '(' <search_expr> ')'

<predicates> ::= <predicate> ( AND <predicate> )*

<predicate> ::= <in_predicate>
              | <contains_all_predicate>
              | <equal_predicate>
              | <not_equal_predicate>
              | <query_predicate>
              | <between_predicate>
              | <range_predicate>
              | <time_predicate>
              | <match_predicate>
              | <like_predicate>
              | <null_predicate>

<in_predicate> ::= <column_name> [NOT] IN <value_list> [<except_clause>] [<predicate_props>]

<contains_all_predicate> ::= <column_name> CONTAINS ALL <value_list> [<except_clause>]
                             [<predicate_props>]

<equal_predicate> ::= <column_name> '=' <value> [<predicate_props>]

<not_equal_predicate> ::= <column_name> '<>' <value> [<predicate_props>]

<query_predicate> ::= QUERY IS <quoted_string>

<between_predicate> ::= <column_name> [NOT] BETWEEN <value> AND <value>

<range_predicate> ::= <column_name> <range_op> <numeric>

<time_predicate> ::= <column_name> IN LAST <time_span>
                   | <column_name> ( SINCE | AFTER | BEFORE ) <time_expr>

<match_predicate> ::= [NOT] MATCH '(' <column_name_list> ')' AGAINST '(' <quoted_string> ')'

<like_predicate> ::= <column_name> [NOT] LIKE <quoted_string>

<null_predicate> ::= <column_name> IS [NOT] NULL

<value_list> ::= <non_variable_value_list> | <variable>

<non_variable_value_list> ::= '(' <value> ( ',' <value> )* ')'

<python_style_list> ::= '[' <python_style_value> ( ',' <python_style_value> )* ']'

<python_style_dict> ::= '{' <key_value_pair> ( ',' <key_value_pair> )* '}'

<python_style_value> ::= <value>
                       | <python_style_list>
                       | <python_style_dict>

<value> ::= <numeric>
          | <quoted_string>
          | TRUE
          | FALSE
          | <variable>

<range_op> ::= '<' | '<=' | '>=' | '>'

<except_clause> ::= EXCEPT <value_list>

<predicate_props> ::= WITH <prop_list>

<prop_list> ::= '(' <key_value_pair> ( ',' <key_value_pair> )* ')'

<key_value_pair> ::= <quoted_string> ':' 
                     ( <value> | <python_style_list> | <python_style_dict> )

<given_clause> ::= GIVEN FACET PARAM <facet_param_list>

<facet_param_list> ::= <facet_param> ( ',' <facet_param> )*

<facet_param> ::= '(' <facet_name> <facet_param_name> <facet_param_type> <facet_param_value> ')'

<facet_param_name> ::= <quoted_string>

<facet_param_type> ::= BOOLEAN | INT | LONG | STRING | BYTEARRAY | DOUBLE

<facet_param_value> ::= <quoted_string>

<additional_clauses> ::= ( <additional_clause> )+

<additional_clause> ::= <order_by_clause>
                      | <limit_clause>
                      | <group_by_clause>
                      | <browse_by_clause>
                      | <fetching_stored_clause>
                      | <route_by_clause>
                      | <relevance_model_clause>

<order_by_clause> ::= ORDER BY <sort_specs>

<sort_specs> ::= <sort_spec> ( ',' <sort_spec> )*

<sort_spec> ::= <column_name> [<ordering_spec>]

<ordering_spec> ::= ASC | DESC

<group_by_clause> ::= GROUP BY <group_spec>

<group_spec> ::= <or_column_name_list> [TOP <max_per_group>]

<or_column_name_list> ::= <column_name> ( OR <column_name> )*

<limit_clause> ::= LIMIT [<offset> ','] <count>

<offset> ::= ( <digit> )+

<count> ::= ( <digit> )+

<browse_by_clause> ::= BROWSE BY <facet_specs>

<facet_specs> ::= <facet_spec> ( ',' <facet_spec> )*

<facet_spec> ::= <facet_name> [<facet_expression>]

<facet_expression> ::= '(' <expand_flag> <count> <count> <facet_ordering> ')'

<expand_flag> ::= TRUE | FALSE

<facet_ordering> ::= HITS | VALUE

<fetching_stored_clause> ::= FETCHING STORED [<fetching_flag>]

<fetching_flag> ::= TRUE | FALSE

<route_by_clause> ::= ROUTE BY <quoted_string>

<relevance_model_clause> ::= USING RELEVANCE MODEL <identifier> <prop_list>
                             [<relevance_model>]

<relevance_model> ::= DEFINED AS <formal_parameters> BEGIN <model_block> END

<formal_parameters> ::= '(' <formal_parameter_decls> ')'

<formal_parameter_decls> ::= <formal_parameter_decl> ( ',' <formal_parameter_decl> )*

<formal_parameter_decl> ::= <variable_modifiers> <type> <variable_declarator_id>

<variable_modifiers> ::= ( <variable_modifier> )*

<variable_modifier> ::= 'final'

<type> ::= <class_or_interface_type> ('[' ']')*
         | <primitive_type> ('[' ']')*
         | <boxed_type> ('[' ']')*
         | <limited_type> ('[' ']')*

<class_or_interface_type> ::= <fast_util_data_type>

<fast_util_data_type> ::= 'IntOpenHashSet'
                        | 'FloatOpenHashSet'
                        | 'DoubleOpenHashSet'
                        | 'LongOpenHashSet'
                        | 'ObjectOpenHashSet'
                        | 'Int2IntOpenHashMap'
                        | 'Int2FloatOpenHashMap'
                        | 'Int2DoubleOpenHashMap'
                        | 'Int2LongOpenHashMap'
                        | 'Int2ObjectOpenHashMap'
                        | 'Object2IntOpenHashMap'
                        | 'Object2FloatOpenHashMap'
                        | 'Object2DoubleOpenHashMap'
                        | 'Object2LongOpenHashMap'
                        | 'Object2ObjectOpenHashMap'

<primitive_type> ::= 'boolean' | 'char' | 'byte' | 'short' 
                   | 'int' | 'long' | 'float' | 'double'

<boxed_type> ::= 'Boolean' | 'Character' | 'Byte' | 'Short' 
               | 'Integer' | 'Long' | 'Float' | 'Double'

<limited_type> ::= 'String' | 'System' | 'Math'

<model_block> ::= ( <block_statement> )+

<block_statement> ::= <local_variable_declaration_stmt>
                    | <java_statement>

<local_variable_declaration_stmt> ::= <local_variable_declaration> ';'

<local_variable_declaration> ::= <variable_modifiers> <type> <variable_declarators>

<java_statement> ::= <block>
                   | 'if' <par_expression> <java_statement> [ <else_statement> ]
                   | 'for' '(' <for_control> ')' <java_statement>
                   | 'while' <par_expression> <java_statement>
                   | 'do' <java_statement> 'while> <par_expression> ';'
                   | 'switch' <par_expression> '{' <switch_block_statement_groups> '}'
                   | 'return' <expression> ';'
                   | 'break' [<identifier>] ';'
                   | 'continue' [<identifier>] ';'
                   | ';'
                   | <statement_expression> ';'

<block> ::= '{' ( <block_statement> )* '}'

<else_statement> ::= 'else' <java_statement>

<switch_block_statement_groups> ::= ( <switch_block_statement_group> )*

<switch_block_statement_group> ::= ( <switch_label> )+ ( <block_statement> )*

<switch_label> ::= 'case' <constant_expression> ':'
                 | 'case' <enum_constant_name> ':'
                 | 'default' ':'

<for_control> ::= <enhanced_for_control>
                | [<for_init>] ';' [<expression>] ';' [<for_update>]

<for_init> ::= <local_variable_declaration>
             | <expression_list>

<enhanced_for_control> ::= <variable_modifiers> <type> <identifier> ':' <expression>

<for_update> ::= <expression_list>

<par_expression> ::= '(' <expression> ')'

<expression_list> ::= <expression> ( ',' <expression> )*

<statement_expression> ::= <expression>

<constant_expression> ::= <expression>

<enum_constant_name> ::= <identifier>

<variable_declarators> ::= <variable_declarator> ( ',' <variable_declarator> )*

<variable_declarator> ::= <variable_declarator_id> '=' <variable_initializer>

<variable_declarator_id> ::= <identifier> ('[' ']')*

<variable_initializer> ::= <array_initializer>
                         | <expression>

<array_initializer> ::= '{' [ <variable_initializer> ( ',' <variable_initializer> )* [','] ] '}'

<expression> ::= <conditional_expression> [ <assignment_operator> <expression> ]

<assignment_operator> ::= '=' | '+=' | '-=' | '*=' | '/=' | '&=' | '|=' | '^=' |
                        | '%=' | '<<=' | '>>>=' | '>>='

<conditional_expression> ::= <conditional_or_expression> [ '?' <expression> ':' <expression> ]

<conditional_or_expression> ::= <conditional_and_expression> ( '||' <conditional_and_expression> )*

<conditional_and_expression> ::= <inclusive_or_expression> ('&&' <inclusive_or_expression> )*

<inclusive_or_expression> ::= <exclusive_or_expression> ('|' <exclusive_or_expression> )*

<exclusive_or_expression> ::= <and_expression> ('^' <and_expression> )*

<and_expression> ::= <equality_expression> ( '&' <equality_expression> )*

<equality_expression> ::= <instanceof_expression> ( ('==' | '!=') <instanceof_expression> )*

<instanceof_expression> ::= <relational_expression> [ 'instanceof' <type> ]

<relational_expression> ::= <shift_expression> ( <relational_op> <shift_expression> )*

<shift_expression> ::= <additive_expression> ( <shift_op> <additive_expression> )*

<relational_op> ::= '<=' | '>=' | '<' | '>'

<shift_op> ::= '<<' | '>>>' | '>>'

<additive_expression> ::= <multiplicative_expression> ( ('+' | '-') <multiplicative_expression> )*

<multiplicative_expression> ::= <unary_expression> ( ( '*' | '/' | '%' ) <unary_expression> )*

<unary_expression> ::= '+' <unary_expression>
                     | '-' <unary_expression>
                     | '++' <unary_expression>
                     | '--' <unary_expression>
                     | <unary_expression_not_plus_minus>

<unary_expression_not_plus_minus> ::= '~' <unary_expression>
                                    | '!' <unary_expression>
                                    | <cast_expression>
                                    | <primary> <selector>* [ ('++'|'--') ]

<cast_expression> ::= '(' <primitive_type> ')' <unary_expression>
                    | '(' (<type> | <expression>) ')' <unary_expression_not_plus_minus>

<primary> ::= <par_expression>
            | <literal>
            | <java_ident> ('.' <java_method>)* [<identifier_suffix>]

<java_ident> ::= <boxed_type>
               | <limited_type>
               | <identifier>

<java_method> ::= <identifier>

<identifier_suffix> ::= ('[' ']')+ '.' 'class'
                      | <arguments>
                      | '.' 'class'
                      | '.' 'this'
                      | '.' 'super' <arguments>

<literal> ::= <integer>
            | <real>
            | <floating_point_literal>
            | <character_literal>
            | <quoted_string>
            | <boolean_literal>
            | 'null'

<boolean_literal> ::= 'true' | 'false'

<selector> ::= '.' <identifier> <arguments>
             | '.' 'this'
             | '[' <expression> ']'

<arguments> ::= '(' [<expression_list>] ')'

<quoted_string> ::= '"' ( <char> )* '"'
                  | "'" ( <char> )* "'"

<identifier> ::= <identifier_start> ( <identifier_part> )*

<identifier_start> ::= <alpha> | '-' | '_'

<identifier_part> ::= <identifier_start> | <digit>

<variable> ::= '$' ( <alpha> | <digit> | '_' )+

<column_name> ::= <identifier> | <quoted_string>

<facet_name> ::= <identifier>

<alpha> ::= <alpha_lower_case> | <alpha_upper_case>

<alpha_upper_case> ::= A | B | C | D | E | F | G | H | I | J | K | L | M | N | O
                     | P | Q | R | S | T | U | V | W | X | Y | Z

<alpha_lower_case> ::= a | b | c | d | e | f | g | h | i | j | k | l | m | n | o
                     | p | q | r | s | t | u | v | w | x | y | z

<digit> ::= 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9

<numeric> ::= <time_expr> 
            | <integer>
            | <real>

<integer> ::= ( <digit> )+

<real> ::= ( <digit> )+ '.' ( <digit> )+

<time_expr> ::= <time_span> AGO
              | <date_time_string>
              | NOW

<time_span> ::= [<time_week_part>] [<time_day_part>] [<time_hour_part>]
                [<time_minute_part>] [<time_second_part>] [<time_millisecond_part>]

<time_week_part> ::= <integer> ( 'week' | 'weeks' )

<time_day_part>  ::= <integer> ( 'day'  | 'days' )

<time_hour_part> ::= <integer> ( 'hour' | 'hours' )

<time_minute_part> ::= <integer> ( 'minute' | 'minutes' | 'min' | 'mins')

<time_second_part> ::= <integer> ( 'second' | 'seconds' | 'sec' | 'secs')

<time_millisecond_part> ::= <integer> ( 'millisecond' | 'milliseconds' | 'msec' | 'msecs')

<date_time_string> ::= <date> [<time>]

<date> ::= <digit><digit><digit><digit> ('-' | '/' | '.') <digit><digit>
           ('-' | '/' | '.') <digit><digit>

<time> ::= DIGIT DIGIT ':' DIGIT DIGIT ':' DIGIT DIGIT

*******************************************************************************************/
@SuppressWarnings({"all", "warnings", "unchecked"})
public class BQLParser extends Parser {
    public static final String[] tokenNames = new String[] {
        "<invalid>", "<EOR>", "<DOWN>", "<UP>", "AFTER", "AGAINST", "AGO", "ALL", "ALPHA", "AND", "AND_PRED", "AS", "ASC", "BEFORE", "BEGIN", "BETWEEN", "BOOLEAN", "BROWSE", "BY", "BYTE", "BYTEARRAY", "CHARACTER_LITERAL", "COLON", "COLUMN_LIST", "COMMA", "COMMENT", "CONTAINS", "DATE", "DAYS", "DEFINED", "DESC", "DESCRIBE", "DIGIT", "DOUBLE", "ELSE", "END", "EQUAL", "EQUAL_PRED", "ESCAPE_SEQUENCE", "EXCEPT", "EXPONENT", "FACET", "FALSE", "FAST_UTIL_DATA_TYPE", "FETCHING", "FLOATING_POINT_LITERAL", "FLOAT_TYPE_SUFFIX", "FROM", "GIVEN", "GROUP", "GT", "GTE", "HEX_DIGIT", "HEX_LITERAL", "HITS", "HOURS", "IDENT", "IN", "INT", "INTEGER", "INTEGER_TYPE_SUFFIX", "IS", "LAST", "LIKE", "LIMIT", "LINE_COMMENT", "LONG", "LPAR", "LT", "LTE", "MATCH", "MILLISECONDS", "MINS", "MINUTES", "MODEL", "MSECS", "NOT", "NOT_EQUAL", "NOW", "NULL", "OCTAL_ESCAPE", "OCTAL_LITERAL", "OR", "ORDER", "OR_PRED", "PARAM", "QUERY", "RANGE_PRED", "REAL", "RELEVANCE", "ROUTE", "RPAR", "SECONDS", "SECS", "SELECT", "SEMI", "SINCE", "STORED", "STRING", "STRING_LITERAL", "TIME", "TOP", "TRUE", "UNICODE_ESCAPE", "USING", "VALUE", "VARIABLE", "WEEKS", "WHERE", "WITH", "WS", "'!'", "'!='", "'%'", "'%='", "'&&'", "'&'", "'&='", "'*'", "'*='", "'+'", "'++'", "'+='", "'-'", "'--'", "'-='", "'.'", "'/'", "'/='", "'=='", "'?'", "'Character'", "'Float'", "'Integer'", "'Math'", "'Short'", "'String'", "'System'", "'['", "']'", "'^'", "'^='", "'break'", "'case'", "'char'", "'class'", "'continue'", "'default'", "'do'", "'extends'", "'final'", "'float'", "'for'", "'if'", "'instanceof'", "'return'", "'short'", "'super'", "'switch'", "'this'", "'while'", "'{'", "'|'", "'|='", "'||'", "'}'", "'~'"
    };

    public static final int EOF=-1;
    public static final int T__111=111;
    public static final int T__112=112;
    public static final int T__113=113;
    public static final int T__114=114;
    public static final int T__115=115;
    public static final int T__116=116;
    public static final int T__117=117;
    public static final int T__118=118;
    public static final int T__119=119;
    public static final int T__120=120;
    public static final int T__121=121;
    public static final int T__122=122;
    public static final int T__123=123;
    public static final int T__124=124;
    public static final int T__125=125;
    public static final int T__126=126;
    public static final int T__127=127;
    public static final int T__128=128;
    public static final int T__129=129;
    public static final int T__130=130;
    public static final int T__131=131;
    public static final int T__132=132;
    public static final int T__133=133;
    public static final int T__134=134;
    public static final int T__135=135;
    public static final int T__136=136;
    public static final int T__137=137;
    public static final int T__138=138;
    public static final int T__139=139;
    public static final int T__140=140;
    public static final int T__141=141;
    public static final int T__142=142;
    public static final int T__143=143;
    public static final int T__144=144;
    public static final int T__145=145;
    public static final int T__146=146;
    public static final int T__147=147;
    public static final int T__148=148;
    public static final int T__149=149;
    public static final int T__150=150;
    public static final int T__151=151;
    public static final int T__152=152;
    public static final int T__153=153;
    public static final int T__154=154;
    public static final int T__155=155;
    public static final int T__156=156;
    public static final int T__157=157;
    public static final int T__158=158;
    public static final int T__159=159;
    public static final int T__160=160;
    public static final int T__161=161;
    public static final int T__162=162;
    public static final int T__163=163;
    public static final int T__164=164;
    public static final int T__165=165;
    public static final int T__166=166;
    public static final int AFTER=4;
    public static final int AGAINST=5;
    public static final int AGO=6;
    public static final int ALL=7;
    public static final int ALPHA=8;
    public static final int AND=9;
    public static final int AND_PRED=10;
    public static final int AS=11;
    public static final int ASC=12;
    public static final int BEFORE=13;
    public static final int BEGIN=14;
    public static final int BETWEEN=15;
    public static final int BOOLEAN=16;
    public static final int BROWSE=17;
    public static final int BY=18;
    public static final int BYTE=19;
    public static final int BYTEARRAY=20;
    public static final int CHARACTER_LITERAL=21;
    public static final int COLON=22;
    public static final int COLUMN_LIST=23;
    public static final int COMMA=24;
    public static final int COMMENT=25;
    public static final int CONTAINS=26;
    public static final int DATE=27;
    public static final int DAYS=28;
    public static final int DEFINED=29;
    public static final int DESC=30;
    public static final int DESCRIBE=31;
    public static final int DIGIT=32;
    public static final int DOUBLE=33;
    public static final int ELSE=34;
    public static final int END=35;
    public static final int EQUAL=36;
    public static final int EQUAL_PRED=37;
    public static final int ESCAPE_SEQUENCE=38;
    public static final int EXCEPT=39;
    public static final int EXPONENT=40;
    public static final int FACET=41;
    public static final int FALSE=42;
    public static final int FAST_UTIL_DATA_TYPE=43;
    public static final int FETCHING=44;
    public static final int FLOATING_POINT_LITERAL=45;
    public static final int FLOAT_TYPE_SUFFIX=46;
    public static final int FROM=47;
    public static final int GIVEN=48;
    public static final int GROUP=49;
    public static final int GT=50;
    public static final int GTE=51;
    public static final int HEX_DIGIT=52;
    public static final int HEX_LITERAL=53;
    public static final int HITS=54;
    public static final int HOURS=55;
    public static final int IDENT=56;
    public static final int IN=57;
    public static final int INT=58;
    public static final int INTEGER=59;
    public static final int INTEGER_TYPE_SUFFIX=60;
    public static final int IS=61;
    public static final int LAST=62;
    public static final int LIKE=63;
    public static final int LIMIT=64;
    public static final int LINE_COMMENT=65;
    public static final int LONG=66;
    public static final int LPAR=67;
    public static final int LT=68;
    public static final int LTE=69;
    public static final int MATCH=70;
    public static final int MILLISECONDS=71;
    public static final int MINS=72;
    public static final int MINUTES=73;
    public static final int MODEL=74;
    public static final int MSECS=75;
    public static final int NOT=76;
    public static final int NOT_EQUAL=77;
    public static final int NOW=78;
    public static final int NULL=79;
    public static final int OCTAL_ESCAPE=80;
    public static final int OCTAL_LITERAL=81;
    public static final int OR=82;
    public static final int ORDER=83;
    public static final int OR_PRED=84;
    public static final int PARAM=85;
    public static final int QUERY=86;
    public static final int RANGE_PRED=87;
    public static final int REAL=88;
    public static final int RELEVANCE=89;
    public static final int ROUTE=90;
    public static final int RPAR=91;
    public static final int SECONDS=92;
    public static final int SECS=93;
    public static final int SELECT=94;
    public static final int SEMI=95;
    public static final int SINCE=96;
    public static final int STORED=97;
    public static final int STRING=98;
    public static final int STRING_LITERAL=99;
    public static final int TIME=100;
    public static final int TOP=101;
    public static final int TRUE=102;
    public static final int UNICODE_ESCAPE=103;
    public static final int USING=104;
    public static final int VALUE=105;
    public static final int VARIABLE=106;
    public static final int WEEKS=107;
    public static final int WHERE=108;
    public static final int WITH=109;
    public static final int WS=110;

    // delegates
    public Parser[] getDelegates() {
        return new Parser[] {};
    }

    // delegators


    public BQLParser(TokenStream input) {
        this(input, new RecognizerSharedState());
    }
    public BQLParser(TokenStream input, RecognizerSharedState state) {
        super(input, state);
        this.state.ruleMemo = new HashMap[356+1];
         

    }

protected TreeAdaptor adaptor = new CommonTreeAdaptor();

public void setTreeAdaptor(TreeAdaptor adaptor) {
    this.adaptor = adaptor;
}
public TreeAdaptor getTreeAdaptor() {
    return adaptor;
}
    public String[] getTokenNames() { return BQLParser.tokenNames; }
    public String getGrammarFileName() { return "com/senseidb/bql/parsers/BQL.g"; }



        private static final int DEFAULT_REQUEST_OFFSET = 0;
        private static final int DEFAULT_REQUEST_COUNT = 10;
        private static final int DEFAULT_REQUEST_MAX_PER_GROUP = 10;
        private static final int DEFAULT_FACET_MINHIT = 1;
        private static final int DEFAULT_FACET_MAXHIT = 10;
        private static final Map<String, String> _fastutilTypeMap;
        private static final Map<String, String> _internalVarMap;
        private static final Set<String> _supportedClasses;
        private static Map<String, Set<String>> _compatibleFacetTypes;

        private Map<String, String[]> _facetInfoMap;
        private long _now;
        private HashSet<String> _variables;
        private SimpleDateFormat[] _format1 = new SimpleDateFormat[2];
        private SimpleDateFormat[] _format2 = new SimpleDateFormat[2];

        private LinkedList<Map<String, String>> _symbolTable;
        private Map<String, String> _currentScope;
        private Set<String> _usedFacets; // Facets used by relevance model
        private Set<String> _usedInternalVars; // Internal variables used by relevance model

        static {
            _fastutilTypeMap = new HashMap<String, String>();
            _fastutilTypeMap.put("IntOpenHashSet", "set_int");
            _fastutilTypeMap.put("FloatOpenHashSet", "set_float");
            _fastutilTypeMap.put("DoubleOpenHashSet", "set_double");
            _fastutilTypeMap.put("LongOpenHashSet", "set_long");
            _fastutilTypeMap.put("ObjectOpenHashSet", "set_string");

            _fastutilTypeMap.put("Int2IntOpenHashMap", "map_int_int");
            _fastutilTypeMap.put("Int2FloatOpenHashMap", "map_int_float");
            _fastutilTypeMap.put("Int2DoubleOpenHashMap", "map_int_double");
            _fastutilTypeMap.put("Int2LongOpenHashMap", "map_int_long");
            _fastutilTypeMap.put("Int2ObjectOpenHashMap", "map_int_string");

            _fastutilTypeMap.put("Object2IntOpenHashMap", "map_string_int");
            _fastutilTypeMap.put("Object2FloatOpenHashMap", "map_string_float");
            _fastutilTypeMap.put("Object2DoubleOpenHashMap", "map_string_double");
            _fastutilTypeMap.put("Object2LongOpenHashMap", "map_string_long");
            _fastutilTypeMap.put("Object2ObjectOpenHashMap", "map_string_string");

            _internalVarMap = new HashMap<String, String>();
            _internalVarMap.put("_NOW", "long");
            _internalVarMap.put("_INNER_SCORE", "float");

            _supportedClasses = new HashSet<String>();
            _supportedClasses.add("Boolean");
            _supportedClasses.add("Byte");
            _supportedClasses.add("Character");
            _supportedClasses.add("Double");
            _supportedClasses.add("Integer");
            _supportedClasses.add("Long");
            _supportedClasses.add("Short");

            _supportedClasses.add("Math");
            _supportedClasses.add("String");
            _supportedClasses.add("System");

            _compatibleFacetTypes = new HashMap<String, Set<String>>();
            _compatibleFacetTypes.put("range", new HashSet<String>(Arrays.asList(new String[]
                                                                   {
                                                                       "simple",
                                                                       "multi"
                                                                   })));
        }

        public BQLParser(TokenStream input, Map<String, String[]> facetInfoMap)
        {
            this(input);
            _facetInfoMap = facetInfoMap;
            _facetInfoMap.put("_uid", new String[]{"simple", "long"});
        }

        // The following two overridden methods are used to force ANTLR to
        // stop parsing upon the very first error.

        // @Override
        // protected void mismatch(IntStream input, int ttype, BitSet follow)
        //     throws RecognitionException
        // {
        //     throw new MismatchedTokenException(ttype, input);
        // }

        @Override
        protected Object recoverFromMismatchedToken(IntStream input, int ttype, BitSet follow)
            throws RecognitionException
        {
            throw new MismatchedTokenException(ttype, input);
        }

        @Override
        public Object recoverFromMismatchedSet(IntStream input,
                                               RecognitionException e,
                                               BitSet follow)
            throws RecognitionException
        {
            throw e;
        }

        @Override
        public String getErrorMessage(RecognitionException err, String[] tokenNames) 
        {
            // List stack = getRuleInvocationStack(err, this.getClass().getName());
            String msg = null; 
            if (err instanceof NoViableAltException) {
                NoViableAltException nvae = (NoViableAltException) err;
                // msg = "No viable alt; token=" + err.token.getText() +
                //     " (decision=" + nvae.decisionNumber +
                //     " state "+nvae.stateNumber+")" +
                //     " decision=<<" + nvae.grammarDecisionDescription + ">>";
                msg = "[line:" + err.line + ", col:" + err.charPositionInLine + "] " +
                    "No viable alternative (token=" + err.token.getText() + ")";
            }
            else if (err instanceof MismatchedTokenException) {
                MismatchedTokenException mte = (MismatchedTokenException) err;
                String tokenName = (mte.expecting == Token.EOF) ? "EOF" : tokenNames[mte.expecting];
                msg = "[line:" + mte.line + ", col:" + mte.charPositionInLine + "] " +
                    "Expecting " + tokenName +
                    " (token=" + err.token.getText() + ")";
            }
            else if (err instanceof FailedPredicateException) {
                FailedPredicateException fpe = (FailedPredicateException) err;
                msg = "[line:" + fpe.line + ", col:" + fpe.charPositionInLine + "] " +
                    fpe.predicateText +
                    " (token=" + fpe.token.getText() + ")";
            }
            else if (err instanceof MismatchedSetException) {
                MismatchedSetException mse = (MismatchedSetException) err;
                msg = "[line:" + mse.line + ", col:" + mse.charPositionInLine + "] " +
                    "Mismatched input (token=" + mse.token.getText() + ")";
            }
            else {
                msg = super.getErrorMessage(err, tokenNames); 
            }
            return msg;
        } 

        @Override
        public String getTokenErrorDisplay(Token t)
        {
            return t.toString();
        }

        private String predType(JSONObject pred)
        {
            return (String) pred.keys().next();
        }

        private String predField(JSONObject pred) throws JSONException
        {
            String type = (String) pred.keys().next();
            JSONObject fieldSpec = pred.getJSONObject(type);
            return (String) fieldSpec.keys().next();
        }

        private boolean verifyFacetType(final String field, final String expectedType)
        {
            String[] facetInfo = _facetInfoMap.get(field);
            if (facetInfo != null) {
                Set<String> compatibleTypes = _compatibleFacetTypes.get(expectedType);
                return (expectedType.equals(facetInfo[0]) ||
                        "custom".equals(facetInfo[0]) ||
                        (compatibleTypes != null && compatibleTypes.contains(facetInfo[0])));
            }
            else {
                return true;
            }
        }

        private boolean verifyValueType(Object value, final String columnType)
        {
            if (value instanceof String &&
                !((String) value).isEmpty() &&
                ((String) value).matches("\\$[^$].*"))
            {
                // This "value" is a variable, return true always
                return true;
            }

            if (columnType.equals("long") || columnType.equals("int") || columnType.equals("short")) {
                return !(value instanceof Float || value instanceof String || value instanceof Boolean);
            }
            else if (columnType.equals("float") || columnType.equals("double")) {
                return !(value instanceof String || value instanceof Boolean);
            }
            else if (columnType.equals("string") || columnType.equals("char")) {
                return (value instanceof String);
            }
            else if (columnType.equals("boolean")) {
                return (value instanceof Boolean);
            }
            else if (columnType.isEmpty()) {
                // For a custom facet, the data type is unknown (empty
                // string).  We accept all value types here.
                return true;
            }
            else {
                return false;
            }
        }

        private boolean verifyFieldDataType(final String field, Object value)
            throws FailedPredicateException
        {
            String[] facetInfo = _facetInfoMap.get(field);

            if (value instanceof String &&
                !((String) value).isEmpty() &&
                ((String) value).matches("\\$[^$].*"))
            {
                // This "value" is a variable, return true always
                return true;
            }
            else if (value instanceof JSONArray)
            {
                try {
                    if (facetInfo != null) {
                        String columnType = facetInfo[1];
                        for (int i = 0; i < ((JSONArray) value).length(); ++i) {
                            if (!verifyValueType(((JSONArray) value).get(i), columnType)) {
                                return false;
                            }
                        }
                    }
                    return true;
                }
                catch (JSONException err) {
                    throw new FailedPredicateException(input, "verifyFieldDataType", "JSONException: " + err.getMessage());
                }
            }
            else
            {
                if (facetInfo != null) {
                    return verifyValueType(value, facetInfo[1]);
                }
                else {
                    // Field is not a facet
                    return true;
                }
            }
        }

        private boolean verifyFieldDataType(final String field, Object[] values)
        {
            String[] facetInfo = _facetInfoMap.get(field);
            if (facetInfo != null) {
                String columnType = facetInfo[1];
                for (Object value: values) {
                    if (!verifyValueType(value, columnType)) {
                        return false;
                    }
                }
            }
            return true;
        }

        private void extractSelectionInfo(JSONObject where,
                                          JSONArray selections,
                                          JSONObject filter,
                                          JSONObject query) throws JSONException {

            JSONObject queryPred = where.optJSONObject("query");
            JSONArray andPreds = null;

            if (queryPred != null) {
                query.put("query", queryPred);
            }
            else if ((andPreds = where.optJSONArray("and")) != null) {
                JSONArray filter_list = new JSONArray();
                for (int i = 0; i < andPreds.length(); ++i) {
                    JSONObject pred = andPreds.getJSONObject(i);
                    queryPred = pred.optJSONObject("query");
                    if (queryPred != null) {
                        if (!query.has("query")) {
                            query.put("query", queryPred);
                        }
                        else {
                            filter_list.put(pred);
                        }
                    }
                    else if (pred.has("and") || pred.has("or") || pred.has("isNull")) {
                        filter_list.put(pred);
                    }
                    else {
                        String[] facetInfo = _facetInfoMap.get(predField(pred));
                        if (facetInfo != null) {
                            if ("range".equals(predType(pred)) && !"range".equals(facetInfo[0])) {
                                filter_list.put(pred);
                            }
                            else {
                                selections.put(pred);
                            }
                        }
                        else {
                            filter_list.put(pred);
                        }
                    }
                }
                if (filter_list.length() > 1) {
                    filter.put("filter", new JSONObject().put("and", filter_list));
                }
                else if (filter_list.length() == 1) {
                    filter.put("filter", filter_list.get(0));
                }
            }
            else if (where.has("or") || where.has("isNull")) {
                filter.put("filter", where);
            }
            else {
                String[] facetInfo = _facetInfoMap.get(predField(where));
                if (facetInfo != null) {
                    if ("range".equals(predType(where)) && !"range".equals(facetInfo[0])) {
                        filter.put("filter", where);
                    }
                    else {
                        selections.put(where);
                    }
                }
                else {
                    filter.put("filter", where);
                }
            }
        }

        private int compareValues(Object v1, Object v2)
        {
            if (v1 instanceof String) {
                return ((String) v1).compareTo((String) v2);
            }
            else if (v1 instanceof Integer) {
                return ((Integer) v1).compareTo((Integer) v2);
            }
            else if (v1 instanceof Long) {
                return ((Long) v1).compareTo((Long) v2);
            }
            else if (v1 instanceof Float) {
                return ((Float) v1).compareTo((Float) v2);
            }
            return 0;
        }

        private Object[] getMax(Object value1, boolean include1, Object value2, boolean include2)
        {
            Object value;
            Boolean include;
            if (value1 == null) {
                value = value2;
                include = include2;
            }
            else if (value2 == null) {
                value = value1;
                include = include1;
            }
            else {
                int comp = compareValues(value1, value2);
                if (comp > 0) {
                    value = value1;
                    include = include1;
                }
                else if (comp == 0) {
                    value = value1;
                    include = (include1 && include2);
                }
                else {
                    value = value2;
                    include = include2;
                }
            }
            return new Object[]{value, include};
        }

        private Object[] getMin(Object value1, boolean include1, Object value2, boolean include2)
        {
            Object value;
            Boolean include;
            if (value1 == null) {
                value = value2;
                include = include2;
            }
            else if (value2 == null) {
                value = value1;
                include = include1;
            }
            else {
                int comp = compareValues(value1, value2);
                if (comp > 0) {
                    value = value2;
                    include = include2;
                }
                else if (comp == 0) {
                    value = value1;
                    include = (include1 && include2);
                }
                else {
                    value = value1;
                    include = include1;
                }
            }
            return new Object[]{value, include};
        }

        private void accumulateRangePred(TokenStream input, JSONObject fieldMap, JSONObject pred) 
            throws JSONException, RecognitionException
        {
            String field = predField(pred);
            if (!fieldMap.has(field)) {
                fieldMap.put(field, pred);
                return;
            }
            JSONObject oldRange = (JSONObject) fieldMap.get(field);
            JSONObject oldSpec = (JSONObject) ((JSONObject) oldRange.get("range")).get(field);
            Object oldFrom = oldSpec.opt("from");
            Object oldTo = oldSpec.opt("to");
            Boolean oldIncludeLower = oldSpec.optBoolean("include_lower", false);
            Boolean oldIncludeUpper = oldSpec.optBoolean("include_upper", false);

            JSONObject curSpec = (JSONObject) ((JSONObject) pred.get("range")).get(field);
            Object curFrom = curSpec.opt("from");
            Object curTo = curSpec.opt("to");
            Boolean curIncludeLower = curSpec.optBoolean("include_lower", false);
            Boolean curIncludeUpper = curSpec.optBoolean("include_upper", false);

            Object[] result = getMax(oldFrom, oldIncludeLower, curFrom, curIncludeLower);
            Object newFrom = result[0];
            Boolean newIncludeLower = (Boolean) result[1];
            result = getMin(oldTo, oldIncludeUpper, curTo, curIncludeUpper);
            Object newTo = result[0];
            Boolean newIncludeUpper = (Boolean) result[1];

            if (newFrom != null && newTo != null) {
                if (compareValues(newFrom, newTo) > 0 ||
                    (compareValues(newFrom, newTo) == 0) && (!newIncludeLower || !newIncludeUpper)) {
                    // This error is in general detected late, so the token
                    // can be a little off, but hopefully the col index info
                    // is good enough.
                    throw new FailedPredicateException(input, "", "Inconsistent ranges detected for column: " + field);
                }
            }
            
            JSONObject newSpec = new JSONObject();
            if (newFrom != null) {
                newSpec.put("from", newFrom);
                newSpec.put("include_lower", newIncludeLower);
            }
            if (newTo != null) {
                newSpec.put("to", newTo);
                newSpec.put("include_upper", newIncludeUpper);
            }

            fieldMap.put(field, new JSONObject().put("range",
                                                     new JSONObject().put(field, newSpec)));
        }
        
        private void processRelevanceModelParam(TokenStream input,
                                                JSONObject json,
                                                Set<String> params,
                                                String typeName,
                                                final String varName)
            throws JSONException, RecognitionException
        {
            if (_facetInfoMap.containsKey(varName)) {
                throw new FailedPredicateException(input, "", "Facet name \"" + varName + "\" cannot be used as a relevance model parameter.");
            }

            if (_internalVarMap.containsKey(varName)) {
                throw new FailedPredicateException(input, "", "Internal variable \"" + varName + "\" cannot be used as a relevance model parameter.");
            }

            if (params.contains(varName)) {
                throw new FailedPredicateException(input, "", "Parameter name \"" + varName + "\" has already been used.");
            }

            if ("String".equals(typeName)) {
                typeName = "string";
            }

            JSONArray funcParams = json.optJSONArray("function_params");
            if (funcParams == null) {
                funcParams = new JSONArray();
                json.put("function_params", funcParams);
            }

            funcParams.put(varName);
            params.add(varName);

            JSONObject variables = json.optJSONObject("variables");
            if (variables == null) {
                variables = new JSONObject();
                json.put("variables", variables);
            }

            JSONArray varsWithSameType = variables.optJSONArray(typeName);
            if (varsWithSameType == null) {
                varsWithSameType = new JSONArray();
                variables.put(typeName, varsWithSameType);
            }
            varsWithSameType.put(varName);
        }

        // Check whether a variable is defined.
        private boolean verifyVariable(final String variable) {
            Iterator<Map<String, String>> itr = _symbolTable.descendingIterator();
            while (itr.hasNext()) {
                Map<String, String> scope = itr.next();
                if (scope.containsKey(variable)) {
                    return true;
                }
            }
            return false;
        }


    public static class statement_return extends ParserRuleReturnScope {
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "statement"
    // com/senseidb/bql/parsers/BQL.g:1180:1: statement returns [Object json] : ( select_stmt | describe_stmt ) ( SEMI )? EOF ;
    public final BQLParser.statement_return statement() throws RecognitionException {
        BQLParser.statement_return retval = new BQLParser.statement_return();
        retval.start = input.LT(1);

        int statement_StartIndex = input.index();

        Object root_0 = null;

        Token SEMI3=null;
        Token EOF4=null;
        BQLParser.select_stmt_return select_stmt1 =null;

        BQLParser.describe_stmt_return describe_stmt2 =null;


        Object SEMI3_tree=null;
        Object EOF4_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 1) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1181:5: ( ( select_stmt | describe_stmt ) ( SEMI )? EOF )
            // com/senseidb/bql/parsers/BQL.g:1181:9: ( select_stmt | describe_stmt ) ( SEMI )? EOF
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:1181:9: ( select_stmt | describe_stmt )
            int alt1=2;
            switch ( input.LA(1) ) {
            case SELECT:
                {
                alt1=1;
                }
                break;
            case DESCRIBE:
                {
                alt1=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 1, 0, input);

                throw nvae;

            }

            switch (alt1) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1181:13: select_stmt
                    {
                    pushFollow(FOLLOW_select_stmt_in_statement3529);
                    select_stmt1=select_stmt();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, select_stmt1.getTree());

                    if ( state.backtracking==0 ) { retval.json = (select_stmt1!=null?select_stmt1.json:null); }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1182:13: describe_stmt
                    {
                    pushFollow(FOLLOW_describe_stmt_in_statement3545);
                    describe_stmt2=describe_stmt();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, describe_stmt2.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:1183:13: ( SEMI )?
            int alt2=2;
            switch ( input.LA(1) ) {
                case SEMI:
                    {
                    alt2=1;
                    }
                    break;
            }

            switch (alt2) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1183:13: SEMI
                    {
                    SEMI3=(Token)match(input,SEMI,FOLLOW_SEMI_in_statement3559); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    SEMI3_tree = 
                    (Object)adaptor.create(SEMI3)
                    ;
                    adaptor.addChild(root_0, SEMI3_tree);
                    }

                    }
                    break;

            }


            EOF4=(Token)match(input,EOF,FOLLOW_EOF_in_statement3562); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            EOF4_tree = 
            (Object)adaptor.create(EOF4)
            ;
            adaptor.addChild(root_0, EOF4_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 1, statement_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "statement"


    public static class select_stmt_return extends ParserRuleReturnScope {
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "select_stmt"
    // com/senseidb/bql/parsers/BQL.g:1186:1: select_stmt returns [Object json] : SELECT ( '*' |cols= column_name_list ) ( FROM ( IDENT | STRING_LITERAL ) )? (w= where )? (given= given_clause )? (order_by= order_by_clause |limit= limit_clause |group_by= group_by_clause |browse_by= browse_by_clause |fetch_stored= fetching_stored_clause |route_param= route_by_clause |rel_model= relevance_model_clause )* ;
    public final BQLParser.select_stmt_return select_stmt() throws RecognitionException {
        BQLParser.select_stmt_return retval = new BQLParser.select_stmt_return();
        retval.start = input.LT(1);

        int select_stmt_StartIndex = input.index();

        Object root_0 = null;

        Token SELECT5=null;
        Token char_literal6=null;
        Token FROM7=null;
        Token set8=null;
        BQLParser.column_name_list_return cols =null;

        BQLParser.where_return w =null;

        BQLParser.given_clause_return given =null;

        BQLParser.order_by_clause_return order_by =null;

        BQLParser.limit_clause_return limit =null;

        BQLParser.group_by_clause_return group_by =null;

        BQLParser.browse_by_clause_return browse_by =null;

        BQLParser.fetching_stored_clause_return fetch_stored =null;

        BQLParser.route_by_clause_return route_param =null;

        BQLParser.relevance_model_clause_return rel_model =null;


        Object SELECT5_tree=null;
        Object char_literal6_tree=null;
        Object FROM7_tree=null;
        Object set8_tree=null;


            _now = System.currentTimeMillis();
            _variables = new HashSet<String>();
            boolean seenOrderBy = false;
            boolean seenLimit = false;
            boolean seenGroupBy = false;
            boolean seenBrowseBy = false;
            boolean seenFetchStored = false;
            boolean seenRouteBy = false;
            boolean seenRelevanceModel = false;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 2) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1198:5: ( SELECT ( '*' |cols= column_name_list ) ( FROM ( IDENT | STRING_LITERAL ) )? (w= where )? (given= given_clause )? (order_by= order_by_clause |limit= limit_clause |group_by= group_by_clause |browse_by= browse_by_clause |fetch_stored= fetching_stored_clause |route_param= route_by_clause |rel_model= relevance_model_clause )* )
            // com/senseidb/bql/parsers/BQL.g:1198:9: SELECT ( '*' |cols= column_name_list ) ( FROM ( IDENT | STRING_LITERAL ) )? (w= where )? (given= given_clause )? (order_by= order_by_clause |limit= limit_clause |group_by= group_by_clause |browse_by= browse_by_clause |fetch_stored= fetching_stored_clause |route_param= route_by_clause |rel_model= relevance_model_clause )*
            {
            root_0 = (Object)adaptor.nil();


            SELECT5=(Token)match(input,SELECT,FOLLOW_SELECT_in_select_stmt3590); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            SELECT5_tree = 
            (Object)adaptor.create(SELECT5)
            ;
            adaptor.addChild(root_0, SELECT5_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:1198:16: ( '*' |cols= column_name_list )
            int alt3=2;
            switch ( input.LA(1) ) {
            case 118:
                {
                alt3=1;
                }
                break;
            case IDENT:
            case STRING_LITERAL:
                {
                alt3=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 3, 0, input);

                throw nvae;

            }

            switch (alt3) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1198:17: '*'
                    {
                    char_literal6=(Token)match(input,118,FOLLOW_118_in_select_stmt3593); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal6_tree = 
                    (Object)adaptor.create(char_literal6)
                    ;
                    adaptor.addChild(root_0, char_literal6_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1198:23: cols= column_name_list
                    {
                    pushFollow(FOLLOW_column_name_list_in_select_stmt3599);
                    cols=column_name_list();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, cols.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:1199:9: ( FROM ( IDENT | STRING_LITERAL ) )?
            int alt4=2;
            switch ( input.LA(1) ) {
                case FROM:
                    {
                    alt4=1;
                    }
                    break;
            }

            switch (alt4) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1199:10: FROM ( IDENT | STRING_LITERAL )
                    {
                    FROM7=(Token)match(input,FROM,FOLLOW_FROM_in_select_stmt3611); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    FROM7_tree = 
                    (Object)adaptor.create(FROM7)
                    ;
                    adaptor.addChild(root_0, FROM7_tree);
                    }

                    set8=(Token)input.LT(1);

                    if ( input.LA(1)==IDENT||input.LA(1)==STRING_LITERAL ) {
                        input.consume();
                        if ( state.backtracking==0 ) adaptor.addChild(root_0, 
                        (Object)adaptor.create(set8)
                        );
                        state.errorRecovery=false;
                        state.failed=false;
                    }
                    else {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        MismatchedSetException mse = new MismatchedSetException(null,input);
                        throw mse;
                    }


                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:1200:10: (w= where )?
            int alt5=2;
            switch ( input.LA(1) ) {
                case WHERE:
                    {
                    alt5=1;
                    }
                    break;
            }

            switch (alt5) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1200:10: w= where
                    {
                    pushFollow(FOLLOW_where_in_select_stmt3633);
                    w=where();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, w.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:1201:14: (given= given_clause )?
            int alt6=2;
            switch ( input.LA(1) ) {
                case GIVEN:
                    {
                    alt6=1;
                    }
                    break;
            }

            switch (alt6) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1201:14: given= given_clause
                    {
                    pushFollow(FOLLOW_given_clause_in_select_stmt3646);
                    given=given_clause();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, given.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:1202:9: (order_by= order_by_clause |limit= limit_clause |group_by= group_by_clause |browse_by= browse_by_clause |fetch_stored= fetching_stored_clause |route_param= route_by_clause |rel_model= relevance_model_clause )*
            loop7:
            do {
                int alt7=8;
                switch ( input.LA(1) ) {
                case ORDER:
                    {
                    alt7=1;
                    }
                    break;
                case LIMIT:
                    {
                    alt7=2;
                    }
                    break;
                case GROUP:
                    {
                    alt7=3;
                    }
                    break;
                case BROWSE:
                    {
                    alt7=4;
                    }
                    break;
                case FETCHING:
                    {
                    alt7=5;
                    }
                    break;
                case ROUTE:
                    {
                    alt7=6;
                    }
                    break;
                case USING:
                    {
                    alt7=7;
                    }
                    break;

                }

                switch (alt7) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1202:13: order_by= order_by_clause
            	    {
            	    pushFollow(FOLLOW_order_by_clause_in_select_stmt3665);
            	    order_by=order_by_clause();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, order_by.getTree());

            	    if ( state.backtracking==0 ) { 
            	                    if (seenOrderBy) {
            	                        throw new FailedPredicateException(input, "select_stmt", "ORDER BY clause can only appear once.");
            	                    }
            	                    else {
            	                        seenOrderBy = true;
            	                    }
            	                }

            	    }
            	    break;
            	case 2 :
            	    // com/senseidb/bql/parsers/BQL.g:1211:13: limit= limit_clause
            	    {
            	    pushFollow(FOLLOW_limit_clause_in_select_stmt3698);
            	    limit=limit_clause();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, limit.getTree());

            	    if ( state.backtracking==0 ) { 
            	                    if (seenLimit) {
            	                        throw new FailedPredicateException(input, "select_stmt", "LIMIT clause can only appear once.");
            	                    }
            	                    else {
            	                        seenLimit = true;
            	                    }
            	                }

            	    }
            	    break;
            	case 3 :
            	    // com/senseidb/bql/parsers/BQL.g:1220:13: group_by= group_by_clause
            	    {
            	    pushFollow(FOLLOW_group_by_clause_in_select_stmt3730);
            	    group_by=group_by_clause();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, group_by.getTree());

            	    if ( state.backtracking==0 ) { 
            	                    if (seenGroupBy) {
            	                        throw new FailedPredicateException(input, "select_stmt", "GROUP BY clause can only appear once.");
            	                    }
            	                    else {
            	                        seenGroupBy = true;
            	                    }
            	                }

            	    }
            	    break;
            	case 4 :
            	    // com/senseidb/bql/parsers/BQL.g:1229:13: browse_by= browse_by_clause
            	    {
            	    pushFollow(FOLLOW_browse_by_clause_in_select_stmt3762);
            	    browse_by=browse_by_clause();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, browse_by.getTree());

            	    if ( state.backtracking==0 ) { 
            	                    if (seenBrowseBy) {
            	                        throw new FailedPredicateException(input, "select_stmt", "BROWSE BY clause can only appear once.");
            	                    }
            	                    else {
            	                        seenBrowseBy = true;
            	                    }
            	                }

            	    }
            	    break;
            	case 5 :
            	    // com/senseidb/bql/parsers/BQL.g:1238:13: fetch_stored= fetching_stored_clause
            	    {
            	    pushFollow(FOLLOW_fetching_stored_clause_in_select_stmt3794);
            	    fetch_stored=fetching_stored_clause();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, fetch_stored.getTree());

            	    if ( state.backtracking==0 ) { 
            	                    if (seenFetchStored) {
            	                        throw new FailedPredicateException(input, "select_stmt", "FETCHING STORED clause can only appear once.");
            	                    }
            	                    else {
            	                        seenFetchStored = true;
            	                    }
            	                }

            	    }
            	    break;
            	case 6 :
            	    // com/senseidb/bql/parsers/BQL.g:1247:13: route_param= route_by_clause
            	    {
            	    pushFollow(FOLLOW_route_by_clause_in_select_stmt3826);
            	    route_param=route_by_clause();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, route_param.getTree());

            	    if ( state.backtracking==0 ) {
            	                    if (seenRouteBy) {
            	                        throw new FailedPredicateException(input, "select_stmt", "ROUTE BY clause can only appear once.");
            	                    }
            	                    else {
            	                        seenRouteBy = true;
            	                    }
            	                }

            	    }
            	    break;
            	case 7 :
            	    // com/senseidb/bql/parsers/BQL.g:1256:13: rel_model= relevance_model_clause
            	    {
            	    pushFollow(FOLLOW_relevance_model_clause_in_select_stmt3858);
            	    rel_model=relevance_model_clause();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, rel_model.getTree());

            	    if ( state.backtracking==0 ) {
            	                    if (seenRelevanceModel) {
            	                        throw new FailedPredicateException(input, "select_stmt", "USING RELEVANCE MODEL clause can only appear once.");
            	                    }
            	                    else {
            	                        seenRelevanceModel = true;
            	                    }
            	                }

            	    }
            	    break;

            	default :
            	    break loop7;
                }
            } while (true);


            if ( state.backtracking==0 ) {
                        JSONObject jsonObj = new JSONObject();
                        JSONArray selections = new JSONArray();
                        JSONObject filter = new JSONObject();
                        JSONObject query = new JSONObject();

                        try {
                            JSONObject metaData = new JSONObject();
                            if (cols == null) {
                                metaData.put("select_list", new JSONArray().put("*"));
                            }
                            else {
                               metaData.put("select_list", (cols!=null?cols.json:null));
                               if ((cols!=null?cols.fetchStored:false)) {
                                   jsonObj.put("fetchStored", true);
                               }
                            }

                            if (_variables.size() > 0)
                            {
                                metaData.put("variables", new JSONArray(_variables));
                            }

                            jsonObj.put("meta", metaData);

                            if (order_by != null) {
                                if ((order_by!=null?order_by.isRelevance:false)) {
                                    jsonObj.put("sort", "relevance");
                                }
                                else {
                                    jsonObj.put("sort", (order_by!=null?order_by.json:null));
                                }
                            }
                            if (limit != null) {
                                jsonObj.put("from", (limit!=null?limit.offset:0));
                                jsonObj.put("size", (limit!=null?limit.count:0));
                            }
                            if (group_by != null) {
                                jsonObj.put("groupBy", (group_by!=null?group_by.json:null));
                            }
                            if (browse_by != null) {
                                jsonObj.put("facets", (browse_by!=null?browse_by.json:null));
                            }
                            if (fetch_stored != null) {
                                if (!(fetch_stored!=null?fetch_stored.val:false) && (cols != null && (cols!=null?cols.fetchStored:false))) {
                                    throw new FailedPredicateException(input, 
                                                                       "select_stmt",
                                                                       "FETCHING STORED cannot be false when _srcdata is selected.");
                                }
                                else if ((fetch_stored!=null?fetch_stored.val:false)) {
                                    // Default is false
                                    jsonObj.put("fetchStored", (fetch_stored!=null?fetch_stored.val:false));
                                }
                            }
                            if (route_param != null) {
                                jsonObj.put("routeParam", (route_param!=null?route_param.val:null));
                            }

                            if (w != null) {
                                extractSelectionInfo((JSONObject) (w!=null?w.json:null), selections, filter, query);
                                JSONObject queryPred = query.optJSONObject("query");
                                if (queryPred != null) {
                                    jsonObj.put("query", queryPred);
                                }
                                if (selections.length() > 0) {
                                    jsonObj.put("selections", selections);
                                }
                                JSONObject f = filter.optJSONObject("filter");
                                if (f != null) {
                                    jsonObj.put("filter", f);
                                }
                            }

                            if (given != null) {
                                jsonObj.put("facetInit", (given!=null?given.json:null));
                            }

                            if (rel_model != null) {
                                JSONObject queryPred = jsonObj.optJSONObject("query");
                                if (queryPred == null) {
                                    queryPred = new JSONObject().put("query_string",
                                                                     new JSONObject().put("query", "")
                                                                                     .put("relevance", (rel_model!=null?rel_model.json:null)));
                                    jsonObj.put("query", queryPred);
                                }
                            }
                            
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "select_stmt", "JSONException: " + err.getMessage());
                        }

                        retval.json = jsonObj;
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 2, select_stmt_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "select_stmt"


    public static class describe_stmt_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "describe_stmt"
    // com/senseidb/bql/parsers/BQL.g:1364:1: describe_stmt : DESCRIBE ( IDENT | STRING_LITERAL ) ;
    public final BQLParser.describe_stmt_return describe_stmt() throws RecognitionException {
        BQLParser.describe_stmt_return retval = new BQLParser.describe_stmt_return();
        retval.start = input.LT(1);

        int describe_stmt_StartIndex = input.index();

        Object root_0 = null;

        Token DESCRIBE9=null;
        Token set10=null;

        Object DESCRIBE9_tree=null;
        Object set10_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 3) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1365:5: ( DESCRIBE ( IDENT | STRING_LITERAL ) )
            // com/senseidb/bql/parsers/BQL.g:1365:9: DESCRIBE ( IDENT | STRING_LITERAL )
            {
            root_0 = (Object)adaptor.nil();


            DESCRIBE9=(Token)match(input,DESCRIBE,FOLLOW_DESCRIBE_in_describe_stmt3922); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            DESCRIBE9_tree = 
            (Object)adaptor.create(DESCRIBE9)
            ;
            adaptor.addChild(root_0, DESCRIBE9_tree);
            }

            set10=(Token)input.LT(1);

            if ( input.LA(1)==IDENT||input.LA(1)==STRING_LITERAL ) {
                input.consume();
                if ( state.backtracking==0 ) adaptor.addChild(root_0, 
                (Object)adaptor.create(set10)
                );
                state.errorRecovery=false;
                state.failed=false;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                MismatchedSetException mse = new MismatchedSetException(null,input);
                throw mse;
            }


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 3, describe_stmt_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "describe_stmt"


    public static class column_name_list_return extends ParserRuleReturnScope {
        public boolean fetchStored;
        public JSONArray json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "column_name_list"
    // com/senseidb/bql/parsers/BQL.g:1368:1: column_name_list returns [boolean fetchStored, JSONArray json] : col= column_name ( COMMA col= column_name )* -> ^( COLUMN_LIST ( column_name )+ ) ;
    public final BQLParser.column_name_list_return column_name_list() throws RecognitionException {
        BQLParser.column_name_list_return retval = new BQLParser.column_name_list_return();
        retval.start = input.LT(1);

        int column_name_list_StartIndex = input.index();

        Object root_0 = null;

        Token COMMA11=null;
        BQLParser.column_name_return col =null;


        Object COMMA11_tree=null;
        RewriteRuleTokenStream stream_COMMA=new RewriteRuleTokenStream(adaptor,"token COMMA");
        RewriteRuleSubtreeStream stream_column_name=new RewriteRuleSubtreeStream(adaptor,"rule column_name");

            retval.fetchStored = false;
            retval.json = new JSONArray();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 4) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1373:5: (col= column_name ( COMMA col= column_name )* -> ^( COLUMN_LIST ( column_name )+ ) )
            // com/senseidb/bql/parsers/BQL.g:1373:9: col= column_name ( COMMA col= column_name )*
            {
            pushFollow(FOLLOW_column_name_in_column_name_list3960);
            col=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_column_name.add(col.getTree());

            if ( state.backtracking==0 ) {
                        String colName = (col!=null?col.text:null);
                        if (colName != null) {
                            retval.json.put((col!=null?col.text:null)); 
                            if ("_srcdata".equals(colName) || colName.startsWith("_srcdata.")) {
                                retval.fetchStored = true;
                            }
                        }
                    }

            // com/senseidb/bql/parsers/BQL.g:1383:9: ( COMMA col= column_name )*
            loop8:
            do {
                int alt8=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt8=1;
                    }
                    break;

                }

                switch (alt8) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1383:10: COMMA col= column_name
            	    {
            	    COMMA11=(Token)match(input,COMMA,FOLLOW_COMMA_in_column_name_list3981); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_COMMA.add(COMMA11);


            	    pushFollow(FOLLOW_column_name_in_column_name_list3985);
            	    col=column_name();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_column_name.add(col.getTree());

            	    if ( state.backtracking==0 ) {
            	                    String colName = (col!=null?col.text:null);
            	                    if (colName != null) {
            	                        retval.json.put((col!=null?col.text:null)); 
            	                        if ("_srcdata".equals(colName) || colName.startsWith("_srcdata.")) {
            	                            retval.fetchStored = true;
            	                        }
            	                    }
            	                }

            	    }
            	    break;

            	default :
            	    break loop8;
                }
            } while (true);


            // AST REWRITE
            // elements: column_name
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Object)adaptor.nil();
            // 1394:9: -> ^( COLUMN_LIST ( column_name )+ )
            {
                // com/senseidb/bql/parsers/BQL.g:1394:12: ^( COLUMN_LIST ( column_name )+ )
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(
                (Object)adaptor.create(COLUMN_LIST, "COLUMN_LIST")
                , root_1);

                if ( !(stream_column_name.hasNext()) ) {
                    throw new RewriteEarlyExitException();
                }
                while ( stream_column_name.hasNext() ) {
                    adaptor.addChild(root_1, stream_column_name.nextTree());

                }
                stream_column_name.reset();

                adaptor.addChild(root_0, root_1);
                }

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 4, column_name_list_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "column_name_list"


    public static class column_name_return extends ParserRuleReturnScope {
        public String text;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "column_name"
    // com/senseidb/bql/parsers/BQL.g:1397:1: column_name returns [String text] : (id= IDENT |str= STRING_LITERAL ) ( '.' (id2= IDENT |str2= STRING_LITERAL ) )* ;
    public final BQLParser.column_name_return column_name() throws RecognitionException {
        BQLParser.column_name_return retval = new BQLParser.column_name_return();
        retval.start = input.LT(1);

        int column_name_StartIndex = input.index();

        Object root_0 = null;

        Token id=null;
        Token str=null;
        Token id2=null;
        Token str2=null;
        Token char_literal12=null;

        Object id_tree=null;
        Object str_tree=null;
        Object id2_tree=null;
        Object str2_tree=null;
        Object char_literal12_tree=null;


            StringBuilder builder = new StringBuilder();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 5) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1401:5: ( (id= IDENT |str= STRING_LITERAL ) ( '.' (id2= IDENT |str2= STRING_LITERAL ) )* )
            // com/senseidb/bql/parsers/BQL.g:1401:9: (id= IDENT |str= STRING_LITERAL ) ( '.' (id2= IDENT |str2= STRING_LITERAL ) )*
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:1401:9: (id= IDENT |str= STRING_LITERAL )
            int alt9=2;
            switch ( input.LA(1) ) {
            case IDENT:
                {
                alt9=1;
                }
                break;
            case STRING_LITERAL:
                {
                alt9=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 9, 0, input);

                throw nvae;

            }

            switch (alt9) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1401:10: id= IDENT
                    {
                    id=(Token)match(input,IDENT,FOLLOW_IDENT_in_column_name4058); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    id_tree = 
                    (Object)adaptor.create(id)
                    ;
                    adaptor.addChild(root_0, id_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1401:21: str= STRING_LITERAL
                    {
                    str=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_column_name4064); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    str_tree = 
                    (Object)adaptor.create(str)
                    ;
                    adaptor.addChild(root_0, str_tree);
                    }

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        if (id != null) {
                            builder.append((id!=null?id.getText():null));
                        }
                        else {
                            String orig = (str!=null?str.getText():null);
                            builder.append(orig.substring(1, orig.length() - 1));
                        }
                    }

            // com/senseidb/bql/parsers/BQL.g:1411:9: ( '.' (id2= IDENT |str2= STRING_LITERAL ) )*
            loop11:
            do {
                int alt11=2;
                switch ( input.LA(1) ) {
                case 126:
                    {
                    alt11=1;
                    }
                    break;

                }

                switch (alt11) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1411:10: '.' (id2= IDENT |str2= STRING_LITERAL )
            	    {
            	    char_literal12=(Token)match(input,126,FOLLOW_126_in_column_name4086); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    char_literal12_tree = 
            	    (Object)adaptor.create(char_literal12)
            	    ;
            	    adaptor.addChild(root_0, char_literal12_tree);
            	    }

            	    // com/senseidb/bql/parsers/BQL.g:1411:14: (id2= IDENT |str2= STRING_LITERAL )
            	    int alt10=2;
            	    switch ( input.LA(1) ) {
            	    case IDENT:
            	        {
            	        alt10=1;
            	        }
            	        break;
            	    case STRING_LITERAL:
            	        {
            	        alt10=2;
            	        }
            	        break;
            	    default:
            	        if (state.backtracking>0) {state.failed=true; return retval;}
            	        NoViableAltException nvae =
            	            new NoViableAltException("", 10, 0, input);

            	        throw nvae;

            	    }

            	    switch (alt10) {
            	        case 1 :
            	            // com/senseidb/bql/parsers/BQL.g:1411:15: id2= IDENT
            	            {
            	            id2=(Token)match(input,IDENT,FOLLOW_IDENT_in_column_name4091); if (state.failed) return retval;
            	            if ( state.backtracking==0 ) {
            	            id2_tree = 
            	            (Object)adaptor.create(id2)
            	            ;
            	            adaptor.addChild(root_0, id2_tree);
            	            }

            	            }
            	            break;
            	        case 2 :
            	            // com/senseidb/bql/parsers/BQL.g:1411:27: str2= STRING_LITERAL
            	            {
            	            str2=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_column_name4097); if (state.failed) return retval;
            	            if ( state.backtracking==0 ) {
            	            str2_tree = 
            	            (Object)adaptor.create(str2)
            	            ;
            	            adaptor.addChild(root_0, str2_tree);
            	            }

            	            }
            	            break;

            	    }


            	    if ( state.backtracking==0 ) {
            	                builder.append(".");
            	                if (id2 != null) {
            	                    builder.append((id2!=null?id2.getText():null));
            	                }
            	                else {
            	                    String orig = (str2!=null?str2.getText():null);
            	                    builder.append(orig.substring(1, orig.length() - 1));
            	                }
            	            }

            	    }
            	    break;

            	default :
            	    break loop11;
                }
            } while (true);


            if ( state.backtracking==0 ) { retval.text = builder.toString(); }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 5, column_name_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "column_name"


    public static class where_return extends ParserRuleReturnScope {
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "where"
    // com/senseidb/bql/parsers/BQL.g:1426:1: where returns [Object json] : WHERE ^ search_expr ;
    public final BQLParser.where_return where() throws RecognitionException {
        BQLParser.where_return retval = new BQLParser.where_return();
        retval.start = input.LT(1);

        int where_StartIndex = input.index();

        Object root_0 = null;

        Token WHERE13=null;
        BQLParser.search_expr_return search_expr14 =null;


        Object WHERE13_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 6) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1427:5: ( WHERE ^ search_expr )
            // com/senseidb/bql/parsers/BQL.g:1427:9: WHERE ^ search_expr
            {
            root_0 = (Object)adaptor.nil();


            WHERE13=(Token)match(input,WHERE,FOLLOW_WHERE_in_where4152); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            WHERE13_tree = 
            (Object)adaptor.create(WHERE13)
            ;
            root_0 = (Object)adaptor.becomeRoot(WHERE13_tree, root_0);
            }

            pushFollow(FOLLOW_search_expr_in_where4155);
            search_expr14=search_expr();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, search_expr14.getTree());

            if ( state.backtracking==0 ) {
                        retval.json = (search_expr14!=null?search_expr14.json:null);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 6, where_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "where"


    public static class order_by_clause_return extends ParserRuleReturnScope {
        public boolean isRelevance;
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "order_by_clause"
    // com/senseidb/bql/parsers/BQL.g:1433:1: order_by_clause returns [boolean isRelevance, Object json] : ORDER BY ( RELEVANCE | sort_specs ) ;
    public final BQLParser.order_by_clause_return order_by_clause() throws RecognitionException {
        BQLParser.order_by_clause_return retval = new BQLParser.order_by_clause_return();
        retval.start = input.LT(1);

        int order_by_clause_StartIndex = input.index();

        Object root_0 = null;

        Token ORDER15=null;
        Token BY16=null;
        Token RELEVANCE17=null;
        BQLParser.sort_specs_return sort_specs18 =null;


        Object ORDER15_tree=null;
        Object BY16_tree=null;
        Object RELEVANCE17_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 7) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1434:5: ( ORDER BY ( RELEVANCE | sort_specs ) )
            // com/senseidb/bql/parsers/BQL.g:1434:9: ORDER BY ( RELEVANCE | sort_specs )
            {
            root_0 = (Object)adaptor.nil();


            ORDER15=(Token)match(input,ORDER,FOLLOW_ORDER_in_order_by_clause4188); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            ORDER15_tree = 
            (Object)adaptor.create(ORDER15)
            ;
            adaptor.addChild(root_0, ORDER15_tree);
            }

            BY16=(Token)match(input,BY,FOLLOW_BY_in_order_by_clause4190); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            BY16_tree = 
            (Object)adaptor.create(BY16)
            ;
            adaptor.addChild(root_0, BY16_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:1434:18: ( RELEVANCE | sort_specs )
            int alt12=2;
            switch ( input.LA(1) ) {
            case RELEVANCE:
                {
                alt12=1;
                }
                break;
            case IDENT:
            case STRING_LITERAL:
                {
                alt12=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 12, 0, input);

                throw nvae;

            }

            switch (alt12) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1434:19: RELEVANCE
                    {
                    RELEVANCE17=(Token)match(input,RELEVANCE,FOLLOW_RELEVANCE_in_order_by_clause4193); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    RELEVANCE17_tree = 
                    (Object)adaptor.create(RELEVANCE17)
                    ;
                    adaptor.addChild(root_0, RELEVANCE17_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1434:31: sort_specs
                    {
                    pushFollow(FOLLOW_sort_specs_in_order_by_clause4197);
                    sort_specs18=sort_specs();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, sort_specs18.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        if (RELEVANCE17 != null) {
                            retval.isRelevance = true;
                        }
                        else {
                            retval.isRelevance = false;
                            retval.json = (sort_specs18!=null?sort_specs18.json:null);
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 7, order_by_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "order_by_clause"


    public static class sort_specs_return extends ParserRuleReturnScope {
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "sort_specs"
    // com/senseidb/bql/parsers/BQL.g:1446:1: sort_specs returns [Object json] : sort= sort_spec ( COMMA sort= sort_spec )* ;
    public final BQLParser.sort_specs_return sort_specs() throws RecognitionException {
        BQLParser.sort_specs_return retval = new BQLParser.sort_specs_return();
        retval.start = input.LT(1);

        int sort_specs_StartIndex = input.index();

        Object root_0 = null;

        Token COMMA19=null;
        BQLParser.sort_spec_return sort =null;


        Object COMMA19_tree=null;


            JSONArray sortArray = new JSONArray();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 8) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1450:5: (sort= sort_spec ( COMMA sort= sort_spec )* )
            // com/senseidb/bql/parsers/BQL.g:1450:9: sort= sort_spec ( COMMA sort= sort_spec )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_sort_spec_in_sort_specs4238);
            sort=sort_spec();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, sort.getTree());

            if ( state.backtracking==0 ) {
                        sortArray.put((sort!=null?sort.json:null));
                    }

            // com/senseidb/bql/parsers/BQL.g:1454:9: ( COMMA sort= sort_spec )*
            loop13:
            do {
                int alt13=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt13=1;
                    }
                    break;

                }

                switch (alt13) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1454:10: COMMA sort= sort_spec
            	    {
            	    COMMA19=(Token)match(input,COMMA,FOLLOW_COMMA_in_sort_specs4259); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA19_tree = 
            	    (Object)adaptor.create(COMMA19)
            	    ;
            	    adaptor.addChild(root_0, COMMA19_tree);
            	    }

            	    pushFollow(FOLLOW_sort_spec_in_sort_specs4263);
            	    sort=sort_spec();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, sort.getTree());

            	    if ( state.backtracking==0 ) {
            	                    sortArray.put((sort!=null?sort.json:null));
            	                }

            	    }
            	    break;

            	default :
            	    break loop13;
                }
            } while (true);


            if ( state.backtracking==0 ) {
                        retval.json = sortArray;
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 8, sort_specs_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "sort_specs"


    public static class sort_spec_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "sort_spec"
    // com/senseidb/bql/parsers/BQL.g:1464:1: sort_spec returns [JSONObject json] : column_name (ordering= ASC |ordering= DESC )? ;
    public final BQLParser.sort_spec_return sort_spec() throws RecognitionException {
        BQLParser.sort_spec_return retval = new BQLParser.sort_spec_return();
        retval.start = input.LT(1);

        int sort_spec_StartIndex = input.index();

        Object root_0 = null;

        Token ordering=null;
        BQLParser.column_name_return column_name20 =null;


        Object ordering_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 9) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1465:5: ( column_name (ordering= ASC |ordering= DESC )? )
            // com/senseidb/bql/parsers/BQL.g:1465:9: column_name (ordering= ASC |ordering= DESC )?
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_column_name_in_sort_spec4324);
            column_name20=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, column_name20.getTree());

            // com/senseidb/bql/parsers/BQL.g:1465:21: (ordering= ASC |ordering= DESC )?
            int alt14=3;
            switch ( input.LA(1) ) {
                case ASC:
                    {
                    alt14=1;
                    }
                    break;
                case DESC:
                    {
                    alt14=2;
                    }
                    break;
            }

            switch (alt14) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1465:22: ordering= ASC
                    {
                    ordering=(Token)match(input,ASC,FOLLOW_ASC_in_sort_spec4329); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    ordering_tree = 
                    (Object)adaptor.create(ordering)
                    ;
                    adaptor.addChild(root_0, ordering_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1465:37: ordering= DESC
                    {
                    ordering=(Token)match(input,DESC,FOLLOW_DESC_in_sort_spec4335); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    ordering_tree = 
                    (Object)adaptor.create(ordering)
                    ;
                    adaptor.addChild(root_0, ordering_tree);
                    }

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        retval.json = new JSONObject();
                        try {
                            if (ordering == null) {
                                retval.json.put((column_name20!=null?column_name20.text:null), "asc");
                            }
                            else {
                                retval.json.put((column_name20!=null?column_name20.text:null), (ordering!=null?ordering.getText():null).toLowerCase());
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "sort_spec", "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 9, sort_spec_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "sort_spec"


    public static class limit_clause_return extends ParserRuleReturnScope {
        public int offset;
        public int count;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "limit_clause"
    // com/senseidb/bql/parsers/BQL.g:1482:1: limit_clause returns [int offset, int count] : LIMIT (n1= INTEGER COMMA )? n2= INTEGER ;
    public final BQLParser.limit_clause_return limit_clause() throws RecognitionException {
        BQLParser.limit_clause_return retval = new BQLParser.limit_clause_return();
        retval.start = input.LT(1);

        int limit_clause_StartIndex = input.index();

        Object root_0 = null;

        Token n1=null;
        Token n2=null;
        Token LIMIT21=null;
        Token COMMA22=null;

        Object n1_tree=null;
        Object n2_tree=null;
        Object LIMIT21_tree=null;
        Object COMMA22_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 10) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1483:5: ( LIMIT (n1= INTEGER COMMA )? n2= INTEGER )
            // com/senseidb/bql/parsers/BQL.g:1483:9: LIMIT (n1= INTEGER COMMA )? n2= INTEGER
            {
            root_0 = (Object)adaptor.nil();


            LIMIT21=(Token)match(input,LIMIT,FOLLOW_LIMIT_in_limit_clause4370); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LIMIT21_tree = 
            (Object)adaptor.create(LIMIT21)
            ;
            adaptor.addChild(root_0, LIMIT21_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:1483:15: (n1= INTEGER COMMA )?
            int alt15=2;
            switch ( input.LA(1) ) {
                case INTEGER:
                    {
                    switch ( input.LA(2) ) {
                        case COMMA:
                            {
                            alt15=1;
                            }
                            break;
                    }

                    }
                    break;
            }

            switch (alt15) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1483:16: n1= INTEGER COMMA
                    {
                    n1=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_limit_clause4375); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    n1_tree = 
                    (Object)adaptor.create(n1)
                    ;
                    adaptor.addChild(root_0, n1_tree);
                    }

                    COMMA22=(Token)match(input,COMMA,FOLLOW_COMMA_in_limit_clause4377); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    COMMA22_tree = 
                    (Object)adaptor.create(COMMA22)
                    ;
                    adaptor.addChild(root_0, COMMA22_tree);
                    }

                    }
                    break;

            }


            n2=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_limit_clause4383); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            n2_tree = 
            (Object)adaptor.create(n2)
            ;
            adaptor.addChild(root_0, n2_tree);
            }

            if ( state.backtracking==0 ) {
                        if (n1 != null) {
                            retval.offset = Integer.parseInt((n1!=null?n1.getText():null));
                        }
                        else {
                            retval.offset = DEFAULT_REQUEST_OFFSET;
                        }
                        retval.count = Integer.parseInt((n2!=null?n2.getText():null));
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 10, limit_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "limit_clause"


    public static class or_column_name_list_return extends ParserRuleReturnScope {
        public JSONArray json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "or_column_name_list"
    // com/senseidb/bql/parsers/BQL.g:1495:1: or_column_name_list returns [JSONArray json] : col= column_name ( OR col= column_name )* ;
    public final BQLParser.or_column_name_list_return or_column_name_list() throws RecognitionException {
        BQLParser.or_column_name_list_return retval = new BQLParser.or_column_name_list_return();
        retval.start = input.LT(1);

        int or_column_name_list_StartIndex = input.index();

        Object root_0 = null;

        Token OR23=null;
        BQLParser.column_name_return col =null;


        Object OR23_tree=null;


            retval.json = new JSONArray();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 11) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1499:5: (col= column_name ( OR col= column_name )* )
            // com/senseidb/bql/parsers/BQL.g:1499:9: col= column_name ( OR col= column_name )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_column_name_in_or_column_name_list4423);
            col=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, col.getTree());

            if ( state.backtracking==0 ) {
                        String colName = (col!=null?col.text:null);
                        if (colName != null) {
                            retval.json.put((col!=null?col.text:null)); 
                        }
                    }

            // com/senseidb/bql/parsers/BQL.g:1506:9: ( OR col= column_name )*
            loop16:
            do {
                int alt16=2;
                switch ( input.LA(1) ) {
                case OR:
                    {
                    alt16=1;
                    }
                    break;

                }

                switch (alt16) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1506:10: OR col= column_name
            	    {
            	    OR23=(Token)match(input,OR,FOLLOW_OR_in_or_column_name_list4444); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    OR23_tree = 
            	    (Object)adaptor.create(OR23)
            	    ;
            	    adaptor.addChild(root_0, OR23_tree);
            	    }

            	    pushFollow(FOLLOW_column_name_in_or_column_name_list4448);
            	    col=column_name();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, col.getTree());

            	    if ( state.backtracking==0 ) {
            	                    String colName = (col!=null?col.text:null);
            	                    if (colName != null) {
            	                        retval.json.put((col!=null?col.text:null)); 
            	                    }
            	                }

            	    }
            	    break;

            	default :
            	    break loop16;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 11, or_column_name_list_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "or_column_name_list"


    public static class group_by_clause_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "group_by_clause"
    // com/senseidb/bql/parsers/BQL.g:1516:1: group_by_clause returns [JSONObject json] : GROUP BY or_column_name_list ( TOP top= INTEGER )? ;
    public final BQLParser.group_by_clause_return group_by_clause() throws RecognitionException {
        BQLParser.group_by_clause_return retval = new BQLParser.group_by_clause_return();
        retval.start = input.LT(1);

        int group_by_clause_StartIndex = input.index();

        Object root_0 = null;

        Token top=null;
        Token GROUP24=null;
        Token BY25=null;
        Token TOP27=null;
        BQLParser.or_column_name_list_return or_column_name_list26 =null;


        Object top_tree=null;
        Object GROUP24_tree=null;
        Object BY25_tree=null;
        Object TOP27_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 12) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1517:5: ( GROUP BY or_column_name_list ( TOP top= INTEGER )? )
            // com/senseidb/bql/parsers/BQL.g:1517:9: GROUP BY or_column_name_list ( TOP top= INTEGER )?
            {
            root_0 = (Object)adaptor.nil();


            GROUP24=(Token)match(input,GROUP,FOLLOW_GROUP_in_group_by_clause4496); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            GROUP24_tree = 
            (Object)adaptor.create(GROUP24)
            ;
            adaptor.addChild(root_0, GROUP24_tree);
            }

            BY25=(Token)match(input,BY,FOLLOW_BY_in_group_by_clause4498); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            BY25_tree = 
            (Object)adaptor.create(BY25)
            ;
            adaptor.addChild(root_0, BY25_tree);
            }

            pushFollow(FOLLOW_or_column_name_list_in_group_by_clause4500);
            or_column_name_list26=or_column_name_list();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, or_column_name_list26.getTree());

            // com/senseidb/bql/parsers/BQL.g:1517:38: ( TOP top= INTEGER )?
            int alt17=2;
            switch ( input.LA(1) ) {
                case TOP:
                    {
                    alt17=1;
                    }
                    break;
            }

            switch (alt17) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1517:39: TOP top= INTEGER
                    {
                    TOP27=(Token)match(input,TOP,FOLLOW_TOP_in_group_by_clause4503); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    TOP27_tree = 
                    (Object)adaptor.create(TOP27)
                    ;
                    adaptor.addChild(root_0, TOP27_tree);
                    }

                    top=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_group_by_clause4507); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    top_tree = 
                    (Object)adaptor.create(top)
                    ;
                    adaptor.addChild(root_0, top_tree);
                    }

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        retval.json = new JSONObject();
                        try {
                            JSONArray cols = (or_column_name_list26!=null?or_column_name_list26.json:null);
                            for (int i = 0; i < cols.length(); ++i) {
                                String col = cols.getString(i);
                                String[] facetInfo = _facetInfoMap.get(col);
                                if (facetInfo != null && (facetInfo[0].equals("range") ||
                                                          facetInfo[0].equals("multi") ||
                                                          facetInfo[0].equals("path"))) {
                                    throw new FailedPredicateException(input, 
                                                                       "group_by_clause",
                                                                       "Range/multi/path facet, \"" + col + "\", cannot be used in the GROUP BY clause.");
                                }
                            }
                            retval.json.put("columns", cols);
                            if (top != null) {
                                retval.json.put("top", Integer.parseInt(top.getText()));
                            }
                            else {
                                retval.json.put("top", DEFAULT_REQUEST_MAX_PER_GROUP);
                            }                    
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "group_by_clause", "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 12, group_by_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "group_by_clause"


    public static class browse_by_clause_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "browse_by_clause"
    // com/senseidb/bql/parsers/BQL.g:1547:1: browse_by_clause returns [JSONObject json] : BROWSE BY f= facet_spec ( COMMA f= facet_spec )* ;
    public final BQLParser.browse_by_clause_return browse_by_clause() throws RecognitionException {
        BQLParser.browse_by_clause_return retval = new BQLParser.browse_by_clause_return();
        retval.start = input.LT(1);

        int browse_by_clause_StartIndex = input.index();

        Object root_0 = null;

        Token BROWSE28=null;
        Token BY29=null;
        Token COMMA30=null;
        BQLParser.facet_spec_return f =null;


        Object BROWSE28_tree=null;
        Object BY29_tree=null;
        Object COMMA30_tree=null;


            retval.json = new JSONObject();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 13) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1551:5: ( BROWSE BY f= facet_spec ( COMMA f= facet_spec )* )
            // com/senseidb/bql/parsers/BQL.g:1551:9: BROWSE BY f= facet_spec ( COMMA f= facet_spec )*
            {
            root_0 = (Object)adaptor.nil();


            BROWSE28=(Token)match(input,BROWSE,FOLLOW_BROWSE_in_browse_by_clause4547); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            BROWSE28_tree = 
            (Object)adaptor.create(BROWSE28)
            ;
            adaptor.addChild(root_0, BROWSE28_tree);
            }

            BY29=(Token)match(input,BY,FOLLOW_BY_in_browse_by_clause4549); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            BY29_tree = 
            (Object)adaptor.create(BY29)
            ;
            adaptor.addChild(root_0, BY29_tree);
            }

            pushFollow(FOLLOW_facet_spec_in_browse_by_clause4553);
            f=facet_spec();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, f.getTree());

            if ( state.backtracking==0 ) {
                        try {
                            retval.json.put((f!=null?f.column:null), (f!=null?f.spec:null));
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "browse_by_clause", "JSONException: " + err.getMessage());
                        }                    
                    }

            // com/senseidb/bql/parsers/BQL.g:1560:9: ( COMMA f= facet_spec )*
            loop18:
            do {
                int alt18=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt18=1;
                    }
                    break;

                }

                switch (alt18) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1560:10: COMMA f= facet_spec
            	    {
            	    COMMA30=(Token)match(input,COMMA,FOLLOW_COMMA_in_browse_by_clause4574); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA30_tree = 
            	    (Object)adaptor.create(COMMA30)
            	    ;
            	    adaptor.addChild(root_0, COMMA30_tree);
            	    }

            	    pushFollow(FOLLOW_facet_spec_in_browse_by_clause4578);
            	    f=facet_spec();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, f.getTree());

            	    if ( state.backtracking==0 ) {
            	                    try {
            	                        retval.json.put((f!=null?f.column:null), (f!=null?f.spec:null));
            	                    }
            	                    catch (JSONException err) {
            	                        throw new FailedPredicateException(input, "browse_by_clause", "JSONException: " + err.getMessage());
            	                    }
            	                }

            	    }
            	    break;

            	default :
            	    break loop18;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 13, browse_by_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "browse_by_clause"


    public static class facet_spec_return extends ParserRuleReturnScope {
        public String column;
        public JSONObject spec;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "facet_spec"
    // com/senseidb/bql/parsers/BQL.g:1572:1: facet_spec returns [String column, JSONObject spec] : column_name ( LPAR ( TRUE | FALSE ) COMMA n1= INTEGER COMMA n2= INTEGER COMMA ( HITS | VALUE ) RPAR )* ;
    public final BQLParser.facet_spec_return facet_spec() throws RecognitionException {
        BQLParser.facet_spec_return retval = new BQLParser.facet_spec_return();
        retval.start = input.LT(1);

        int facet_spec_StartIndex = input.index();

        Object root_0 = null;

        Token n1=null;
        Token n2=null;
        Token LPAR32=null;
        Token TRUE33=null;
        Token FALSE34=null;
        Token COMMA35=null;
        Token COMMA36=null;
        Token COMMA37=null;
        Token HITS38=null;
        Token VALUE39=null;
        Token RPAR40=null;
        BQLParser.column_name_return column_name31 =null;


        Object n1_tree=null;
        Object n2_tree=null;
        Object LPAR32_tree=null;
        Object TRUE33_tree=null;
        Object FALSE34_tree=null;
        Object COMMA35_tree=null;
        Object COMMA36_tree=null;
        Object COMMA37_tree=null;
        Object HITS38_tree=null;
        Object VALUE39_tree=null;
        Object RPAR40_tree=null;


            boolean expand = false;
            int minhit = DEFAULT_FACET_MINHIT;
            int max = DEFAULT_FACET_MAXHIT;
            String orderBy = "hits";

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 14) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1579:5: ( column_name ( LPAR ( TRUE | FALSE ) COMMA n1= INTEGER COMMA n2= INTEGER COMMA ( HITS | VALUE ) RPAR )* )
            // com/senseidb/bql/parsers/BQL.g:1579:9: column_name ( LPAR ( TRUE | FALSE ) COMMA n1= INTEGER COMMA n2= INTEGER COMMA ( HITS | VALUE ) RPAR )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_column_name_in_facet_spec4631);
            column_name31=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, column_name31.getTree());

            // com/senseidb/bql/parsers/BQL.g:1580:9: ( LPAR ( TRUE | FALSE ) COMMA n1= INTEGER COMMA n2= INTEGER COMMA ( HITS | VALUE ) RPAR )*
            loop21:
            do {
                int alt21=2;
                switch ( input.LA(1) ) {
                case LPAR:
                    {
                    alt21=1;
                    }
                    break;

                }

                switch (alt21) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1581:13: LPAR ( TRUE | FALSE ) COMMA n1= INTEGER COMMA n2= INTEGER COMMA ( HITS | VALUE ) RPAR
            	    {
            	    LPAR32=(Token)match(input,LPAR,FOLLOW_LPAR_in_facet_spec4655); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    LPAR32_tree = 
            	    (Object)adaptor.create(LPAR32)
            	    ;
            	    adaptor.addChild(root_0, LPAR32_tree);
            	    }

            	    // com/senseidb/bql/parsers/BQL.g:1582:13: ( TRUE | FALSE )
            	    int alt19=2;
            	    switch ( input.LA(1) ) {
            	    case TRUE:
            	        {
            	        alt19=1;
            	        }
            	        break;
            	    case FALSE:
            	        {
            	        alt19=2;
            	        }
            	        break;
            	    default:
            	        if (state.backtracking>0) {state.failed=true; return retval;}
            	        NoViableAltException nvae =
            	            new NoViableAltException("", 19, 0, input);

            	        throw nvae;

            	    }

            	    switch (alt19) {
            	        case 1 :
            	            // com/senseidb/bql/parsers/BQL.g:1582:14: TRUE
            	            {
            	            TRUE33=(Token)match(input,TRUE,FOLLOW_TRUE_in_facet_spec4671); if (state.failed) return retval;
            	            if ( state.backtracking==0 ) {
            	            TRUE33_tree = 
            	            (Object)adaptor.create(TRUE33)
            	            ;
            	            adaptor.addChild(root_0, TRUE33_tree);
            	            }

            	            if ( state.backtracking==0 ) {expand = true;}

            	            }
            	            break;
            	        case 2 :
            	            // com/senseidb/bql/parsers/BQL.g:1582:38: FALSE
            	            {
            	            FALSE34=(Token)match(input,FALSE,FOLLOW_FALSE_in_facet_spec4677); if (state.failed) return retval;
            	            if ( state.backtracking==0 ) {
            	            FALSE34_tree = 
            	            (Object)adaptor.create(FALSE34)
            	            ;
            	            adaptor.addChild(root_0, FALSE34_tree);
            	            }

            	            }
            	            break;

            	    }


            	    COMMA35=(Token)match(input,COMMA,FOLLOW_COMMA_in_facet_spec4680); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA35_tree = 
            	    (Object)adaptor.create(COMMA35)
            	    ;
            	    adaptor.addChild(root_0, COMMA35_tree);
            	    }

            	    n1=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_facet_spec4696); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    n1_tree = 
            	    (Object)adaptor.create(n1)
            	    ;
            	    adaptor.addChild(root_0, n1_tree);
            	    }

            	    COMMA36=(Token)match(input,COMMA,FOLLOW_COMMA_in_facet_spec4698); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA36_tree = 
            	    (Object)adaptor.create(COMMA36)
            	    ;
            	    adaptor.addChild(root_0, COMMA36_tree);
            	    }

            	    n2=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_facet_spec4714); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    n2_tree = 
            	    (Object)adaptor.create(n2)
            	    ;
            	    adaptor.addChild(root_0, n2_tree);
            	    }

            	    COMMA37=(Token)match(input,COMMA,FOLLOW_COMMA_in_facet_spec4716); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA37_tree = 
            	    (Object)adaptor.create(COMMA37)
            	    ;
            	    adaptor.addChild(root_0, COMMA37_tree);
            	    }

            	    // com/senseidb/bql/parsers/BQL.g:1585:13: ( HITS | VALUE )
            	    int alt20=2;
            	    switch ( input.LA(1) ) {
            	    case HITS:
            	        {
            	        alt20=1;
            	        }
            	        break;
            	    case VALUE:
            	        {
            	        alt20=2;
            	        }
            	        break;
            	    default:
            	        if (state.backtracking>0) {state.failed=true; return retval;}
            	        NoViableAltException nvae =
            	            new NoViableAltException("", 20, 0, input);

            	        throw nvae;

            	    }

            	    switch (alt20) {
            	        case 1 :
            	            // com/senseidb/bql/parsers/BQL.g:1585:14: HITS
            	            {
            	            HITS38=(Token)match(input,HITS,FOLLOW_HITS_in_facet_spec4731); if (state.failed) return retval;
            	            if ( state.backtracking==0 ) {
            	            HITS38_tree = 
            	            (Object)adaptor.create(HITS38)
            	            ;
            	            adaptor.addChild(root_0, HITS38_tree);
            	            }

            	            }
            	            break;
            	        case 2 :
            	            // com/senseidb/bql/parsers/BQL.g:1585:21: VALUE
            	            {
            	            VALUE39=(Token)match(input,VALUE,FOLLOW_VALUE_in_facet_spec4735); if (state.failed) return retval;
            	            if ( state.backtracking==0 ) {
            	            VALUE39_tree = 
            	            (Object)adaptor.create(VALUE39)
            	            ;
            	            adaptor.addChild(root_0, VALUE39_tree);
            	            }

            	            if ( state.backtracking==0 ) {orderBy = "val";}

            	            }
            	            break;

            	    }


            	    RPAR40=(Token)match(input,RPAR,FOLLOW_RPAR_in_facet_spec4752); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    RPAR40_tree = 
            	    (Object)adaptor.create(RPAR40)
            	    ;
            	    adaptor.addChild(root_0, RPAR40_tree);
            	    }

            	    }
            	    break;

            	default :
            	    break loop21;
                }
            } while (true);


            if ( state.backtracking==0 ) {
                        retval.column = (column_name31!=null?column_name31.text:null);
                        if (n1 != null) {
                            minhit = Integer.parseInt((n1!=null?n1.getText():null));
                        }
                        if (n2 != null) {
                            max = Integer.parseInt((n2!=null?n2.getText():null));
                        }
                        try {
                            retval.spec = new JSONObject().put("expand", expand)
                                                    .put("minhit", minhit)
                                                    .put("max", max)
                                                    .put("order", orderBy);
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "facet_spec", "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 14, facet_spec_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "facet_spec"


    public static class fetching_stored_clause_return extends ParserRuleReturnScope {
        public boolean val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "fetching_stored_clause"
    // com/senseidb/bql/parsers/BQL.g:1608:1: fetching_stored_clause returns [boolean val] : FETCHING STORED ( ( TRUE | FALSE ) )* ;
    public final BQLParser.fetching_stored_clause_return fetching_stored_clause() throws RecognitionException {
        BQLParser.fetching_stored_clause_return retval = new BQLParser.fetching_stored_clause_return();
        retval.start = input.LT(1);

        int fetching_stored_clause_StartIndex = input.index();

        Object root_0 = null;

        Token FETCHING41=null;
        Token STORED42=null;
        Token TRUE43=null;
        Token FALSE44=null;

        Object FETCHING41_tree=null;
        Object STORED42_tree=null;
        Object TRUE43_tree=null;
        Object FALSE44_tree=null;


            retval.val = true;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 15) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1612:5: ( FETCHING STORED ( ( TRUE | FALSE ) )* )
            // com/senseidb/bql/parsers/BQL.g:1612:9: FETCHING STORED ( ( TRUE | FALSE ) )*
            {
            root_0 = (Object)adaptor.nil();


            FETCHING41=(Token)match(input,FETCHING,FOLLOW_FETCHING_in_fetching_stored_clause4801); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            FETCHING41_tree = 
            (Object)adaptor.create(FETCHING41)
            ;
            adaptor.addChild(root_0, FETCHING41_tree);
            }

            STORED42=(Token)match(input,STORED,FOLLOW_STORED_in_fetching_stored_clause4803); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            STORED42_tree = 
            (Object)adaptor.create(STORED42)
            ;
            adaptor.addChild(root_0, STORED42_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:1613:9: ( ( TRUE | FALSE ) )*
            loop23:
            do {
                int alt23=2;
                switch ( input.LA(1) ) {
                case FALSE:
                case TRUE:
                    {
                    alt23=1;
                    }
                    break;

                }

                switch (alt23) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1613:10: ( TRUE | FALSE )
            	    {
            	    // com/senseidb/bql/parsers/BQL.g:1613:10: ( TRUE | FALSE )
            	    int alt22=2;
            	    switch ( input.LA(1) ) {
            	    case TRUE:
            	        {
            	        alt22=1;
            	        }
            	        break;
            	    case FALSE:
            	        {
            	        alt22=2;
            	        }
            	        break;
            	    default:
            	        if (state.backtracking>0) {state.failed=true; return retval;}
            	        NoViableAltException nvae =
            	            new NoViableAltException("", 22, 0, input);

            	        throw nvae;

            	    }

            	    switch (alt22) {
            	        case 1 :
            	            // com/senseidb/bql/parsers/BQL.g:1613:11: TRUE
            	            {
            	            TRUE43=(Token)match(input,TRUE,FOLLOW_TRUE_in_fetching_stored_clause4815); if (state.failed) return retval;
            	            if ( state.backtracking==0 ) {
            	            TRUE43_tree = 
            	            (Object)adaptor.create(TRUE43)
            	            ;
            	            adaptor.addChild(root_0, TRUE43_tree);
            	            }

            	            }
            	            break;
            	        case 2 :
            	            // com/senseidb/bql/parsers/BQL.g:1613:18: FALSE
            	            {
            	            FALSE44=(Token)match(input,FALSE,FOLLOW_FALSE_in_fetching_stored_clause4819); if (state.failed) return retval;
            	            if ( state.backtracking==0 ) {
            	            FALSE44_tree = 
            	            (Object)adaptor.create(FALSE44)
            	            ;
            	            adaptor.addChild(root_0, FALSE44_tree);
            	            }

            	            if ( state.backtracking==0 ) {retval.val = false;}

            	            }
            	            break;

            	    }


            	    }
            	    break;

            	default :
            	    break loop23;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 15, fetching_stored_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "fetching_stored_clause"


    public static class route_by_clause_return extends ParserRuleReturnScope {
        public String val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "route_by_clause"
    // com/senseidb/bql/parsers/BQL.g:1617:1: route_by_clause returns [String val] : ROUTE BY STRING_LITERAL ;
    public final BQLParser.route_by_clause_return route_by_clause() throws RecognitionException {
        BQLParser.route_by_clause_return retval = new BQLParser.route_by_clause_return();
        retval.start = input.LT(1);

        int route_by_clause_StartIndex = input.index();

        Object root_0 = null;

        Token ROUTE45=null;
        Token BY46=null;
        Token STRING_LITERAL47=null;

        Object ROUTE45_tree=null;
        Object BY46_tree=null;
        Object STRING_LITERAL47_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 16) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1618:5: ( ROUTE BY STRING_LITERAL )
            // com/senseidb/bql/parsers/BQL.g:1618:9: ROUTE BY STRING_LITERAL
            {
            root_0 = (Object)adaptor.nil();


            ROUTE45=(Token)match(input,ROUTE,FOLLOW_ROUTE_in_route_by_clause4856); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            ROUTE45_tree = 
            (Object)adaptor.create(ROUTE45)
            ;
            adaptor.addChild(root_0, ROUTE45_tree);
            }

            BY46=(Token)match(input,BY,FOLLOW_BY_in_route_by_clause4858); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            BY46_tree = 
            (Object)adaptor.create(BY46)
            ;
            adaptor.addChild(root_0, BY46_tree);
            }

            STRING_LITERAL47=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_route_by_clause4860); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            STRING_LITERAL47_tree = 
            (Object)adaptor.create(STRING_LITERAL47)
            ;
            adaptor.addChild(root_0, STRING_LITERAL47_tree);
            }

            if ( state.backtracking==0 ) { 
                        String orig = (STRING_LITERAL47!=null?STRING_LITERAL47.getText():null);
                        retval.val = orig.substring(1, orig.length() - 1);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 16, route_by_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "route_by_clause"


    public static class search_expr_return extends ParserRuleReturnScope {
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "search_expr"
    // com/senseidb/bql/parsers/BQL.g:1625:1: search_expr returns [Object json] : t= term_expr ( OR t= term_expr )* -> {array.length() > 1}? ^( OR_PRED ( term_expr )+ ) -> term_expr ;
    public final BQLParser.search_expr_return search_expr() throws RecognitionException {
        BQLParser.search_expr_return retval = new BQLParser.search_expr_return();
        retval.start = input.LT(1);

        int search_expr_StartIndex = input.index();

        Object root_0 = null;

        Token OR48=null;
        BQLParser.term_expr_return t =null;


        Object OR48_tree=null;
        RewriteRuleTokenStream stream_OR=new RewriteRuleTokenStream(adaptor,"token OR");
        RewriteRuleSubtreeStream stream_term_expr=new RewriteRuleSubtreeStream(adaptor,"rule term_expr");

            JSONArray array = new JSONArray();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 17) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1629:5: (t= term_expr ( OR t= term_expr )* -> {array.length() > 1}? ^( OR_PRED ( term_expr )+ ) -> term_expr )
            // com/senseidb/bql/parsers/BQL.g:1629:9: t= term_expr ( OR t= term_expr )*
            {
            pushFollow(FOLLOW_term_expr_in_search_expr4901);
            t=term_expr();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_term_expr.add(t.getTree());

            if ( state.backtracking==0 ) { array.put((t!=null?t.json:null)); }

            // com/senseidb/bql/parsers/BQL.g:1630:9: ( OR t= term_expr )*
            loop24:
            do {
                int alt24=2;
                switch ( input.LA(1) ) {
                case OR:
                    {
                    alt24=1;
                    }
                    break;

                }

                switch (alt24) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1630:10: OR t= term_expr
            	    {
            	    OR48=(Token)match(input,OR,FOLLOW_OR_in_search_expr4914); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_OR.add(OR48);


            	    pushFollow(FOLLOW_term_expr_in_search_expr4918);
            	    t=term_expr();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_term_expr.add(t.getTree());

            	    if ( state.backtracking==0 ) { array.put((t!=null?t.json:null)); }

            	    }
            	    break;

            	default :
            	    break loop24;
                }
            } while (true);


            if ( state.backtracking==0 ) {
                        try {
                            if (array.length() == 1) {
                                retval.json = array.get(0);
                            }
                            else {
                                retval.json = new JSONObject().put("or", array);
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "search_expr", "JSONException: " + err.getMessage());
                        }
                    }

            // AST REWRITE
            // elements: term_expr, term_expr
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Object)adaptor.nil();
            // 1644:9: -> {array.length() > 1}? ^( OR_PRED ( term_expr )+ )
            if (array.length() > 1) {
                // com/senseidb/bql/parsers/BQL.g:1644:34: ^( OR_PRED ( term_expr )+ )
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(
                (Object)adaptor.create(OR_PRED, "OR_PRED")
                , root_1);

                if ( !(stream_term_expr.hasNext()) ) {
                    throw new RewriteEarlyExitException();
                }
                while ( stream_term_expr.hasNext() ) {
                    adaptor.addChild(root_1, stream_term_expr.nextTree());

                }
                stream_term_expr.reset();

                adaptor.addChild(root_0, root_1);
                }

            }

            else // 1645:9: -> term_expr
            {
                adaptor.addChild(root_0, stream_term_expr.nextTree());

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 17, search_expr_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "search_expr"


    public static class term_expr_return extends ParserRuleReturnScope {
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "term_expr"
    // com/senseidb/bql/parsers/BQL.g:1648:1: term_expr returns [Object json] : f= factor_expr ( AND f= factor_expr )* -> { array.length() > 1}? ^( AND_PRED ( factor_expr )+ ) -> factor_expr ;
    public final BQLParser.term_expr_return term_expr() throws RecognitionException {
        BQLParser.term_expr_return retval = new BQLParser.term_expr_return();
        retval.start = input.LT(1);

        int term_expr_StartIndex = input.index();

        Object root_0 = null;

        Token AND49=null;
        BQLParser.factor_expr_return f =null;


        Object AND49_tree=null;
        RewriteRuleTokenStream stream_AND=new RewriteRuleTokenStream(adaptor,"token AND");
        RewriteRuleSubtreeStream stream_factor_expr=new RewriteRuleSubtreeStream(adaptor,"rule factor_expr");

            JSONArray array = new JSONArray();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 18) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1652:5: (f= factor_expr ( AND f= factor_expr )* -> { array.length() > 1}? ^( AND_PRED ( factor_expr )+ ) -> factor_expr )
            // com/senseidb/bql/parsers/BQL.g:1652:9: f= factor_expr ( AND f= factor_expr )*
            {
            pushFollow(FOLLOW_factor_expr_in_term_expr4994);
            f=factor_expr();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_factor_expr.add(f.getTree());

            if ( state.backtracking==0 ) { array.put((f!=null?f.json:null)); }

            // com/senseidb/bql/parsers/BQL.g:1653:9: ( AND f= factor_expr )*
            loop25:
            do {
                int alt25=2;
                switch ( input.LA(1) ) {
                case AND:
                    {
                    alt25=1;
                    }
                    break;

                }

                switch (alt25) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1653:10: AND f= factor_expr
            	    {
            	    AND49=(Token)match(input,AND,FOLLOW_AND_in_term_expr5007); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_AND.add(AND49);


            	    pushFollow(FOLLOW_factor_expr_in_term_expr5011);
            	    f=factor_expr();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_factor_expr.add(f.getTree());

            	    if ( state.backtracking==0 ) { array.put((f!=null?f.json:null)); }

            	    }
            	    break;

            	default :
            	    break loop25;
                }
            } while (true);


            if ( state.backtracking==0 ) {
                        try {
                            JSONArray newArray = new JSONArray();
                            JSONObject fieldMap = new JSONObject();
                            for (int i = 0; i < array.length(); ++i) {
                                JSONObject pred = (JSONObject) array.get(i);
                                if (!"range".equals(predType(pred))) {
                                    newArray.put(pred);
                                }
                                else {
                                    accumulateRangePred(input, fieldMap, pred);
                                }
                            }
                            Iterator<String> itr = fieldMap.keys();
                            while (itr.hasNext()) {
                                newArray.put(fieldMap.get(itr.next()));
                            }
                            if (newArray.length() == 1) {
                                retval.json = newArray.get(0);
                            }
                            else {
                                retval.json = new JSONObject().put("and", newArray);
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "term_expr", "JSONException: " + err.getMessage());
                        }
                    }

            // AST REWRITE
            // elements: factor_expr, factor_expr
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Object)adaptor.nil();
            // 1682:9: -> { array.length() > 1}? ^( AND_PRED ( factor_expr )+ )
            if ( array.length() > 1) {
                // com/senseidb/bql/parsers/BQL.g:1682:35: ^( AND_PRED ( factor_expr )+ )
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(
                (Object)adaptor.create(AND_PRED, "AND_PRED")
                , root_1);

                if ( !(stream_factor_expr.hasNext()) ) {
                    throw new RewriteEarlyExitException();
                }
                while ( stream_factor_expr.hasNext() ) {
                    adaptor.addChild(root_1, stream_factor_expr.nextTree());

                }
                stream_factor_expr.reset();

                adaptor.addChild(root_0, root_1);
                }

            }

            else // 1683:9: -> factor_expr
            {
                adaptor.addChild(root_0, stream_factor_expr.nextTree());

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 18, term_expr_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "term_expr"


    public static class factor_expr_return extends ParserRuleReturnScope {
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "factor_expr"
    // com/senseidb/bql/parsers/BQL.g:1686:1: factor_expr returns [Object json] : ( predicate | LPAR search_expr RPAR -> search_expr );
    public final BQLParser.factor_expr_return factor_expr() throws RecognitionException {
        BQLParser.factor_expr_return retval = new BQLParser.factor_expr_return();
        retval.start = input.LT(1);

        int factor_expr_StartIndex = input.index();

        Object root_0 = null;

        Token LPAR51=null;
        Token RPAR53=null;
        BQLParser.predicate_return predicate50 =null;

        BQLParser.search_expr_return search_expr52 =null;


        Object LPAR51_tree=null;
        Object RPAR53_tree=null;
        RewriteRuleTokenStream stream_RPAR=new RewriteRuleTokenStream(adaptor,"token RPAR");
        RewriteRuleTokenStream stream_LPAR=new RewriteRuleTokenStream(adaptor,"token LPAR");
        RewriteRuleSubtreeStream stream_search_expr=new RewriteRuleSubtreeStream(adaptor,"rule search_expr");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 19) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1687:5: ( predicate | LPAR search_expr RPAR -> search_expr )
            int alt26=2;
            switch ( input.LA(1) ) {
            case IDENT:
            case MATCH:
            case NOT:
            case QUERY:
            case STRING_LITERAL:
                {
                alt26=1;
                }
                break;
            case LPAR:
                {
                alt26=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 26, 0, input);

                throw nvae;

            }

            switch (alt26) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1687:9: predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_predicate_in_factor_expr5080);
                    predicate50=predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, predicate50.getTree());

                    if ( state.backtracking==0 ) { retval.json = (predicate50!=null?predicate50.json:null); }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1688:9: LPAR search_expr RPAR
                    {
                    LPAR51=(Token)match(input,LPAR,FOLLOW_LPAR_in_factor_expr5092); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_LPAR.add(LPAR51);


                    pushFollow(FOLLOW_search_expr_in_factor_expr5094);
                    search_expr52=search_expr();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_search_expr.add(search_expr52.getTree());

                    RPAR53=(Token)match(input,RPAR,FOLLOW_RPAR_in_factor_expr5096); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_RPAR.add(RPAR53);


                    if ( state.backtracking==0 ) {retval.json = (search_expr52!=null?search_expr52.json:null);}

                    // AST REWRITE
                    // elements: search_expr
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {

                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Object)adaptor.nil();
                    // 1690:9: -> search_expr
                    {
                        adaptor.addChild(root_0, stream_search_expr.nextTree());

                    }


                    retval.tree = root_0;
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 19, factor_expr_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "factor_expr"


    public static class predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "predicate"
    // com/senseidb/bql/parsers/BQL.g:1693:1: predicate returns [JSONObject json] : ( in_predicate | contains_all_predicate | equal_predicate | not_equal_predicate | query_predicate | between_predicate | range_predicate | time_predicate | match_predicate | like_predicate | null_predicate );
    public final BQLParser.predicate_return predicate() throws RecognitionException {
        BQLParser.predicate_return retval = new BQLParser.predicate_return();
        retval.start = input.LT(1);

        int predicate_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.in_predicate_return in_predicate54 =null;

        BQLParser.contains_all_predicate_return contains_all_predicate55 =null;

        BQLParser.equal_predicate_return equal_predicate56 =null;

        BQLParser.not_equal_predicate_return not_equal_predicate57 =null;

        BQLParser.query_predicate_return query_predicate58 =null;

        BQLParser.between_predicate_return between_predicate59 =null;

        BQLParser.range_predicate_return range_predicate60 =null;

        BQLParser.time_predicate_return time_predicate61 =null;

        BQLParser.match_predicate_return match_predicate62 =null;

        BQLParser.like_predicate_return like_predicate63 =null;

        BQLParser.null_predicate_return null_predicate64 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 20) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1694:5: ( in_predicate | contains_all_predicate | equal_predicate | not_equal_predicate | query_predicate | between_predicate | range_predicate | time_predicate | match_predicate | like_predicate | null_predicate )
            int alt27=11;
            alt27 = dfa27.predict(input);
            switch (alt27) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1694:9: in_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_in_predicate_in_predicate5141);
                    in_predicate54=in_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, in_predicate54.getTree());

                    if ( state.backtracking==0 ) { retval.json = (in_predicate54!=null?in_predicate54.json:null); }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1695:9: contains_all_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_contains_all_predicate_in_predicate5153);
                    contains_all_predicate55=contains_all_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, contains_all_predicate55.getTree());

                    if ( state.backtracking==0 ) { retval.json = (contains_all_predicate55!=null?contains_all_predicate55.json:null); }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:1696:9: equal_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_equal_predicate_in_predicate5165);
                    equal_predicate56=equal_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, equal_predicate56.getTree());

                    if ( state.backtracking==0 ) { retval.json = (equal_predicate56!=null?equal_predicate56.json:null); }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:1697:9: not_equal_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_not_equal_predicate_in_predicate5177);
                    not_equal_predicate57=not_equal_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, not_equal_predicate57.getTree());

                    if ( state.backtracking==0 ) { retval.json = (not_equal_predicate57!=null?not_equal_predicate57.json:null); }

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:1698:9: query_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_query_predicate_in_predicate5189);
                    query_predicate58=query_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, query_predicate58.getTree());

                    if ( state.backtracking==0 ) { retval.json = (query_predicate58!=null?query_predicate58.json:null); }

                    }
                    break;
                case 6 :
                    // com/senseidb/bql/parsers/BQL.g:1699:9: between_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_between_predicate_in_predicate5201);
                    between_predicate59=between_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, between_predicate59.getTree());

                    if ( state.backtracking==0 ) { retval.json = (between_predicate59!=null?between_predicate59.json:null); }

                    }
                    break;
                case 7 :
                    // com/senseidb/bql/parsers/BQL.g:1700:9: range_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_range_predicate_in_predicate5213);
                    range_predicate60=range_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, range_predicate60.getTree());

                    if ( state.backtracking==0 ) { retval.json = (range_predicate60!=null?range_predicate60.json:null); }

                    }
                    break;
                case 8 :
                    // com/senseidb/bql/parsers/BQL.g:1701:9: time_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_time_predicate_in_predicate5225);
                    time_predicate61=time_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, time_predicate61.getTree());

                    if ( state.backtracking==0 ) { retval.json = (time_predicate61!=null?time_predicate61.json:null); }

                    }
                    break;
                case 9 :
                    // com/senseidb/bql/parsers/BQL.g:1702:9: match_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_match_predicate_in_predicate5237);
                    match_predicate62=match_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, match_predicate62.getTree());

                    if ( state.backtracking==0 ) { retval.json = (match_predicate62!=null?match_predicate62.json:null); }

                    }
                    break;
                case 10 :
                    // com/senseidb/bql/parsers/BQL.g:1703:9: like_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_like_predicate_in_predicate5249);
                    like_predicate63=like_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, like_predicate63.getTree());

                    if ( state.backtracking==0 ) { retval.json = (like_predicate63!=null?like_predicate63.json:null); }

                    }
                    break;
                case 11 :
                    // com/senseidb/bql/parsers/BQL.g:1704:9: null_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_null_predicate_in_predicate5261);
                    null_predicate64=null_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, null_predicate64.getTree());

                    if ( state.backtracking==0 ) { retval.json = (null_predicate64!=null?null_predicate64.json:null); }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 20, predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "predicate"


    public static class in_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "in_predicate"
    // com/senseidb/bql/parsers/BQL.g:1707:1: in_predicate returns [JSONObject json] : column_name (not= NOT )? IN value_list (except= except_clause )? ( predicate_props )? -> ^( IN ( NOT )? ^( column_name value_list ) ( except_clause )? ( predicate_props )? ) ;
    public final BQLParser.in_predicate_return in_predicate() throws RecognitionException {
        BQLParser.in_predicate_return retval = new BQLParser.in_predicate_return();
        retval.start = input.LT(1);

        int in_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token not=null;
        Token IN66=null;
        BQLParser.except_clause_return except =null;

        BQLParser.column_name_return column_name65 =null;

        BQLParser.value_list_return value_list67 =null;

        BQLParser.predicate_props_return predicate_props68 =null;


        Object not_tree=null;
        Object IN66_tree=null;
        RewriteRuleTokenStream stream_IN=new RewriteRuleTokenStream(adaptor,"token IN");
        RewriteRuleTokenStream stream_NOT=new RewriteRuleTokenStream(adaptor,"token NOT");
        RewriteRuleSubtreeStream stream_predicate_props=new RewriteRuleSubtreeStream(adaptor,"rule predicate_props");
        RewriteRuleSubtreeStream stream_except_clause=new RewriteRuleSubtreeStream(adaptor,"rule except_clause");
        RewriteRuleSubtreeStream stream_column_name=new RewriteRuleSubtreeStream(adaptor,"rule column_name");
        RewriteRuleSubtreeStream stream_value_list=new RewriteRuleSubtreeStream(adaptor,"rule value_list");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 21) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1708:5: ( column_name (not= NOT )? IN value_list (except= except_clause )? ( predicate_props )? -> ^( IN ( NOT )? ^( column_name value_list ) ( except_clause )? ( predicate_props )? ) )
            // com/senseidb/bql/parsers/BQL.g:1708:9: column_name (not= NOT )? IN value_list (except= except_clause )? ( predicate_props )?
            {
            pushFollow(FOLLOW_column_name_in_in_predicate5286);
            column_name65=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_column_name.add(column_name65.getTree());

            // com/senseidb/bql/parsers/BQL.g:1708:24: (not= NOT )?
            int alt28=2;
            switch ( input.LA(1) ) {
                case NOT:
                    {
                    alt28=1;
                    }
                    break;
            }

            switch (alt28) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1708:24: not= NOT
                    {
                    not=(Token)match(input,NOT,FOLLOW_NOT_in_in_predicate5290); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_NOT.add(not);


                    }
                    break;

            }


            IN66=(Token)match(input,IN,FOLLOW_IN_in_in_predicate5293); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_IN.add(IN66);


            pushFollow(FOLLOW_value_list_in_in_predicate5295);
            value_list67=value_list();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_value_list.add(value_list67.getTree());

            // com/senseidb/bql/parsers/BQL.g:1708:50: (except= except_clause )?
            int alt29=2;
            switch ( input.LA(1) ) {
                case EXCEPT:
                    {
                    alt29=1;
                    }
                    break;
            }

            switch (alt29) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1708:50: except= except_clause
                    {
                    pushFollow(FOLLOW_except_clause_in_in_predicate5299);
                    except=except_clause();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_except_clause.add(except.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:1708:66: ( predicate_props )?
            int alt30=2;
            switch ( input.LA(1) ) {
                case WITH:
                    {
                    alt30=1;
                    }
                    break;
            }

            switch (alt30) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1708:66: predicate_props
                    {
                    pushFollow(FOLLOW_predicate_props_in_in_predicate5302);
                    predicate_props68=predicate_props();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_predicate_props.add(predicate_props68.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        String col = (column_name65!=null?column_name65.text:null);
                        String[] facetInfo = _facetInfoMap.get(col);

                        if (facetInfo != null && facetInfo[0].equals("range")) {
                            throw new FailedPredicateException(input, 
                                                               "in_predicate",
                                                               "Range facet \"" + col + "\" cannot be used in IN predicates.");
                        }
                        if (!verifyFieldDataType(col, (value_list67!=null?value_list67.json:null))) {
                            throw new FailedPredicateException(input,
                                                               "in_predicate",
                                                               "Value list for IN predicate of facet \"" + col + "\" contains incompatible value(s).");
                        }

                        if (except != null && !verifyFieldDataType(col, (except!=null?except.json:null))) {
                            throw new FailedPredicateException(input,
                                                               "in_predicate",
                                                               "EXCEPT value list for IN predicate of facet \"" + col + "\" contains incompatible value(s).");
                        }

                        try {
                            JSONObject dict = new JSONObject();
                            dict.put("operator", "or");
                            if (not == null) {
                                dict.put("values", (value_list67!=null?value_list67.json:null));
                                if (except != null) {
                                    dict.put("excludes", (except!=null?except.json:null));
                                }
                                else {
                                    dict.put("excludes", new JSONArray());
                                }
                            }
                            else {
                                dict.put("excludes", (value_list67!=null?value_list67.json:null));
                                if (except != null) {
                                    dict.put("values", (except!=null?except.json:null));
                                }
                                else {
                                    dict.put("values", new JSONArray());
                                }
                            }
                            if (_facetInfoMap.get(col) == null) {
                                dict.put("_noOptimize", true);
                            }
                            retval.json = new JSONObject().put("terms",
                                                         new JSONObject().put(col, dict));
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "in_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            // AST REWRITE
            // elements: IN, except_clause, NOT, predicate_props, value_list, column_name
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Object)adaptor.nil();
            // 1761:9: -> ^( IN ( NOT )? ^( column_name value_list ) ( except_clause )? ( predicate_props )? )
            {
                // com/senseidb/bql/parsers/BQL.g:1761:12: ^( IN ( NOT )? ^( column_name value_list ) ( except_clause )? ( predicate_props )? )
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(
                stream_IN.nextNode()
                , root_1);

                // com/senseidb/bql/parsers/BQL.g:1761:17: ( NOT )?
                if ( stream_NOT.hasNext() ) {
                    adaptor.addChild(root_1, 
                    stream_NOT.nextNode()
                    );

                }
                stream_NOT.reset();

                // com/senseidb/bql/parsers/BQL.g:1761:22: ^( column_name value_list )
                {
                Object root_2 = (Object)adaptor.nil();
                root_2 = (Object)adaptor.becomeRoot(stream_column_name.nextNode(), root_2);

                adaptor.addChild(root_2, stream_value_list.nextTree());

                adaptor.addChild(root_1, root_2);
                }

                // com/senseidb/bql/parsers/BQL.g:1761:48: ( except_clause )?
                if ( stream_except_clause.hasNext() ) {
                    adaptor.addChild(root_1, stream_except_clause.nextTree());

                }
                stream_except_clause.reset();

                // com/senseidb/bql/parsers/BQL.g:1761:63: ( predicate_props )?
                if ( stream_predicate_props.hasNext() ) {
                    adaptor.addChild(root_1, stream_predicate_props.nextTree());

                }
                stream_predicate_props.reset();

                adaptor.addChild(root_0, root_1);
                }

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 21, in_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "in_predicate"


    public static class contains_all_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "contains_all_predicate"
    // com/senseidb/bql/parsers/BQL.g:1764:1: contains_all_predicate returns [JSONObject json] : column_name CONTAINS ALL value_list (except= except_clause )? ( predicate_props )? -> ^( CONTAINS ^( column_name value_list ) ( except_clause )? ( predicate_props )? ) ;
    public final BQLParser.contains_all_predicate_return contains_all_predicate() throws RecognitionException {
        BQLParser.contains_all_predicate_return retval = new BQLParser.contains_all_predicate_return();
        retval.start = input.LT(1);

        int contains_all_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token CONTAINS70=null;
        Token ALL71=null;
        BQLParser.except_clause_return except =null;

        BQLParser.column_name_return column_name69 =null;

        BQLParser.value_list_return value_list72 =null;

        BQLParser.predicate_props_return predicate_props73 =null;


        Object CONTAINS70_tree=null;
        Object ALL71_tree=null;
        RewriteRuleTokenStream stream_CONTAINS=new RewriteRuleTokenStream(adaptor,"token CONTAINS");
        RewriteRuleTokenStream stream_ALL=new RewriteRuleTokenStream(adaptor,"token ALL");
        RewriteRuleSubtreeStream stream_predicate_props=new RewriteRuleSubtreeStream(adaptor,"rule predicate_props");
        RewriteRuleSubtreeStream stream_except_clause=new RewriteRuleSubtreeStream(adaptor,"rule except_clause");
        RewriteRuleSubtreeStream stream_column_name=new RewriteRuleSubtreeStream(adaptor,"rule column_name");
        RewriteRuleSubtreeStream stream_value_list=new RewriteRuleSubtreeStream(adaptor,"rule value_list");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 22) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1765:5: ( column_name CONTAINS ALL value_list (except= except_clause )? ( predicate_props )? -> ^( CONTAINS ^( column_name value_list ) ( except_clause )? ( predicate_props )? ) )
            // com/senseidb/bql/parsers/BQL.g:1765:9: column_name CONTAINS ALL value_list (except= except_clause )? ( predicate_props )?
            {
            pushFollow(FOLLOW_column_name_in_contains_all_predicate5365);
            column_name69=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_column_name.add(column_name69.getTree());

            CONTAINS70=(Token)match(input,CONTAINS,FOLLOW_CONTAINS_in_contains_all_predicate5367); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_CONTAINS.add(CONTAINS70);


            ALL71=(Token)match(input,ALL,FOLLOW_ALL_in_contains_all_predicate5369); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_ALL.add(ALL71);


            pushFollow(FOLLOW_value_list_in_contains_all_predicate5371);
            value_list72=value_list();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_value_list.add(value_list72.getTree());

            // com/senseidb/bql/parsers/BQL.g:1765:51: (except= except_clause )?
            int alt31=2;
            switch ( input.LA(1) ) {
                case EXCEPT:
                    {
                    alt31=1;
                    }
                    break;
            }

            switch (alt31) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1765:51: except= except_clause
                    {
                    pushFollow(FOLLOW_except_clause_in_contains_all_predicate5375);
                    except=except_clause();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_except_clause.add(except.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:1765:67: ( predicate_props )?
            int alt32=2;
            switch ( input.LA(1) ) {
                case WITH:
                    {
                    alt32=1;
                    }
                    break;
            }

            switch (alt32) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1765:67: predicate_props
                    {
                    pushFollow(FOLLOW_predicate_props_in_contains_all_predicate5378);
                    predicate_props73=predicate_props();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_predicate_props.add(predicate_props73.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        String col = (column_name69!=null?column_name69.text:null);
                        String[] facetInfo = _facetInfoMap.get(col);
                        if (facetInfo != null && facetInfo[0].equals("range")) {
                            throw new FailedPredicateException(input, 
                                                               "contains_all_predicate", 
                                                               "Range facet column \"" + col + "\" cannot be used in CONTAINS ALL predicates.");
                        }
                        if (!verifyFieldDataType(col, (value_list72!=null?value_list72.json:null))) {
                            throw new FailedPredicateException(input,
                                                               "contains_all_predicate",
                                                               "Value list for CONTAINS ALL predicate of facet \"" + col + "\" contains incompatible value(s).");
                        }

                        if (except != null && !verifyFieldDataType(col, (except!=null?except.json:null))) {
                            throw new FailedPredicateException(input,
                                                               "contains_all_predicate",
                                                               "EXCEPT value list for CONTAINS ALL predicate of facet \"" + col + "\" contains incompatible value(s).");
                        }

                        try {
                            JSONObject dict = new JSONObject();
                            dict.put("operator", "and");
                            dict.put("values", (value_list72!=null?value_list72.json:null));
                            if (except != null) {
                                dict.put("excludes", (except!=null?except.json:null));
                            }
                            else {
                                dict.put("excludes", new JSONArray());
                            }
                            if (_facetInfoMap.get(col) == null) {
                                dict.put("_noOptimize", true);
                            }
                            retval.json = new JSONObject().put("terms",
                                                         new JSONObject().put(col, dict));
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "contains_all_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            // AST REWRITE
            // elements: value_list, except_clause, predicate_props, column_name, CONTAINS
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Object)adaptor.nil();
            // 1806:9: -> ^( CONTAINS ^( column_name value_list ) ( except_clause )? ( predicate_props )? )
            {
                // com/senseidb/bql/parsers/BQL.g:1806:12: ^( CONTAINS ^( column_name value_list ) ( except_clause )? ( predicate_props )? )
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(
                stream_CONTAINS.nextNode()
                , root_1);

                // com/senseidb/bql/parsers/BQL.g:1806:23: ^( column_name value_list )
                {
                Object root_2 = (Object)adaptor.nil();
                root_2 = (Object)adaptor.becomeRoot(stream_column_name.nextNode(), root_2);

                adaptor.addChild(root_2, stream_value_list.nextTree());

                adaptor.addChild(root_1, root_2);
                }

                // com/senseidb/bql/parsers/BQL.g:1806:49: ( except_clause )?
                if ( stream_except_clause.hasNext() ) {
                    adaptor.addChild(root_1, stream_except_clause.nextTree());

                }
                stream_except_clause.reset();

                // com/senseidb/bql/parsers/BQL.g:1806:64: ( predicate_props )?
                if ( stream_predicate_props.hasNext() ) {
                    adaptor.addChild(root_1, stream_predicate_props.nextTree());

                }
                stream_predicate_props.reset();

                adaptor.addChild(root_0, root_1);
                }

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 22, contains_all_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "contains_all_predicate"


    public static class equal_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "equal_predicate"
    // com/senseidb/bql/parsers/BQL.g:1809:1: equal_predicate returns [JSONObject json] : column_name EQUAL value (props= predicate_props )? -> ^( EQUAL column_name value ( predicate_props )? ) ;
    public final BQLParser.equal_predicate_return equal_predicate() throws RecognitionException {
        BQLParser.equal_predicate_return retval = new BQLParser.equal_predicate_return();
        retval.start = input.LT(1);

        int equal_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token EQUAL75=null;
        BQLParser.predicate_props_return props =null;

        BQLParser.column_name_return column_name74 =null;

        BQLParser.value_return value76 =null;


        Object EQUAL75_tree=null;
        RewriteRuleTokenStream stream_EQUAL=new RewriteRuleTokenStream(adaptor,"token EQUAL");
        RewriteRuleSubtreeStream stream_predicate_props=new RewriteRuleSubtreeStream(adaptor,"rule predicate_props");
        RewriteRuleSubtreeStream stream_value=new RewriteRuleSubtreeStream(adaptor,"rule value");
        RewriteRuleSubtreeStream stream_column_name=new RewriteRuleSubtreeStream(adaptor,"rule column_name");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 23) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1810:5: ( column_name EQUAL value (props= predicate_props )? -> ^( EQUAL column_name value ( predicate_props )? ) )
            // com/senseidb/bql/parsers/BQL.g:1810:9: column_name EQUAL value (props= predicate_props )?
            {
            pushFollow(FOLLOW_column_name_in_equal_predicate5439);
            column_name74=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_column_name.add(column_name74.getTree());

            EQUAL75=(Token)match(input,EQUAL,FOLLOW_EQUAL_in_equal_predicate5441); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_EQUAL.add(EQUAL75);


            pushFollow(FOLLOW_value_in_equal_predicate5443);
            value76=value();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_value.add(value76.getTree());

            // com/senseidb/bql/parsers/BQL.g:1810:38: (props= predicate_props )?
            int alt33=2;
            switch ( input.LA(1) ) {
                case WITH:
                    {
                    alt33=1;
                    }
                    break;
            }

            switch (alt33) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1810:38: props= predicate_props
                    {
                    pushFollow(FOLLOW_predicate_props_in_equal_predicate5447);
                    props=predicate_props();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_predicate_props.add(props.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        String col = (column_name74!=null?column_name74.text:null);
                        if (!verifyFieldDataType(col, (value76!=null?value76.val:null))) {
                            throw new FailedPredicateException(input, 
                                                               "equal_predicate", 
                                                               "Incompatible data type was found in an EQUAL predicate for column \"" + col + "\".");
                        }
                        try {
                            String[] facetInfo = _facetInfoMap.get(col);
                            if (facetInfo != null && facetInfo[0].equals("range")) {
                                retval.json = new JSONObject().put("range",
                                                             new JSONObject().put(col,
                                                                                  new JSONObject().put("from", (value76!=null?value76.val:null))
                                                                                                  .put("to", (value76!=null?value76.val:null))
                                                                                                  .put("include_lower", true)
                                                                                                  .put("include_upper", true)));
                            }
                            else if (facetInfo != null && facetInfo[0].equals("path")) {
                                JSONObject valObj = new JSONObject();
                                valObj.put("value", (value76!=null?value76.val:null));
                                if (props != null) {
                                    JSONObject propsJson = (props!=null?props.json:null);
                                    Iterator<String> itr = propsJson.keys();
                                    while (itr.hasNext()) {
                                        String key = itr.next();
                                        if (key.equals("strict") || key.equals("depth")) {
                                            valObj.put(key, propsJson.get(key));
                                        }
                                        else {
                                            throw new FailedPredicateException(input,
                                                                               "equal_predicate", 
                                                                               "Unsupported property was found in an EQUAL predicate for path facet column \"" + col + "\": " + key + ".");
                                        }
                                    }
                                }
                                retval.json = new JSONObject().put("path", new JSONObject().put(col, valObj));
                            }
                            else {
                                JSONObject valSpec = new JSONObject().put("value", (value76!=null?value76.val:null));
                                if (_facetInfoMap.get(col) == null) {
                                    valSpec.put("_noOptimize", true);
                                }
                                retval.json = new JSONObject().put("term", 
                                                             new JSONObject().put(col, valSpec));
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "equal_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            // AST REWRITE
            // elements: predicate_props, column_name, value, EQUAL
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Object)adaptor.nil();
            // 1861:9: -> ^( EQUAL column_name value ( predicate_props )? )
            {
                // com/senseidb/bql/parsers/BQL.g:1861:12: ^( EQUAL column_name value ( predicate_props )? )
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(
                stream_EQUAL.nextNode()
                , root_1);

                adaptor.addChild(root_1, stream_column_name.nextTree());

                adaptor.addChild(root_1, stream_value.nextTree());

                // com/senseidb/bql/parsers/BQL.g:1861:38: ( predicate_props )?
                if ( stream_predicate_props.hasNext() ) {
                    adaptor.addChild(root_1, stream_predicate_props.nextTree());

                }
                stream_predicate_props.reset();

                adaptor.addChild(root_0, root_1);
                }

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 23, equal_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "equal_predicate"


    public static class not_equal_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "not_equal_predicate"
    // com/senseidb/bql/parsers/BQL.g:1864:1: not_equal_predicate returns [JSONObject json] : column_name NOT_EQUAL value ( predicate_props )? -> ^( NOT_EQUAL column_name value ( predicate_props )? ) ;
    public final BQLParser.not_equal_predicate_return not_equal_predicate() throws RecognitionException {
        BQLParser.not_equal_predicate_return retval = new BQLParser.not_equal_predicate_return();
        retval.start = input.LT(1);

        int not_equal_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token NOT_EQUAL78=null;
        BQLParser.column_name_return column_name77 =null;

        BQLParser.value_return value79 =null;

        BQLParser.predicate_props_return predicate_props80 =null;


        Object NOT_EQUAL78_tree=null;
        RewriteRuleTokenStream stream_NOT_EQUAL=new RewriteRuleTokenStream(adaptor,"token NOT_EQUAL");
        RewriteRuleSubtreeStream stream_predicate_props=new RewriteRuleSubtreeStream(adaptor,"rule predicate_props");
        RewriteRuleSubtreeStream stream_value=new RewriteRuleSubtreeStream(adaptor,"rule value");
        RewriteRuleSubtreeStream stream_column_name=new RewriteRuleSubtreeStream(adaptor,"rule column_name");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 24) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1865:5: ( column_name NOT_EQUAL value ( predicate_props )? -> ^( NOT_EQUAL column_name value ( predicate_props )? ) )
            // com/senseidb/bql/parsers/BQL.g:1865:9: column_name NOT_EQUAL value ( predicate_props )?
            {
            pushFollow(FOLLOW_column_name_in_not_equal_predicate5502);
            column_name77=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_column_name.add(column_name77.getTree());

            NOT_EQUAL78=(Token)match(input,NOT_EQUAL,FOLLOW_NOT_EQUAL_in_not_equal_predicate5504); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_NOT_EQUAL.add(NOT_EQUAL78);


            pushFollow(FOLLOW_value_in_not_equal_predicate5506);
            value79=value();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_value.add(value79.getTree());

            // com/senseidb/bql/parsers/BQL.g:1865:37: ( predicate_props )?
            int alt34=2;
            switch ( input.LA(1) ) {
                case WITH:
                    {
                    alt34=1;
                    }
                    break;
            }

            switch (alt34) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1865:37: predicate_props
                    {
                    pushFollow(FOLLOW_predicate_props_in_not_equal_predicate5508);
                    predicate_props80=predicate_props();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_predicate_props.add(predicate_props80.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        String col = (column_name77!=null?column_name77.text:null);
                        if (!verifyFieldDataType(col, (value79!=null?value79.val:null))) {
                            throw new FailedPredicateException(input, 
                                                               "not_equal_predicate", 
                                                               "Incompatible data type was found in a NOT EQUAL predicate for column \"" + col + "\".");
                        }
                        try {
                            String[] facetInfo = _facetInfoMap.get(col);
                            if (facetInfo != null && facetInfo[0].equals("range")) {
                                JSONObject left = new JSONObject().put("range",
                                                                       new JSONObject().put(col,
                                                                                            new JSONObject().put("to", (value79!=null?value79.val:null))
                                                                                                            .put("include_upper", false)));
                                JSONObject right = new JSONObject().put("range",
                                                                        new JSONObject().put(col,
                                                                                             new JSONObject().put("from", (value79!=null?value79.val:null))
                                                                                                             .put("include_lower", false)));
                                retval.json = new JSONObject().put("or", new JSONArray().put(left).put(right));
                            }
                            else if (facetInfo != null && facetInfo[0].equals("path")) {
                                throw new FailedPredicateException(input, 
                                                                   "not_equal_predicate",
                                                                   "NOT EQUAL predicate is not supported for path facets (column \"" + col + "\").");
                            }
                            else {
                                JSONObject valObj = new JSONObject();
                                valObj.put("operator", "or");
                                valObj.put("values", new JSONArray());
                                valObj.put("excludes", new JSONArray().put((value79!=null?value79.val:null)));
                                if (_facetInfoMap.get(col) == null) {
                                    valObj.put("_noOptimize", true);
                                }
                                retval.json = new JSONObject().put("terms",
                                                             new JSONObject().put(col, valObj));
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "not_equal_predicate", "JSONException: " + err.getMessage());
                        }                                         
                    }

            // AST REWRITE
            // elements: column_name, predicate_props, NOT_EQUAL, value
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Object)adaptor.nil();
            // 1907:9: -> ^( NOT_EQUAL column_name value ( predicate_props )? )
            {
                // com/senseidb/bql/parsers/BQL.g:1907:12: ^( NOT_EQUAL column_name value ( predicate_props )? )
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(
                stream_NOT_EQUAL.nextNode()
                , root_1);

                adaptor.addChild(root_1, stream_column_name.nextTree());

                adaptor.addChild(root_1, stream_value.nextTree());

                // com/senseidb/bql/parsers/BQL.g:1907:42: ( predicate_props )?
                if ( stream_predicate_props.hasNext() ) {
                    adaptor.addChild(root_1, stream_predicate_props.nextTree());

                }
                stream_predicate_props.reset();

                adaptor.addChild(root_0, root_1);
                }

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 24, not_equal_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "not_equal_predicate"


    public static class query_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "query_predicate"
    // com/senseidb/bql/parsers/BQL.g:1910:1: query_predicate returns [JSONObject json] : QUERY IS STRING_LITERAL -> ^( QUERY STRING_LITERAL ) ;
    public final BQLParser.query_predicate_return query_predicate() throws RecognitionException {
        BQLParser.query_predicate_return retval = new BQLParser.query_predicate_return();
        retval.start = input.LT(1);

        int query_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token QUERY81=null;
        Token IS82=null;
        Token STRING_LITERAL83=null;

        Object QUERY81_tree=null;
        Object IS82_tree=null;
        Object STRING_LITERAL83_tree=null;
        RewriteRuleTokenStream stream_QUERY=new RewriteRuleTokenStream(adaptor,"token QUERY");
        RewriteRuleTokenStream stream_STRING_LITERAL=new RewriteRuleTokenStream(adaptor,"token STRING_LITERAL");
        RewriteRuleTokenStream stream_IS=new RewriteRuleTokenStream(adaptor,"token IS");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 25) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1911:5: ( QUERY IS STRING_LITERAL -> ^( QUERY STRING_LITERAL ) )
            // com/senseidb/bql/parsers/BQL.g:1911:9: QUERY IS STRING_LITERAL
            {
            QUERY81=(Token)match(input,QUERY,FOLLOW_QUERY_in_query_predicate5563); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_QUERY.add(QUERY81);


            IS82=(Token)match(input,IS,FOLLOW_IS_in_query_predicate5565); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_IS.add(IS82);


            STRING_LITERAL83=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_query_predicate5567); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_STRING_LITERAL.add(STRING_LITERAL83);


            if ( state.backtracking==0 ) {
                        try {
                            String orig = (STRING_LITERAL83!=null?STRING_LITERAL83.getText():null);
                            orig = orig.substring(1, orig.length() - 1);
                            retval.json = new JSONObject().put("query",
                                                         new JSONObject().put("query_string",
                                                                              new JSONObject().put("query", orig)));
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "query_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            // AST REWRITE
            // elements: STRING_LITERAL, QUERY
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Object)adaptor.nil();
            // 1924:9: -> ^( QUERY STRING_LITERAL )
            {
                // com/senseidb/bql/parsers/BQL.g:1924:12: ^( QUERY STRING_LITERAL )
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(
                stream_QUERY.nextNode()
                , root_1);

                adaptor.addChild(root_1, 
                stream_STRING_LITERAL.nextNode()
                );

                adaptor.addChild(root_0, root_1);
                }

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 25, query_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "query_predicate"


    public static class between_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "between_predicate"
    // com/senseidb/bql/parsers/BQL.g:1927:1: between_predicate returns [JSONObject json] : column_name (not= NOT )? BETWEEN val1= value AND val2= value -> ^( BETWEEN ( NOT )? $val1 $val2) ;
    public final BQLParser.between_predicate_return between_predicate() throws RecognitionException {
        BQLParser.between_predicate_return retval = new BQLParser.between_predicate_return();
        retval.start = input.LT(1);

        int between_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token not=null;
        Token BETWEEN85=null;
        Token AND86=null;
        BQLParser.value_return val1 =null;

        BQLParser.value_return val2 =null;

        BQLParser.column_name_return column_name84 =null;


        Object not_tree=null;
        Object BETWEEN85_tree=null;
        Object AND86_tree=null;
        RewriteRuleTokenStream stream_NOT=new RewriteRuleTokenStream(adaptor,"token NOT");
        RewriteRuleTokenStream stream_AND=new RewriteRuleTokenStream(adaptor,"token AND");
        RewriteRuleTokenStream stream_BETWEEN=new RewriteRuleTokenStream(adaptor,"token BETWEEN");
        RewriteRuleSubtreeStream stream_value=new RewriteRuleSubtreeStream(adaptor,"rule value");
        RewriteRuleSubtreeStream stream_column_name=new RewriteRuleSubtreeStream(adaptor,"rule column_name");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 26) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1928:5: ( column_name (not= NOT )? BETWEEN val1= value AND val2= value -> ^( BETWEEN ( NOT )? $val1 $val2) )
            // com/senseidb/bql/parsers/BQL.g:1928:9: column_name (not= NOT )? BETWEEN val1= value AND val2= value
            {
            pushFollow(FOLLOW_column_name_in_between_predicate5616);
            column_name84=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_column_name.add(column_name84.getTree());

            // com/senseidb/bql/parsers/BQL.g:1928:24: (not= NOT )?
            int alt35=2;
            switch ( input.LA(1) ) {
                case NOT:
                    {
                    alt35=1;
                    }
                    break;
            }

            switch (alt35) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1928:24: not= NOT
                    {
                    not=(Token)match(input,NOT,FOLLOW_NOT_in_between_predicate5620); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_NOT.add(not);


                    }
                    break;

            }


            BETWEEN85=(Token)match(input,BETWEEN,FOLLOW_BETWEEN_in_between_predicate5623); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_BETWEEN.add(BETWEEN85);


            pushFollow(FOLLOW_value_in_between_predicate5627);
            val1=value();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_value.add(val1.getTree());

            AND86=(Token)match(input,AND,FOLLOW_AND_in_between_predicate5629); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_AND.add(AND86);


            pushFollow(FOLLOW_value_in_between_predicate5633);
            val2=value();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_value.add(val2.getTree());

            if ( state.backtracking==0 ) {
                        String col = (column_name84!=null?column_name84.text:null);
                        if (!verifyFacetType(col, "range")) {
                            throw new FailedPredicateException(input, 
                                                               "between_predicate",
                                                               "Non-rangable facet column \"" + col + "\" cannot be used in BETWEEN predicates.");
                        }

                        if (!verifyFieldDataType(col, new Object[]{(val1!=null?val1.val:null), (val2!=null?val2.val:null)})) {
                            throw new FailedPredicateException(input,
                                                               "between_predicate", 
                                                               "Incompatible data type was found in a BETWEEN predicate for column \"" + col + "\".");
                        }

                        try {
                            if (not == null) {
                                retval.json = new JSONObject().put("range",
                                                             new JSONObject().put(col,
                                                                                  new JSONObject().put("from", (val1!=null?val1.val:null))
                                                                                                  .put("to", (val2!=null?val2.val:null))
                                                                                                  .put("include_lower", true)
                                                                                                  .put("include_upper", true)));
                            }
                            else {
                                JSONObject range1 = 
                                    new JSONObject().put("range",
                                                         new JSONObject().put(col,
                                                                              new JSONObject().put("to", (val1!=null?val1.val:null))
                                                                                              .put("include_upper", false)));
                                JSONObject range2 = 
                                    new JSONObject().put("range",
                                                         new JSONObject().put(col,
                                                                              new JSONObject().put("from", (val2!=null?val2.val:null))
                                                                                              .put("include_lower", false)));

                                retval.json = new JSONObject().put("or", new JSONArray().put(range1).put(range2));
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "between_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            // AST REWRITE
            // elements: val2, NOT, val1, BETWEEN
            // token labels: 
            // rule labels: retval, val1, val2
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
            RewriteRuleSubtreeStream stream_val1=new RewriteRuleSubtreeStream(adaptor,"rule val1",val1!=null?val1.tree:null);
            RewriteRuleSubtreeStream stream_val2=new RewriteRuleSubtreeStream(adaptor,"rule val2",val2!=null?val2.tree:null);

            root_0 = (Object)adaptor.nil();
            // 1971:9: -> ^( BETWEEN ( NOT )? $val1 $val2)
            {
                // com/senseidb/bql/parsers/BQL.g:1971:12: ^( BETWEEN ( NOT )? $val1 $val2)
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(
                stream_BETWEEN.nextNode()
                , root_1);

                // com/senseidb/bql/parsers/BQL.g:1971:22: ( NOT )?
                if ( stream_NOT.hasNext() ) {
                    adaptor.addChild(root_1, 
                    stream_NOT.nextNode()
                    );

                }
                stream_NOT.reset();

                adaptor.addChild(root_1, stream_val1.nextTree());

                adaptor.addChild(root_1, stream_val2.nextTree());

                adaptor.addChild(root_0, root_1);
                }

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 26, between_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "between_predicate"


    public static class range_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "range_predicate"
    // com/senseidb/bql/parsers/BQL.g:1974:1: range_predicate returns [JSONObject json] : column_name (op= GT |op= GTE |op= LT |op= LTE ) val= value -> ^( $op column_name value ) ;
    public final BQLParser.range_predicate_return range_predicate() throws RecognitionException {
        BQLParser.range_predicate_return retval = new BQLParser.range_predicate_return();
        retval.start = input.LT(1);

        int range_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token op=null;
        BQLParser.value_return val =null;

        BQLParser.column_name_return column_name87 =null;


        Object op_tree=null;
        RewriteRuleTokenStream stream_GT=new RewriteRuleTokenStream(adaptor,"token GT");
        RewriteRuleTokenStream stream_LT=new RewriteRuleTokenStream(adaptor,"token LT");
        RewriteRuleTokenStream stream_LTE=new RewriteRuleTokenStream(adaptor,"token LTE");
        RewriteRuleTokenStream stream_GTE=new RewriteRuleTokenStream(adaptor,"token GTE");
        RewriteRuleSubtreeStream stream_value=new RewriteRuleSubtreeStream(adaptor,"rule value");
        RewriteRuleSubtreeStream stream_column_name=new RewriteRuleSubtreeStream(adaptor,"rule column_name");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 27) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1975:5: ( column_name (op= GT |op= GTE |op= LT |op= LTE ) val= value -> ^( $op column_name value ) )
            // com/senseidb/bql/parsers/BQL.g:1975:9: column_name (op= GT |op= GTE |op= LT |op= LTE ) val= value
            {
            pushFollow(FOLLOW_column_name_in_range_predicate5689);
            column_name87=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_column_name.add(column_name87.getTree());

            // com/senseidb/bql/parsers/BQL.g:1975:21: (op= GT |op= GTE |op= LT |op= LTE )
            int alt36=4;
            switch ( input.LA(1) ) {
            case GT:
                {
                alt36=1;
                }
                break;
            case GTE:
                {
                alt36=2;
                }
                break;
            case LT:
                {
                alt36=3;
                }
                break;
            case LTE:
                {
                alt36=4;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 36, 0, input);

                throw nvae;

            }

            switch (alt36) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1975:22: op= GT
                    {
                    op=(Token)match(input,GT,FOLLOW_GT_in_range_predicate5694); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_GT.add(op);


                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1975:30: op= GTE
                    {
                    op=(Token)match(input,GTE,FOLLOW_GTE_in_range_predicate5700); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_GTE.add(op);


                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:1975:39: op= LT
                    {
                    op=(Token)match(input,LT,FOLLOW_LT_in_range_predicate5706); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_LT.add(op);


                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:1975:47: op= LTE
                    {
                    op=(Token)match(input,LTE,FOLLOW_LTE_in_range_predicate5712); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_LTE.add(op);


                    }
                    break;

            }


            pushFollow(FOLLOW_value_in_range_predicate5717);
            val=value();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_value.add(val.getTree());

            if ( state.backtracking==0 ) {
                        String col = (column_name87!=null?column_name87.text:null);
                        if (!verifyFacetType(col, "range")) {
                            throw new FailedPredicateException(input, 
                                                               "range_predicate",
                                                               "Non-rangable facet column \"" + col + "\" cannot be used in RANGE predicates.");
                        }

                        if (!verifyFieldDataType(col, (val!=null?val.val:null))) {
                            throw new FailedPredicateException(input,
                                                               "range_predicate", 
                                                               "Incompatible data type was found in a RANGE predicate for column \"" + col + "\".");
                        }

                        try {
                            if ((op!=null?op.getText():null).charAt(0) == '>') {
                                retval.json = new JSONObject().put("range",
                                                             new JSONObject().put(col,
                                                                                  new JSONObject().put("from", (val!=null?val.val:null))
                                                                                                  .put("include_lower", ">=".equals((op!=null?op.getText():null)))));
                            }
                            else {
                                retval.json = new JSONObject().put("range",
                                                             new JSONObject().put(col,
                                                                                  new JSONObject().put("to", (val!=null?val.val:null))
                                                                                                  .put("include_upper", "<=".equals((op!=null?op.getText():null)))));
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "range_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            // AST REWRITE
            // elements: op, column_name, value
            // token labels: op
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleTokenStream stream_op=new RewriteRuleTokenStream(adaptor,"token op",op);
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Object)adaptor.nil();
            // 2008:9: -> ^( $op column_name value )
            {
                // com/senseidb/bql/parsers/BQL.g:2008:12: ^( $op column_name value )
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(stream_op.nextNode(), root_1);

                adaptor.addChild(root_1, stream_column_name.nextTree());

                adaptor.addChild(root_1, stream_value.nextTree());

                adaptor.addChild(root_0, root_1);
                }

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 27, range_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "range_predicate"


    public static class time_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "time_predicate"
    // com/senseidb/bql/parsers/BQL.g:2011:1: time_predicate returns [JSONObject json] : ( column_name ( NOT )? IN LAST time_span | column_name ( NOT )? (since= SINCE |since= AFTER |before= BEFORE ) time_expr );
    public final BQLParser.time_predicate_return time_predicate() throws RecognitionException {
        BQLParser.time_predicate_return retval = new BQLParser.time_predicate_return();
        retval.start = input.LT(1);

        int time_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token since=null;
        Token before=null;
        Token NOT89=null;
        Token IN90=null;
        Token LAST91=null;
        Token NOT94=null;
        BQLParser.column_name_return column_name88 =null;

        BQLParser.time_span_return time_span92 =null;

        BQLParser.column_name_return column_name93 =null;

        BQLParser.time_expr_return time_expr95 =null;


        Object since_tree=null;
        Object before_tree=null;
        Object NOT89_tree=null;
        Object IN90_tree=null;
        Object LAST91_tree=null;
        Object NOT94_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 28) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2012:5: ( column_name ( NOT )? IN LAST time_span | column_name ( NOT )? (since= SINCE |since= AFTER |before= BEFORE ) time_expr )
            int alt40=2;
            alt40 = dfa40.predict(input);
            switch (alt40) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2012:9: column_name ( NOT )? IN LAST time_span
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_column_name_in_time_predicate5769);
                    column_name88=column_name();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, column_name88.getTree());

                    // com/senseidb/bql/parsers/BQL.g:2012:21: ( NOT )?
                    int alt37=2;
                    switch ( input.LA(1) ) {
                        case NOT:
                            {
                            alt37=1;
                            }
                            break;
                    }

                    switch (alt37) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2012:22: NOT
                            {
                            NOT89=(Token)match(input,NOT,FOLLOW_NOT_in_time_predicate5772); if (state.failed) return retval;
                            if ( state.backtracking==0 ) {
                            NOT89_tree = 
                            (Object)adaptor.create(NOT89)
                            ;
                            adaptor.addChild(root_0, NOT89_tree);
                            }

                            }
                            break;

                    }


                    IN90=(Token)match(input,IN,FOLLOW_IN_in_time_predicate5776); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    IN90_tree = 
                    (Object)adaptor.create(IN90)
                    ;
                    adaptor.addChild(root_0, IN90_tree);
                    }

                    LAST91=(Token)match(input,LAST,FOLLOW_LAST_in_time_predicate5778); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    LAST91_tree = 
                    (Object)adaptor.create(LAST91)
                    ;
                    adaptor.addChild(root_0, LAST91_tree);
                    }

                    pushFollow(FOLLOW_time_span_in_time_predicate5780);
                    time_span92=time_span();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, time_span92.getTree());

                    if ( state.backtracking==0 ) {
                                String col = (column_name88!=null?column_name88.text:null);
                                if (!verifyFacetType(col, "range")) {
                                    throw new FailedPredicateException(input, 
                                                                       "range_predicate",
                                                                       "Non-rangable facet column \"" + col + "\" cannot be used in TIME predicates.");
                                }

                                try {
                                    if (NOT89 == null) {
                                        retval.json = new JSONObject().put("range",
                                                                     new JSONObject().put(col,
                                                                                          new JSONObject().put("from", (time_span92!=null?time_span92.val:0))
                                                                                                          .put("include_lower", false)));
                                    }
                                    else {
                                        retval.json = new JSONObject().put("range",
                                                                     new JSONObject().put(col,
                                                                                          new JSONObject().put("to", (time_span92!=null?time_span92.val:0))
                                                                                                          .put("include_upper", true)));
                                    }
                                }
                                catch (JSONException err) {
                                    throw new FailedPredicateException(input, "time_predicate", "JSONException: " + err.getMessage());
                                }
                            }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2039:9: column_name ( NOT )? (since= SINCE |since= AFTER |before= BEFORE ) time_expr
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_column_name_in_time_predicate5800);
                    column_name93=column_name();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, column_name93.getTree());

                    // com/senseidb/bql/parsers/BQL.g:2039:21: ( NOT )?
                    int alt38=2;
                    switch ( input.LA(1) ) {
                        case NOT:
                            {
                            alt38=1;
                            }
                            break;
                    }

                    switch (alt38) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2039:22: NOT
                            {
                            NOT94=(Token)match(input,NOT,FOLLOW_NOT_in_time_predicate5803); if (state.failed) return retval;
                            if ( state.backtracking==0 ) {
                            NOT94_tree = 
                            (Object)adaptor.create(NOT94)
                            ;
                            adaptor.addChild(root_0, NOT94_tree);
                            }

                            }
                            break;

                    }


                    // com/senseidb/bql/parsers/BQL.g:2039:28: (since= SINCE |since= AFTER |before= BEFORE )
                    int alt39=3;
                    switch ( input.LA(1) ) {
                    case SINCE:
                        {
                        alt39=1;
                        }
                        break;
                    case AFTER:
                        {
                        alt39=2;
                        }
                        break;
                    case BEFORE:
                        {
                        alt39=3;
                        }
                        break;
                    default:
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        NoViableAltException nvae =
                            new NoViableAltException("", 39, 0, input);

                        throw nvae;

                    }

                    switch (alt39) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2039:29: since= SINCE
                            {
                            since=(Token)match(input,SINCE,FOLLOW_SINCE_in_time_predicate5810); if (state.failed) return retval;
                            if ( state.backtracking==0 ) {
                            since_tree = 
                            (Object)adaptor.create(since)
                            ;
                            adaptor.addChild(root_0, since_tree);
                            }

                            }
                            break;
                        case 2 :
                            // com/senseidb/bql/parsers/BQL.g:2039:43: since= AFTER
                            {
                            since=(Token)match(input,AFTER,FOLLOW_AFTER_in_time_predicate5816); if (state.failed) return retval;
                            if ( state.backtracking==0 ) {
                            since_tree = 
                            (Object)adaptor.create(since)
                            ;
                            adaptor.addChild(root_0, since_tree);
                            }

                            }
                            break;
                        case 3 :
                            // com/senseidb/bql/parsers/BQL.g:2039:57: before= BEFORE
                            {
                            before=(Token)match(input,BEFORE,FOLLOW_BEFORE_in_time_predicate5822); if (state.failed) return retval;
                            if ( state.backtracking==0 ) {
                            before_tree = 
                            (Object)adaptor.create(before)
                            ;
                            adaptor.addChild(root_0, before_tree);
                            }

                            }
                            break;

                    }


                    pushFollow(FOLLOW_time_expr_in_time_predicate5825);
                    time_expr95=time_expr();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, time_expr95.getTree());

                    if ( state.backtracking==0 ) {
                                String col = (column_name93!=null?column_name93.text:null);
                                if (!verifyFacetType(col, "range")) {
                                    throw new FailedPredicateException(input, 
                                                                       "range_predicate",
                                                                       "Non-rangable facet column \"" + col + "\" cannot be used in TIME predicates.");
                                }

                                try {
                                    if (since != null && NOT94 == null ||
                                        since == null && NOT94 != null) {
                                        retval.json = new JSONObject().put("range",
                                                                     new JSONObject().put(col,
                                                                                          new JSONObject().put("from", (time_expr95!=null?time_expr95.val:0))
                                                                                                          .put("include_lower", false)));
                                    }
                                    else {
                                        retval.json = new JSONObject().put("range",
                                                                     new JSONObject().put(col,
                                                                                          new JSONObject().put("to", (time_expr95!=null?time_expr95.val:0))
                                                                                                          .put("include_upper", false)));
                                    }
                                }
                                catch (JSONException err) {
                                    throw new FailedPredicateException(input, "time_predicate", "JSONException: " + err.getMessage());
                                }
                            }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 28, time_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "time_predicate"


    public static class time_span_return extends ParserRuleReturnScope {
        public long val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "time_span"
    // com/senseidb/bql/parsers/BQL.g:2069:1: time_span returns [long val] : (week= time_week_part )? (day= time_day_part )? (hour= time_hour_part )? (minute= time_minute_part )? (second= time_second_part )? (msec= time_millisecond_part )? ;
    public final BQLParser.time_span_return time_span() throws RecognitionException {
        BQLParser.time_span_return retval = new BQLParser.time_span_return();
        retval.start = input.LT(1);

        int time_span_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.time_week_part_return week =null;

        BQLParser.time_day_part_return day =null;

        BQLParser.time_hour_part_return hour =null;

        BQLParser.time_minute_part_return minute =null;

        BQLParser.time_second_part_return second =null;

        BQLParser.time_millisecond_part_return msec =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 29) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2070:5: ( (week= time_week_part )? (day= time_day_part )? (hour= time_hour_part )? (minute= time_minute_part )? (second= time_second_part )? (msec= time_millisecond_part )? )
            // com/senseidb/bql/parsers/BQL.g:2070:9: (week= time_week_part )? (day= time_day_part )? (hour= time_hour_part )? (minute= time_minute_part )? (second= time_second_part )? (msec= time_millisecond_part )?
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:2070:13: (week= time_week_part )?
            int alt41=2;
            switch ( input.LA(1) ) {
                case INTEGER:
                    {
                    switch ( input.LA(2) ) {
                        case WEEKS:
                            {
                            alt41=1;
                            }
                            break;
                    }

                    }
                    break;
            }

            switch (alt41) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2070:13: week= time_week_part
                    {
                    pushFollow(FOLLOW_time_week_part_in_time_span5860);
                    week=time_week_part();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, week.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:2070:33: (day= time_day_part )?
            int alt42=2;
            switch ( input.LA(1) ) {
                case INTEGER:
                    {
                    switch ( input.LA(2) ) {
                        case DAYS:
                            {
                            alt42=1;
                            }
                            break;
                    }

                    }
                    break;
            }

            switch (alt42) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2070:33: day= time_day_part
                    {
                    pushFollow(FOLLOW_time_day_part_in_time_span5865);
                    day=time_day_part();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, day.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:2070:53: (hour= time_hour_part )?
            int alt43=2;
            switch ( input.LA(1) ) {
                case INTEGER:
                    {
                    switch ( input.LA(2) ) {
                        case HOURS:
                            {
                            alt43=1;
                            }
                            break;
                    }

                    }
                    break;
            }

            switch (alt43) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2070:53: hour= time_hour_part
                    {
                    pushFollow(FOLLOW_time_hour_part_in_time_span5870);
                    hour=time_hour_part();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, hour.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:2071:15: (minute= time_minute_part )?
            int alt44=2;
            switch ( input.LA(1) ) {
                case INTEGER:
                    {
                    switch ( input.LA(2) ) {
                        case MINS:
                        case MINUTES:
                            {
                            alt44=1;
                            }
                            break;
                    }

                    }
                    break;
            }

            switch (alt44) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2071:15: minute= time_minute_part
                    {
                    pushFollow(FOLLOW_time_minute_part_in_time_span5884);
                    minute=time_minute_part();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, minute.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:2071:40: (second= time_second_part )?
            int alt45=2;
            switch ( input.LA(1) ) {
                case INTEGER:
                    {
                    switch ( input.LA(2) ) {
                        case SECONDS:
                        case SECS:
                            {
                            alt45=1;
                            }
                            break;
                    }

                    }
                    break;
            }

            switch (alt45) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2071:40: second= time_second_part
                    {
                    pushFollow(FOLLOW_time_second_part_in_time_span5889);
                    second=time_second_part();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, second.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:2071:63: (msec= time_millisecond_part )?
            int alt46=2;
            switch ( input.LA(1) ) {
                case INTEGER:
                    {
                    alt46=1;
                    }
                    break;
            }

            switch (alt46) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2071:63: msec= time_millisecond_part
                    {
                    pushFollow(FOLLOW_time_millisecond_part_in_time_span5894);
                    msec=time_millisecond_part();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, msec.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        retval.val = 0;
                        if (week != null) retval.val += (week!=null?week.val:0);
                        if (day != null) retval.val += (day!=null?day.val:0);
                        if (hour != null) retval.val += (hour!=null?hour.val:0);
                        if (minute != null) retval.val += (minute!=null?minute.val:0);
                        if (second != null) retval.val += (second!=null?second.val:0);
                        if (msec != null) retval.val += (msec!=null?msec.val:0);
                        retval.val = _now - retval.val;
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 29, time_span_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "time_span"


    public static class time_week_part_return extends ParserRuleReturnScope {
        public long val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "time_week_part"
    // com/senseidb/bql/parsers/BQL.g:2084:1: time_week_part returns [long val] : INTEGER WEEKS ;
    public final BQLParser.time_week_part_return time_week_part() throws RecognitionException {
        BQLParser.time_week_part_return retval = new BQLParser.time_week_part_return();
        retval.start = input.LT(1);

        int time_week_part_StartIndex = input.index();

        Object root_0 = null;

        Token INTEGER96=null;
        Token WEEKS97=null;

        Object INTEGER96_tree=null;
        Object WEEKS97_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 30) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2085:5: ( INTEGER WEEKS )
            // com/senseidb/bql/parsers/BQL.g:2085:9: INTEGER WEEKS
            {
            root_0 = (Object)adaptor.nil();


            INTEGER96=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_time_week_part5928); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            INTEGER96_tree = 
            (Object)adaptor.create(INTEGER96)
            ;
            adaptor.addChild(root_0, INTEGER96_tree);
            }

            WEEKS97=(Token)match(input,WEEKS,FOLLOW_WEEKS_in_time_week_part5930); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            WEEKS97_tree = 
            (Object)adaptor.create(WEEKS97)
            ;
            adaptor.addChild(root_0, WEEKS97_tree);
            }

            if ( state.backtracking==0 ) {
                        retval.val = Integer.parseInt((INTEGER96!=null?INTEGER96.getText():null)) * 7 * 24 * 60 * 60 * 1000L;
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 30, time_week_part_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "time_week_part"


    public static class time_day_part_return extends ParserRuleReturnScope {
        public long val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "time_day_part"
    // com/senseidb/bql/parsers/BQL.g:2091:1: time_day_part returns [long val] : INTEGER DAYS ;
    public final BQLParser.time_day_part_return time_day_part() throws RecognitionException {
        BQLParser.time_day_part_return retval = new BQLParser.time_day_part_return();
        retval.start = input.LT(1);

        int time_day_part_StartIndex = input.index();

        Object root_0 = null;

        Token INTEGER98=null;
        Token DAYS99=null;

        Object INTEGER98_tree=null;
        Object DAYS99_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 31) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2092:5: ( INTEGER DAYS )
            // com/senseidb/bql/parsers/BQL.g:2092:9: INTEGER DAYS
            {
            root_0 = (Object)adaptor.nil();


            INTEGER98=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_time_day_part5963); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            INTEGER98_tree = 
            (Object)adaptor.create(INTEGER98)
            ;
            adaptor.addChild(root_0, INTEGER98_tree);
            }

            DAYS99=(Token)match(input,DAYS,FOLLOW_DAYS_in_time_day_part5965); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            DAYS99_tree = 
            (Object)adaptor.create(DAYS99)
            ;
            adaptor.addChild(root_0, DAYS99_tree);
            }

            if ( state.backtracking==0 ) {
                      retval.val = Integer.parseInt((INTEGER98!=null?INTEGER98.getText():null)) * 24 * 60 * 60 * 1000L;
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 31, time_day_part_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "time_day_part"


    public static class time_hour_part_return extends ParserRuleReturnScope {
        public long val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "time_hour_part"
    // com/senseidb/bql/parsers/BQL.g:2098:1: time_hour_part returns [long val] : INTEGER HOURS ;
    public final BQLParser.time_hour_part_return time_hour_part() throws RecognitionException {
        BQLParser.time_hour_part_return retval = new BQLParser.time_hour_part_return();
        retval.start = input.LT(1);

        int time_hour_part_StartIndex = input.index();

        Object root_0 = null;

        Token INTEGER100=null;
        Token HOURS101=null;

        Object INTEGER100_tree=null;
        Object HOURS101_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 32) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2099:5: ( INTEGER HOURS )
            // com/senseidb/bql/parsers/BQL.g:2099:9: INTEGER HOURS
            {
            root_0 = (Object)adaptor.nil();


            INTEGER100=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_time_hour_part5998); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            INTEGER100_tree = 
            (Object)adaptor.create(INTEGER100)
            ;
            adaptor.addChild(root_0, INTEGER100_tree);
            }

            HOURS101=(Token)match(input,HOURS,FOLLOW_HOURS_in_time_hour_part6000); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            HOURS101_tree = 
            (Object)adaptor.create(HOURS101)
            ;
            adaptor.addChild(root_0, HOURS101_tree);
            }

            if ( state.backtracking==0 ) {
                      retval.val = Integer.parseInt((INTEGER100!=null?INTEGER100.getText():null)) * 60 * 60 * 1000L;
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 32, time_hour_part_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "time_hour_part"


    public static class time_minute_part_return extends ParserRuleReturnScope {
        public long val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "time_minute_part"
    // com/senseidb/bql/parsers/BQL.g:2105:1: time_minute_part returns [long val] : INTEGER ( MINUTES | MINS ) ;
    public final BQLParser.time_minute_part_return time_minute_part() throws RecognitionException {
        BQLParser.time_minute_part_return retval = new BQLParser.time_minute_part_return();
        retval.start = input.LT(1);

        int time_minute_part_StartIndex = input.index();

        Object root_0 = null;

        Token INTEGER102=null;
        Token set103=null;

        Object INTEGER102_tree=null;
        Object set103_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 33) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2106:5: ( INTEGER ( MINUTES | MINS ) )
            // com/senseidb/bql/parsers/BQL.g:2106:9: INTEGER ( MINUTES | MINS )
            {
            root_0 = (Object)adaptor.nil();


            INTEGER102=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_time_minute_part6033); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            INTEGER102_tree = 
            (Object)adaptor.create(INTEGER102)
            ;
            adaptor.addChild(root_0, INTEGER102_tree);
            }

            set103=(Token)input.LT(1);

            if ( (input.LA(1) >= MINS && input.LA(1) <= MINUTES) ) {
                input.consume();
                if ( state.backtracking==0 ) adaptor.addChild(root_0, 
                (Object)adaptor.create(set103)
                );
                state.errorRecovery=false;
                state.failed=false;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                MismatchedSetException mse = new MismatchedSetException(null,input);
                throw mse;
            }


            if ( state.backtracking==0 ) {
                      retval.val = Integer.parseInt((INTEGER102!=null?INTEGER102.getText():null)) * 60 * 1000L;
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 33, time_minute_part_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "time_minute_part"


    public static class time_second_part_return extends ParserRuleReturnScope {
        public long val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "time_second_part"
    // com/senseidb/bql/parsers/BQL.g:2112:1: time_second_part returns [long val] : INTEGER ( SECONDS | SECS ) ;
    public final BQLParser.time_second_part_return time_second_part() throws RecognitionException {
        BQLParser.time_second_part_return retval = new BQLParser.time_second_part_return();
        retval.start = input.LT(1);

        int time_second_part_StartIndex = input.index();

        Object root_0 = null;

        Token INTEGER104=null;
        Token set105=null;

        Object INTEGER104_tree=null;
        Object set105_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 34) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2113:5: ( INTEGER ( SECONDS | SECS ) )
            // com/senseidb/bql/parsers/BQL.g:2113:9: INTEGER ( SECONDS | SECS )
            {
            root_0 = (Object)adaptor.nil();


            INTEGER104=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_time_second_part6074); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            INTEGER104_tree = 
            (Object)adaptor.create(INTEGER104)
            ;
            adaptor.addChild(root_0, INTEGER104_tree);
            }

            set105=(Token)input.LT(1);

            if ( (input.LA(1) >= SECONDS && input.LA(1) <= SECS) ) {
                input.consume();
                if ( state.backtracking==0 ) adaptor.addChild(root_0, 
                (Object)adaptor.create(set105)
                );
                state.errorRecovery=false;
                state.failed=false;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                MismatchedSetException mse = new MismatchedSetException(null,input);
                throw mse;
            }


            if ( state.backtracking==0 ) {
                      retval.val = Integer.parseInt((INTEGER104!=null?INTEGER104.getText():null)) * 1000L;
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 34, time_second_part_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "time_second_part"


    public static class time_millisecond_part_return extends ParserRuleReturnScope {
        public long val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "time_millisecond_part"
    // com/senseidb/bql/parsers/BQL.g:2119:1: time_millisecond_part returns [long val] : INTEGER ( MILLISECONDS | MSECS ) ;
    public final BQLParser.time_millisecond_part_return time_millisecond_part() throws RecognitionException {
        BQLParser.time_millisecond_part_return retval = new BQLParser.time_millisecond_part_return();
        retval.start = input.LT(1);

        int time_millisecond_part_StartIndex = input.index();

        Object root_0 = null;

        Token INTEGER106=null;
        Token set107=null;

        Object INTEGER106_tree=null;
        Object set107_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 35) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2120:5: ( INTEGER ( MILLISECONDS | MSECS ) )
            // com/senseidb/bql/parsers/BQL.g:2120:9: INTEGER ( MILLISECONDS | MSECS )
            {
            root_0 = (Object)adaptor.nil();


            INTEGER106=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_time_millisecond_part6116); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            INTEGER106_tree = 
            (Object)adaptor.create(INTEGER106)
            ;
            adaptor.addChild(root_0, INTEGER106_tree);
            }

            set107=(Token)input.LT(1);

            if ( input.LA(1)==MILLISECONDS||input.LA(1)==MSECS ) {
                input.consume();
                if ( state.backtracking==0 ) adaptor.addChild(root_0, 
                (Object)adaptor.create(set107)
                );
                state.errorRecovery=false;
                state.failed=false;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                MismatchedSetException mse = new MismatchedSetException(null,input);
                throw mse;
            }


            if ( state.backtracking==0 ) {
                      retval.val = Integer.parseInt((INTEGER106!=null?INTEGER106.getText():null));
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 35, time_millisecond_part_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "time_millisecond_part"


    public static class time_expr_return extends ParserRuleReturnScope {
        public long val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "time_expr"
    // com/senseidb/bql/parsers/BQL.g:2126:1: time_expr returns [long val] : ( time_span AGO | date_time_string | NOW );
    public final BQLParser.time_expr_return time_expr() throws RecognitionException {
        BQLParser.time_expr_return retval = new BQLParser.time_expr_return();
        retval.start = input.LT(1);

        int time_expr_StartIndex = input.index();

        Object root_0 = null;

        Token AGO109=null;
        Token NOW111=null;
        BQLParser.time_span_return time_span108 =null;

        BQLParser.date_time_string_return date_time_string110 =null;


        Object AGO109_tree=null;
        Object NOW111_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 36) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2127:5: ( time_span AGO | date_time_string | NOW )
            int alt47=3;
            switch ( input.LA(1) ) {
            case EOF:
            case AGO:
            case AND:
            case BROWSE:
            case FETCHING:
            case GIVEN:
            case GROUP:
            case INTEGER:
            case LIMIT:
            case OR:
            case ORDER:
            case ROUTE:
            case RPAR:
            case SEMI:
            case USING:
                {
                alt47=1;
                }
                break;
            case DATE:
                {
                alt47=2;
                }
                break;
            case NOW:
                {
                alt47=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 47, 0, input);

                throw nvae;

            }

            switch (alt47) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2127:9: time_span AGO
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_time_span_in_time_expr6157);
                    time_span108=time_span();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, time_span108.getTree());

                    AGO109=(Token)match(input,AGO,FOLLOW_AGO_in_time_expr6159); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    AGO109_tree = 
                    (Object)adaptor.create(AGO109)
                    ;
                    adaptor.addChild(root_0, AGO109_tree);
                    }

                    if ( state.backtracking==0 ) {
                                retval.val = (time_span108!=null?time_span108.val:0);
                            }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2131:9: date_time_string
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_date_time_string_in_time_expr6179);
                    date_time_string110=date_time_string();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, date_time_string110.getTree());

                    if ( state.backtracking==0 ) {
                                retval.val = (date_time_string110!=null?date_time_string110.val:0);
                            }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2135:9: NOW
                    {
                    root_0 = (Object)adaptor.nil();


                    NOW111=(Token)match(input,NOW,FOLLOW_NOW_in_time_expr6199); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    NOW111_tree = 
                    (Object)adaptor.create(NOW111)
                    ;
                    adaptor.addChild(root_0, NOW111_tree);
                    }

                    if ( state.backtracking==0 ) {
                                retval.val = _now;
                            }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 36, time_expr_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "time_expr"


    public static class date_time_string_return extends ParserRuleReturnScope {
        public long val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "date_time_string"
    // com/senseidb/bql/parsers/BQL.g:2141:1: date_time_string returns [long val] : DATE ( TIME )? ;
    public final BQLParser.date_time_string_return date_time_string() throws RecognitionException {
        BQLParser.date_time_string_return retval = new BQLParser.date_time_string_return();
        retval.start = input.LT(1);

        int date_time_string_StartIndex = input.index();

        Object root_0 = null;

        Token DATE112=null;
        Token TIME113=null;

        Object DATE112_tree=null;
        Object TIME113_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 37) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2142:5: ( DATE ( TIME )? )
            // com/senseidb/bql/parsers/BQL.g:2142:9: DATE ( TIME )?
            {
            root_0 = (Object)adaptor.nil();


            DATE112=(Token)match(input,DATE,FOLLOW_DATE_in_date_time_string6232); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            DATE112_tree = 
            (Object)adaptor.create(DATE112)
            ;
            adaptor.addChild(root_0, DATE112_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:2142:14: ( TIME )?
            int alt48=2;
            switch ( input.LA(1) ) {
                case TIME:
                    {
                    alt48=1;
                    }
                    break;
            }

            switch (alt48) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2142:14: TIME
                    {
                    TIME113=(Token)match(input,TIME,FOLLOW_TIME_in_date_time_string6234); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    TIME113_tree = 
                    (Object)adaptor.create(TIME113)
                    ;
                    adaptor.addChild(root_0, TIME113_tree);
                    }

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        SimpleDateFormat format;
                        String dateTimeStr = (DATE112!=null?DATE112.getText():null);
                        char separator = dateTimeStr.charAt(4);
                        if (TIME113 != null) {
                            dateTimeStr = dateTimeStr + " " + (TIME113!=null?TIME113.getText():null);
                        }
                        int formatIdx = (separator == '-' ? 0 : 1);

                        if (TIME113 == null) {
                            if (_format1[formatIdx] != null) {
                                format = _format1[formatIdx];
                            }
                            else {
                                format = _format1[formatIdx] = new SimpleDateFormat("yyyy" + separator + "MM" + separator + "dd");
                            }
                        }
                        else {
                            if (_format2[formatIdx] != null) {
                                format = _format2[formatIdx];
                            }
                            else {
                                format = _format2[formatIdx] = new SimpleDateFormat("yyyy" + separator + "MM" + separator + "dd HH:mm:ss");
                            }
                        }
                        try {
                            retval.val = format.parse(dateTimeStr).getTime();
                            if (!dateTimeStr.equals(format.format(retval.val))) {
                                throw new FailedPredicateException(input,
                                                                   "date_time_string", 
                                                                   "Date string contains invalid date/time: \"" + dateTimeStr + "\".");
                            }
                        }
                        catch (ParseException err) {
                            throw new FailedPredicateException(input,
                                                               "date_time_string", 
                                                               "ParseException happened for \"" + dateTimeStr + "\": " + 
                                                               err.getMessage() + ".");
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 37, date_time_string_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "date_time_string"


    public static class match_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "match_predicate"
    // com/senseidb/bql/parsers/BQL.g:2185:1: match_predicate returns [JSONObject json] : ( NOT )? MATCH LPAR column_name_list RPAR AGAINST LPAR STRING_LITERAL RPAR ;
    public final BQLParser.match_predicate_return match_predicate() throws RecognitionException {
        BQLParser.match_predicate_return retval = new BQLParser.match_predicate_return();
        retval.start = input.LT(1);

        int match_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token NOT114=null;
        Token MATCH115=null;
        Token LPAR116=null;
        Token RPAR118=null;
        Token AGAINST119=null;
        Token LPAR120=null;
        Token STRING_LITERAL121=null;
        Token RPAR122=null;
        BQLParser.column_name_list_return column_name_list117 =null;


        Object NOT114_tree=null;
        Object MATCH115_tree=null;
        Object LPAR116_tree=null;
        Object RPAR118_tree=null;
        Object AGAINST119_tree=null;
        Object LPAR120_tree=null;
        Object STRING_LITERAL121_tree=null;
        Object RPAR122_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 38) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2186:5: ( ( NOT )? MATCH LPAR column_name_list RPAR AGAINST LPAR STRING_LITERAL RPAR )
            // com/senseidb/bql/parsers/BQL.g:2186:9: ( NOT )? MATCH LPAR column_name_list RPAR AGAINST LPAR STRING_LITERAL RPAR
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:2186:9: ( NOT )?
            int alt49=2;
            switch ( input.LA(1) ) {
                case NOT:
                    {
                    alt49=1;
                    }
                    break;
            }

            switch (alt49) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2186:10: NOT
                    {
                    NOT114=(Token)match(input,NOT,FOLLOW_NOT_in_match_predicate6269); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    NOT114_tree = 
                    (Object)adaptor.create(NOT114)
                    ;
                    adaptor.addChild(root_0, NOT114_tree);
                    }

                    }
                    break;

            }


            MATCH115=(Token)match(input,MATCH,FOLLOW_MATCH_in_match_predicate6273); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            MATCH115_tree = 
            (Object)adaptor.create(MATCH115)
            ;
            adaptor.addChild(root_0, MATCH115_tree);
            }

            LPAR116=(Token)match(input,LPAR,FOLLOW_LPAR_in_match_predicate6275); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LPAR116_tree = 
            (Object)adaptor.create(LPAR116)
            ;
            adaptor.addChild(root_0, LPAR116_tree);
            }

            pushFollow(FOLLOW_column_name_list_in_match_predicate6277);
            column_name_list117=column_name_list();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, column_name_list117.getTree());

            RPAR118=(Token)match(input,RPAR,FOLLOW_RPAR_in_match_predicate6279); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            RPAR118_tree = 
            (Object)adaptor.create(RPAR118)
            ;
            adaptor.addChild(root_0, RPAR118_tree);
            }

            AGAINST119=(Token)match(input,AGAINST,FOLLOW_AGAINST_in_match_predicate6281); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            AGAINST119_tree = 
            (Object)adaptor.create(AGAINST119)
            ;
            adaptor.addChild(root_0, AGAINST119_tree);
            }

            LPAR120=(Token)match(input,LPAR,FOLLOW_LPAR_in_match_predicate6283); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LPAR120_tree = 
            (Object)adaptor.create(LPAR120)
            ;
            adaptor.addChild(root_0, LPAR120_tree);
            }

            STRING_LITERAL121=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_match_predicate6285); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            STRING_LITERAL121_tree = 
            (Object)adaptor.create(STRING_LITERAL121)
            ;
            adaptor.addChild(root_0, STRING_LITERAL121_tree);
            }

            RPAR122=(Token)match(input,RPAR,FOLLOW_RPAR_in_match_predicate6287); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            RPAR122_tree = 
            (Object)adaptor.create(RPAR122)
            ;
            adaptor.addChild(root_0, RPAR122_tree);
            }

            if ( state.backtracking==0 ) {
                        try {
                            JSONArray cols = (column_name_list117!=null?column_name_list117.json:null);
                            for (int i = 0; i < cols.length(); ++i) {
                                String col = cols.getString(i);
                                String[] facetInfo = _facetInfoMap.get(col);
                                if (facetInfo != null && !facetInfo[1].equals("string")) {
                                    throw new FailedPredicateException(input, 
                                                                       "match_predicate", 
                                                                       "Non-string type column \"" + col + "\" cannot be used in MATCH AGAINST predicates.");
                                }
                            }

                            String orig = (STRING_LITERAL121!=null?STRING_LITERAL121.getText():null);
                            orig = orig.substring(1, orig.length() - 1);
                            retval.json = new JSONObject().put("query",
                                                         new JSONObject().put("query_string",
                                                                              new JSONObject().put("fields", cols)
                                                                                              .put("query", orig)));
                            if (NOT114 != null) {
                                retval.json = new JSONObject().put("bool",
                                                             new JSONObject().put("must_not", retval.json));
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "match_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 38, match_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "match_predicate"


    public static class like_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "like_predicate"
    // com/senseidb/bql/parsers/BQL.g:2217:1: like_predicate returns [JSONObject json] : column_name ( NOT )? LIKE STRING_LITERAL ;
    public final BQLParser.like_predicate_return like_predicate() throws RecognitionException {
        BQLParser.like_predicate_return retval = new BQLParser.like_predicate_return();
        retval.start = input.LT(1);

        int like_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token NOT124=null;
        Token LIKE125=null;
        Token STRING_LITERAL126=null;
        BQLParser.column_name_return column_name123 =null;


        Object NOT124_tree=null;
        Object LIKE125_tree=null;
        Object STRING_LITERAL126_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 39) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2218:5: ( column_name ( NOT )? LIKE STRING_LITERAL )
            // com/senseidb/bql/parsers/BQL.g:2218:9: column_name ( NOT )? LIKE STRING_LITERAL
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_column_name_in_like_predicate6320);
            column_name123=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, column_name123.getTree());

            // com/senseidb/bql/parsers/BQL.g:2218:21: ( NOT )?
            int alt50=2;
            switch ( input.LA(1) ) {
                case NOT:
                    {
                    alt50=1;
                    }
                    break;
            }

            switch (alt50) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2218:22: NOT
                    {
                    NOT124=(Token)match(input,NOT,FOLLOW_NOT_in_like_predicate6323); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    NOT124_tree = 
                    (Object)adaptor.create(NOT124)
                    ;
                    adaptor.addChild(root_0, NOT124_tree);
                    }

                    }
                    break;

            }


            LIKE125=(Token)match(input,LIKE,FOLLOW_LIKE_in_like_predicate6327); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LIKE125_tree = 
            (Object)adaptor.create(LIKE125)
            ;
            adaptor.addChild(root_0, LIKE125_tree);
            }

            STRING_LITERAL126=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_like_predicate6329); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            STRING_LITERAL126_tree = 
            (Object)adaptor.create(STRING_LITERAL126)
            ;
            adaptor.addChild(root_0, STRING_LITERAL126_tree);
            }

            if ( state.backtracking==0 ) {
                        String col = (column_name123!=null?column_name123.text:null);
                        String[] facetInfo = _facetInfoMap.get(col);
                        if (facetInfo != null && !facetInfo[1].equals("string")) {
                            throw new FailedPredicateException(input, 
                                                               "match_predicate", 
                                                               "Non-string type column \"" + col + "\" cannot be used in LIKE predicates.");
                        }
                        String orig = (STRING_LITERAL126!=null?STRING_LITERAL126.getText():null);
                        orig = orig.substring(1, orig.length() - 1);
                        String likeString = orig.replace('%', '*').replace('_', '?');
                        try {
                            retval.json = new JSONObject().put("query",
                                                         new JSONObject().put("wildcard",
                                                                              new JSONObject().put(col, likeString)));
                            if (NOT124 != null) {
                                retval.json = new JSONObject().put("bool",
                                                             new JSONObject().put("must_not", retval.json));
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "like_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 39, like_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "like_predicate"


    public static class null_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "null_predicate"
    // com/senseidb/bql/parsers/BQL.g:2245:1: null_predicate returns [JSONObject json] : column_name IS ( NOT )? NULL ;
    public final BQLParser.null_predicate_return null_predicate() throws RecognitionException {
        BQLParser.null_predicate_return retval = new BQLParser.null_predicate_return();
        retval.start = input.LT(1);

        int null_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token IS128=null;
        Token NOT129=null;
        Token NULL130=null;
        BQLParser.column_name_return column_name127 =null;


        Object IS128_tree=null;
        Object NOT129_tree=null;
        Object NULL130_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 40) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2246:5: ( column_name IS ( NOT )? NULL )
            // com/senseidb/bql/parsers/BQL.g:2246:9: column_name IS ( NOT )? NULL
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_column_name_in_null_predicate6362);
            column_name127=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, column_name127.getTree());

            IS128=(Token)match(input,IS,FOLLOW_IS_in_null_predicate6364); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            IS128_tree = 
            (Object)adaptor.create(IS128)
            ;
            adaptor.addChild(root_0, IS128_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:2246:24: ( NOT )?
            int alt51=2;
            switch ( input.LA(1) ) {
                case NOT:
                    {
                    alt51=1;
                    }
                    break;
            }

            switch (alt51) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2246:25: NOT
                    {
                    NOT129=(Token)match(input,NOT,FOLLOW_NOT_in_null_predicate6367); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    NOT129_tree = 
                    (Object)adaptor.create(NOT129)
                    ;
                    adaptor.addChild(root_0, NOT129_tree);
                    }

                    }
                    break;

            }


            NULL130=(Token)match(input,NULL,FOLLOW_NULL_in_null_predicate6371); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            NULL130_tree = 
            (Object)adaptor.create(NULL130)
            ;
            adaptor.addChild(root_0, NULL130_tree);
            }

            if ( state.backtracking==0 ) {
                        String col = (column_name127!=null?column_name127.text:null);
                        try {
                            retval.json = new JSONObject().put("isNull", col);
                            if (NOT129 != null) {
                                retval.json = new JSONObject().put("bool",
                                                             new JSONObject().put("must_not", retval.json));
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "null_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 40, null_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "null_predicate"


    public static class non_variable_value_list_return extends ParserRuleReturnScope {
        public JSONArray json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "non_variable_value_list"
    // com/senseidb/bql/parsers/BQL.g:2262:1: non_variable_value_list returns [JSONArray json] : LPAR v= value ( COMMA v= value )* RPAR ;
    public final BQLParser.non_variable_value_list_return non_variable_value_list() throws RecognitionException {
        BQLParser.non_variable_value_list_return retval = new BQLParser.non_variable_value_list_return();
        retval.start = input.LT(1);

        int non_variable_value_list_StartIndex = input.index();

        Object root_0 = null;

        Token LPAR131=null;
        Token COMMA132=null;
        Token RPAR133=null;
        BQLParser.value_return v =null;


        Object LPAR131_tree=null;
        Object COMMA132_tree=null;
        Object RPAR133_tree=null;


            retval.json = new JSONArray();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 41) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2266:5: ( LPAR v= value ( COMMA v= value )* RPAR )
            // com/senseidb/bql/parsers/BQL.g:2266:9: LPAR v= value ( COMMA v= value )* RPAR
            {
            root_0 = (Object)adaptor.nil();


            LPAR131=(Token)match(input,LPAR,FOLLOW_LPAR_in_non_variable_value_list6409); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LPAR131_tree = 
            (Object)adaptor.create(LPAR131)
            ;
            adaptor.addChild(root_0, LPAR131_tree);
            }

            pushFollow(FOLLOW_value_in_non_variable_value_list6413);
            v=value();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, v.getTree());

            if ( state.backtracking==0 ) {
                        retval.json.put((v!=null?v.val:null));
                    }

            // com/senseidb/bql/parsers/BQL.g:2270:9: ( COMMA v= value )*
            loop52:
            do {
                int alt52=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt52=1;
                    }
                    break;

                }

                switch (alt52) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2270:10: COMMA v= value
            	    {
            	    COMMA132=(Token)match(input,COMMA,FOLLOW_COMMA_in_non_variable_value_list6434); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA132_tree = 
            	    (Object)adaptor.create(COMMA132)
            	    ;
            	    adaptor.addChild(root_0, COMMA132_tree);
            	    }

            	    pushFollow(FOLLOW_value_in_non_variable_value_list6438);
            	    v=value();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, v.getTree());

            	    if ( state.backtracking==0 ) {
            	                    retval.json.put((v!=null?v.val:null));
            	                }

            	    }
            	    break;

            	default :
            	    break loop52;
                }
            } while (true);


            RPAR133=(Token)match(input,RPAR,FOLLOW_RPAR_in_non_variable_value_list6465); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            RPAR133_tree = 
            (Object)adaptor.create(RPAR133)
            ;
            adaptor.addChild(root_0, RPAR133_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 41, non_variable_value_list_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "non_variable_value_list"


    public static class python_style_list_return extends ParserRuleReturnScope {
        public JSONArray json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "python_style_list"
    // com/senseidb/bql/parsers/BQL.g:2277:1: python_style_list returns [JSONArray json] : '[' v= python_style_value ( COMMA v= python_style_value )* ']' ;
    public final BQLParser.python_style_list_return python_style_list() throws RecognitionException {
        BQLParser.python_style_list_return retval = new BQLParser.python_style_list_return();
        retval.start = input.LT(1);

        int python_style_list_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal134=null;
        Token COMMA135=null;
        Token char_literal136=null;
        BQLParser.python_style_value_return v =null;


        Object char_literal134_tree=null;
        Object COMMA135_tree=null;
        Object char_literal136_tree=null;


            retval.json = new JSONArray();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 42) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2281:5: ( '[' v= python_style_value ( COMMA v= python_style_value )* ']' )
            // com/senseidb/bql/parsers/BQL.g:2281:9: '[' v= python_style_value ( COMMA v= python_style_value )* ']'
            {
            root_0 = (Object)adaptor.nil();


            char_literal134=(Token)match(input,138,FOLLOW_138_in_python_style_list6493); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal134_tree = 
            (Object)adaptor.create(char_literal134)
            ;
            adaptor.addChild(root_0, char_literal134_tree);
            }

            pushFollow(FOLLOW_python_style_value_in_python_style_list6497);
            v=python_style_value();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, v.getTree());

            if ( state.backtracking==0 ) {
                        retval.json.put((v!=null?v.val:null));
                    }

            // com/senseidb/bql/parsers/BQL.g:2285:9: ( COMMA v= python_style_value )*
            loop53:
            do {
                int alt53=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt53=1;
                    }
                    break;

                }

                switch (alt53) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2285:10: COMMA v= python_style_value
            	    {
            	    COMMA135=(Token)match(input,COMMA,FOLLOW_COMMA_in_python_style_list6518); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA135_tree = 
            	    (Object)adaptor.create(COMMA135)
            	    ;
            	    adaptor.addChild(root_0, COMMA135_tree);
            	    }

            	    pushFollow(FOLLOW_python_style_value_in_python_style_list6522);
            	    v=python_style_value();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, v.getTree());

            	    if ( state.backtracking==0 ) {
            	                    retval.json.put((v!=null?v.val:null));
            	                }

            	    }
            	    break;

            	default :
            	    break loop53;
                }
            } while (true);


            char_literal136=(Token)match(input,139,FOLLOW_139_in_python_style_list6549); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal136_tree = 
            (Object)adaptor.create(char_literal136)
            ;
            adaptor.addChild(root_0, char_literal136_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 42, python_style_list_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "python_style_list"


    public static class python_style_dict_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "python_style_dict"
    // com/senseidb/bql/parsers/BQL.g:2292:1: python_style_dict returns [JSONObject json] : '{' p= key_value_pair[true] ( COMMA p= key_value_pair[true] )* '}' ;
    public final BQLParser.python_style_dict_return python_style_dict() throws RecognitionException {
        BQLParser.python_style_dict_return retval = new BQLParser.python_style_dict_return();
        retval.start = input.LT(1);

        int python_style_dict_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal137=null;
        Token COMMA138=null;
        Token char_literal139=null;
        BQLParser.key_value_pair_return p =null;


        Object char_literal137_tree=null;
        Object COMMA138_tree=null;
        Object char_literal139_tree=null;


            retval.json = new JSONObject();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 43) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2296:5: ( '{' p= key_value_pair[true] ( COMMA p= key_value_pair[true] )* '}' )
            // com/senseidb/bql/parsers/BQL.g:2296:9: '{' p= key_value_pair[true] ( COMMA p= key_value_pair[true] )* '}'
            {
            root_0 = (Object)adaptor.nil();


            char_literal137=(Token)match(input,161,FOLLOW_161_in_python_style_dict6577); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal137_tree = 
            (Object)adaptor.create(char_literal137)
            ;
            adaptor.addChild(root_0, char_literal137_tree);
            }

            pushFollow(FOLLOW_key_value_pair_in_python_style_dict6581);
            p=key_value_pair(true);

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, p.getTree());

            if ( state.backtracking==0 ) {
                        try {
                            retval.json.put((p!=null?p.key:null), (p!=null?p.val:null));
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "python_style_dict", "JSONException: " + err.getMessage());
                        }
                    }

            // com/senseidb/bql/parsers/BQL.g:2305:9: ( COMMA p= key_value_pair[true] )*
            loop54:
            do {
                int alt54=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt54=1;
                    }
                    break;

                }

                switch (alt54) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2305:10: COMMA p= key_value_pair[true]
            	    {
            	    COMMA138=(Token)match(input,COMMA,FOLLOW_COMMA_in_python_style_dict6603); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA138_tree = 
            	    (Object)adaptor.create(COMMA138)
            	    ;
            	    adaptor.addChild(root_0, COMMA138_tree);
            	    }

            	    pushFollow(FOLLOW_key_value_pair_in_python_style_dict6607);
            	    p=key_value_pair(true);

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, p.getTree());

            	    if ( state.backtracking==0 ) {
            	                    try {
            	                        retval.json.put((p!=null?p.key:null), (p!=null?p.val:null));
            	                    }
            	                    catch (JSONException err) {
            	                        throw new FailedPredicateException(input, "python_style_dict", "JSONException: " + err.getMessage());
            	                    }
            	                }

            	    }
            	    break;

            	default :
            	    break loop54;
                }
            } while (true);


            char_literal139=(Token)match(input,165,FOLLOW_165_in_python_style_dict6635); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal139_tree = 
            (Object)adaptor.create(char_literal139)
            ;
            adaptor.addChild(root_0, char_literal139_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 43, python_style_dict_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "python_style_dict"


    public static class python_style_value_return extends ParserRuleReturnScope {
        public Object val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "python_style_value"
    // com/senseidb/bql/parsers/BQL.g:2317:1: python_style_value returns [Object val] : ( value | python_style_list | python_style_dict );
    public final BQLParser.python_style_value_return python_style_value() throws RecognitionException {
        BQLParser.python_style_value_return retval = new BQLParser.python_style_value_return();
        retval.start = input.LT(1);

        int python_style_value_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.value_return value140 =null;

        BQLParser.python_style_list_return python_style_list141 =null;

        BQLParser.python_style_dict_return python_style_dict142 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 44) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2318:5: ( value | python_style_list | python_style_dict )
            int alt55=3;
            switch ( input.LA(1) ) {
            case EOF:
            case AGO:
            case AND:
            case BROWSE:
            case DATE:
            case FALSE:
            case FETCHING:
            case GIVEN:
            case GROUP:
            case INTEGER:
            case LIMIT:
            case NOW:
            case OR:
            case ORDER:
            case REAL:
            case ROUTE:
            case RPAR:
            case SEMI:
            case STRING_LITERAL:
            case TRUE:
            case USING:
            case VARIABLE:
                {
                alt55=1;
                }
                break;
            case 138:
                {
                alt55=2;
                }
                break;
            case 161:
                {
                alt55=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 55, 0, input);

                throw nvae;

            }

            switch (alt55) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2318:9: value
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_value_in_python_style_value6658);
                    value140=value();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, value140.getTree());

                    if ( state.backtracking==0 ) { retval.val = (value140!=null?value140.val:null); }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2319:9: python_style_list
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_python_style_list_in_python_style_value6670);
                    python_style_list141=python_style_list();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, python_style_list141.getTree());

                    if ( state.backtracking==0 ) { retval.val = (python_style_list141!=null?python_style_list141.json:null); }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2320:9: python_style_dict
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_python_style_dict_in_python_style_value6682);
                    python_style_dict142=python_style_dict();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, python_style_dict142.getTree());

                    if ( state.backtracking==0 ) { retval.val = (python_style_dict142!=null?python_style_dict142.json:null); }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 44, python_style_value_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "python_style_value"


    public static class value_list_return extends ParserRuleReturnScope {
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "value_list"
    // com/senseidb/bql/parsers/BQL.g:2323:1: value_list returns [Object json] : ( non_variable_value_list | VARIABLE );
    public final BQLParser.value_list_return value_list() throws RecognitionException {
        BQLParser.value_list_return retval = new BQLParser.value_list_return();
        retval.start = input.LT(1);

        int value_list_StartIndex = input.index();

        Object root_0 = null;

        Token VARIABLE144=null;
        BQLParser.non_variable_value_list_return non_variable_value_list143 =null;


        Object VARIABLE144_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 45) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2324:5: ( non_variable_value_list | VARIABLE )
            int alt56=2;
            switch ( input.LA(1) ) {
            case LPAR:
                {
                alt56=1;
                }
                break;
            case VARIABLE:
                {
                alt56=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 56, 0, input);

                throw nvae;

            }

            switch (alt56) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2324:9: non_variable_value_list
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_non_variable_value_list_in_value_list6707);
                    non_variable_value_list143=non_variable_value_list();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, non_variable_value_list143.getTree());

                    if ( state.backtracking==0 ) {
                                retval.json = (non_variable_value_list143!=null?non_variable_value_list143.json:null);
                            }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2328:9: VARIABLE
                    {
                    root_0 = (Object)adaptor.nil();


                    VARIABLE144=(Token)match(input,VARIABLE,FOLLOW_VARIABLE_in_value_list6727); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    VARIABLE144_tree = 
                    (Object)adaptor.create(VARIABLE144)
                    ;
                    adaptor.addChild(root_0, VARIABLE144_tree);
                    }

                    if ( state.backtracking==0 ) {
                                retval.json = (VARIABLE144!=null?VARIABLE144.getText():null);
                                _variables.add(((VARIABLE144!=null?VARIABLE144.getText():null)).substring(1));
                            }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 45, value_list_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "value_list"


    public static class value_return extends ParserRuleReturnScope {
        public Object val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "value"
    // com/senseidb/bql/parsers/BQL.g:2335:1: value returns [Object val] : ( numeric | STRING_LITERAL | TRUE | FALSE | VARIABLE );
    public final BQLParser.value_return value() throws RecognitionException {
        BQLParser.value_return retval = new BQLParser.value_return();
        retval.start = input.LT(1);

        int value_StartIndex = input.index();

        Object root_0 = null;

        Token STRING_LITERAL146=null;
        Token TRUE147=null;
        Token FALSE148=null;
        Token VARIABLE149=null;
        BQLParser.numeric_return numeric145 =null;


        Object STRING_LITERAL146_tree=null;
        Object TRUE147_tree=null;
        Object FALSE148_tree=null;
        Object VARIABLE149_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 46) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2336:5: ( numeric | STRING_LITERAL | TRUE | FALSE | VARIABLE )
            int alt57=5;
            switch ( input.LA(1) ) {
            case EOF:
            case AGO:
            case AND:
            case BROWSE:
            case DATE:
            case FETCHING:
            case GIVEN:
            case GROUP:
            case INTEGER:
            case LIMIT:
            case NOW:
            case OR:
            case ORDER:
            case REAL:
            case ROUTE:
            case RPAR:
            case SEMI:
            case USING:
                {
                alt57=1;
                }
                break;
            case STRING_LITERAL:
                {
                alt57=2;
                }
                break;
            case TRUE:
                {
                alt57=3;
                }
                break;
            case FALSE:
                {
                alt57=4;
                }
                break;
            case VARIABLE:
                {
                alt57=5;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 57, 0, input);

                throw nvae;

            }

            switch (alt57) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2336:9: numeric
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_numeric_in_value6760);
                    numeric145=numeric();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, numeric145.getTree());

                    if ( state.backtracking==0 ) { retval.val = (numeric145!=null?numeric145.val:null); }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2337:9: STRING_LITERAL
                    {
                    root_0 = (Object)adaptor.nil();


                    STRING_LITERAL146=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_value6772); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    STRING_LITERAL146_tree = 
                    (Object)adaptor.create(STRING_LITERAL146)
                    ;
                    adaptor.addChild(root_0, STRING_LITERAL146_tree);
                    }

                    if ( state.backtracking==0 ) {
                                String orig = (STRING_LITERAL146!=null?STRING_LITERAL146.getText():null);
                                orig = orig.substring(1, orig.length() - 1);
                                retval.val = orig;
                            }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2343:9: TRUE
                    {
                    root_0 = (Object)adaptor.nil();


                    TRUE147=(Token)match(input,TRUE,FOLLOW_TRUE_in_value6792); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    TRUE147_tree = 
                    (Object)adaptor.create(TRUE147)
                    ;
                    adaptor.addChild(root_0, TRUE147_tree);
                    }

                    if ( state.backtracking==0 ) { retval.val = true; }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:2344:9: FALSE
                    {
                    root_0 = (Object)adaptor.nil();


                    FALSE148=(Token)match(input,FALSE,FOLLOW_FALSE_in_value6804); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    FALSE148_tree = 
                    (Object)adaptor.create(FALSE148)
                    ;
                    adaptor.addChild(root_0, FALSE148_tree);
                    }

                    if ( state.backtracking==0 ) { retval.val = false; }

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:2345:9: VARIABLE
                    {
                    root_0 = (Object)adaptor.nil();


                    VARIABLE149=(Token)match(input,VARIABLE,FOLLOW_VARIABLE_in_value6816); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    VARIABLE149_tree = 
                    (Object)adaptor.create(VARIABLE149)
                    ;
                    adaptor.addChild(root_0, VARIABLE149_tree);
                    }

                    if ( state.backtracking==0 ) {
                                retval.val = (VARIABLE149!=null?VARIABLE149.getText():null);
                                _variables.add(((VARIABLE149!=null?VARIABLE149.getText():null)).substring(1));
                            }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 46, value_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "value"


    public static class numeric_return extends ParserRuleReturnScope {
        public Object val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "numeric"
    // com/senseidb/bql/parsers/BQL.g:2352:1: numeric returns [Object val] : ( time_expr | INTEGER | REAL );
    public final BQLParser.numeric_return numeric() throws RecognitionException {
        BQLParser.numeric_return retval = new BQLParser.numeric_return();
        retval.start = input.LT(1);

        int numeric_StartIndex = input.index();

        Object root_0 = null;

        Token INTEGER151=null;
        Token REAL152=null;
        BQLParser.time_expr_return time_expr150 =null;


        Object INTEGER151_tree=null;
        Object REAL152_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 47) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2353:5: ( time_expr | INTEGER | REAL )
            int alt58=3;
            switch ( input.LA(1) ) {
            case INTEGER:
                {
                switch ( input.LA(2) ) {
                case DAYS:
                case HOURS:
                case MILLISECONDS:
                case MINS:
                case MINUTES:
                case MSECS:
                case SECONDS:
                case SECS:
                case WEEKS:
                    {
                    alt58=1;
                    }
                    break;
                case EOF:
                case AND:
                case BROWSE:
                case COMMA:
                case FETCHING:
                case GIVEN:
                case GROUP:
                case LIMIT:
                case OR:
                case ORDER:
                case ROUTE:
                case RPAR:
                case SEMI:
                case USING:
                case WITH:
                case 139:
                case 165:
                    {
                    alt58=2;
                    }
                    break;
                default:
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 58, 1, input);

                    throw nvae;

                }

                }
                break;
            case AGO:
            case DATE:
            case NOW:
                {
                alt58=1;
                }
                break;
            case REAL:
                {
                alt58=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 58, 0, input);

                throw nvae;

            }

            switch (alt58) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2353:9: time_expr
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_time_expr_in_numeric6849);
                    time_expr150=time_expr();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, time_expr150.getTree());

                    if ( state.backtracking==0 ) { retval.val = (time_expr150!=null?time_expr150.val:0); }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2354:9: INTEGER
                    {
                    root_0 = (Object)adaptor.nil();


                    INTEGER151=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_numeric6861); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    INTEGER151_tree = 
                    (Object)adaptor.create(INTEGER151)
                    ;
                    adaptor.addChild(root_0, INTEGER151_tree);
                    }

                    if ( state.backtracking==0 ) {
                                try {
                                    retval.val = Long.parseLong((INTEGER151!=null?INTEGER151.getText():null));
                                }
                                catch (NumberFormatException err) {
                                    throw new FailedPredicateException(input, "numeric", "Hit NumberFormatException: " + err.getMessage());
                                }
                            }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2362:9: REAL
                    {
                    root_0 = (Object)adaptor.nil();


                    REAL152=(Token)match(input,REAL,FOLLOW_REAL_in_numeric6873); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    REAL152_tree = 
                    (Object)adaptor.create(REAL152)
                    ;
                    adaptor.addChild(root_0, REAL152_tree);
                    }

                    if ( state.backtracking==0 ) {
                                try {
                                    retval.val = Float.parseFloat((REAL152!=null?REAL152.getText():null));
                                }
                                catch (NumberFormatException err) {
                                    throw new FailedPredicateException(input, "numeric", "Hit NumberFormatException: " + err.getMessage());
                                }
                            }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 47, numeric_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "numeric"


    public static class except_clause_return extends ParserRuleReturnScope {
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "except_clause"
    // com/senseidb/bql/parsers/BQL.g:2372:1: except_clause returns [Object json] : EXCEPT ^ value_list ;
    public final BQLParser.except_clause_return except_clause() throws RecognitionException {
        BQLParser.except_clause_return retval = new BQLParser.except_clause_return();
        retval.start = input.LT(1);

        int except_clause_StartIndex = input.index();

        Object root_0 = null;

        Token EXCEPT153=null;
        BQLParser.value_list_return value_list154 =null;


        Object EXCEPT153_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 48) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2373:5: ( EXCEPT ^ value_list )
            // com/senseidb/bql/parsers/BQL.g:2373:9: EXCEPT ^ value_list
            {
            root_0 = (Object)adaptor.nil();


            EXCEPT153=(Token)match(input,EXCEPT,FOLLOW_EXCEPT_in_except_clause6898); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            EXCEPT153_tree = 
            (Object)adaptor.create(EXCEPT153)
            ;
            root_0 = (Object)adaptor.becomeRoot(EXCEPT153_tree, root_0);
            }

            pushFollow(FOLLOW_value_list_in_except_clause6901);
            value_list154=value_list();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, value_list154.getTree());

            if ( state.backtracking==0 ) {
                        retval.json = (value_list154!=null?value_list154.json:null);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 48, except_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "except_clause"


    public static class predicate_props_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "predicate_props"
    // com/senseidb/bql/parsers/BQL.g:2379:1: predicate_props returns [JSONObject json] : WITH ^ prop_list[true] ;
    public final BQLParser.predicate_props_return predicate_props() throws RecognitionException {
        BQLParser.predicate_props_return retval = new BQLParser.predicate_props_return();
        retval.start = input.LT(1);

        int predicate_props_StartIndex = input.index();

        Object root_0 = null;

        Token WITH155=null;
        BQLParser.prop_list_return prop_list156 =null;


        Object WITH155_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 49) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2380:5: ( WITH ^ prop_list[true] )
            // com/senseidb/bql/parsers/BQL.g:2380:9: WITH ^ prop_list[true]
            {
            root_0 = (Object)adaptor.nil();


            WITH155=(Token)match(input,WITH,FOLLOW_WITH_in_predicate_props6936); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            WITH155_tree = 
            (Object)adaptor.create(WITH155)
            ;
            root_0 = (Object)adaptor.becomeRoot(WITH155_tree, root_0);
            }

            pushFollow(FOLLOW_prop_list_in_predicate_props6939);
            prop_list156=prop_list(true);

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, prop_list156.getTree());

            if ( state.backtracking==0 ) {
                        retval.json = (prop_list156!=null?prop_list156.json:null);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 49, predicate_props_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "predicate_props"


    public static class prop_list_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "prop_list"
    // com/senseidb/bql/parsers/BQL.g:2386:1: prop_list[boolean needKeyInString] returns [JSONObject json] : LPAR p= key_value_pair[needKeyInString] ( COMMA p= key_value_pair[needKeyInString] )* RPAR ;
    public final BQLParser.prop_list_return prop_list(boolean needKeyInString) throws RecognitionException {
        BQLParser.prop_list_return retval = new BQLParser.prop_list_return();
        retval.start = input.LT(1);

        int prop_list_StartIndex = input.index();

        Object root_0 = null;

        Token LPAR157=null;
        Token COMMA158=null;
        Token RPAR159=null;
        BQLParser.key_value_pair_return p =null;


        Object LPAR157_tree=null;
        Object COMMA158_tree=null;
        Object RPAR159_tree=null;


            retval.json = new JSONObject();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 50) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2390:5: ( LPAR p= key_value_pair[needKeyInString] ( COMMA p= key_value_pair[needKeyInString] )* RPAR )
            // com/senseidb/bql/parsers/BQL.g:2390:9: LPAR p= key_value_pair[needKeyInString] ( COMMA p= key_value_pair[needKeyInString] )* RPAR
            {
            root_0 = (Object)adaptor.nil();


            LPAR157=(Token)match(input,LPAR,FOLLOW_LPAR_in_prop_list6979); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LPAR157_tree = 
            (Object)adaptor.create(LPAR157)
            ;
            adaptor.addChild(root_0, LPAR157_tree);
            }

            pushFollow(FOLLOW_key_value_pair_in_prop_list6983);
            p=key_value_pair(needKeyInString);

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, p.getTree());

            if ( state.backtracking==0 ) {
                        try {
                            retval.json.put((p!=null?p.key:null), (p!=null?p.val:null));
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "prop_list", "JSONException: " + err.getMessage());
                        }
                    }

            // com/senseidb/bql/parsers/BQL.g:2399:9: ( COMMA p= key_value_pair[needKeyInString] )*
            loop59:
            do {
                int alt59=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt59=1;
                    }
                    break;

                }

                switch (alt59) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2399:10: COMMA p= key_value_pair[needKeyInString]
            	    {
            	    COMMA158=(Token)match(input,COMMA,FOLLOW_COMMA_in_prop_list7005); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA158_tree = 
            	    (Object)adaptor.create(COMMA158)
            	    ;
            	    adaptor.addChild(root_0, COMMA158_tree);
            	    }

            	    pushFollow(FOLLOW_key_value_pair_in_prop_list7009);
            	    p=key_value_pair(needKeyInString);

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, p.getTree());

            	    if ( state.backtracking==0 ) {
            	                    try {
            	                        retval.json.put((p!=null?p.key:null), (p!=null?p.val:null));
            	                    }
            	                    catch (JSONException err) {
            	                        throw new FailedPredicateException(input, "prop_list", "JSONException: " + err.getMessage());
            	                    }
            	                }

            	    }
            	    break;

            	default :
            	    break loop59;
                }
            } while (true);


            RPAR159=(Token)match(input,RPAR,FOLLOW_RPAR_in_prop_list7037); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            RPAR159_tree = 
            (Object)adaptor.create(RPAR159)
            ;
            adaptor.addChild(root_0, RPAR159_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 50, prop_list_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "prop_list"


    protected static class key_value_pair_scope {
        boolean needString;
    }
    protected Stack key_value_pair_stack = new Stack();


    public static class key_value_pair_return extends ParserRuleReturnScope {
        public String key;
        public Object val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "key_value_pair"
    // com/senseidb/bql/parsers/BQL.g:2411:1: key_value_pair[boolean needKeyInString] returns [String key, Object val] : ({...}? => STRING_LITERAL |{...}? => IDENT ) COLON (v= value |vs= python_style_list |vd= python_style_dict ) ;
    public final BQLParser.key_value_pair_return key_value_pair(boolean needKeyInString) throws RecognitionException {
        key_value_pair_stack.push(new key_value_pair_scope());
        BQLParser.key_value_pair_return retval = new BQLParser.key_value_pair_return();
        retval.start = input.LT(1);

        int key_value_pair_StartIndex = input.index();

        Object root_0 = null;

        Token STRING_LITERAL160=null;
        Token IDENT161=null;
        Token COLON162=null;
        BQLParser.value_return v =null;

        BQLParser.python_style_list_return vs =null;

        BQLParser.python_style_dict_return vd =null;


        Object STRING_LITERAL160_tree=null;
        Object IDENT161_tree=null;
        Object COLON162_tree=null;


            ((key_value_pair_scope)key_value_pair_stack.peek()).needString = needKeyInString;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 51) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2418:5: ( ({...}? => STRING_LITERAL |{...}? => IDENT ) COLON (v= value |vs= python_style_list |vd= python_style_dict ) )
            // com/senseidb/bql/parsers/BQL.g:2418:9: ({...}? => STRING_LITERAL |{...}? => IDENT ) COLON (v= value |vs= python_style_list |vd= python_style_dict )
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:2418:9: ({...}? => STRING_LITERAL |{...}? => IDENT )
            int alt60=2;
            int LA60_0 = input.LA(1);

            if ( (LA60_0==STRING_LITERAL) && ((((key_value_pair_scope)key_value_pair_stack.peek()).needString))) {
                alt60=1;
            }
            else if ( (LA60_0==IDENT) && ((!((key_value_pair_scope)key_value_pair_stack.peek()).needString))) {
                alt60=2;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 60, 0, input);

                throw nvae;

            }
            switch (alt60) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2418:11: {...}? => STRING_LITERAL
                    {
                    if ( !((((key_value_pair_scope)key_value_pair_stack.peek()).needString)) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "key_value_pair", "$key_value_pair::needString");
                    }

                    STRING_LITERAL160=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_key_value_pair7075); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    STRING_LITERAL160_tree = 
                    (Object)adaptor.create(STRING_LITERAL160)
                    ;
                    adaptor.addChild(root_0, STRING_LITERAL160_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2419:11: {...}? => IDENT
                    {
                    if ( !((!((key_value_pair_scope)key_value_pair_stack.peek()).needString)) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "key_value_pair", "!$key_value_pair::needString");
                    }

                    IDENT161=(Token)match(input,IDENT,FOLLOW_IDENT_in_key_value_pair7090); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    IDENT161_tree = 
                    (Object)adaptor.create(IDENT161)
                    ;
                    adaptor.addChild(root_0, IDENT161_tree);
                    }

                    }
                    break;

            }


            COLON162=(Token)match(input,COLON,FOLLOW_COLON_in_key_value_pair7110); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            COLON162_tree = 
            (Object)adaptor.create(COLON162)
            ;
            adaptor.addChild(root_0, COLON162_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:2421:15: (v= value |vs= python_style_list |vd= python_style_dict )
            int alt61=3;
            switch ( input.LA(1) ) {
            case EOF:
            case AGO:
            case AND:
            case BROWSE:
            case DATE:
            case FALSE:
            case FETCHING:
            case GIVEN:
            case GROUP:
            case INTEGER:
            case LIMIT:
            case NOW:
            case OR:
            case ORDER:
            case REAL:
            case ROUTE:
            case RPAR:
            case SEMI:
            case STRING_LITERAL:
            case TRUE:
            case USING:
            case VARIABLE:
                {
                alt61=1;
                }
                break;
            case 138:
                {
                alt61=2;
                }
                break;
            case 161:
                {
                alt61=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 61, 0, input);

                throw nvae;

            }

            switch (alt61) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2421:16: v= value
                    {
                    pushFollow(FOLLOW_value_in_key_value_pair7115);
                    v=value();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, v.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2421:26: vs= python_style_list
                    {
                    pushFollow(FOLLOW_python_style_list_in_key_value_pair7121);
                    vs=python_style_list();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, vs.getTree());

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2421:49: vd= python_style_dict
                    {
                    pushFollow(FOLLOW_python_style_dict_in_key_value_pair7127);
                    vd=python_style_dict();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, vd.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        if (STRING_LITERAL160 != null) {
                            String orig = (STRING_LITERAL160!=null?STRING_LITERAL160.getText():null);
                            retval.key = orig.substring(1, orig.length() - 1);
                        }
                        else {
                            retval.key = (IDENT161!=null?IDENT161.getText():null);
                        }
                        if (v != null) {
                            retval.val = (v!=null?v.val:null);
                        }
                        else if (vs != null) {
                            retval.val = (vs!=null?vs.json:null);
                        }
                        else {
                            retval.val = (vd!=null?vd.json:null);
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 51, key_value_pair_StartIndex); }

            key_value_pair_stack.pop();
        }
        return retval;
    }
    // $ANTLR end "key_value_pair"


    public static class given_clause_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "given_clause"
    // com/senseidb/bql/parsers/BQL.g:2442:1: given_clause returns [JSONObject json] : GIVEN FACET PARAM facet_param_list ;
    public final BQLParser.given_clause_return given_clause() throws RecognitionException {
        BQLParser.given_clause_return retval = new BQLParser.given_clause_return();
        retval.start = input.LT(1);

        int given_clause_StartIndex = input.index();

        Object root_0 = null;

        Token GIVEN163=null;
        Token FACET164=null;
        Token PARAM165=null;
        BQLParser.facet_param_list_return facet_param_list166 =null;


        Object GIVEN163_tree=null;
        Object FACET164_tree=null;
        Object PARAM165_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 52) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2443:5: ( GIVEN FACET PARAM facet_param_list )
            // com/senseidb/bql/parsers/BQL.g:2443:9: GIVEN FACET PARAM facet_param_list
            {
            root_0 = (Object)adaptor.nil();


            GIVEN163=(Token)match(input,GIVEN,FOLLOW_GIVEN_in_given_clause7161); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            GIVEN163_tree = 
            (Object)adaptor.create(GIVEN163)
            ;
            adaptor.addChild(root_0, GIVEN163_tree);
            }

            FACET164=(Token)match(input,FACET,FOLLOW_FACET_in_given_clause7163); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            FACET164_tree = 
            (Object)adaptor.create(FACET164)
            ;
            adaptor.addChild(root_0, FACET164_tree);
            }

            PARAM165=(Token)match(input,PARAM,FOLLOW_PARAM_in_given_clause7165); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            PARAM165_tree = 
            (Object)adaptor.create(PARAM165)
            ;
            adaptor.addChild(root_0, PARAM165_tree);
            }

            pushFollow(FOLLOW_facet_param_list_in_given_clause7167);
            facet_param_list166=facet_param_list();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, facet_param_list166.getTree());

            if ( state.backtracking==0 ) {
                        retval.json = (facet_param_list166!=null?facet_param_list166.json:null);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 52, given_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "given_clause"


    public static class variable_declarators_return extends ParserRuleReturnScope {
        public JSONArray json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "variable_declarators"
    // com/senseidb/bql/parsers/BQL.g:2454:1: variable_declarators returns [JSONArray json] : var1= variable_declarator ( COMMA var2= variable_declarator )* ;
    public final BQLParser.variable_declarators_return variable_declarators() throws RecognitionException {
        BQLParser.variable_declarators_return retval = new BQLParser.variable_declarators_return();
        retval.start = input.LT(1);

        int variable_declarators_StartIndex = input.index();

        Object root_0 = null;

        Token COMMA167=null;
        BQLParser.variable_declarator_return var1 =null;

        BQLParser.variable_declarator_return var2 =null;


        Object COMMA167_tree=null;


            retval.json = new JSONArray();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 53) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2458:5: (var1= variable_declarator ( COMMA var2= variable_declarator )* )
            // com/senseidb/bql/parsers/BQL.g:2458:9: var1= variable_declarator ( COMMA var2= variable_declarator )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_variable_declarator_in_variable_declarators7212);
            var1=variable_declarator();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, var1.getTree());

            if ( state.backtracking==0 ) {
                        retval.json.put((var1!=null?var1.varName:null));
                    }

            // com/senseidb/bql/parsers/BQL.g:2462:9: ( COMMA var2= variable_declarator )*
            loop62:
            do {
                int alt62=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt62=1;
                    }
                    break;

                }

                switch (alt62) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2462:10: COMMA var2= variable_declarator
            	    {
            	    COMMA167=(Token)match(input,COMMA,FOLLOW_COMMA_in_variable_declarators7233); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA167_tree = 
            	    (Object)adaptor.create(COMMA167)
            	    ;
            	    adaptor.addChild(root_0, COMMA167_tree);
            	    }

            	    pushFollow(FOLLOW_variable_declarator_in_variable_declarators7237);
            	    var2=variable_declarator();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, var2.getTree());

            	    if ( state.backtracking==0 ) {
            	                    retval.json.put((var2!=null?var2.varName:null));
            	                }

            	    }
            	    break;

            	default :
            	    break loop62;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 53, variable_declarators_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "variable_declarators"


    public static class variable_declarator_return extends ParserRuleReturnScope {
        public String varName;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "variable_declarator"
    // com/senseidb/bql/parsers/BQL.g:2469:1: variable_declarator returns [String varName] : variable_declarator_id ( '=' variable_initializer )? ;
    public final BQLParser.variable_declarator_return variable_declarator() throws RecognitionException {
        BQLParser.variable_declarator_return retval = new BQLParser.variable_declarator_return();
        retval.start = input.LT(1);

        int variable_declarator_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal169=null;
        BQLParser.variable_declarator_id_return variable_declarator_id168 =null;

        BQLParser.variable_initializer_return variable_initializer170 =null;


        Object char_literal169_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 54) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2470:5: ( variable_declarator_id ( '=' variable_initializer )? )
            // com/senseidb/bql/parsers/BQL.g:2470:9: variable_declarator_id ( '=' variable_initializer )?
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_variable_declarator_id_in_variable_declarator7285);
            variable_declarator_id168=variable_declarator_id();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_declarator_id168.getTree());

            // com/senseidb/bql/parsers/BQL.g:2470:32: ( '=' variable_initializer )?
            int alt63=2;
            switch ( input.LA(1) ) {
                case EQUAL:
                    {
                    alt63=1;
                    }
                    break;
            }

            switch (alt63) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2470:33: '=' variable_initializer
                    {
                    char_literal169=(Token)match(input,EQUAL,FOLLOW_EQUAL_in_variable_declarator7288); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal169_tree = 
                    (Object)adaptor.create(char_literal169)
                    ;
                    adaptor.addChild(root_0, char_literal169_tree);
                    }

                    pushFollow(FOLLOW_variable_initializer_in_variable_declarator7290);
                    variable_initializer170=variable_initializer();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_initializer170.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        retval.varName = (variable_declarator_id168!=null?variable_declarator_id168.varName:null);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 54, variable_declarator_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "variable_declarator"


    public static class variable_declarator_id_return extends ParserRuleReturnScope {
        public String varName;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "variable_declarator_id"
    // com/senseidb/bql/parsers/BQL.g:2476:1: variable_declarator_id returns [String varName] : IDENT ( '[' ']' )* ;
    public final BQLParser.variable_declarator_id_return variable_declarator_id() throws RecognitionException {
        BQLParser.variable_declarator_id_return retval = new BQLParser.variable_declarator_id_return();
        retval.start = input.LT(1);

        int variable_declarator_id_StartIndex = input.index();

        Object root_0 = null;

        Token IDENT171=null;
        Token char_literal172=null;
        Token char_literal173=null;

        Object IDENT171_tree=null;
        Object char_literal172_tree=null;
        Object char_literal173_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 55) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2477:5: ( IDENT ( '[' ']' )* )
            // com/senseidb/bql/parsers/BQL.g:2477:9: IDENT ( '[' ']' )*
            {
            root_0 = (Object)adaptor.nil();


            IDENT171=(Token)match(input,IDENT,FOLLOW_IDENT_in_variable_declarator_id7325); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            IDENT171_tree = 
            (Object)adaptor.create(IDENT171)
            ;
            adaptor.addChild(root_0, IDENT171_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:2477:15: ( '[' ']' )*
            loop64:
            do {
                int alt64=2;
                switch ( input.LA(1) ) {
                case 138:
                    {
                    alt64=1;
                    }
                    break;

                }

                switch (alt64) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2477:16: '[' ']'
            	    {
            	    char_literal172=(Token)match(input,138,FOLLOW_138_in_variable_declarator_id7328); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    char_literal172_tree = 
            	    (Object)adaptor.create(char_literal172)
            	    ;
            	    adaptor.addChild(root_0, char_literal172_tree);
            	    }

            	    char_literal173=(Token)match(input,139,FOLLOW_139_in_variable_declarator_id7330); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    char_literal173_tree = 
            	    (Object)adaptor.create(char_literal173)
            	    ;
            	    adaptor.addChild(root_0, char_literal173_tree);
            	    }

            	    }
            	    break;

            	default :
            	    break loop64;
                }
            } while (true);


            if ( state.backtracking==0 ) {
                        retval.varName = (IDENT171!=null?IDENT171.getText():null);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 55, variable_declarator_id_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "variable_declarator_id"


    public static class variable_initializer_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "variable_initializer"
    // com/senseidb/bql/parsers/BQL.g:2483:1: variable_initializer : ( array_initializer | expression );
    public final BQLParser.variable_initializer_return variable_initializer() throws RecognitionException {
        BQLParser.variable_initializer_return retval = new BQLParser.variable_initializer_return();
        retval.start = input.LT(1);

        int variable_initializer_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.array_initializer_return array_initializer174 =null;

        BQLParser.expression_return expression175 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 56) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2484:5: ( array_initializer | expression )
            int alt65=2;
            switch ( input.LA(1) ) {
            case 161:
                {
                alt65=1;
                }
                break;
            case BOOLEAN:
            case BYTE:
            case CHARACTER_LITERAL:
            case DOUBLE:
            case FALSE:
            case FLOATING_POINT_LITERAL:
            case HEX_LITERAL:
            case IDENT:
            case INTEGER:
            case LONG:
            case LPAR:
            case NULL:
            case OCTAL_LITERAL:
            case REAL:
            case STRING_LITERAL:
            case TRUE:
            case 111:
            case 120:
            case 121:
            case 123:
            case 124:
            case 131:
            case 132:
            case 133:
            case 134:
            case 135:
            case 136:
            case 137:
            case 166:
                {
                alt65=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 65, 0, input);

                throw nvae;

            }

            switch (alt65) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2484:9: array_initializer
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_array_initializer_in_variable_initializer7361);
                    array_initializer174=array_initializer();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, array_initializer174.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2485:9: expression
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_expression_in_variable_initializer7371);
                    expression175=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression175.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 56, variable_initializer_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "variable_initializer"


    public static class array_initializer_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "array_initializer"
    // com/senseidb/bql/parsers/BQL.g:2488:1: array_initializer : '{' ( variable_initializer ( ',' variable_initializer )* ( ',' )? )? '}' ;
    public final BQLParser.array_initializer_return array_initializer() throws RecognitionException {
        BQLParser.array_initializer_return retval = new BQLParser.array_initializer_return();
        retval.start = input.LT(1);

        int array_initializer_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal176=null;
        Token char_literal178=null;
        Token char_literal180=null;
        Token char_literal181=null;
        BQLParser.variable_initializer_return variable_initializer177 =null;

        BQLParser.variable_initializer_return variable_initializer179 =null;


        Object char_literal176_tree=null;
        Object char_literal178_tree=null;
        Object char_literal180_tree=null;
        Object char_literal181_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 57) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2489:5: ( '{' ( variable_initializer ( ',' variable_initializer )* ( ',' )? )? '}' )
            // com/senseidb/bql/parsers/BQL.g:2489:9: '{' ( variable_initializer ( ',' variable_initializer )* ( ',' )? )? '}'
            {
            root_0 = (Object)adaptor.nil();


            char_literal176=(Token)match(input,161,FOLLOW_161_in_array_initializer7390); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal176_tree = 
            (Object)adaptor.create(char_literal176)
            ;
            adaptor.addChild(root_0, char_literal176_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:2489:13: ( variable_initializer ( ',' variable_initializer )* ( ',' )? )?
            int alt68=2;
            switch ( input.LA(1) ) {
                case BOOLEAN:
                case BYTE:
                case CHARACTER_LITERAL:
                case DOUBLE:
                case FALSE:
                case FLOATING_POINT_LITERAL:
                case HEX_LITERAL:
                case IDENT:
                case INTEGER:
                case LONG:
                case LPAR:
                case NULL:
                case OCTAL_LITERAL:
                case REAL:
                case STRING_LITERAL:
                case TRUE:
                case 111:
                case 120:
                case 121:
                case 123:
                case 124:
                case 131:
                case 132:
                case 133:
                case 134:
                case 135:
                case 136:
                case 137:
                case 161:
                case 166:
                    {
                    alt68=1;
                    }
                    break;
            }

            switch (alt68) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2489:14: variable_initializer ( ',' variable_initializer )* ( ',' )?
                    {
                    pushFollow(FOLLOW_variable_initializer_in_array_initializer7393);
                    variable_initializer177=variable_initializer();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_initializer177.getTree());

                    // com/senseidb/bql/parsers/BQL.g:2489:35: ( ',' variable_initializer )*
                    loop66:
                    do {
                        int alt66=2;
                        switch ( input.LA(1) ) {
                        case COMMA:
                            {
                            switch ( input.LA(2) ) {
                            case BOOLEAN:
                            case BYTE:
                            case CHARACTER_LITERAL:
                            case DOUBLE:
                            case FALSE:
                            case FLOATING_POINT_LITERAL:
                            case HEX_LITERAL:
                            case IDENT:
                            case INTEGER:
                            case LONG:
                            case LPAR:
                            case NULL:
                            case OCTAL_LITERAL:
                            case REAL:
                            case STRING_LITERAL:
                            case TRUE:
                            case 111:
                            case 120:
                            case 121:
                            case 123:
                            case 124:
                            case 131:
                            case 132:
                            case 133:
                            case 134:
                            case 135:
                            case 136:
                            case 137:
                            case 161:
                            case 166:
                                {
                                alt66=1;
                                }
                                break;

                            }

                            }
                            break;

                        }

                        switch (alt66) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:2489:36: ',' variable_initializer
                    	    {
                    	    char_literal178=(Token)match(input,COMMA,FOLLOW_COMMA_in_array_initializer7396); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal178_tree = 
                    	    (Object)adaptor.create(char_literal178)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal178_tree);
                    	    }

                    	    pushFollow(FOLLOW_variable_initializer_in_array_initializer7398);
                    	    variable_initializer179=variable_initializer();

                    	    state._fsp--;
                    	    if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_initializer179.getTree());

                    	    }
                    	    break;

                    	default :
                    	    break loop66;
                        }
                    } while (true);


                    // com/senseidb/bql/parsers/BQL.g:2489:63: ( ',' )?
                    int alt67=2;
                    switch ( input.LA(1) ) {
                        case COMMA:
                            {
                            alt67=1;
                            }
                            break;
                    }

                    switch (alt67) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2489:64: ','
                            {
                            char_literal180=(Token)match(input,COMMA,FOLLOW_COMMA_in_array_initializer7403); if (state.failed) return retval;
                            if ( state.backtracking==0 ) {
                            char_literal180_tree = 
                            (Object)adaptor.create(char_literal180)
                            ;
                            adaptor.addChild(root_0, char_literal180_tree);
                            }

                            }
                            break;

                    }


                    }
                    break;

            }


            char_literal181=(Token)match(input,165,FOLLOW_165_in_array_initializer7409); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal181_tree = 
            (Object)adaptor.create(char_literal181)
            ;
            adaptor.addChild(root_0, char_literal181_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 57, array_initializer_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "array_initializer"


    public static class type_return extends ParserRuleReturnScope {
        public String typeName;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "type"
    // com/senseidb/bql/parsers/BQL.g:2492:1: type returns [String typeName] : ( class_or_interface_type ( '[' ']' )* | primitive_type ( '[' ']' )* | boxed_type ( '[' ']' )* | limited_type ( '[' ']' )* );
    public final BQLParser.type_return type() throws RecognitionException {
        BQLParser.type_return retval = new BQLParser.type_return();
        retval.start = input.LT(1);

        int type_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal183=null;
        Token char_literal184=null;
        Token char_literal186=null;
        Token char_literal187=null;
        Token char_literal189=null;
        Token char_literal190=null;
        Token char_literal192=null;
        Token char_literal193=null;
        BQLParser.class_or_interface_type_return class_or_interface_type182 =null;

        BQLParser.primitive_type_return primitive_type185 =null;

        BQLParser.boxed_type_return boxed_type188 =null;

        BQLParser.limited_type_return limited_type191 =null;


        Object char_literal183_tree=null;
        Object char_literal184_tree=null;
        Object char_literal186_tree=null;
        Object char_literal187_tree=null;
        Object char_literal189_tree=null;
        Object char_literal190_tree=null;
        Object char_literal192_tree=null;
        Object char_literal193_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 58) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2493:5: ( class_or_interface_type ( '[' ']' )* | primitive_type ( '[' ']' )* | boxed_type ( '[' ']' )* | limited_type ( '[' ']' )* )
            int alt73=4;
            switch ( input.LA(1) ) {
            case FAST_UTIL_DATA_TYPE:
                {
                alt73=1;
                }
                break;
            case BOOLEAN:
                {
                int LA73_2 = input.LA(2);

                if ( (((( "boolean".equals(input.LT(1).getText()) )&&( "boolean".equals(input.LT(1).getText()) ))&&synpred103_BQL())) ) {
                    alt73=2;
                }
                else if ( (((synpred105_BQL()&&synpred105_BQL())&&( "Boolean".equals(input.LT(1).getText()) ))) ) {
                    alt73=3;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 73, 2, input);

                    throw nvae;

                }
                }
                break;
            case INT:
            case 144:
            case 151:
            case 156:
                {
                alt73=2;
                }
                break;
            case BYTE:
                {
                int LA73_4 = input.LA(2);

                if ( (((( "byte".equals(input.LT(1).getText()) )&&( "byte".equals(input.LT(1).getText()) ))&&synpred103_BQL())) ) {
                    alt73=2;
                }
                else if ( (((( "Byte".equals(input.LT(1).getText()) )&&( "Byte".equals(input.LT(1).getText()) ))&&synpred105_BQL())) ) {
                    alt73=3;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 73, 4, input);

                    throw nvae;

                }
                }
                break;
            case LONG:
                {
                int LA73_5 = input.LA(2);

                if ( (((( "long".equals(input.LT(1).getText()) )&&( "long".equals(input.LT(1).getText()) ))&&synpred103_BQL())) ) {
                    alt73=2;
                }
                else if ( (((synpred105_BQL()&&synpred105_BQL())&&( "Long".equals(input.LT(1).getText()) ))) ) {
                    alt73=3;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 73, 5, input);

                    throw nvae;

                }
                }
                break;
            case DOUBLE:
                {
                int LA73_6 = input.LA(2);

                if ( (((( "double".equals(input.LT(1).getText()) )&&( "double".equals(input.LT(1).getText()) ))&&synpred103_BQL())) ) {
                    alt73=2;
                }
                else if ( (((synpred105_BQL()&&synpred105_BQL())&&( "Double".equals(input.LT(1).getText()) ))) ) {
                    alt73=3;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 73, 6, input);

                    throw nvae;

                }
                }
                break;
            case 131:
            case 132:
            case 133:
            case 135:
                {
                alt73=3;
                }
                break;
            case 134:
            case 136:
            case 137:
                {
                alt73=4;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 73, 0, input);

                throw nvae;

            }

            switch (alt73) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2493:9: class_or_interface_type ( '[' ']' )*
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_class_or_interface_type_in_type7432);
                    class_or_interface_type182=class_or_interface_type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, class_or_interface_type182.getTree());

                    // com/senseidb/bql/parsers/BQL.g:2493:33: ( '[' ']' )*
                    loop69:
                    do {
                        int alt69=2;
                        switch ( input.LA(1) ) {
                        case 138:
                            {
                            alt69=1;
                            }
                            break;

                        }

                        switch (alt69) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:2493:34: '[' ']'
                    	    {
                    	    char_literal183=(Token)match(input,138,FOLLOW_138_in_type7435); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal183_tree = 
                    	    (Object)adaptor.create(char_literal183)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal183_tree);
                    	    }

                    	    char_literal184=(Token)match(input,139,FOLLOW_139_in_type7437); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal184_tree = 
                    	    (Object)adaptor.create(char_literal184)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal184_tree);
                    	    }

                    	    }
                    	    break;

                    	default :
                    	    break loop69;
                        }
                    } while (true);


                    if ( state.backtracking==0 ) { retval.typeName = (class_or_interface_type182!=null?class_or_interface_type182.typeName:null); }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2495:9: primitive_type ( '[' ']' )*
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_primitive_type_in_type7459);
                    primitive_type185=primitive_type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, primitive_type185.getTree());

                    // com/senseidb/bql/parsers/BQL.g:2495:24: ( '[' ']' )*
                    loop70:
                    do {
                        int alt70=2;
                        switch ( input.LA(1) ) {
                        case 138:
                            {
                            alt70=1;
                            }
                            break;

                        }

                        switch (alt70) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:2495:25: '[' ']'
                    	    {
                    	    char_literal186=(Token)match(input,138,FOLLOW_138_in_type7462); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal186_tree = 
                    	    (Object)adaptor.create(char_literal186)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal186_tree);
                    	    }

                    	    char_literal187=(Token)match(input,139,FOLLOW_139_in_type7464); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal187_tree = 
                    	    (Object)adaptor.create(char_literal187)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal187_tree);
                    	    }

                    	    }
                    	    break;

                    	default :
                    	    break loop70;
                        }
                    } while (true);


                    if ( state.backtracking==0 ) { retval.typeName = (primitive_type185!=null?input.toString(primitive_type185.start,primitive_type185.stop):null);}

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2497:9: boxed_type ( '[' ']' )*
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_boxed_type_in_type7486);
                    boxed_type188=boxed_type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, boxed_type188.getTree());

                    // com/senseidb/bql/parsers/BQL.g:2497:20: ( '[' ']' )*
                    loop71:
                    do {
                        int alt71=2;
                        switch ( input.LA(1) ) {
                        case 138:
                            {
                            alt71=1;
                            }
                            break;

                        }

                        switch (alt71) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:2497:21: '[' ']'
                    	    {
                    	    char_literal189=(Token)match(input,138,FOLLOW_138_in_type7489); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal189_tree = 
                    	    (Object)adaptor.create(char_literal189)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal189_tree);
                    	    }

                    	    char_literal190=(Token)match(input,139,FOLLOW_139_in_type7491); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal190_tree = 
                    	    (Object)adaptor.create(char_literal190)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal190_tree);
                    	    }

                    	    }
                    	    break;

                    	default :
                    	    break loop71;
                        }
                    } while (true);


                    if ( state.backtracking==0 ) { retval.typeName = (boxed_type188!=null?input.toString(boxed_type188.start,boxed_type188.stop):null);}

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:2499:9: limited_type ( '[' ']' )*
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_limited_type_in_type7513);
                    limited_type191=limited_type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, limited_type191.getTree());

                    // com/senseidb/bql/parsers/BQL.g:2499:22: ( '[' ']' )*
                    loop72:
                    do {
                        int alt72=2;
                        switch ( input.LA(1) ) {
                        case 138:
                            {
                            alt72=1;
                            }
                            break;

                        }

                        switch (alt72) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:2499:23: '[' ']'
                    	    {
                    	    char_literal192=(Token)match(input,138,FOLLOW_138_in_type7516); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal192_tree = 
                    	    (Object)adaptor.create(char_literal192)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal192_tree);
                    	    }

                    	    char_literal193=(Token)match(input,139,FOLLOW_139_in_type7518); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal193_tree = 
                    	    (Object)adaptor.create(char_literal193)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal193_tree);
                    	    }

                    	    }
                    	    break;

                    	default :
                    	    break loop72;
                        }
                    } while (true);


                    if ( state.backtracking==0 ) { retval.typeName = (limited_type191!=null?input.toString(limited_type191.start,limited_type191.stop):null);}

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 58, type_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "type"


    public static class class_or_interface_type_return extends ParserRuleReturnScope {
        public String typeName;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "class_or_interface_type"
    // com/senseidb/bql/parsers/BQL.g:2503:1: class_or_interface_type returns [String typeName] : FAST_UTIL_DATA_TYPE ;
    public final BQLParser.class_or_interface_type_return class_or_interface_type() throws RecognitionException {
        BQLParser.class_or_interface_type_return retval = new BQLParser.class_or_interface_type_return();
        retval.start = input.LT(1);

        int class_or_interface_type_StartIndex = input.index();

        Object root_0 = null;

        Token FAST_UTIL_DATA_TYPE194=null;

        Object FAST_UTIL_DATA_TYPE194_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 59) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2504:5: ( FAST_UTIL_DATA_TYPE )
            // com/senseidb/bql/parsers/BQL.g:2504:9: FAST_UTIL_DATA_TYPE
            {
            root_0 = (Object)adaptor.nil();


            FAST_UTIL_DATA_TYPE194=(Token)match(input,FAST_UTIL_DATA_TYPE,FOLLOW_FAST_UTIL_DATA_TYPE_in_class_or_interface_type7553); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            FAST_UTIL_DATA_TYPE194_tree = 
            (Object)adaptor.create(FAST_UTIL_DATA_TYPE194)
            ;
            adaptor.addChild(root_0, FAST_UTIL_DATA_TYPE194_tree);
            }

            if ( state.backtracking==0 ) { retval.typeName = _fastutilTypeMap.get((FAST_UTIL_DATA_TYPE194!=null?FAST_UTIL_DATA_TYPE194.getText():null)); }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 59, class_or_interface_type_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "class_or_interface_type"


    public static class type_arguments_return extends ParserRuleReturnScope {
        public String typeArgs;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "type_arguments"
    // com/senseidb/bql/parsers/BQL.g:2508:1: type_arguments returns [String typeArgs] : '<' ta1= type_argument ( COMMA ta2= type_argument )* '>' ;
    public final BQLParser.type_arguments_return type_arguments() throws RecognitionException {
        BQLParser.type_arguments_return retval = new BQLParser.type_arguments_return();
        retval.start = input.LT(1);

        int type_arguments_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal195=null;
        Token COMMA196=null;
        Token char_literal197=null;
        BQLParser.type_argument_return ta1 =null;

        BQLParser.type_argument_return ta2 =null;


        Object char_literal195_tree=null;
        Object COMMA196_tree=null;
        Object char_literal197_tree=null;


            StringBuilder builder = new StringBuilder();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 60) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2512:5: ( '<' ta1= type_argument ( COMMA ta2= type_argument )* '>' )
            // com/senseidb/bql/parsers/BQL.g:2512:9: '<' ta1= type_argument ( COMMA ta2= type_argument )* '>'
            {
            root_0 = (Object)adaptor.nil();


            char_literal195=(Token)match(input,LT,FOLLOW_LT_in_type_arguments7591); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal195_tree = 
            (Object)adaptor.create(char_literal195)
            ;
            adaptor.addChild(root_0, char_literal195_tree);
            }

            pushFollow(FOLLOW_type_argument_in_type_arguments7603);
            ta1=type_argument();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, ta1.getTree());

            if ( state.backtracking==0 ) { builder.append((ta1!=null?input.toString(ta1.start,ta1.stop):null)); }

            // com/senseidb/bql/parsers/BQL.g:2515:9: ( COMMA ta2= type_argument )*
            loop74:
            do {
                int alt74=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt74=1;
                    }
                    break;

                }

                switch (alt74) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2515:10: COMMA ta2= type_argument
            	    {
            	    COMMA196=(Token)match(input,COMMA,FOLLOW_COMMA_in_type_arguments7624); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA196_tree = 
            	    (Object)adaptor.create(COMMA196)
            	    ;
            	    adaptor.addChild(root_0, COMMA196_tree);
            	    }

            	    pushFollow(FOLLOW_type_argument_in_type_arguments7628);
            	    ta2=type_argument();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, ta2.getTree());

            	    if ( state.backtracking==0 ) { builder.append("_").append((ta2!=null?input.toString(ta2.start,ta2.stop):null)); }

            	    }
            	    break;

            	default :
            	    break loop74;
                }
            } while (true);


            char_literal197=(Token)match(input,GT,FOLLOW_GT_in_type_arguments7655); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal197_tree = 
            (Object)adaptor.create(char_literal197)
            ;
            adaptor.addChild(root_0, char_literal197_tree);
            }

            if ( state.backtracking==0 ) {
                        retval.typeArgs = builder.toString();
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 60, type_arguments_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "type_arguments"


    public static class type_argument_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "type_argument"
    // com/senseidb/bql/parsers/BQL.g:2523:1: type_argument : ( type | '?' ( ( 'extends' | 'super' ) type )? );
    public final BQLParser.type_argument_return type_argument() throws RecognitionException {
        BQLParser.type_argument_return retval = new BQLParser.type_argument_return();
        retval.start = input.LT(1);

        int type_argument_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal199=null;
        Token set200=null;
        BQLParser.type_return type198 =null;

        BQLParser.type_return type201 =null;


        Object char_literal199_tree=null;
        Object set200_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 61) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2524:5: ( type | '?' ( ( 'extends' | 'super' ) type )? )
            int alt76=2;
            switch ( input.LA(1) ) {
            case BOOLEAN:
            case BYTE:
            case DOUBLE:
            case FAST_UTIL_DATA_TYPE:
            case INT:
            case LONG:
            case 131:
            case 132:
            case 133:
            case 134:
            case 135:
            case 136:
            case 137:
            case 144:
            case 151:
            case 156:
                {
                alt76=1;
                }
                break;
            case 130:
                {
                alt76=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 76, 0, input);

                throw nvae;

            }

            switch (alt76) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2524:9: type
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_type_in_type_argument7684);
                    type198=type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, type198.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2525:9: '?' ( ( 'extends' | 'super' ) type )?
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal199=(Token)match(input,130,FOLLOW_130_in_type_argument7694); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal199_tree = 
                    (Object)adaptor.create(char_literal199)
                    ;
                    adaptor.addChild(root_0, char_literal199_tree);
                    }

                    // com/senseidb/bql/parsers/BQL.g:2525:13: ( ( 'extends' | 'super' ) type )?
                    int alt75=2;
                    switch ( input.LA(1) ) {
                        case 149:
                        case 157:
                            {
                            alt75=1;
                            }
                            break;
                    }

                    switch (alt75) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2525:14: ( 'extends' | 'super' ) type
                            {
                            set200=(Token)input.LT(1);

                            if ( input.LA(1)==149||input.LA(1)==157 ) {
                                input.consume();
                                if ( state.backtracking==0 ) adaptor.addChild(root_0, 
                                (Object)adaptor.create(set200)
                                );
                                state.errorRecovery=false;
                                state.failed=false;
                            }
                            else {
                                if (state.backtracking>0) {state.failed=true; return retval;}
                                MismatchedSetException mse = new MismatchedSetException(null,input);
                                throw mse;
                            }


                            pushFollow(FOLLOW_type_in_type_argument7705);
                            type201=type();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) adaptor.addChild(root_0, type201.getTree());

                            }
                            break;

                    }


                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 61, type_argument_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "type_argument"


    public static class formal_parameters_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "formal_parameters"
    // com/senseidb/bql/parsers/BQL.g:2528:1: formal_parameters returns [JSONObject json] : LPAR formal_parameter_decls RPAR ;
    public final BQLParser.formal_parameters_return formal_parameters() throws RecognitionException {
        BQLParser.formal_parameters_return retval = new BQLParser.formal_parameters_return();
        retval.start = input.LT(1);

        int formal_parameters_StartIndex = input.index();

        Object root_0 = null;

        Token LPAR202=null;
        Token RPAR204=null;
        BQLParser.formal_parameter_decls_return formal_parameter_decls203 =null;


        Object LPAR202_tree=null;
        Object RPAR204_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 62) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2529:5: ( LPAR formal_parameter_decls RPAR )
            // com/senseidb/bql/parsers/BQL.g:2529:9: LPAR formal_parameter_decls RPAR
            {
            root_0 = (Object)adaptor.nil();


            LPAR202=(Token)match(input,LPAR,FOLLOW_LPAR_in_formal_parameters7730); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LPAR202_tree = 
            (Object)adaptor.create(LPAR202)
            ;
            adaptor.addChild(root_0, LPAR202_tree);
            }

            pushFollow(FOLLOW_formal_parameter_decls_in_formal_parameters7732);
            formal_parameter_decls203=formal_parameter_decls();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, formal_parameter_decls203.getTree());

            RPAR204=(Token)match(input,RPAR,FOLLOW_RPAR_in_formal_parameters7734); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            RPAR204_tree = 
            (Object)adaptor.create(RPAR204)
            ;
            adaptor.addChild(root_0, RPAR204_tree);
            }

            if ( state.backtracking==0 ) {
                        retval.json = (formal_parameter_decls203!=null?formal_parameter_decls203.json:null);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 62, formal_parameters_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "formal_parameters"


    public static class formal_parameter_decls_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "formal_parameter_decls"
    // com/senseidb/bql/parsers/BQL.g:2535:1: formal_parameter_decls returns [JSONObject json] : decl= formal_parameter_decl ( COMMA decl= formal_parameter_decl )* ;
    public final BQLParser.formal_parameter_decls_return formal_parameter_decls() throws RecognitionException {
        BQLParser.formal_parameter_decls_return retval = new BQLParser.formal_parameter_decls_return();
        retval.start = input.LT(1);

        int formal_parameter_decls_StartIndex = input.index();

        Object root_0 = null;

        Token COMMA205=null;
        BQLParser.formal_parameter_decl_return decl =null;


        Object COMMA205_tree=null;


            retval.json = new JSONObject();
            Set<String> params = new HashSet<String>();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 63) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2540:5: (decl= formal_parameter_decl ( COMMA decl= formal_parameter_decl )* )
            // com/senseidb/bql/parsers/BQL.g:2540:9: decl= formal_parameter_decl ( COMMA decl= formal_parameter_decl )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_formal_parameter_decl_in_formal_parameter_decls7774);
            decl=formal_parameter_decl();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, decl.getTree());

            if ( state.backtracking==0 ) {
                        try {
                            processRelevanceModelParam(input, retval.json, params, (decl!=null?decl.typeName:null), (decl!=null?decl.varName:null));
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input,
                                                               "formal_parameter_decls",
                                                               "JSONException: " + err.getMessage());
                        }
                    }

            // com/senseidb/bql/parsers/BQL.g:2551:9: ( COMMA decl= formal_parameter_decl )*
            loop77:
            do {
                int alt77=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt77=1;
                    }
                    break;

                }

                switch (alt77) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2551:10: COMMA decl= formal_parameter_decl
            	    {
            	    COMMA205=(Token)match(input,COMMA,FOLLOW_COMMA_in_formal_parameter_decls7795); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA205_tree = 
            	    (Object)adaptor.create(COMMA205)
            	    ;
            	    adaptor.addChild(root_0, COMMA205_tree);
            	    }

            	    pushFollow(FOLLOW_formal_parameter_decl_in_formal_parameter_decls7799);
            	    decl=formal_parameter_decl();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, decl.getTree());

            	    if ( state.backtracking==0 ) {
            	                    try {
            	                        processRelevanceModelParam(input, retval.json, params, (decl!=null?decl.typeName:null), (decl!=null?decl.varName:null));
            	                    }
            	                    catch (JSONException err) {
            	                        throw new FailedPredicateException(input,
            	                                                           "formal_parameter_decl",
            	                                                           "JSONException: " + err.getMessage());
            	                    }
            	                }

            	    }
            	    break;

            	default :
            	    break loop77;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 63, formal_parameter_decls_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "formal_parameter_decls"


    public static class formal_parameter_decl_return extends ParserRuleReturnScope {
        public String typeName;
        public String varName;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "formal_parameter_decl"
    // com/senseidb/bql/parsers/BQL.g:2565:1: formal_parameter_decl returns [String typeName, String varName] : variable_modifiers type variable_declarator_id ;
    public final BQLParser.formal_parameter_decl_return formal_parameter_decl() throws RecognitionException {
        BQLParser.formal_parameter_decl_return retval = new BQLParser.formal_parameter_decl_return();
        retval.start = input.LT(1);

        int formal_parameter_decl_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.variable_modifiers_return variable_modifiers206 =null;

        BQLParser.type_return type207 =null;

        BQLParser.variable_declarator_id_return variable_declarator_id208 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 64) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2566:5: ( variable_modifiers type variable_declarator_id )
            // com/senseidb/bql/parsers/BQL.g:2566:9: variable_modifiers type variable_declarator_id
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_variable_modifiers_in_formal_parameter_decl7851);
            variable_modifiers206=variable_modifiers();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_modifiers206.getTree());

            pushFollow(FOLLOW_type_in_formal_parameter_decl7853);
            type207=type();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, type207.getTree());

            pushFollow(FOLLOW_variable_declarator_id_in_formal_parameter_decl7855);
            variable_declarator_id208=variable_declarator_id();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_declarator_id208.getTree());

            if ( state.backtracking==0 ) {
                        retval.typeName = (type207!=null?type207.typeName:null);
                        retval.varName = (variable_declarator_id208!=null?variable_declarator_id208.varName:null);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 64, formal_parameter_decl_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "formal_parameter_decl"


    public static class primitive_type_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "primitive_type"
    // com/senseidb/bql/parsers/BQL.g:2573:1: primitive_type : ({...}? BOOLEAN | 'char' |{...}? BYTE | 'short' |{...}? INT |{...}? LONG | 'float' |{...}? DOUBLE );
    public final BQLParser.primitive_type_return primitive_type() throws RecognitionException {
        BQLParser.primitive_type_return retval = new BQLParser.primitive_type_return();
        retval.start = input.LT(1);

        int primitive_type_StartIndex = input.index();

        Object root_0 = null;

        Token BOOLEAN209=null;
        Token string_literal210=null;
        Token BYTE211=null;
        Token string_literal212=null;
        Token INT213=null;
        Token LONG214=null;
        Token string_literal215=null;
        Token DOUBLE216=null;

        Object BOOLEAN209_tree=null;
        Object string_literal210_tree=null;
        Object BYTE211_tree=null;
        Object string_literal212_tree=null;
        Object INT213_tree=null;
        Object LONG214_tree=null;
        Object string_literal215_tree=null;
        Object DOUBLE216_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 65) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2574:5: ({...}? BOOLEAN | 'char' |{...}? BYTE | 'short' |{...}? INT |{...}? LONG | 'float' |{...}? DOUBLE )
            int alt78=8;
            switch ( input.LA(1) ) {
            case BOOLEAN:
                {
                alt78=1;
                }
                break;
            case 144:
                {
                alt78=2;
                }
                break;
            case BYTE:
                {
                alt78=3;
                }
                break;
            case 156:
                {
                alt78=4;
                }
                break;
            case INT:
                {
                alt78=5;
                }
                break;
            case LONG:
                {
                alt78=6;
                }
                break;
            case 151:
                {
                alt78=7;
                }
                break;
            case DOUBLE:
                {
                alt78=8;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 78, 0, input);

                throw nvae;

            }

            switch (alt78) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2574:9: {...}? BOOLEAN
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "boolean".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "primitive_type", " \"boolean\".equals(input.LT(1).getText()) ");
                    }

                    BOOLEAN209=(Token)match(input,BOOLEAN,FOLLOW_BOOLEAN_in_primitive_type7886); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    BOOLEAN209_tree = 
                    (Object)adaptor.create(BOOLEAN209)
                    ;
                    adaptor.addChild(root_0, BOOLEAN209_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2575:9: 'char'
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal210=(Token)match(input,144,FOLLOW_144_in_primitive_type7896); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal210_tree = 
                    (Object)adaptor.create(string_literal210)
                    ;
                    adaptor.addChild(root_0, string_literal210_tree);
                    }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2576:9: {...}? BYTE
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "byte".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "primitive_type", " \"byte\".equals(input.LT(1).getText()) ");
                    }

                    BYTE211=(Token)match(input,BYTE,FOLLOW_BYTE_in_primitive_type7908); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    BYTE211_tree = 
                    (Object)adaptor.create(BYTE211)
                    ;
                    adaptor.addChild(root_0, BYTE211_tree);
                    }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:2577:9: 'short'
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal212=(Token)match(input,156,FOLLOW_156_in_primitive_type7918); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal212_tree = 
                    (Object)adaptor.create(string_literal212)
                    ;
                    adaptor.addChild(root_0, string_literal212_tree);
                    }

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:2578:9: {...}? INT
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "int".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "primitive_type", " \"int\".equals(input.LT(1).getText()) ");
                    }

                    INT213=(Token)match(input,INT,FOLLOW_INT_in_primitive_type7930); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    INT213_tree = 
                    (Object)adaptor.create(INT213)
                    ;
                    adaptor.addChild(root_0, INT213_tree);
                    }

                    }
                    break;
                case 6 :
                    // com/senseidb/bql/parsers/BQL.g:2579:9: {...}? LONG
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "long".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "primitive_type", " \"long\".equals(input.LT(1).getText()) ");
                    }

                    LONG214=(Token)match(input,LONG,FOLLOW_LONG_in_primitive_type7942); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    LONG214_tree = 
                    (Object)adaptor.create(LONG214)
                    ;
                    adaptor.addChild(root_0, LONG214_tree);
                    }

                    }
                    break;
                case 7 :
                    // com/senseidb/bql/parsers/BQL.g:2580:9: 'float'
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal215=(Token)match(input,151,FOLLOW_151_in_primitive_type7952); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal215_tree = 
                    (Object)adaptor.create(string_literal215)
                    ;
                    adaptor.addChild(root_0, string_literal215_tree);
                    }

                    }
                    break;
                case 8 :
                    // com/senseidb/bql/parsers/BQL.g:2581:9: {...}? DOUBLE
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "double".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "primitive_type", " \"double\".equals(input.LT(1).getText()) ");
                    }

                    DOUBLE216=(Token)match(input,DOUBLE,FOLLOW_DOUBLE_in_primitive_type7964); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    DOUBLE216_tree = 
                    (Object)adaptor.create(DOUBLE216)
                    ;
                    adaptor.addChild(root_0, DOUBLE216_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 65, primitive_type_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "primitive_type"


    public static class boxed_type_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "boxed_type"
    // com/senseidb/bql/parsers/BQL.g:2584:1: boxed_type : ({...}? BOOLEAN | 'Character' |{...}? BYTE | 'Short' | 'Integer' |{...}? LONG | 'Float' |{...}? DOUBLE );
    public final BQLParser.boxed_type_return boxed_type() throws RecognitionException {
        BQLParser.boxed_type_return retval = new BQLParser.boxed_type_return();
        retval.start = input.LT(1);

        int boxed_type_StartIndex = input.index();

        Object root_0 = null;

        Token BOOLEAN217=null;
        Token string_literal218=null;
        Token BYTE219=null;
        Token string_literal220=null;
        Token string_literal221=null;
        Token LONG222=null;
        Token string_literal223=null;
        Token DOUBLE224=null;

        Object BOOLEAN217_tree=null;
        Object string_literal218_tree=null;
        Object BYTE219_tree=null;
        Object string_literal220_tree=null;
        Object string_literal221_tree=null;
        Object LONG222_tree=null;
        Object string_literal223_tree=null;
        Object DOUBLE224_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 66) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2585:5: ({...}? BOOLEAN | 'Character' |{...}? BYTE | 'Short' | 'Integer' |{...}? LONG | 'Float' |{...}? DOUBLE )
            int alt79=8;
            switch ( input.LA(1) ) {
            case BOOLEAN:
                {
                alt79=1;
                }
                break;
            case 131:
                {
                alt79=2;
                }
                break;
            case BYTE:
                {
                alt79=3;
                }
                break;
            case 135:
                {
                alt79=4;
                }
                break;
            case 133:
                {
                alt79=5;
                }
                break;
            case LONG:
                {
                alt79=6;
                }
                break;
            case 132:
                {
                alt79=7;
                }
                break;
            case DOUBLE:
                {
                alt79=8;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 79, 0, input);

                throw nvae;

            }

            switch (alt79) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2585:9: {...}? BOOLEAN
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "Boolean".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "boxed_type", " \"Boolean\".equals(input.LT(1).getText()) ");
                    }

                    BOOLEAN217=(Token)match(input,BOOLEAN,FOLLOW_BOOLEAN_in_boxed_type7985); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    BOOLEAN217_tree = 
                    (Object)adaptor.create(BOOLEAN217)
                    ;
                    adaptor.addChild(root_0, BOOLEAN217_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2586:9: 'Character'
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal218=(Token)match(input,131,FOLLOW_131_in_boxed_type7995); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal218_tree = 
                    (Object)adaptor.create(string_literal218)
                    ;
                    adaptor.addChild(root_0, string_literal218_tree);
                    }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2587:9: {...}? BYTE
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "Byte".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "boxed_type", " \"Byte\".equals(input.LT(1).getText()) ");
                    }

                    BYTE219=(Token)match(input,BYTE,FOLLOW_BYTE_in_boxed_type8007); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    BYTE219_tree = 
                    (Object)adaptor.create(BYTE219)
                    ;
                    adaptor.addChild(root_0, BYTE219_tree);
                    }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:2588:9: 'Short'
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal220=(Token)match(input,135,FOLLOW_135_in_boxed_type8017); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal220_tree = 
                    (Object)adaptor.create(string_literal220)
                    ;
                    adaptor.addChild(root_0, string_literal220_tree);
                    }

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:2589:9: 'Integer'
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal221=(Token)match(input,133,FOLLOW_133_in_boxed_type8027); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal221_tree = 
                    (Object)adaptor.create(string_literal221)
                    ;
                    adaptor.addChild(root_0, string_literal221_tree);
                    }

                    }
                    break;
                case 6 :
                    // com/senseidb/bql/parsers/BQL.g:2590:9: {...}? LONG
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "Long".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "boxed_type", " \"Long\".equals(input.LT(1).getText()) ");
                    }

                    LONG222=(Token)match(input,LONG,FOLLOW_LONG_in_boxed_type8039); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    LONG222_tree = 
                    (Object)adaptor.create(LONG222)
                    ;
                    adaptor.addChild(root_0, LONG222_tree);
                    }

                    }
                    break;
                case 7 :
                    // com/senseidb/bql/parsers/BQL.g:2591:9: 'Float'
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal223=(Token)match(input,132,FOLLOW_132_in_boxed_type8049); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal223_tree = 
                    (Object)adaptor.create(string_literal223)
                    ;
                    adaptor.addChild(root_0, string_literal223_tree);
                    }

                    }
                    break;
                case 8 :
                    // com/senseidb/bql/parsers/BQL.g:2592:9: {...}? DOUBLE
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "Double".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "boxed_type", " \"Double\".equals(input.LT(1).getText()) ");
                    }

                    DOUBLE224=(Token)match(input,DOUBLE,FOLLOW_DOUBLE_in_boxed_type8061); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    DOUBLE224_tree = 
                    (Object)adaptor.create(DOUBLE224)
                    ;
                    adaptor.addChild(root_0, DOUBLE224_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 66, boxed_type_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "boxed_type"


    public static class limited_type_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "limited_type"
    // com/senseidb/bql/parsers/BQL.g:2595:1: limited_type : ( 'String' | 'System' | 'Math' );
    public final BQLParser.limited_type_return limited_type() throws RecognitionException {
        BQLParser.limited_type_return retval = new BQLParser.limited_type_return();
        retval.start = input.LT(1);

        int limited_type_StartIndex = input.index();

        Object root_0 = null;

        Token set225=null;

        Object set225_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 67) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2596:5: ( 'String' | 'System' | 'Math' )
            // com/senseidb/bql/parsers/BQL.g:
            {
            root_0 = (Object)adaptor.nil();


            set225=(Token)input.LT(1);

            if ( input.LA(1)==134||(input.LA(1) >= 136 && input.LA(1) <= 137) ) {
                input.consume();
                if ( state.backtracking==0 ) adaptor.addChild(root_0, 
                (Object)adaptor.create(set225)
                );
                state.errorRecovery=false;
                state.failed=false;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                MismatchedSetException mse = new MismatchedSetException(null,input);
                throw mse;
            }


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 67, limited_type_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "limited_type"


    public static class variable_modifier_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "variable_modifier"
    // com/senseidb/bql/parsers/BQL.g:2601:1: variable_modifier : 'final' ;
    public final BQLParser.variable_modifier_return variable_modifier() throws RecognitionException {
        BQLParser.variable_modifier_return retval = new BQLParser.variable_modifier_return();
        retval.start = input.LT(1);

        int variable_modifier_StartIndex = input.index();

        Object root_0 = null;

        Token string_literal226=null;

        Object string_literal226_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 68) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2602:5: ( 'final' )
            // com/senseidb/bql/parsers/BQL.g:2602:9: 'final'
            {
            root_0 = (Object)adaptor.nil();


            string_literal226=(Token)match(input,150,FOLLOW_150_in_variable_modifier8119); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            string_literal226_tree = 
            (Object)adaptor.create(string_literal226)
            ;
            adaptor.addChild(root_0, string_literal226_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 68, variable_modifier_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "variable_modifier"


    public static class relevance_model_return extends ParserRuleReturnScope {
        public String functionBody;
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "relevance_model"
    // com/senseidb/bql/parsers/BQL.g:2605:1: relevance_model returns [String functionBody, JSONObject json] : DEFINED AS params= formal_parameters BEGIN model_block END ;
    public final BQLParser.relevance_model_return relevance_model() throws RecognitionException {
        BQLParser.relevance_model_return retval = new BQLParser.relevance_model_return();
        retval.start = input.LT(1);

        int relevance_model_StartIndex = input.index();

        Object root_0 = null;

        Token DEFINED227=null;
        Token AS228=null;
        Token BEGIN229=null;
        Token END231=null;
        BQLParser.formal_parameters_return params =null;

        BQLParser.model_block_return model_block230 =null;


        Object DEFINED227_tree=null;
        Object AS228_tree=null;
        Object BEGIN229_tree=null;
        Object END231_tree=null;


            _usedFacets = new HashSet<String>();
            _usedInternalVars = new HashSet<String>();
            _symbolTable = new LinkedList<Map<String, String>>();
            _currentScope = new HashMap<String, String>();
            _symbolTable.offerLast(_currentScope);

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 69) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2613:5: ( DEFINED AS params= formal_parameters BEGIN model_block END )
            // com/senseidb/bql/parsers/BQL.g:2613:9: DEFINED AS params= formal_parameters BEGIN model_block END
            {
            root_0 = (Object)adaptor.nil();


            DEFINED227=(Token)match(input,DEFINED,FOLLOW_DEFINED_in_relevance_model8147); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            DEFINED227_tree = 
            (Object)adaptor.create(DEFINED227)
            ;
            adaptor.addChild(root_0, DEFINED227_tree);
            }

            AS228=(Token)match(input,AS,FOLLOW_AS_in_relevance_model8149); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            AS228_tree = 
            (Object)adaptor.create(AS228)
            ;
            adaptor.addChild(root_0, AS228_tree);
            }

            pushFollow(FOLLOW_formal_parameters_in_relevance_model8153);
            params=formal_parameters();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, params.getTree());

            if ( state.backtracking==0 ) {
                        try {
                            JSONObject varParams = (params!=null?params.json:null).optJSONObject("variables");
                            if (varParams != null) {
                                Iterator<String> itr = varParams.keys();
                                while (itr.hasNext()) {
                                    String key = (String) itr.next();
                                    JSONArray vars = varParams.getJSONArray(key);
                                    for (int i = 0; i < vars.length(); ++i) {
                                        _currentScope.put(vars.getString(i), key);
                                    }
                                }
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "relevance_model", "JSONException: " + err.getMessage());
                        }
                    }

            BEGIN229=(Token)match(input,BEGIN,FOLLOW_BEGIN_in_relevance_model8173); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            BEGIN229_tree = 
            (Object)adaptor.create(BEGIN229)
            ;
            adaptor.addChild(root_0, BEGIN229_tree);
            }

            pushFollow(FOLLOW_model_block_in_relevance_model8175);
            model_block230=model_block();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, model_block230.getTree());

            END231=(Token)match(input,END,FOLLOW_END_in_relevance_model8177); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            END231_tree = 
            (Object)adaptor.create(END231)
            ;
            adaptor.addChild(root_0, END231_tree);
            }

            if ( state.backtracking==0 ) {
                        retval.functionBody = (model_block230!=null?input.toString(model_block230.start,model_block230.stop):null);
                        retval.json = (params!=null?params.json:null);

                        // Append facets and internal variable to "function_params".
                        try {
                            JSONArray funcParams = retval.json.getJSONArray("function_params");

                            JSONObject facets = new JSONObject();
                            retval.json.put("facets", facets);

                            for (String facet: _usedFacets) {
                                funcParams.put(facet);
                                String[] facetInfo = _facetInfoMap.get(facet);
                                String typeName = (facetInfo[0].equals("multi") ? "m" : "")
                                                  + _facetInfoMap.get(facet)[1];
                                JSONArray facetsWithSameType = facets.optJSONArray(typeName);
                                if (facetsWithSameType == null) {
                                    facetsWithSameType = new JSONArray();
                                    facets.put(typeName, facetsWithSameType);
                                }
                                facetsWithSameType.put(facet);
                            }

                            // Internal variables, like _NOW, do not need to be
                            // included in "variables".
                            for (String varName: _usedInternalVars) {
                                funcParams.put(varName);
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input,
                                                               "formal_parameter_decl",
                                                               "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 69, relevance_model_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "relevance_model"


    public static class model_block_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "model_block"
    // com/senseidb/bql/parsers/BQL.g:2671:1: model_block : ( block_statement )+ ;
    public final BQLParser.model_block_return model_block() throws RecognitionException {
        BQLParser.model_block_return retval = new BQLParser.model_block_return();
        retval.start = input.LT(1);

        int model_block_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.block_statement_return block_statement232 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 70) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2672:5: ( ( block_statement )+ )
            // com/senseidb/bql/parsers/BQL.g:2672:9: ( block_statement )+
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:2672:9: ( block_statement )+
            int cnt80=0;
            loop80:
            do {
                int alt80=2;
                switch ( input.LA(1) ) {
                case BOOLEAN:
                case BYTE:
                case CHARACTER_LITERAL:
                case DOUBLE:
                case FALSE:
                case FAST_UTIL_DATA_TYPE:
                case FLOATING_POINT_LITERAL:
                case HEX_LITERAL:
                case IDENT:
                case INT:
                case INTEGER:
                case LONG:
                case LPAR:
                case NULL:
                case OCTAL_LITERAL:
                case REAL:
                case SEMI:
                case STRING_LITERAL:
                case TRUE:
                case 111:
                case 120:
                case 121:
                case 123:
                case 124:
                case 131:
                case 132:
                case 133:
                case 134:
                case 135:
                case 136:
                case 137:
                case 142:
                case 144:
                case 146:
                case 148:
                case 150:
                case 151:
                case 152:
                case 153:
                case 155:
                case 156:
                case 158:
                case 160:
                case 161:
                case 166:
                    {
                    alt80=1;
                    }
                    break;

                }

                switch (alt80) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2672:9: block_statement
            	    {
            	    pushFollow(FOLLOW_block_statement_in_model_block8206);
            	    block_statement232=block_statement();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, block_statement232.getTree());

            	    }
            	    break;

            	default :
            	    if ( cnt80 >= 1 ) break loop80;
            	    if (state.backtracking>0) {state.failed=true; return retval;}
                        EarlyExitException eee =
                            new EarlyExitException(80, input);
                        throw eee;
                }
                cnt80++;
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 70, model_block_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "model_block"


    public static class block_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "block"
    // com/senseidb/bql/parsers/BQL.g:2675:1: block : '{' ( block_statement )* '}' ;
    public final BQLParser.block_return block() throws RecognitionException {
        BQLParser.block_return retval = new BQLParser.block_return();
        retval.start = input.LT(1);

        int block_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal233=null;
        Token char_literal235=null;
        BQLParser.block_statement_return block_statement234 =null;


        Object char_literal233_tree=null;
        Object char_literal235_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 71) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2676:5: ( '{' ( block_statement )* '}' )
            // com/senseidb/bql/parsers/BQL.g:2676:9: '{' ( block_statement )* '}'
            {
            root_0 = (Object)adaptor.nil();


            char_literal233=(Token)match(input,161,FOLLOW_161_in_block8226); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal233_tree = 
            (Object)adaptor.create(char_literal233)
            ;
            adaptor.addChild(root_0, char_literal233_tree);
            }

            if ( state.backtracking==0 ) {
                        _currentScope = new HashMap<String, String>();
                        _symbolTable.offerLast(_currentScope);
                    }

            // com/senseidb/bql/parsers/BQL.g:2681:9: ( block_statement )*
            loop81:
            do {
                int alt81=2;
                switch ( input.LA(1) ) {
                case BOOLEAN:
                case BYTE:
                case CHARACTER_LITERAL:
                case DOUBLE:
                case FALSE:
                case FAST_UTIL_DATA_TYPE:
                case FLOATING_POINT_LITERAL:
                case HEX_LITERAL:
                case IDENT:
                case INT:
                case INTEGER:
                case LONG:
                case LPAR:
                case NULL:
                case OCTAL_LITERAL:
                case REAL:
                case SEMI:
                case STRING_LITERAL:
                case TRUE:
                case 111:
                case 120:
                case 121:
                case 123:
                case 124:
                case 131:
                case 132:
                case 133:
                case 134:
                case 135:
                case 136:
                case 137:
                case 142:
                case 144:
                case 146:
                case 148:
                case 150:
                case 151:
                case 152:
                case 153:
                case 155:
                case 156:
                case 158:
                case 160:
                case 161:
                case 166:
                    {
                    alt81=1;
                    }
                    break;

                }

                switch (alt81) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2681:9: block_statement
            	    {
            	    pushFollow(FOLLOW_block_statement_in_block8247);
            	    block_statement234=block_statement();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, block_statement234.getTree());

            	    }
            	    break;

            	default :
            	    break loop81;
                }
            } while (true);


            if ( state.backtracking==0 ) {
                        _symbolTable.pollLast();
                        _currentScope = _symbolTable.peekLast();
                    }

            char_literal235=(Token)match(input,165,FOLLOW_165_in_block8269); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal235_tree = 
            (Object)adaptor.create(char_literal235)
            ;
            adaptor.addChild(root_0, char_literal235_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 71, block_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "block"


    public static class block_statement_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "block_statement"
    // com/senseidb/bql/parsers/BQL.g:2689:1: block_statement : ( local_variable_declaration_stmt | java_statement );
    public final BQLParser.block_statement_return block_statement() throws RecognitionException {
        BQLParser.block_statement_return retval = new BQLParser.block_statement_return();
        retval.start = input.LT(1);

        int block_statement_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.local_variable_declaration_stmt_return local_variable_declaration_stmt236 =null;

        BQLParser.java_statement_return java_statement237 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 72) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2690:5: ( local_variable_declaration_stmt | java_statement )
            int alt82=2;
            alt82 = dfa82.predict(input);
            switch (alt82) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2690:9: local_variable_declaration_stmt
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_local_variable_declaration_stmt_in_block_statement8288);
                    local_variable_declaration_stmt236=local_variable_declaration_stmt();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, local_variable_declaration_stmt236.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2691:9: java_statement
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_java_statement_in_block_statement8298);
                    java_statement237=java_statement();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, java_statement237.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 72, block_statement_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "block_statement"


    public static class local_variable_declaration_stmt_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "local_variable_declaration_stmt"
    // com/senseidb/bql/parsers/BQL.g:2694:1: local_variable_declaration_stmt : local_variable_declaration SEMI ;
    public final BQLParser.local_variable_declaration_stmt_return local_variable_declaration_stmt() throws RecognitionException {
        BQLParser.local_variable_declaration_stmt_return retval = new BQLParser.local_variable_declaration_stmt_return();
        retval.start = input.LT(1);

        int local_variable_declaration_stmt_StartIndex = input.index();

        Object root_0 = null;

        Token SEMI239=null;
        BQLParser.local_variable_declaration_return local_variable_declaration238 =null;


        Object SEMI239_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 73) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2695:5: ( local_variable_declaration SEMI )
            // com/senseidb/bql/parsers/BQL.g:2695:9: local_variable_declaration SEMI
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_local_variable_declaration_in_local_variable_declaration_stmt8317);
            local_variable_declaration238=local_variable_declaration();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, local_variable_declaration238.getTree());

            SEMI239=(Token)match(input,SEMI,FOLLOW_SEMI_in_local_variable_declaration_stmt8319); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            SEMI239_tree = 
            (Object)adaptor.create(SEMI239)
            ;
            adaptor.addChild(root_0, SEMI239_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 73, local_variable_declaration_stmt_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "local_variable_declaration_stmt"


    public static class local_variable_declaration_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "local_variable_declaration"
    // com/senseidb/bql/parsers/BQL.g:2698:1: local_variable_declaration : variable_modifiers type variable_declarators ;
    public final BQLParser.local_variable_declaration_return local_variable_declaration() throws RecognitionException {
        BQLParser.local_variable_declaration_return retval = new BQLParser.local_variable_declaration_return();
        retval.start = input.LT(1);

        int local_variable_declaration_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.variable_modifiers_return variable_modifiers240 =null;

        BQLParser.type_return type241 =null;

        BQLParser.variable_declarators_return variable_declarators242 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 74) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2699:5: ( variable_modifiers type variable_declarators )
            // com/senseidb/bql/parsers/BQL.g:2699:9: variable_modifiers type variable_declarators
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_variable_modifiers_in_local_variable_declaration8338);
            variable_modifiers240=variable_modifiers();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_modifiers240.getTree());

            pushFollow(FOLLOW_type_in_local_variable_declaration8340);
            type241=type();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, type241.getTree());

            pushFollow(FOLLOW_variable_declarators_in_local_variable_declaration8342);
            variable_declarators242=variable_declarators();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_declarators242.getTree());

            if ( state.backtracking==0 ) {
                        try {
                            JSONArray vars = (variable_declarators242!=null?variable_declarators242.json:null);
                            for (int i = 0; i < vars.length(); ++i) {
                                String var = vars.getString(i);
                                if (_facetInfoMap.containsKey(var)) {
                                    throw new FailedPredicateException(input,
                                                                       "local_variable_declaration",
                                                                       "Facet name \"" + var + "\" cannot be used to declare a variable.");
                                }
                                else if (_internalVarMap.containsKey(var)) {
                                    throw new FailedPredicateException(input,
                                                                       "local_variable_declaration",
                                                                       "Internal variable \"" + var + "\" cannot be re-used to declare another variable.");
                                }
                                else if (verifyVariable(var)) {
                                    throw new FailedPredicateException(input,
                                                                       "local_variable_declaration",
                                                                       "Variable \"" + var + "\" is already defined.");
                                }
                                else {
                                    _currentScope.put(var, (type241!=null?type241.typeName:null));
                                }
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "local_variable_declaration", "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 74, local_variable_declaration_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "local_variable_declaration"


    public static class variable_modifiers_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "variable_modifiers"
    // com/senseidb/bql/parsers/BQL.g:2731:1: variable_modifiers : ( variable_modifier )* ;
    public final BQLParser.variable_modifiers_return variable_modifiers() throws RecognitionException {
        BQLParser.variable_modifiers_return retval = new BQLParser.variable_modifiers_return();
        retval.start = input.LT(1);

        int variable_modifiers_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.variable_modifier_return variable_modifier243 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 75) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2732:5: ( ( variable_modifier )* )
            // com/senseidb/bql/parsers/BQL.g:2732:9: ( variable_modifier )*
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:2732:9: ( variable_modifier )*
            loop83:
            do {
                int alt83=2;
                switch ( input.LA(1) ) {
                case 150:
                    {
                    alt83=1;
                    }
                    break;

                }

                switch (alt83) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2732:9: variable_modifier
            	    {
            	    pushFollow(FOLLOW_variable_modifier_in_variable_modifiers8371);
            	    variable_modifier243=variable_modifier();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_modifier243.getTree());

            	    }
            	    break;

            	default :
            	    break loop83;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 75, variable_modifiers_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "variable_modifiers"


    public static class java_statement_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "java_statement"
    // com/senseidb/bql/parsers/BQL.g:2735:1: java_statement : ( block | 'if' par_expression java_statement ( else_statement )? | 'for' LPAR for_control RPAR java_statement | 'while' par_expression java_statement | 'do' java_statement 'while' par_expression SEMI | 'switch' par_expression '{' switch_block_statement_groups '}' | 'return' expression SEMI | 'break' ( IDENT )? SEMI | 'continue' ( IDENT )? SEMI | SEMI | statement_expression SEMI );
    public final BQLParser.java_statement_return java_statement() throws RecognitionException {
        BQLParser.java_statement_return retval = new BQLParser.java_statement_return();
        retval.start = input.LT(1);

        int java_statement_StartIndex = input.index();

        Object root_0 = null;

        Token string_literal245=null;
        Token string_literal249=null;
        Token LPAR250=null;
        Token RPAR252=null;
        Token string_literal254=null;
        Token string_literal257=null;
        Token string_literal259=null;
        Token SEMI261=null;
        Token string_literal262=null;
        Token char_literal264=null;
        Token char_literal266=null;
        Token string_literal267=null;
        Token SEMI269=null;
        Token string_literal270=null;
        Token IDENT271=null;
        Token SEMI272=null;
        Token string_literal273=null;
        Token IDENT274=null;
        Token SEMI275=null;
        Token SEMI276=null;
        Token SEMI278=null;
        BQLParser.block_return block244 =null;

        BQLParser.par_expression_return par_expression246 =null;

        BQLParser.java_statement_return java_statement247 =null;

        BQLParser.else_statement_return else_statement248 =null;

        BQLParser.for_control_return for_control251 =null;

        BQLParser.java_statement_return java_statement253 =null;

        BQLParser.par_expression_return par_expression255 =null;

        BQLParser.java_statement_return java_statement256 =null;

        BQLParser.java_statement_return java_statement258 =null;

        BQLParser.par_expression_return par_expression260 =null;

        BQLParser.par_expression_return par_expression263 =null;

        BQLParser.switch_block_statement_groups_return switch_block_statement_groups265 =null;

        BQLParser.expression_return expression268 =null;

        BQLParser.statement_expression_return statement_expression277 =null;


        Object string_literal245_tree=null;
        Object string_literal249_tree=null;
        Object LPAR250_tree=null;
        Object RPAR252_tree=null;
        Object string_literal254_tree=null;
        Object string_literal257_tree=null;
        Object string_literal259_tree=null;
        Object SEMI261_tree=null;
        Object string_literal262_tree=null;
        Object char_literal264_tree=null;
        Object char_literal266_tree=null;
        Object string_literal267_tree=null;
        Object SEMI269_tree=null;
        Object string_literal270_tree=null;
        Object IDENT271_tree=null;
        Object SEMI272_tree=null;
        Object string_literal273_tree=null;
        Object IDENT274_tree=null;
        Object SEMI275_tree=null;
        Object SEMI276_tree=null;
        Object SEMI278_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 76) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2736:5: ( block | 'if' par_expression java_statement ( else_statement )? | 'for' LPAR for_control RPAR java_statement | 'while' par_expression java_statement | 'do' java_statement 'while' par_expression SEMI | 'switch' par_expression '{' switch_block_statement_groups '}' | 'return' expression SEMI | 'break' ( IDENT )? SEMI | 'continue' ( IDENT )? SEMI | SEMI | statement_expression SEMI )
            int alt87=11;
            switch ( input.LA(1) ) {
            case 161:
                {
                alt87=1;
                }
                break;
            case 153:
                {
                alt87=2;
                }
                break;
            case 152:
                {
                alt87=3;
                }
                break;
            case 160:
                {
                alt87=4;
                }
                break;
            case 148:
                {
                alt87=5;
                }
                break;
            case 158:
                {
                alt87=6;
                }
                break;
            case 155:
                {
                alt87=7;
                }
                break;
            case 142:
                {
                alt87=8;
                }
                break;
            case 146:
                {
                alt87=9;
                }
                break;
            case SEMI:
                {
                alt87=10;
                }
                break;
            case BOOLEAN:
            case BYTE:
            case CHARACTER_LITERAL:
            case DOUBLE:
            case FALSE:
            case FLOATING_POINT_LITERAL:
            case HEX_LITERAL:
            case IDENT:
            case INTEGER:
            case LONG:
            case LPAR:
            case NULL:
            case OCTAL_LITERAL:
            case REAL:
            case STRING_LITERAL:
            case TRUE:
            case 111:
            case 120:
            case 121:
            case 123:
            case 124:
            case 131:
            case 132:
            case 133:
            case 134:
            case 135:
            case 136:
            case 137:
            case 166:
                {
                alt87=11;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 87, 0, input);

                throw nvae;

            }

            switch (alt87) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2736:9: block
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_block_in_java_statement8391);
                    block244=block();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, block244.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2737:9: 'if' par_expression java_statement ( else_statement )?
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal245=(Token)match(input,153,FOLLOW_153_in_java_statement8401); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal245_tree = 
                    (Object)adaptor.create(string_literal245)
                    ;
                    adaptor.addChild(root_0, string_literal245_tree);
                    }

                    pushFollow(FOLLOW_par_expression_in_java_statement8403);
                    par_expression246=par_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, par_expression246.getTree());

                    pushFollow(FOLLOW_java_statement_in_java_statement8405);
                    java_statement247=java_statement();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, java_statement247.getTree());

                    // com/senseidb/bql/parsers/BQL.g:2737:44: ( else_statement )?
                    int alt84=2;
                    switch ( input.LA(1) ) {
                        case ELSE:
                            {
                            int LA84_1 = input.LA(2);

                            if ( (((synpred133_BQL()&&synpred133_BQL())&&( "else".equals(input.LT(1).getText()) ))) ) {
                                alt84=1;
                            }
                            }
                            break;
                    }

                    switch (alt84) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2737:45: else_statement
                            {
                            pushFollow(FOLLOW_else_statement_in_java_statement8408);
                            else_statement248=else_statement();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) adaptor.addChild(root_0, else_statement248.getTree());

                            }
                            break;

                    }


                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2738:9: 'for' LPAR for_control RPAR java_statement
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal249=(Token)match(input,152,FOLLOW_152_in_java_statement8420); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal249_tree = 
                    (Object)adaptor.create(string_literal249)
                    ;
                    adaptor.addChild(root_0, string_literal249_tree);
                    }

                    LPAR250=(Token)match(input,LPAR,FOLLOW_LPAR_in_java_statement8422); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    LPAR250_tree = 
                    (Object)adaptor.create(LPAR250)
                    ;
                    adaptor.addChild(root_0, LPAR250_tree);
                    }

                    if ( state.backtracking==0 ) {
                                _currentScope = new HashMap<String, String>();
                                _symbolTable.offerLast(_currentScope);
                            }

                    pushFollow(FOLLOW_for_control_in_java_statement8442);
                    for_control251=for_control();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, for_control251.getTree());

                    RPAR252=(Token)match(input,RPAR,FOLLOW_RPAR_in_java_statement8444); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    RPAR252_tree = 
                    (Object)adaptor.create(RPAR252)
                    ;
                    adaptor.addChild(root_0, RPAR252_tree);
                    }

                    pushFollow(FOLLOW_java_statement_in_java_statement8446);
                    java_statement253=java_statement();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, java_statement253.getTree());

                    if ( state.backtracking==0 ) {
                                _symbolTable.pollLast();
                                _currentScope = _symbolTable.peekLast();
                            }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:2748:9: 'while' par_expression java_statement
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal254=(Token)match(input,160,FOLLOW_160_in_java_statement8466); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal254_tree = 
                    (Object)adaptor.create(string_literal254)
                    ;
                    adaptor.addChild(root_0, string_literal254_tree);
                    }

                    pushFollow(FOLLOW_par_expression_in_java_statement8468);
                    par_expression255=par_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, par_expression255.getTree());

                    pushFollow(FOLLOW_java_statement_in_java_statement8470);
                    java_statement256=java_statement();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, java_statement256.getTree());

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:2749:9: 'do' java_statement 'while' par_expression SEMI
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal257=(Token)match(input,148,FOLLOW_148_in_java_statement8480); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal257_tree = 
                    (Object)adaptor.create(string_literal257)
                    ;
                    adaptor.addChild(root_0, string_literal257_tree);
                    }

                    pushFollow(FOLLOW_java_statement_in_java_statement8482);
                    java_statement258=java_statement();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, java_statement258.getTree());

                    string_literal259=(Token)match(input,160,FOLLOW_160_in_java_statement8484); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal259_tree = 
                    (Object)adaptor.create(string_literal259)
                    ;
                    adaptor.addChild(root_0, string_literal259_tree);
                    }

                    pushFollow(FOLLOW_par_expression_in_java_statement8486);
                    par_expression260=par_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, par_expression260.getTree());

                    SEMI261=(Token)match(input,SEMI,FOLLOW_SEMI_in_java_statement8488); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    SEMI261_tree = 
                    (Object)adaptor.create(SEMI261)
                    ;
                    adaptor.addChild(root_0, SEMI261_tree);
                    }

                    }
                    break;
                case 6 :
                    // com/senseidb/bql/parsers/BQL.g:2750:9: 'switch' par_expression '{' switch_block_statement_groups '}'
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal262=(Token)match(input,158,FOLLOW_158_in_java_statement8498); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal262_tree = 
                    (Object)adaptor.create(string_literal262)
                    ;
                    adaptor.addChild(root_0, string_literal262_tree);
                    }

                    pushFollow(FOLLOW_par_expression_in_java_statement8500);
                    par_expression263=par_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, par_expression263.getTree());

                    char_literal264=(Token)match(input,161,FOLLOW_161_in_java_statement8502); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal264_tree = 
                    (Object)adaptor.create(char_literal264)
                    ;
                    adaptor.addChild(root_0, char_literal264_tree);
                    }

                    pushFollow(FOLLOW_switch_block_statement_groups_in_java_statement8504);
                    switch_block_statement_groups265=switch_block_statement_groups();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, switch_block_statement_groups265.getTree());

                    char_literal266=(Token)match(input,165,FOLLOW_165_in_java_statement8506); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal266_tree = 
                    (Object)adaptor.create(char_literal266)
                    ;
                    adaptor.addChild(root_0, char_literal266_tree);
                    }

                    }
                    break;
                case 7 :
                    // com/senseidb/bql/parsers/BQL.g:2751:9: 'return' expression SEMI
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal267=(Token)match(input,155,FOLLOW_155_in_java_statement8516); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal267_tree = 
                    (Object)adaptor.create(string_literal267)
                    ;
                    adaptor.addChild(root_0, string_literal267_tree);
                    }

                    pushFollow(FOLLOW_expression_in_java_statement8518);
                    expression268=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression268.getTree());

                    SEMI269=(Token)match(input,SEMI,FOLLOW_SEMI_in_java_statement8520); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    SEMI269_tree = 
                    (Object)adaptor.create(SEMI269)
                    ;
                    adaptor.addChild(root_0, SEMI269_tree);
                    }

                    }
                    break;
                case 8 :
                    // com/senseidb/bql/parsers/BQL.g:2752:9: 'break' ( IDENT )? SEMI
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal270=(Token)match(input,142,FOLLOW_142_in_java_statement8530); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal270_tree = 
                    (Object)adaptor.create(string_literal270)
                    ;
                    adaptor.addChild(root_0, string_literal270_tree);
                    }

                    // com/senseidb/bql/parsers/BQL.g:2752:17: ( IDENT )?
                    int alt85=2;
                    switch ( input.LA(1) ) {
                        case IDENT:
                            {
                            alt85=1;
                            }
                            break;
                    }

                    switch (alt85) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2752:17: IDENT
                            {
                            IDENT271=(Token)match(input,IDENT,FOLLOW_IDENT_in_java_statement8532); if (state.failed) return retval;
                            if ( state.backtracking==0 ) {
                            IDENT271_tree = 
                            (Object)adaptor.create(IDENT271)
                            ;
                            adaptor.addChild(root_0, IDENT271_tree);
                            }

                            }
                            break;

                    }


                    SEMI272=(Token)match(input,SEMI,FOLLOW_SEMI_in_java_statement8535); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    SEMI272_tree = 
                    (Object)adaptor.create(SEMI272)
                    ;
                    adaptor.addChild(root_0, SEMI272_tree);
                    }

                    }
                    break;
                case 9 :
                    // com/senseidb/bql/parsers/BQL.g:2753:9: 'continue' ( IDENT )? SEMI
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal273=(Token)match(input,146,FOLLOW_146_in_java_statement8545); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal273_tree = 
                    (Object)adaptor.create(string_literal273)
                    ;
                    adaptor.addChild(root_0, string_literal273_tree);
                    }

                    // com/senseidb/bql/parsers/BQL.g:2753:20: ( IDENT )?
                    int alt86=2;
                    switch ( input.LA(1) ) {
                        case IDENT:
                            {
                            alt86=1;
                            }
                            break;
                    }

                    switch (alt86) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2753:20: IDENT
                            {
                            IDENT274=(Token)match(input,IDENT,FOLLOW_IDENT_in_java_statement8547); if (state.failed) return retval;
                            if ( state.backtracking==0 ) {
                            IDENT274_tree = 
                            (Object)adaptor.create(IDENT274)
                            ;
                            adaptor.addChild(root_0, IDENT274_tree);
                            }

                            }
                            break;

                    }


                    SEMI275=(Token)match(input,SEMI,FOLLOW_SEMI_in_java_statement8550); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    SEMI275_tree = 
                    (Object)adaptor.create(SEMI275)
                    ;
                    adaptor.addChild(root_0, SEMI275_tree);
                    }

                    }
                    break;
                case 10 :
                    // com/senseidb/bql/parsers/BQL.g:2754:9: SEMI
                    {
                    root_0 = (Object)adaptor.nil();


                    SEMI276=(Token)match(input,SEMI,FOLLOW_SEMI_in_java_statement8560); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    SEMI276_tree = 
                    (Object)adaptor.create(SEMI276)
                    ;
                    adaptor.addChild(root_0, SEMI276_tree);
                    }

                    }
                    break;
                case 11 :
                    // com/senseidb/bql/parsers/BQL.g:2755:9: statement_expression SEMI
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_statement_expression_in_java_statement8570);
                    statement_expression277=statement_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, statement_expression277.getTree());

                    SEMI278=(Token)match(input,SEMI,FOLLOW_SEMI_in_java_statement8572); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    SEMI278_tree = 
                    (Object)adaptor.create(SEMI278)
                    ;
                    adaptor.addChild(root_0, SEMI278_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 76, java_statement_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "java_statement"


    public static class else_statement_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "else_statement"
    // com/senseidb/bql/parsers/BQL.g:2758:1: else_statement :{...}? ELSE java_statement ;
    public final BQLParser.else_statement_return else_statement() throws RecognitionException {
        BQLParser.else_statement_return retval = new BQLParser.else_statement_return();
        retval.start = input.LT(1);

        int else_statement_StartIndex = input.index();

        Object root_0 = null;

        Token ELSE279=null;
        BQLParser.java_statement_return java_statement280 =null;


        Object ELSE279_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 77) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2759:5: ({...}? ELSE java_statement )
            // com/senseidb/bql/parsers/BQL.g:2759:9: {...}? ELSE java_statement
            {
            root_0 = (Object)adaptor.nil();


            if ( !(( "else".equals(input.LT(1).getText()) )) ) {
                if (state.backtracking>0) {state.failed=true; return retval;}
                throw new FailedPredicateException(input, "else_statement", " \"else\".equals(input.LT(1).getText()) ");
            }

            ELSE279=(Token)match(input,ELSE,FOLLOW_ELSE_in_else_statement8593); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            ELSE279_tree = 
            (Object)adaptor.create(ELSE279)
            ;
            adaptor.addChild(root_0, ELSE279_tree);
            }

            pushFollow(FOLLOW_java_statement_in_else_statement8595);
            java_statement280=java_statement();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, java_statement280.getTree());

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 77, else_statement_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "else_statement"


    public static class switch_block_statement_groups_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "switch_block_statement_groups"
    // com/senseidb/bql/parsers/BQL.g:2762:1: switch_block_statement_groups : ( switch_block_statement_group )* ;
    public final BQLParser.switch_block_statement_groups_return switch_block_statement_groups() throws RecognitionException {
        BQLParser.switch_block_statement_groups_return retval = new BQLParser.switch_block_statement_groups_return();
        retval.start = input.LT(1);

        int switch_block_statement_groups_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.switch_block_statement_group_return switch_block_statement_group281 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 78) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2763:5: ( ( switch_block_statement_group )* )
            // com/senseidb/bql/parsers/BQL.g:2763:9: ( switch_block_statement_group )*
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:2763:9: ( switch_block_statement_group )*
            loop88:
            do {
                int alt88=2;
                switch ( input.LA(1) ) {
                case 143:
                case 147:
                    {
                    alt88=1;
                    }
                    break;

                }

                switch (alt88) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2763:10: switch_block_statement_group
            	    {
            	    pushFollow(FOLLOW_switch_block_statement_group_in_switch_block_statement_groups8615);
            	    switch_block_statement_group281=switch_block_statement_group();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, switch_block_statement_group281.getTree());

            	    }
            	    break;

            	default :
            	    break loop88;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 78, switch_block_statement_groups_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "switch_block_statement_groups"


    public static class switch_block_statement_group_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "switch_block_statement_group"
    // com/senseidb/bql/parsers/BQL.g:2766:1: switch_block_statement_group : ( switch_label )+ ( block_statement )* ;
    public final BQLParser.switch_block_statement_group_return switch_block_statement_group() throws RecognitionException {
        BQLParser.switch_block_statement_group_return retval = new BQLParser.switch_block_statement_group_return();
        retval.start = input.LT(1);

        int switch_block_statement_group_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.switch_label_return switch_label282 =null;

        BQLParser.block_statement_return block_statement283 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 79) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2767:5: ( ( switch_label )+ ( block_statement )* )
            // com/senseidb/bql/parsers/BQL.g:2767:9: ( switch_label )+ ( block_statement )*
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:2767:9: ( switch_label )+
            int cnt89=0;
            loop89:
            do {
                int alt89=2;
                switch ( input.LA(1) ) {
                case 143:
                    {
                    int LA89_2 = input.LA(2);

                    if ( (synpred146_BQL()) ) {
                        alt89=1;
                    }


                    }
                    break;
                case 147:
                    {
                    int LA89_3 = input.LA(2);

                    if ( (synpred146_BQL()) ) {
                        alt89=1;
                    }


                    }
                    break;

                }

                switch (alt89) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2767:9: switch_label
            	    {
            	    pushFollow(FOLLOW_switch_label_in_switch_block_statement_group8636);
            	    switch_label282=switch_label();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, switch_label282.getTree());

            	    }
            	    break;

            	default :
            	    if ( cnt89 >= 1 ) break loop89;
            	    if (state.backtracking>0) {state.failed=true; return retval;}
                        EarlyExitException eee =
                            new EarlyExitException(89, input);
                        throw eee;
                }
                cnt89++;
            } while (true);


            // com/senseidb/bql/parsers/BQL.g:2767:23: ( block_statement )*
            loop90:
            do {
                int alt90=2;
                switch ( input.LA(1) ) {
                case BOOLEAN:
                case BYTE:
                case CHARACTER_LITERAL:
                case DOUBLE:
                case FALSE:
                case FAST_UTIL_DATA_TYPE:
                case FLOATING_POINT_LITERAL:
                case HEX_LITERAL:
                case IDENT:
                case INT:
                case INTEGER:
                case LONG:
                case LPAR:
                case NULL:
                case OCTAL_LITERAL:
                case REAL:
                case SEMI:
                case STRING_LITERAL:
                case TRUE:
                case 111:
                case 120:
                case 121:
                case 123:
                case 124:
                case 131:
                case 132:
                case 133:
                case 134:
                case 135:
                case 136:
                case 137:
                case 142:
                case 144:
                case 146:
                case 148:
                case 150:
                case 151:
                case 152:
                case 153:
                case 155:
                case 156:
                case 158:
                case 160:
                case 161:
                case 166:
                    {
                    alt90=1;
                    }
                    break;

                }

                switch (alt90) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2767:23: block_statement
            	    {
            	    pushFollow(FOLLOW_block_statement_in_switch_block_statement_group8639);
            	    block_statement283=block_statement();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, block_statement283.getTree());

            	    }
            	    break;

            	default :
            	    break loop90;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 79, switch_block_statement_group_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "switch_block_statement_group"


    public static class switch_label_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "switch_label"
    // com/senseidb/bql/parsers/BQL.g:2770:1: switch_label : ( 'case' constant_expression COLON | 'case' enum_constant_name COLON | 'default' COLON );
    public final BQLParser.switch_label_return switch_label() throws RecognitionException {
        BQLParser.switch_label_return retval = new BQLParser.switch_label_return();
        retval.start = input.LT(1);

        int switch_label_StartIndex = input.index();

        Object root_0 = null;

        Token string_literal284=null;
        Token COLON286=null;
        Token string_literal287=null;
        Token COLON289=null;
        Token string_literal290=null;
        Token COLON291=null;
        BQLParser.constant_expression_return constant_expression285 =null;

        BQLParser.enum_constant_name_return enum_constant_name288 =null;


        Object string_literal284_tree=null;
        Object COLON286_tree=null;
        Object string_literal287_tree=null;
        Object COLON289_tree=null;
        Object string_literal290_tree=null;
        Object COLON291_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 80) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2771:5: ( 'case' constant_expression COLON | 'case' enum_constant_name COLON | 'default' COLON )
            int alt91=3;
            switch ( input.LA(1) ) {
            case 143:
                {
                switch ( input.LA(2) ) {
                case BOOLEAN:
                case BYTE:
                case CHARACTER_LITERAL:
                case DOUBLE:
                case FALSE:
                case FLOATING_POINT_LITERAL:
                case HEX_LITERAL:
                case INTEGER:
                case LONG:
                case LPAR:
                case NULL:
                case OCTAL_LITERAL:
                case REAL:
                case STRING_LITERAL:
                case TRUE:
                case 111:
                case 120:
                case 121:
                case 123:
                case 124:
                case 131:
                case 132:
                case 133:
                case 134:
                case 135:
                case 136:
                case 137:
                case 166:
                    {
                    alt91=1;
                    }
                    break;
                case IDENT:
                    {
                    switch ( input.LA(3) ) {
                    case EQUAL:
                    case GT:
                    case LPAR:
                    case LT:
                    case 112:
                    case 113:
                    case 114:
                    case 115:
                    case 116:
                    case 117:
                    case 118:
                    case 119:
                    case 120:
                    case 121:
                    case 122:
                    case 123:
                    case 124:
                    case 125:
                    case 126:
                    case 127:
                    case 128:
                    case 129:
                    case 130:
                    case 138:
                    case 140:
                    case 141:
                    case 154:
                    case 162:
                    case 163:
                    case 164:
                        {
                        alt91=1;
                        }
                        break;
                    case COLON:
                        {
                        int LA91_5 = input.LA(4);

                        if ( (synpred148_BQL()) ) {
                            alt91=1;
                        }
                        else if ( (synpred149_BQL()) ) {
                            alt91=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 91, 5, input);

                            throw nvae;

                        }
                        }
                        break;
                    default:
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        NoViableAltException nvae =
                            new NoViableAltException("", 91, 4, input);

                        throw nvae;

                    }

                    }
                    break;
                default:
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 91, 1, input);

                    throw nvae;

                }

                }
                break;
            case 147:
                {
                alt91=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 91, 0, input);

                throw nvae;

            }

            switch (alt91) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2771:9: 'case' constant_expression COLON
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal284=(Token)match(input,143,FOLLOW_143_in_switch_label8659); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal284_tree = 
                    (Object)adaptor.create(string_literal284)
                    ;
                    adaptor.addChild(root_0, string_literal284_tree);
                    }

                    pushFollow(FOLLOW_constant_expression_in_switch_label8661);
                    constant_expression285=constant_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, constant_expression285.getTree());

                    COLON286=(Token)match(input,COLON,FOLLOW_COLON_in_switch_label8663); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    COLON286_tree = 
                    (Object)adaptor.create(COLON286)
                    ;
                    adaptor.addChild(root_0, COLON286_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2772:9: 'case' enum_constant_name COLON
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal287=(Token)match(input,143,FOLLOW_143_in_switch_label8673); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal287_tree = 
                    (Object)adaptor.create(string_literal287)
                    ;
                    adaptor.addChild(root_0, string_literal287_tree);
                    }

                    pushFollow(FOLLOW_enum_constant_name_in_switch_label8675);
                    enum_constant_name288=enum_constant_name();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, enum_constant_name288.getTree());

                    COLON289=(Token)match(input,COLON,FOLLOW_COLON_in_switch_label8677); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    COLON289_tree = 
                    (Object)adaptor.create(COLON289)
                    ;
                    adaptor.addChild(root_0, COLON289_tree);
                    }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2773:9: 'default' COLON
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal290=(Token)match(input,147,FOLLOW_147_in_switch_label8687); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal290_tree = 
                    (Object)adaptor.create(string_literal290)
                    ;
                    adaptor.addChild(root_0, string_literal290_tree);
                    }

                    COLON291=(Token)match(input,COLON,FOLLOW_COLON_in_switch_label8689); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    COLON291_tree = 
                    (Object)adaptor.create(COLON291)
                    ;
                    adaptor.addChild(root_0, COLON291_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 80, switch_label_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "switch_label"


    public static class for_control_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "for_control"
    // com/senseidb/bql/parsers/BQL.g:2776:1: for_control options {k=3; } : ( enhanced_for_control | ( for_init )? SEMI ( expression )? SEMI ( for_update )? );
    public final BQLParser.for_control_return for_control() throws RecognitionException {
        BQLParser.for_control_return retval = new BQLParser.for_control_return();
        retval.start = input.LT(1);

        int for_control_StartIndex = input.index();

        Object root_0 = null;

        Token SEMI294=null;
        Token SEMI296=null;
        BQLParser.enhanced_for_control_return enhanced_for_control292 =null;

        BQLParser.for_init_return for_init293 =null;

        BQLParser.expression_return expression295 =null;

        BQLParser.for_update_return for_update297 =null;


        Object SEMI294_tree=null;
        Object SEMI296_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 81) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2778:5: ( enhanced_for_control | ( for_init )? SEMI ( expression )? SEMI ( for_update )? )
            int alt95=2;
            alt95 = dfa95.predict(input);
            switch (alt95) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2778:9: enhanced_for_control
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_enhanced_for_control_in_for_control8715);
                    enhanced_for_control292=enhanced_for_control();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, enhanced_for_control292.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2779:9: ( for_init )? SEMI ( expression )? SEMI ( for_update )?
                    {
                    root_0 = (Object)adaptor.nil();


                    // com/senseidb/bql/parsers/BQL.g:2779:9: ( for_init )?
                    int alt92=2;
                    switch ( input.LA(1) ) {
                        case BOOLEAN:
                        case BYTE:
                        case CHARACTER_LITERAL:
                        case DOUBLE:
                        case FALSE:
                        case FAST_UTIL_DATA_TYPE:
                        case FLOATING_POINT_LITERAL:
                        case HEX_LITERAL:
                        case IDENT:
                        case INT:
                        case INTEGER:
                        case LONG:
                        case LPAR:
                        case NULL:
                        case OCTAL_LITERAL:
                        case REAL:
                        case STRING_LITERAL:
                        case TRUE:
                        case 111:
                        case 120:
                        case 121:
                        case 123:
                        case 124:
                        case 131:
                        case 132:
                        case 133:
                        case 134:
                        case 135:
                        case 136:
                        case 137:
                        case 144:
                        case 150:
                        case 151:
                        case 156:
                        case 166:
                            {
                            alt92=1;
                            }
                            break;
                    }

                    switch (alt92) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2779:9: for_init
                            {
                            pushFollow(FOLLOW_for_init_in_for_control8725);
                            for_init293=for_init();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) adaptor.addChild(root_0, for_init293.getTree());

                            }
                            break;

                    }


                    SEMI294=(Token)match(input,SEMI,FOLLOW_SEMI_in_for_control8728); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    SEMI294_tree = 
                    (Object)adaptor.create(SEMI294)
                    ;
                    adaptor.addChild(root_0, SEMI294_tree);
                    }

                    // com/senseidb/bql/parsers/BQL.g:2779:24: ( expression )?
                    int alt93=2;
                    switch ( input.LA(1) ) {
                        case BOOLEAN:
                        case BYTE:
                        case CHARACTER_LITERAL:
                        case DOUBLE:
                        case FALSE:
                        case FLOATING_POINT_LITERAL:
                        case HEX_LITERAL:
                        case IDENT:
                        case INTEGER:
                        case LONG:
                        case LPAR:
                        case NULL:
                        case OCTAL_LITERAL:
                        case REAL:
                        case STRING_LITERAL:
                        case TRUE:
                        case 111:
                        case 120:
                        case 121:
                        case 123:
                        case 124:
                        case 131:
                        case 132:
                        case 133:
                        case 134:
                        case 135:
                        case 136:
                        case 137:
                        case 166:
                            {
                            alt93=1;
                            }
                            break;
                    }

                    switch (alt93) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2779:24: expression
                            {
                            pushFollow(FOLLOW_expression_in_for_control8730);
                            expression295=expression();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression295.getTree());

                            }
                            break;

                    }


                    SEMI296=(Token)match(input,SEMI,FOLLOW_SEMI_in_for_control8733); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    SEMI296_tree = 
                    (Object)adaptor.create(SEMI296)
                    ;
                    adaptor.addChild(root_0, SEMI296_tree);
                    }

                    // com/senseidb/bql/parsers/BQL.g:2779:41: ( for_update )?
                    int alt94=2;
                    switch ( input.LA(1) ) {
                        case BOOLEAN:
                        case BYTE:
                        case CHARACTER_LITERAL:
                        case DOUBLE:
                        case FALSE:
                        case FLOATING_POINT_LITERAL:
                        case HEX_LITERAL:
                        case IDENT:
                        case INTEGER:
                        case LONG:
                        case LPAR:
                        case NULL:
                        case OCTAL_LITERAL:
                        case REAL:
                        case STRING_LITERAL:
                        case TRUE:
                        case 111:
                        case 120:
                        case 121:
                        case 123:
                        case 124:
                        case 131:
                        case 132:
                        case 133:
                        case 134:
                        case 135:
                        case 136:
                        case 137:
                        case 166:
                            {
                            alt94=1;
                            }
                            break;
                    }

                    switch (alt94) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2779:41: for_update
                            {
                            pushFollow(FOLLOW_for_update_in_for_control8735);
                            for_update297=for_update();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) adaptor.addChild(root_0, for_update297.getTree());

                            }
                            break;

                    }


                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 81, for_control_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "for_control"


    public static class for_init_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "for_init"
    // com/senseidb/bql/parsers/BQL.g:2782:1: for_init : ( local_variable_declaration | expression_list );
    public final BQLParser.for_init_return for_init() throws RecognitionException {
        BQLParser.for_init_return retval = new BQLParser.for_init_return();
        retval.start = input.LT(1);

        int for_init_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.local_variable_declaration_return local_variable_declaration298 =null;

        BQLParser.expression_list_return expression_list299 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 82) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2783:5: ( local_variable_declaration | expression_list )
            int alt96=2;
            alt96 = dfa96.predict(input);
            switch (alt96) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2783:9: local_variable_declaration
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_local_variable_declaration_in_for_init8755);
                    local_variable_declaration298=local_variable_declaration();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, local_variable_declaration298.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2784:9: expression_list
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_expression_list_in_for_init8765);
                    expression_list299=expression_list();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression_list299.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 82, for_init_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "for_init"


    public static class enhanced_for_control_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "enhanced_for_control"
    // com/senseidb/bql/parsers/BQL.g:2787:1: enhanced_for_control : variable_modifiers type IDENT COLON expression ;
    public final BQLParser.enhanced_for_control_return enhanced_for_control() throws RecognitionException {
        BQLParser.enhanced_for_control_return retval = new BQLParser.enhanced_for_control_return();
        retval.start = input.LT(1);

        int enhanced_for_control_StartIndex = input.index();

        Object root_0 = null;

        Token IDENT302=null;
        Token COLON303=null;
        BQLParser.variable_modifiers_return variable_modifiers300 =null;

        BQLParser.type_return type301 =null;

        BQLParser.expression_return expression304 =null;


        Object IDENT302_tree=null;
        Object COLON303_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 83) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2788:5: ( variable_modifiers type IDENT COLON expression )
            // com/senseidb/bql/parsers/BQL.g:2788:9: variable_modifiers type IDENT COLON expression
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_variable_modifiers_in_enhanced_for_control8784);
            variable_modifiers300=variable_modifiers();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_modifiers300.getTree());

            pushFollow(FOLLOW_type_in_enhanced_for_control8786);
            type301=type();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, type301.getTree());

            IDENT302=(Token)match(input,IDENT,FOLLOW_IDENT_in_enhanced_for_control8788); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            IDENT302_tree = 
            (Object)adaptor.create(IDENT302)
            ;
            adaptor.addChild(root_0, IDENT302_tree);
            }

            COLON303=(Token)match(input,COLON,FOLLOW_COLON_in_enhanced_for_control8790); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            COLON303_tree = 
            (Object)adaptor.create(COLON303)
            ;
            adaptor.addChild(root_0, COLON303_tree);
            }

            pushFollow(FOLLOW_expression_in_enhanced_for_control8792);
            expression304=expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression304.getTree());

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 83, enhanced_for_control_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "enhanced_for_control"


    public static class for_update_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "for_update"
    // com/senseidb/bql/parsers/BQL.g:2791:1: for_update : expression_list ;
    public final BQLParser.for_update_return for_update() throws RecognitionException {
        BQLParser.for_update_return retval = new BQLParser.for_update_return();
        retval.start = input.LT(1);

        int for_update_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.expression_list_return expression_list305 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 84) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2792:5: ( expression_list )
            // com/senseidb/bql/parsers/BQL.g:2792:9: expression_list
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_expression_list_in_for_update8811);
            expression_list305=expression_list();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression_list305.getTree());

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 84, for_update_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "for_update"


    public static class par_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "par_expression"
    // com/senseidb/bql/parsers/BQL.g:2795:1: par_expression : LPAR expression RPAR ;
    public final BQLParser.par_expression_return par_expression() throws RecognitionException {
        BQLParser.par_expression_return retval = new BQLParser.par_expression_return();
        retval.start = input.LT(1);

        int par_expression_StartIndex = input.index();

        Object root_0 = null;

        Token LPAR306=null;
        Token RPAR308=null;
        BQLParser.expression_return expression307 =null;


        Object LPAR306_tree=null;
        Object RPAR308_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 85) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2796:5: ( LPAR expression RPAR )
            // com/senseidb/bql/parsers/BQL.g:2796:9: LPAR expression RPAR
            {
            root_0 = (Object)adaptor.nil();


            LPAR306=(Token)match(input,LPAR,FOLLOW_LPAR_in_par_expression8830); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LPAR306_tree = 
            (Object)adaptor.create(LPAR306)
            ;
            adaptor.addChild(root_0, LPAR306_tree);
            }

            pushFollow(FOLLOW_expression_in_par_expression8832);
            expression307=expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression307.getTree());

            RPAR308=(Token)match(input,RPAR,FOLLOW_RPAR_in_par_expression8834); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            RPAR308_tree = 
            (Object)adaptor.create(RPAR308)
            ;
            adaptor.addChild(root_0, RPAR308_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 85, par_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "par_expression"


    public static class expression_list_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "expression_list"
    // com/senseidb/bql/parsers/BQL.g:2799:1: expression_list : expression ( ',' expression )* ;
    public final BQLParser.expression_list_return expression_list() throws RecognitionException {
        BQLParser.expression_list_return retval = new BQLParser.expression_list_return();
        retval.start = input.LT(1);

        int expression_list_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal310=null;
        BQLParser.expression_return expression309 =null;

        BQLParser.expression_return expression311 =null;


        Object char_literal310_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 86) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2800:5: ( expression ( ',' expression )* )
            // com/senseidb/bql/parsers/BQL.g:2800:9: expression ( ',' expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_expression_in_expression_list8853);
            expression309=expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression309.getTree());

            // com/senseidb/bql/parsers/BQL.g:2800:20: ( ',' expression )*
            loop97:
            do {
                int alt97=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt97=1;
                    }
                    break;

                }

                switch (alt97) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2800:21: ',' expression
            	    {
            	    char_literal310=(Token)match(input,COMMA,FOLLOW_COMMA_in_expression_list8856); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    char_literal310_tree = 
            	    (Object)adaptor.create(char_literal310)
            	    ;
            	    adaptor.addChild(root_0, char_literal310_tree);
            	    }

            	    pushFollow(FOLLOW_expression_in_expression_list8858);
            	    expression311=expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression311.getTree());

            	    }
            	    break;

            	default :
            	    break loop97;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 86, expression_list_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "expression_list"


    public static class statement_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "statement_expression"
    // com/senseidb/bql/parsers/BQL.g:2803:1: statement_expression : expression ;
    public final BQLParser.statement_expression_return statement_expression() throws RecognitionException {
        BQLParser.statement_expression_return retval = new BQLParser.statement_expression_return();
        retval.start = input.LT(1);

        int statement_expression_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.expression_return expression312 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 87) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2804:5: ( expression )
            // com/senseidb/bql/parsers/BQL.g:2804:9: expression
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_expression_in_statement_expression8879);
            expression312=expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression312.getTree());

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 87, statement_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "statement_expression"


    public static class constant_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "constant_expression"
    // com/senseidb/bql/parsers/BQL.g:2807:1: constant_expression : expression ;
    public final BQLParser.constant_expression_return constant_expression() throws RecognitionException {
        BQLParser.constant_expression_return retval = new BQLParser.constant_expression_return();
        retval.start = input.LT(1);

        int constant_expression_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.expression_return expression313 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 88) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2808:5: ( expression )
            // com/senseidb/bql/parsers/BQL.g:2808:9: expression
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_expression_in_constant_expression8898);
            expression313=expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression313.getTree());

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 88, constant_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "constant_expression"


    public static class enum_constant_name_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "enum_constant_name"
    // com/senseidb/bql/parsers/BQL.g:2811:1: enum_constant_name : IDENT ;
    public final BQLParser.enum_constant_name_return enum_constant_name() throws RecognitionException {
        BQLParser.enum_constant_name_return retval = new BQLParser.enum_constant_name_return();
        retval.start = input.LT(1);

        int enum_constant_name_StartIndex = input.index();

        Object root_0 = null;

        Token IDENT314=null;

        Object IDENT314_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 89) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2812:5: ( IDENT )
            // com/senseidb/bql/parsers/BQL.g:2812:9: IDENT
            {
            root_0 = (Object)adaptor.nil();


            IDENT314=(Token)match(input,IDENT,FOLLOW_IDENT_in_enum_constant_name8917); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            IDENT314_tree = 
            (Object)adaptor.create(IDENT314)
            ;
            adaptor.addChild(root_0, IDENT314_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 89, enum_constant_name_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "enum_constant_name"


    public static class expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "expression"
    // com/senseidb/bql/parsers/BQL.g:2815:1: expression : conditional_expression ( assignment_operator expression )? ;
    public final BQLParser.expression_return expression() throws RecognitionException {
        BQLParser.expression_return retval = new BQLParser.expression_return();
        retval.start = input.LT(1);

        int expression_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.conditional_expression_return conditional_expression315 =null;

        BQLParser.assignment_operator_return assignment_operator316 =null;

        BQLParser.expression_return expression317 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 90) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2816:5: ( conditional_expression ( assignment_operator expression )? )
            // com/senseidb/bql/parsers/BQL.g:2816:9: conditional_expression ( assignment_operator expression )?
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_conditional_expression_in_expression8936);
            conditional_expression315=conditional_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, conditional_expression315.getTree());

            // com/senseidb/bql/parsers/BQL.g:2816:32: ( assignment_operator expression )?
            int alt98=2;
            switch ( input.LA(1) ) {
                case EQUAL:
                    {
                    int LA98_1 = input.LA(2);

                    if ( (synpred156_BQL()) ) {
                        alt98=1;
                    }
                    }
                    break;
                case 122:
                    {
                    int LA98_2 = input.LA(2);

                    if ( (synpred156_BQL()) ) {
                        alt98=1;
                    }
                    }
                    break;
                case 125:
                    {
                    int LA98_3 = input.LA(2);

                    if ( (synpred156_BQL()) ) {
                        alt98=1;
                    }
                    }
                    break;
                case 119:
                    {
                    int LA98_4 = input.LA(2);

                    if ( (synpred156_BQL()) ) {
                        alt98=1;
                    }
                    }
                    break;
                case 128:
                    {
                    int LA98_5 = input.LA(2);

                    if ( (synpred156_BQL()) ) {
                        alt98=1;
                    }
                    }
                    break;
                case 117:
                    {
                    int LA98_6 = input.LA(2);

                    if ( (synpred156_BQL()) ) {
                        alt98=1;
                    }
                    }
                    break;
                case 163:
                    {
                    int LA98_7 = input.LA(2);

                    if ( (synpred156_BQL()) ) {
                        alt98=1;
                    }
                    }
                    break;
                case 141:
                    {
                    int LA98_8 = input.LA(2);

                    if ( (synpred156_BQL()) ) {
                        alt98=1;
                    }
                    }
                    break;
                case 114:
                    {
                    int LA98_9 = input.LA(2);

                    if ( (synpred156_BQL()) ) {
                        alt98=1;
                    }
                    }
                    break;
                case LT:
                    {
                    int LA98_10 = input.LA(2);

                    if ( (synpred156_BQL()) ) {
                        alt98=1;
                    }
                    }
                    break;
                case GT:
                    {
                    int LA98_11 = input.LA(2);

                    if ( (synpred156_BQL()) ) {
                        alt98=1;
                    }
                    }
                    break;
            }

            switch (alt98) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2816:33: assignment_operator expression
                    {
                    pushFollow(FOLLOW_assignment_operator_in_expression8939);
                    assignment_operator316=assignment_operator();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, assignment_operator316.getTree());

                    pushFollow(FOLLOW_expression_in_expression8941);
                    expression317=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression317.getTree());

                    }
                    break;

            }


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 90, expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "expression"


    public static class assignment_operator_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "assignment_operator"
    // com/senseidb/bql/parsers/BQL.g:2819:1: assignment_operator : ( '=' | '+=' | '-=' | '*=' | '/=' | '&=' | '|=' | '^=' | '%=' | ( '<' '<' '=' )=>t1= '<' t2= '<' t3= '=' {...}?| ( '>' '>' '>' '=' )=>t1= '>' t2= '>' t3= '>' t4= '=' {...}?| ( '>' '>' '=' )=>t1= '>' t2= '>' t3= '=' {...}?);
    public final BQLParser.assignment_operator_return assignment_operator() throws RecognitionException {
        BQLParser.assignment_operator_return retval = new BQLParser.assignment_operator_return();
        retval.start = input.LT(1);

        int assignment_operator_StartIndex = input.index();

        Object root_0 = null;

        Token t1=null;
        Token t2=null;
        Token t3=null;
        Token t4=null;
        Token char_literal318=null;
        Token string_literal319=null;
        Token string_literal320=null;
        Token string_literal321=null;
        Token string_literal322=null;
        Token string_literal323=null;
        Token string_literal324=null;
        Token string_literal325=null;
        Token string_literal326=null;

        Object t1_tree=null;
        Object t2_tree=null;
        Object t3_tree=null;
        Object t4_tree=null;
        Object char_literal318_tree=null;
        Object string_literal319_tree=null;
        Object string_literal320_tree=null;
        Object string_literal321_tree=null;
        Object string_literal322_tree=null;
        Object string_literal323_tree=null;
        Object string_literal324_tree=null;
        Object string_literal325_tree=null;
        Object string_literal326_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 91) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2820:5: ( '=' | '+=' | '-=' | '*=' | '/=' | '&=' | '|=' | '^=' | '%=' | ( '<' '<' '=' )=>t1= '<' t2= '<' t3= '=' {...}?| ( '>' '>' '>' '=' )=>t1= '>' t2= '>' t3= '>' t4= '=' {...}?| ( '>' '>' '=' )=>t1= '>' t2= '>' t3= '=' {...}?)
            int alt99=12;
            int LA99_0 = input.LA(1);

            if ( (LA99_0==EQUAL) ) {
                alt99=1;
            }
            else if ( (LA99_0==122) ) {
                alt99=2;
            }
            else if ( (LA99_0==125) ) {
                alt99=3;
            }
            else if ( (LA99_0==119) ) {
                alt99=4;
            }
            else if ( (LA99_0==128) ) {
                alt99=5;
            }
            else if ( (LA99_0==117) ) {
                alt99=6;
            }
            else if ( (LA99_0==163) ) {
                alt99=7;
            }
            else if ( (LA99_0==141) ) {
                alt99=8;
            }
            else if ( (LA99_0==114) ) {
                alt99=9;
            }
            else if ( (LA99_0==LT) && (synpred166_BQL())) {
                alt99=10;
            }
            else if ( (LA99_0==GT) ) {
                switch ( input.LA(2) ) {
                case GT:
                    {
                    int LA99_12 = input.LA(3);

                    if ( (LA99_12==GT) && (synpred167_BQL())) {
                        alt99=11;
                    }
                    else if ( (LA99_12==EQUAL) && (synpred168_BQL())) {
                        alt99=12;
                    }
                    else {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        NoViableAltException nvae =
                            new NoViableAltException("", 99, 12, input);

                        throw nvae;

                    }
                    }
                    break;
                default:
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 99, 11, input);

                    throw nvae;

                }

            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 99, 0, input);

                throw nvae;

            }
            switch (alt99) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2820:9: '='
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal318=(Token)match(input,EQUAL,FOLLOW_EQUAL_in_assignment_operator8962); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal318_tree = 
                    (Object)adaptor.create(char_literal318)
                    ;
                    adaptor.addChild(root_0, char_literal318_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2821:9: '+='
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal319=(Token)match(input,122,FOLLOW_122_in_assignment_operator8972); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal319_tree = 
                    (Object)adaptor.create(string_literal319)
                    ;
                    adaptor.addChild(root_0, string_literal319_tree);
                    }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2822:9: '-='
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal320=(Token)match(input,125,FOLLOW_125_in_assignment_operator8982); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal320_tree = 
                    (Object)adaptor.create(string_literal320)
                    ;
                    adaptor.addChild(root_0, string_literal320_tree);
                    }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:2823:9: '*='
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal321=(Token)match(input,119,FOLLOW_119_in_assignment_operator8992); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal321_tree = 
                    (Object)adaptor.create(string_literal321)
                    ;
                    adaptor.addChild(root_0, string_literal321_tree);
                    }

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:2824:9: '/='
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal322=(Token)match(input,128,FOLLOW_128_in_assignment_operator9002); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal322_tree = 
                    (Object)adaptor.create(string_literal322)
                    ;
                    adaptor.addChild(root_0, string_literal322_tree);
                    }

                    }
                    break;
                case 6 :
                    // com/senseidb/bql/parsers/BQL.g:2825:9: '&='
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal323=(Token)match(input,117,FOLLOW_117_in_assignment_operator9012); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal323_tree = 
                    (Object)adaptor.create(string_literal323)
                    ;
                    adaptor.addChild(root_0, string_literal323_tree);
                    }

                    }
                    break;
                case 7 :
                    // com/senseidb/bql/parsers/BQL.g:2826:9: '|='
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal324=(Token)match(input,163,FOLLOW_163_in_assignment_operator9022); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal324_tree = 
                    (Object)adaptor.create(string_literal324)
                    ;
                    adaptor.addChild(root_0, string_literal324_tree);
                    }

                    }
                    break;
                case 8 :
                    // com/senseidb/bql/parsers/BQL.g:2827:9: '^='
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal325=(Token)match(input,141,FOLLOW_141_in_assignment_operator9032); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal325_tree = 
                    (Object)adaptor.create(string_literal325)
                    ;
                    adaptor.addChild(root_0, string_literal325_tree);
                    }

                    }
                    break;
                case 9 :
                    // com/senseidb/bql/parsers/BQL.g:2828:9: '%='
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal326=(Token)match(input,114,FOLLOW_114_in_assignment_operator9042); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal326_tree = 
                    (Object)adaptor.create(string_literal326)
                    ;
                    adaptor.addChild(root_0, string_literal326_tree);
                    }

                    }
                    break;
                case 10 :
                    // com/senseidb/bql/parsers/BQL.g:2829:9: ( '<' '<' '=' )=>t1= '<' t2= '<' t3= '=' {...}?
                    {
                    root_0 = (Object)adaptor.nil();


                    t1=(Token)match(input,LT,FOLLOW_LT_in_assignment_operator9063); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t1_tree = 
                    (Object)adaptor.create(t1)
                    ;
                    adaptor.addChild(root_0, t1_tree);
                    }

                    t2=(Token)match(input,LT,FOLLOW_LT_in_assignment_operator9067); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t2_tree = 
                    (Object)adaptor.create(t2)
                    ;
                    adaptor.addChild(root_0, t2_tree);
                    }

                    t3=(Token)match(input,EQUAL,FOLLOW_EQUAL_in_assignment_operator9071); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t3_tree = 
                    (Object)adaptor.create(t3)
                    ;
                    adaptor.addChild(root_0, t3_tree);
                    }

                    if ( !(( t1.getLine() == t2.getLine() &&
                              t1.getCharPositionInLine() + 1 == t2.getCharPositionInLine() && 
                              t2.getLine() == t3.getLine() && 
                              t2.getCharPositionInLine() + 1 == t3.getCharPositionInLine() )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "assignment_operator", " $t1.getLine() == $t2.getLine() &&\n          $t1.getCharPositionInLine() + 1 == $t2.getCharPositionInLine() && \n          $t2.getLine() == $t3.getLine() && \n          $t2.getCharPositionInLine() + 1 == $t3.getCharPositionInLine() ");
                    }

                    }
                    break;
                case 11 :
                    // com/senseidb/bql/parsers/BQL.g:2834:9: ( '>' '>' '>' '=' )=>t1= '>' t2= '>' t3= '>' t4= '=' {...}?
                    {
                    root_0 = (Object)adaptor.nil();


                    t1=(Token)match(input,GT,FOLLOW_GT_in_assignment_operator9105); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t1_tree = 
                    (Object)adaptor.create(t1)
                    ;
                    adaptor.addChild(root_0, t1_tree);
                    }

                    t2=(Token)match(input,GT,FOLLOW_GT_in_assignment_operator9109); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t2_tree = 
                    (Object)adaptor.create(t2)
                    ;
                    adaptor.addChild(root_0, t2_tree);
                    }

                    t3=(Token)match(input,GT,FOLLOW_GT_in_assignment_operator9113); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t3_tree = 
                    (Object)adaptor.create(t3)
                    ;
                    adaptor.addChild(root_0, t3_tree);
                    }

                    t4=(Token)match(input,EQUAL,FOLLOW_EQUAL_in_assignment_operator9117); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t4_tree = 
                    (Object)adaptor.create(t4)
                    ;
                    adaptor.addChild(root_0, t4_tree);
                    }

                    if ( !(( t1.getLine() == t2.getLine() && 
                              t1.getCharPositionInLine() + 1 == t2.getCharPositionInLine() &&
                              t2.getLine() == t3.getLine() && 
                              t2.getCharPositionInLine() + 1 == t3.getCharPositionInLine() &&
                              t3.getLine() == t4.getLine() && 
                              t3.getCharPositionInLine() + 1 == t4.getCharPositionInLine() )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "assignment_operator", " $t1.getLine() == $t2.getLine() && \n          $t1.getCharPositionInLine() + 1 == $t2.getCharPositionInLine() &&\n          $t2.getLine() == $t3.getLine() && \n          $t2.getCharPositionInLine() + 1 == $t3.getCharPositionInLine() &&\n          $t3.getLine() == $t4.getLine() && \n          $t3.getCharPositionInLine() + 1 == $t4.getCharPositionInLine() ");
                    }

                    }
                    break;
                case 12 :
                    // com/senseidb/bql/parsers/BQL.g:2841:9: ( '>' '>' '=' )=>t1= '>' t2= '>' t3= '=' {...}?
                    {
                    root_0 = (Object)adaptor.nil();


                    t1=(Token)match(input,GT,FOLLOW_GT_in_assignment_operator9148); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t1_tree = 
                    (Object)adaptor.create(t1)
                    ;
                    adaptor.addChild(root_0, t1_tree);
                    }

                    t2=(Token)match(input,GT,FOLLOW_GT_in_assignment_operator9152); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t2_tree = 
                    (Object)adaptor.create(t2)
                    ;
                    adaptor.addChild(root_0, t2_tree);
                    }

                    t3=(Token)match(input,EQUAL,FOLLOW_EQUAL_in_assignment_operator9156); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t3_tree = 
                    (Object)adaptor.create(t3)
                    ;
                    adaptor.addChild(root_0, t3_tree);
                    }

                    if ( !(( t1.getLine() == t2.getLine() && 
                              t1.getCharPositionInLine() + 1 == t2.getCharPositionInLine() && 
                              t2.getLine() == t3.getLine() && 
                              t2.getCharPositionInLine() + 1 == t3.getCharPositionInLine() )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "assignment_operator", " $t1.getLine() == $t2.getLine() && \n          $t1.getCharPositionInLine() + 1 == $t2.getCharPositionInLine() && \n          $t2.getLine() == $t3.getLine() && \n          $t2.getCharPositionInLine() + 1 == $t3.getCharPositionInLine() ");
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 91, assignment_operator_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "assignment_operator"


    public static class conditional_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "conditional_expression"
    // com/senseidb/bql/parsers/BQL.g:2848:1: conditional_expression : conditional_or_expression ( '?' expression ':' expression )? ;
    public final BQLParser.conditional_expression_return conditional_expression() throws RecognitionException {
        BQLParser.conditional_expression_return retval = new BQLParser.conditional_expression_return();
        retval.start = input.LT(1);

        int conditional_expression_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal328=null;
        Token char_literal330=null;
        BQLParser.conditional_or_expression_return conditional_or_expression327 =null;

        BQLParser.expression_return expression329 =null;

        BQLParser.expression_return expression331 =null;


        Object char_literal328_tree=null;
        Object char_literal330_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 92) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2849:5: ( conditional_or_expression ( '?' expression ':' expression )? )
            // com/senseidb/bql/parsers/BQL.g:2849:9: conditional_or_expression ( '?' expression ':' expression )?
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_conditional_or_expression_in_conditional_expression9185);
            conditional_or_expression327=conditional_or_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, conditional_or_expression327.getTree());

            // com/senseidb/bql/parsers/BQL.g:2849:35: ( '?' expression ':' expression )?
            int alt100=2;
            switch ( input.LA(1) ) {
                case 130:
                    {
                    alt100=1;
                    }
                    break;
            }

            switch (alt100) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2849:37: '?' expression ':' expression
                    {
                    char_literal328=(Token)match(input,130,FOLLOW_130_in_conditional_expression9189); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal328_tree = 
                    (Object)adaptor.create(char_literal328)
                    ;
                    adaptor.addChild(root_0, char_literal328_tree);
                    }

                    pushFollow(FOLLOW_expression_in_conditional_expression9191);
                    expression329=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression329.getTree());

                    char_literal330=(Token)match(input,COLON,FOLLOW_COLON_in_conditional_expression9193); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal330_tree = 
                    (Object)adaptor.create(char_literal330)
                    ;
                    adaptor.addChild(root_0, char_literal330_tree);
                    }

                    pushFollow(FOLLOW_expression_in_conditional_expression9195);
                    expression331=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression331.getTree());

                    }
                    break;

            }


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 92, conditional_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "conditional_expression"


    public static class conditional_or_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "conditional_or_expression"
    // com/senseidb/bql/parsers/BQL.g:2852:1: conditional_or_expression : conditional_and_expression ( '||' conditional_and_expression )* ;
    public final BQLParser.conditional_or_expression_return conditional_or_expression() throws RecognitionException {
        BQLParser.conditional_or_expression_return retval = new BQLParser.conditional_or_expression_return();
        retval.start = input.LT(1);

        int conditional_or_expression_StartIndex = input.index();

        Object root_0 = null;

        Token string_literal333=null;
        BQLParser.conditional_and_expression_return conditional_and_expression332 =null;

        BQLParser.conditional_and_expression_return conditional_and_expression334 =null;


        Object string_literal333_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 93) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2853:5: ( conditional_and_expression ( '||' conditional_and_expression )* )
            // com/senseidb/bql/parsers/BQL.g:2853:9: conditional_and_expression ( '||' conditional_and_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_conditional_and_expression_in_conditional_or_expression9217);
            conditional_and_expression332=conditional_and_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, conditional_and_expression332.getTree());

            // com/senseidb/bql/parsers/BQL.g:2853:36: ( '||' conditional_and_expression )*
            loop101:
            do {
                int alt101=2;
                switch ( input.LA(1) ) {
                case 164:
                    {
                    alt101=1;
                    }
                    break;

                }

                switch (alt101) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2853:38: '||' conditional_and_expression
            	    {
            	    string_literal333=(Token)match(input,164,FOLLOW_164_in_conditional_or_expression9221); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    string_literal333_tree = 
            	    (Object)adaptor.create(string_literal333)
            	    ;
            	    adaptor.addChild(root_0, string_literal333_tree);
            	    }

            	    pushFollow(FOLLOW_conditional_and_expression_in_conditional_or_expression9223);
            	    conditional_and_expression334=conditional_and_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, conditional_and_expression334.getTree());

            	    }
            	    break;

            	default :
            	    break loop101;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 93, conditional_or_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "conditional_or_expression"


    public static class conditional_and_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "conditional_and_expression"
    // com/senseidb/bql/parsers/BQL.g:2856:1: conditional_and_expression : inclusive_or_expression ( '&&' inclusive_or_expression )* ;
    public final BQLParser.conditional_and_expression_return conditional_and_expression() throws RecognitionException {
        BQLParser.conditional_and_expression_return retval = new BQLParser.conditional_and_expression_return();
        retval.start = input.LT(1);

        int conditional_and_expression_StartIndex = input.index();

        Object root_0 = null;

        Token string_literal336=null;
        BQLParser.inclusive_or_expression_return inclusive_or_expression335 =null;

        BQLParser.inclusive_or_expression_return inclusive_or_expression337 =null;


        Object string_literal336_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 94) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2857:5: ( inclusive_or_expression ( '&&' inclusive_or_expression )* )
            // com/senseidb/bql/parsers/BQL.g:2857:9: inclusive_or_expression ( '&&' inclusive_or_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_inclusive_or_expression_in_conditional_and_expression9245);
            inclusive_or_expression335=inclusive_or_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, inclusive_or_expression335.getTree());

            // com/senseidb/bql/parsers/BQL.g:2857:33: ( '&&' inclusive_or_expression )*
            loop102:
            do {
                int alt102=2;
                switch ( input.LA(1) ) {
                case 115:
                    {
                    alt102=1;
                    }
                    break;

                }

                switch (alt102) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2857:34: '&&' inclusive_or_expression
            	    {
            	    string_literal336=(Token)match(input,115,FOLLOW_115_in_conditional_and_expression9248); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    string_literal336_tree = 
            	    (Object)adaptor.create(string_literal336)
            	    ;
            	    adaptor.addChild(root_0, string_literal336_tree);
            	    }

            	    pushFollow(FOLLOW_inclusive_or_expression_in_conditional_and_expression9250);
            	    inclusive_or_expression337=inclusive_or_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, inclusive_or_expression337.getTree());

            	    }
            	    break;

            	default :
            	    break loop102;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 94, conditional_and_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "conditional_and_expression"


    public static class inclusive_or_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "inclusive_or_expression"
    // com/senseidb/bql/parsers/BQL.g:2860:1: inclusive_or_expression : exclusive_or_expression ( '|' exclusive_or_expression )* ;
    public final BQLParser.inclusive_or_expression_return inclusive_or_expression() throws RecognitionException {
        BQLParser.inclusive_or_expression_return retval = new BQLParser.inclusive_or_expression_return();
        retval.start = input.LT(1);

        int inclusive_or_expression_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal339=null;
        BQLParser.exclusive_or_expression_return exclusive_or_expression338 =null;

        BQLParser.exclusive_or_expression_return exclusive_or_expression340 =null;


        Object char_literal339_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 95) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2861:5: ( exclusive_or_expression ( '|' exclusive_or_expression )* )
            // com/senseidb/bql/parsers/BQL.g:2861:9: exclusive_or_expression ( '|' exclusive_or_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_exclusive_or_expression_in_inclusive_or_expression9272);
            exclusive_or_expression338=exclusive_or_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, exclusive_or_expression338.getTree());

            // com/senseidb/bql/parsers/BQL.g:2861:33: ( '|' exclusive_or_expression )*
            loop103:
            do {
                int alt103=2;
                switch ( input.LA(1) ) {
                case 162:
                    {
                    alt103=1;
                    }
                    break;

                }

                switch (alt103) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2861:34: '|' exclusive_or_expression
            	    {
            	    char_literal339=(Token)match(input,162,FOLLOW_162_in_inclusive_or_expression9275); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    char_literal339_tree = 
            	    (Object)adaptor.create(char_literal339)
            	    ;
            	    adaptor.addChild(root_0, char_literal339_tree);
            	    }

            	    pushFollow(FOLLOW_exclusive_or_expression_in_inclusive_or_expression9277);
            	    exclusive_or_expression340=exclusive_or_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, exclusive_or_expression340.getTree());

            	    }
            	    break;

            	default :
            	    break loop103;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 95, inclusive_or_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "inclusive_or_expression"


    public static class exclusive_or_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "exclusive_or_expression"
    // com/senseidb/bql/parsers/BQL.g:2864:1: exclusive_or_expression : and_expression ( '^' and_expression )* ;
    public final BQLParser.exclusive_or_expression_return exclusive_or_expression() throws RecognitionException {
        BQLParser.exclusive_or_expression_return retval = new BQLParser.exclusive_or_expression_return();
        retval.start = input.LT(1);

        int exclusive_or_expression_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal342=null;
        BQLParser.and_expression_return and_expression341 =null;

        BQLParser.and_expression_return and_expression343 =null;


        Object char_literal342_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 96) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2865:5: ( and_expression ( '^' and_expression )* )
            // com/senseidb/bql/parsers/BQL.g:2865:9: and_expression ( '^' and_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_and_expression_in_exclusive_or_expression9299);
            and_expression341=and_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, and_expression341.getTree());

            // com/senseidb/bql/parsers/BQL.g:2865:24: ( '^' and_expression )*
            loop104:
            do {
                int alt104=2;
                switch ( input.LA(1) ) {
                case 140:
                    {
                    alt104=1;
                    }
                    break;

                }

                switch (alt104) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2865:25: '^' and_expression
            	    {
            	    char_literal342=(Token)match(input,140,FOLLOW_140_in_exclusive_or_expression9302); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    char_literal342_tree = 
            	    (Object)adaptor.create(char_literal342)
            	    ;
            	    adaptor.addChild(root_0, char_literal342_tree);
            	    }

            	    pushFollow(FOLLOW_and_expression_in_exclusive_or_expression9304);
            	    and_expression343=and_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, and_expression343.getTree());

            	    }
            	    break;

            	default :
            	    break loop104;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 96, exclusive_or_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "exclusive_or_expression"


    public static class and_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "and_expression"
    // com/senseidb/bql/parsers/BQL.g:2868:1: and_expression : equality_expression ( '&' equality_expression )* ;
    public final BQLParser.and_expression_return and_expression() throws RecognitionException {
        BQLParser.and_expression_return retval = new BQLParser.and_expression_return();
        retval.start = input.LT(1);

        int and_expression_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal345=null;
        BQLParser.equality_expression_return equality_expression344 =null;

        BQLParser.equality_expression_return equality_expression346 =null;


        Object char_literal345_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 97) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2869:5: ( equality_expression ( '&' equality_expression )* )
            // com/senseidb/bql/parsers/BQL.g:2869:9: equality_expression ( '&' equality_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_equality_expression_in_and_expression9326);
            equality_expression344=equality_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, equality_expression344.getTree());

            // com/senseidb/bql/parsers/BQL.g:2869:29: ( '&' equality_expression )*
            loop105:
            do {
                int alt105=2;
                switch ( input.LA(1) ) {
                case 116:
                    {
                    alt105=1;
                    }
                    break;

                }

                switch (alt105) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2869:31: '&' equality_expression
            	    {
            	    char_literal345=(Token)match(input,116,FOLLOW_116_in_and_expression9330); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    char_literal345_tree = 
            	    (Object)adaptor.create(char_literal345)
            	    ;
            	    adaptor.addChild(root_0, char_literal345_tree);
            	    }

            	    pushFollow(FOLLOW_equality_expression_in_and_expression9332);
            	    equality_expression346=equality_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, equality_expression346.getTree());

            	    }
            	    break;

            	default :
            	    break loop105;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 97, and_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "and_expression"


    public static class equality_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "equality_expression"
    // com/senseidb/bql/parsers/BQL.g:2872:1: equality_expression : instanceof_expression ( ( '==' | '!=' ) instanceof_expression )* ;
    public final BQLParser.equality_expression_return equality_expression() throws RecognitionException {
        BQLParser.equality_expression_return retval = new BQLParser.equality_expression_return();
        retval.start = input.LT(1);

        int equality_expression_StartIndex = input.index();

        Object root_0 = null;

        Token set348=null;
        BQLParser.instanceof_expression_return instanceof_expression347 =null;

        BQLParser.instanceof_expression_return instanceof_expression349 =null;


        Object set348_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 98) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2873:5: ( instanceof_expression ( ( '==' | '!=' ) instanceof_expression )* )
            // com/senseidb/bql/parsers/BQL.g:2873:9: instanceof_expression ( ( '==' | '!=' ) instanceof_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_instanceof_expression_in_equality_expression9354);
            instanceof_expression347=instanceof_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, instanceof_expression347.getTree());

            // com/senseidb/bql/parsers/BQL.g:2873:31: ( ( '==' | '!=' ) instanceof_expression )*
            loop106:
            do {
                int alt106=2;
                switch ( input.LA(1) ) {
                case 112:
                case 129:
                    {
                    alt106=1;
                    }
                    break;

                }

                switch (alt106) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2873:33: ( '==' | '!=' ) instanceof_expression
            	    {
            	    set348=(Token)input.LT(1);

            	    if ( input.LA(1)==112||input.LA(1)==129 ) {
            	        input.consume();
            	        if ( state.backtracking==0 ) adaptor.addChild(root_0, 
            	        (Object)adaptor.create(set348)
            	        );
            	        state.errorRecovery=false;
            	        state.failed=false;
            	    }
            	    else {
            	        if (state.backtracking>0) {state.failed=true; return retval;}
            	        MismatchedSetException mse = new MismatchedSetException(null,input);
            	        throw mse;
            	    }


            	    pushFollow(FOLLOW_instanceof_expression_in_equality_expression9366);
            	    instanceof_expression349=instanceof_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, instanceof_expression349.getTree());

            	    }
            	    break;

            	default :
            	    break loop106;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 98, equality_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "equality_expression"


    public static class instanceof_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "instanceof_expression"
    // com/senseidb/bql/parsers/BQL.g:2876:1: instanceof_expression : relational_expression ( 'instanceof' type )? ;
    public final BQLParser.instanceof_expression_return instanceof_expression() throws RecognitionException {
        BQLParser.instanceof_expression_return retval = new BQLParser.instanceof_expression_return();
        retval.start = input.LT(1);

        int instanceof_expression_StartIndex = input.index();

        Object root_0 = null;

        Token string_literal351=null;
        BQLParser.relational_expression_return relational_expression350 =null;

        BQLParser.type_return type352 =null;


        Object string_literal351_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 99) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2877:5: ( relational_expression ( 'instanceof' type )? )
            // com/senseidb/bql/parsers/BQL.g:2877:9: relational_expression ( 'instanceof' type )?
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_relational_expression_in_instanceof_expression9388);
            relational_expression350=relational_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, relational_expression350.getTree());

            // com/senseidb/bql/parsers/BQL.g:2877:31: ( 'instanceof' type )?
            int alt107=2;
            switch ( input.LA(1) ) {
                case 154:
                    {
                    alt107=1;
                    }
                    break;
            }

            switch (alt107) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2877:32: 'instanceof' type
                    {
                    string_literal351=(Token)match(input,154,FOLLOW_154_in_instanceof_expression9391); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal351_tree = 
                    (Object)adaptor.create(string_literal351)
                    ;
                    adaptor.addChild(root_0, string_literal351_tree);
                    }

                    pushFollow(FOLLOW_type_in_instanceof_expression9393);
                    type352=type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, type352.getTree());

                    }
                    break;

            }


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 99, instanceof_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "instanceof_expression"


    public static class relational_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "relational_expression"
    // com/senseidb/bql/parsers/BQL.g:2880:1: relational_expression : shift_expression ( relational_op shift_expression )* ;
    public final BQLParser.relational_expression_return relational_expression() throws RecognitionException {
        BQLParser.relational_expression_return retval = new BQLParser.relational_expression_return();
        retval.start = input.LT(1);

        int relational_expression_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.shift_expression_return shift_expression353 =null;

        BQLParser.relational_op_return relational_op354 =null;

        BQLParser.shift_expression_return shift_expression355 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 100) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2881:5: ( shift_expression ( relational_op shift_expression )* )
            // com/senseidb/bql/parsers/BQL.g:2881:9: shift_expression ( relational_op shift_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_shift_expression_in_relational_expression9414);
            shift_expression353=shift_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, shift_expression353.getTree());

            // com/senseidb/bql/parsers/BQL.g:2881:26: ( relational_op shift_expression )*
            loop108:
            do {
                int alt108=2;
                switch ( input.LA(1) ) {
                case LT:
                    {
                    switch ( input.LA(2) ) {
                    case BOOLEAN:
                    case BYTE:
                    case CHARACTER_LITERAL:
                    case DOUBLE:
                    case EQUAL:
                    case FALSE:
                    case FLOATING_POINT_LITERAL:
                    case HEX_LITERAL:
                    case IDENT:
                    case INTEGER:
                    case LONG:
                    case LPAR:
                    case NULL:
                    case OCTAL_LITERAL:
                    case REAL:
                    case STRING_LITERAL:
                    case TRUE:
                    case 111:
                    case 120:
                    case 121:
                    case 123:
                    case 124:
                    case 131:
                    case 132:
                    case 133:
                    case 134:
                    case 135:
                    case 136:
                    case 137:
                    case 166:
                        {
                        alt108=1;
                        }
                        break;

                    }

                    }
                    break;
                case GT:
                    {
                    switch ( input.LA(2) ) {
                    case BOOLEAN:
                    case BYTE:
                    case CHARACTER_LITERAL:
                    case DOUBLE:
                    case EQUAL:
                    case FALSE:
                    case FLOATING_POINT_LITERAL:
                    case HEX_LITERAL:
                    case IDENT:
                    case INTEGER:
                    case LONG:
                    case LPAR:
                    case NULL:
                    case OCTAL_LITERAL:
                    case REAL:
                    case STRING_LITERAL:
                    case TRUE:
                    case 111:
                    case 120:
                    case 121:
                    case 123:
                    case 124:
                    case 131:
                    case 132:
                    case 133:
                    case 134:
                    case 135:
                    case 136:
                    case 137:
                    case 166:
                        {
                        alt108=1;
                        }
                        break;

                    }

                    }
                    break;

                }

                switch (alt108) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2881:28: relational_op shift_expression
            	    {
            	    pushFollow(FOLLOW_relational_op_in_relational_expression9418);
            	    relational_op354=relational_op();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, relational_op354.getTree());

            	    pushFollow(FOLLOW_shift_expression_in_relational_expression9420);
            	    shift_expression355=shift_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, shift_expression355.getTree());

            	    }
            	    break;

            	default :
            	    break loop108;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 100, relational_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "relational_expression"


    public static class relational_op_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "relational_op"
    // com/senseidb/bql/parsers/BQL.g:2884:1: relational_op : ( ( '<' '=' )=>t1= '<' t2= '=' {...}?| ( '>' '=' )=>t1= '>' t2= '=' {...}?| '<' | '>' );
    public final BQLParser.relational_op_return relational_op() throws RecognitionException {
        BQLParser.relational_op_return retval = new BQLParser.relational_op_return();
        retval.start = input.LT(1);

        int relational_op_StartIndex = input.index();

        Object root_0 = null;

        Token t1=null;
        Token t2=null;
        Token char_literal356=null;
        Token char_literal357=null;

        Object t1_tree=null;
        Object t2_tree=null;
        Object char_literal356_tree=null;
        Object char_literal357_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 101) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2885:5: ( ( '<' '=' )=>t1= '<' t2= '=' {...}?| ( '>' '=' )=>t1= '>' t2= '=' {...}?| '<' | '>' )
            int alt109=4;
            switch ( input.LA(1) ) {
            case LT:
                {
                int LA109_1 = input.LA(2);

                if ( (LA109_1==EQUAL) && (synpred179_BQL())) {
                    alt109=1;
                }
                else if ( (LA109_1==BOOLEAN||LA109_1==BYTE||LA109_1==CHARACTER_LITERAL||LA109_1==DOUBLE||LA109_1==FALSE||LA109_1==FLOATING_POINT_LITERAL||LA109_1==HEX_LITERAL||LA109_1==IDENT||LA109_1==INTEGER||(LA109_1 >= LONG && LA109_1 <= LPAR)||LA109_1==NULL||LA109_1==OCTAL_LITERAL||LA109_1==REAL||LA109_1==STRING_LITERAL||LA109_1==TRUE||LA109_1==111||(LA109_1 >= 120 && LA109_1 <= 121)||(LA109_1 >= 123 && LA109_1 <= 124)||(LA109_1 >= 131 && LA109_1 <= 137)||LA109_1==166) ) {
                    alt109=3;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 109, 1, input);

                    throw nvae;

                }
                }
                break;
            case GT:
                {
                int LA109_2 = input.LA(2);

                if ( (LA109_2==EQUAL) && (synpred180_BQL())) {
                    alt109=2;
                }
                else if ( (LA109_2==BOOLEAN||LA109_2==BYTE||LA109_2==CHARACTER_LITERAL||LA109_2==DOUBLE||LA109_2==FALSE||LA109_2==FLOATING_POINT_LITERAL||LA109_2==HEX_LITERAL||LA109_2==IDENT||LA109_2==INTEGER||(LA109_2 >= LONG && LA109_2 <= LPAR)||LA109_2==NULL||LA109_2==OCTAL_LITERAL||LA109_2==REAL||LA109_2==STRING_LITERAL||LA109_2==TRUE||LA109_2==111||(LA109_2 >= 120 && LA109_2 <= 121)||(LA109_2 >= 123 && LA109_2 <= 124)||(LA109_2 >= 131 && LA109_2 <= 137)||LA109_2==166) ) {
                    alt109=4;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 109, 2, input);

                    throw nvae;

                }
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 109, 0, input);

                throw nvae;

            }

            switch (alt109) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2885:9: ( '<' '=' )=>t1= '<' t2= '=' {...}?
                    {
                    root_0 = (Object)adaptor.nil();


                    t1=(Token)match(input,LT,FOLLOW_LT_in_relational_op9451); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t1_tree = 
                    (Object)adaptor.create(t1)
                    ;
                    adaptor.addChild(root_0, t1_tree);
                    }

                    t2=(Token)match(input,EQUAL,FOLLOW_EQUAL_in_relational_op9455); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t2_tree = 
                    (Object)adaptor.create(t2)
                    ;
                    adaptor.addChild(root_0, t2_tree);
                    }

                    if ( !(( t1.getLine() == t2.getLine() && 
                              t1.getCharPositionInLine() + 1 == t2.getCharPositionInLine() )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "relational_op", " $t1.getLine() == $t2.getLine() && \n          $t1.getCharPositionInLine() + 1 == $t2.getCharPositionInLine() ");
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2888:9: ( '>' '=' )=>t1= '>' t2= '=' {...}?
                    {
                    root_0 = (Object)adaptor.nil();


                    t1=(Token)match(input,GT,FOLLOW_GT_in_relational_op9485); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t1_tree = 
                    (Object)adaptor.create(t1)
                    ;
                    adaptor.addChild(root_0, t1_tree);
                    }

                    t2=(Token)match(input,EQUAL,FOLLOW_EQUAL_in_relational_op9489); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t2_tree = 
                    (Object)adaptor.create(t2)
                    ;
                    adaptor.addChild(root_0, t2_tree);
                    }

                    if ( !(( t1.getLine() == t2.getLine() && 
                              t1.getCharPositionInLine() + 1 == t2.getCharPositionInLine() )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "relational_op", " $t1.getLine() == $t2.getLine() && \n          $t1.getCharPositionInLine() + 1 == $t2.getCharPositionInLine() ");
                    }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2891:9: '<'
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal356=(Token)match(input,LT,FOLLOW_LT_in_relational_op9510); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal356_tree = 
                    (Object)adaptor.create(char_literal356)
                    ;
                    adaptor.addChild(root_0, char_literal356_tree);
                    }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:2892:9: '>'
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal357=(Token)match(input,GT,FOLLOW_GT_in_relational_op9520); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal357_tree = 
                    (Object)adaptor.create(char_literal357)
                    ;
                    adaptor.addChild(root_0, char_literal357_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 101, relational_op_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "relational_op"


    public static class shift_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "shift_expression"
    // com/senseidb/bql/parsers/BQL.g:2895:1: shift_expression : additive_expression ( shift_op additive_expression )* ;
    public final BQLParser.shift_expression_return shift_expression() throws RecognitionException {
        BQLParser.shift_expression_return retval = new BQLParser.shift_expression_return();
        retval.start = input.LT(1);

        int shift_expression_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.additive_expression_return additive_expression358 =null;

        BQLParser.shift_op_return shift_op359 =null;

        BQLParser.additive_expression_return additive_expression360 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 102) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2896:5: ( additive_expression ( shift_op additive_expression )* )
            // com/senseidb/bql/parsers/BQL.g:2896:9: additive_expression ( shift_op additive_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_additive_expression_in_shift_expression9539);
            additive_expression358=additive_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, additive_expression358.getTree());

            // com/senseidb/bql/parsers/BQL.g:2896:29: ( shift_op additive_expression )*
            loop110:
            do {
                int alt110=2;
                switch ( input.LA(1) ) {
                case LT:
                    {
                    switch ( input.LA(2) ) {
                    case LT:
                        {
                        switch ( input.LA(3) ) {
                        case BOOLEAN:
                        case BYTE:
                        case CHARACTER_LITERAL:
                        case DOUBLE:
                        case FALSE:
                        case FLOATING_POINT_LITERAL:
                        case HEX_LITERAL:
                        case IDENT:
                        case INTEGER:
                        case LONG:
                        case LPAR:
                        case NULL:
                        case OCTAL_LITERAL:
                        case REAL:
                        case STRING_LITERAL:
                        case TRUE:
                        case 111:
                        case 120:
                        case 121:
                        case 123:
                        case 124:
                        case 131:
                        case 132:
                        case 133:
                        case 134:
                        case 135:
                        case 136:
                        case 137:
                        case 166:
                            {
                            alt110=1;
                            }
                            break;

                        }

                        }
                        break;

                    }

                    }
                    break;
                case GT:
                    {
                    switch ( input.LA(2) ) {
                    case GT:
                        {
                        switch ( input.LA(3) ) {
                        case GT:
                            {
                            switch ( input.LA(4) ) {
                            case BOOLEAN:
                            case BYTE:
                            case CHARACTER_LITERAL:
                            case DOUBLE:
                            case FALSE:
                            case FLOATING_POINT_LITERAL:
                            case HEX_LITERAL:
                            case IDENT:
                            case INTEGER:
                            case LONG:
                            case LPAR:
                            case NULL:
                            case OCTAL_LITERAL:
                            case REAL:
                            case STRING_LITERAL:
                            case TRUE:
                            case 111:
                            case 120:
                            case 121:
                            case 123:
                            case 124:
                            case 131:
                            case 132:
                            case 133:
                            case 134:
                            case 135:
                            case 136:
                            case 137:
                            case 166:
                                {
                                alt110=1;
                                }
                                break;

                            }

                            }
                            break;
                        case BOOLEAN:
                        case BYTE:
                        case CHARACTER_LITERAL:
                        case DOUBLE:
                        case FALSE:
                        case FLOATING_POINT_LITERAL:
                        case HEX_LITERAL:
                        case IDENT:
                        case INTEGER:
                        case LONG:
                        case LPAR:
                        case NULL:
                        case OCTAL_LITERAL:
                        case REAL:
                        case STRING_LITERAL:
                        case TRUE:
                        case 111:
                        case 120:
                        case 121:
                        case 123:
                        case 124:
                        case 131:
                        case 132:
                        case 133:
                        case 134:
                        case 135:
                        case 136:
                        case 137:
                        case 166:
                            {
                            alt110=1;
                            }
                            break;

                        }

                        }
                        break;

                    }

                    }
                    break;

                }

                switch (alt110) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2896:31: shift_op additive_expression
            	    {
            	    pushFollow(FOLLOW_shift_op_in_shift_expression9543);
            	    shift_op359=shift_op();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, shift_op359.getTree());

            	    pushFollow(FOLLOW_additive_expression_in_shift_expression9545);
            	    additive_expression360=additive_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, additive_expression360.getTree());

            	    }
            	    break;

            	default :
            	    break loop110;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 102, shift_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "shift_expression"


    public static class shift_op_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "shift_op"
    // com/senseidb/bql/parsers/BQL.g:2899:1: shift_op : ( ( '<' '<' )=>t1= '<' t2= '<' {...}?| ( '>' '>' '>' )=>t1= '>' t2= '>' t3= '>' {...}?| ( '>' '>' )=>t1= '>' t2= '>' {...}?);
    public final BQLParser.shift_op_return shift_op() throws RecognitionException {
        BQLParser.shift_op_return retval = new BQLParser.shift_op_return();
        retval.start = input.LT(1);

        int shift_op_StartIndex = input.index();

        Object root_0 = null;

        Token t1=null;
        Token t2=null;
        Token t3=null;

        Object t1_tree=null;
        Object t2_tree=null;
        Object t3_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 103) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2900:5: ( ( '<' '<' )=>t1= '<' t2= '<' {...}?| ( '>' '>' '>' )=>t1= '>' t2= '>' t3= '>' {...}?| ( '>' '>' )=>t1= '>' t2= '>' {...}?)
            int alt111=3;
            int LA111_0 = input.LA(1);

            if ( (LA111_0==LT) && (synpred183_BQL())) {
                alt111=1;
            }
            else if ( (LA111_0==GT) ) {
                switch ( input.LA(2) ) {
                case GT:
                    {
                    int LA111_3 = input.LA(3);

                    if ( (LA111_3==GT) && (synpred184_BQL())) {
                        alt111=2;
                    }
                    else if ( (LA111_3==120) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==123) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==121) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==124) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==166) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==111) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==LPAR) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==HEX_LITERAL||LA111_3==INTEGER||LA111_3==OCTAL_LITERAL) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==REAL) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==FLOATING_POINT_LITERAL) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==CHARACTER_LITERAL) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==STRING_LITERAL) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==TRUE) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==FALSE) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==NULL) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==BOOLEAN) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==131) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==BYTE) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==135) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==133) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==LONG) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==132) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==DOUBLE) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==134||(LA111_3 >= 136 && LA111_3 <= 137)) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else if ( (LA111_3==IDENT) && (synpred185_BQL())) {
                        alt111=3;
                    }
                    else {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        NoViableAltException nvae =
                            new NoViableAltException("", 111, 3, input);

                        throw nvae;

                    }
                    }
                    break;
                default:
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 111, 2, input);

                    throw nvae;

                }

            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 111, 0, input);

                throw nvae;

            }
            switch (alt111) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2900:9: ( '<' '<' )=>t1= '<' t2= '<' {...}?
                    {
                    root_0 = (Object)adaptor.nil();


                    t1=(Token)match(input,LT,FOLLOW_LT_in_shift_op9576); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t1_tree = 
                    (Object)adaptor.create(t1)
                    ;
                    adaptor.addChild(root_0, t1_tree);
                    }

                    t2=(Token)match(input,LT,FOLLOW_LT_in_shift_op9580); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t2_tree = 
                    (Object)adaptor.create(t2)
                    ;
                    adaptor.addChild(root_0, t2_tree);
                    }

                    if ( !(( t1.getLine() == t2.getLine() && 
                              t1.getCharPositionInLine() + 1 == t2.getCharPositionInLine() )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "shift_op", " $t1.getLine() == $t2.getLine() && \n          $t1.getCharPositionInLine() + 1 == $t2.getCharPositionInLine() ");
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2903:9: ( '>' '>' '>' )=>t1= '>' t2= '>' t3= '>' {...}?
                    {
                    root_0 = (Object)adaptor.nil();


                    t1=(Token)match(input,GT,FOLLOW_GT_in_shift_op9612); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t1_tree = 
                    (Object)adaptor.create(t1)
                    ;
                    adaptor.addChild(root_0, t1_tree);
                    }

                    t2=(Token)match(input,GT,FOLLOW_GT_in_shift_op9616); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t2_tree = 
                    (Object)adaptor.create(t2)
                    ;
                    adaptor.addChild(root_0, t2_tree);
                    }

                    t3=(Token)match(input,GT,FOLLOW_GT_in_shift_op9620); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t3_tree = 
                    (Object)adaptor.create(t3)
                    ;
                    adaptor.addChild(root_0, t3_tree);
                    }

                    if ( !(( t1.getLine() == t2.getLine() && 
                              t1.getCharPositionInLine() + 1 == t2.getCharPositionInLine() &&
                              t2.getLine() == t3.getLine() && 
                              t2.getCharPositionInLine() + 1 == t3.getCharPositionInLine() )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "shift_op", " $t1.getLine() == $t2.getLine() && \n          $t1.getCharPositionInLine() + 1 == $t2.getCharPositionInLine() &&\n          $t2.getLine() == $t3.getLine() && \n          $t2.getCharPositionInLine() + 1 == $t3.getCharPositionInLine() ");
                    }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2908:9: ( '>' '>' )=>t1= '>' t2= '>' {...}?
                    {
                    root_0 = (Object)adaptor.nil();


                    t1=(Token)match(input,GT,FOLLOW_GT_in_shift_op9650); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t1_tree = 
                    (Object)adaptor.create(t1)
                    ;
                    adaptor.addChild(root_0, t1_tree);
                    }

                    t2=(Token)match(input,GT,FOLLOW_GT_in_shift_op9654); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t2_tree = 
                    (Object)adaptor.create(t2)
                    ;
                    adaptor.addChild(root_0, t2_tree);
                    }

                    if ( !(( t1.getLine() == t2.getLine() && 
                              t1.getCharPositionInLine() + 1 == t2.getCharPositionInLine() )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "shift_op", " $t1.getLine() == $t2.getLine() && \n          $t1.getCharPositionInLine() + 1 == $t2.getCharPositionInLine() ");
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 103, shift_op_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "shift_op"


    public static class additive_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "additive_expression"
    // com/senseidb/bql/parsers/BQL.g:2913:1: additive_expression : multiplicative_expression ( ( '+' | '-' ) multiplicative_expression )* ;
    public final BQLParser.additive_expression_return additive_expression() throws RecognitionException {
        BQLParser.additive_expression_return retval = new BQLParser.additive_expression_return();
        retval.start = input.LT(1);

        int additive_expression_StartIndex = input.index();

        Object root_0 = null;

        Token set362=null;
        BQLParser.multiplicative_expression_return multiplicative_expression361 =null;

        BQLParser.multiplicative_expression_return multiplicative_expression363 =null;


        Object set362_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 104) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2914:5: ( multiplicative_expression ( ( '+' | '-' ) multiplicative_expression )* )
            // com/senseidb/bql/parsers/BQL.g:2914:9: multiplicative_expression ( ( '+' | '-' ) multiplicative_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_multiplicative_expression_in_additive_expression9683);
            multiplicative_expression361=multiplicative_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, multiplicative_expression361.getTree());

            // com/senseidb/bql/parsers/BQL.g:2914:35: ( ( '+' | '-' ) multiplicative_expression )*
            loop112:
            do {
                int alt112=2;
                switch ( input.LA(1) ) {
                case 120:
                case 123:
                    {
                    alt112=1;
                    }
                    break;

                }

                switch (alt112) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2914:37: ( '+' | '-' ) multiplicative_expression
            	    {
            	    set362=(Token)input.LT(1);

            	    if ( input.LA(1)==120||input.LA(1)==123 ) {
            	        input.consume();
            	        if ( state.backtracking==0 ) adaptor.addChild(root_0, 
            	        (Object)adaptor.create(set362)
            	        );
            	        state.errorRecovery=false;
            	        state.failed=false;
            	    }
            	    else {
            	        if (state.backtracking>0) {state.failed=true; return retval;}
            	        MismatchedSetException mse = new MismatchedSetException(null,input);
            	        throw mse;
            	    }


            	    pushFollow(FOLLOW_multiplicative_expression_in_additive_expression9695);
            	    multiplicative_expression363=multiplicative_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, multiplicative_expression363.getTree());

            	    }
            	    break;

            	default :
            	    break loop112;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 104, additive_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "additive_expression"


    public static class multiplicative_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "multiplicative_expression"
    // com/senseidb/bql/parsers/BQL.g:2917:1: multiplicative_expression : unary_expression ( ( '*' | '/' | '%' ) unary_expression )* ;
    public final BQLParser.multiplicative_expression_return multiplicative_expression() throws RecognitionException {
        BQLParser.multiplicative_expression_return retval = new BQLParser.multiplicative_expression_return();
        retval.start = input.LT(1);

        int multiplicative_expression_StartIndex = input.index();

        Object root_0 = null;

        Token set365=null;
        BQLParser.unary_expression_return unary_expression364 =null;

        BQLParser.unary_expression_return unary_expression366 =null;


        Object set365_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 105) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2918:5: ( unary_expression ( ( '*' | '/' | '%' ) unary_expression )* )
            // com/senseidb/bql/parsers/BQL.g:2918:9: unary_expression ( ( '*' | '/' | '%' ) unary_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_unary_expression_in_multiplicative_expression9717);
            unary_expression364=unary_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression364.getTree());

            // com/senseidb/bql/parsers/BQL.g:2918:26: ( ( '*' | '/' | '%' ) unary_expression )*
            loop113:
            do {
                int alt113=2;
                switch ( input.LA(1) ) {
                case 113:
                case 118:
                case 127:
                    {
                    alt113=1;
                    }
                    break;

                }

                switch (alt113) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2918:28: ( '*' | '/' | '%' ) unary_expression
            	    {
            	    set365=(Token)input.LT(1);

            	    if ( input.LA(1)==113||input.LA(1)==118||input.LA(1)==127 ) {
            	        input.consume();
            	        if ( state.backtracking==0 ) adaptor.addChild(root_0, 
            	        (Object)adaptor.create(set365)
            	        );
            	        state.errorRecovery=false;
            	        state.failed=false;
            	    }
            	    else {
            	        if (state.backtracking>0) {state.failed=true; return retval;}
            	        MismatchedSetException mse = new MismatchedSetException(null,input);
            	        throw mse;
            	    }


            	    pushFollow(FOLLOW_unary_expression_in_multiplicative_expression9735);
            	    unary_expression366=unary_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression366.getTree());

            	    }
            	    break;

            	default :
            	    break loop113;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 105, multiplicative_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "multiplicative_expression"


    public static class unary_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "unary_expression"
    // com/senseidb/bql/parsers/BQL.g:2921:1: unary_expression : ( '+' unary_expression | '-' unary_expression | '++' unary_expression | '--' unary_expression | unary_expression_not_plus_minus );
    public final BQLParser.unary_expression_return unary_expression() throws RecognitionException {
        BQLParser.unary_expression_return retval = new BQLParser.unary_expression_return();
        retval.start = input.LT(1);

        int unary_expression_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal367=null;
        Token char_literal369=null;
        Token string_literal371=null;
        Token string_literal373=null;
        BQLParser.unary_expression_return unary_expression368 =null;

        BQLParser.unary_expression_return unary_expression370 =null;

        BQLParser.unary_expression_return unary_expression372 =null;

        BQLParser.unary_expression_return unary_expression374 =null;

        BQLParser.unary_expression_not_plus_minus_return unary_expression_not_plus_minus375 =null;


        Object char_literal367_tree=null;
        Object char_literal369_tree=null;
        Object string_literal371_tree=null;
        Object string_literal373_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 106) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2922:5: ( '+' unary_expression | '-' unary_expression | '++' unary_expression | '--' unary_expression | unary_expression_not_plus_minus )
            int alt114=5;
            switch ( input.LA(1) ) {
            case 120:
                {
                alt114=1;
                }
                break;
            case 123:
                {
                alt114=2;
                }
                break;
            case 121:
                {
                alt114=3;
                }
                break;
            case 124:
                {
                alt114=4;
                }
                break;
            case BOOLEAN:
            case BYTE:
            case CHARACTER_LITERAL:
            case DOUBLE:
            case FALSE:
            case FLOATING_POINT_LITERAL:
            case HEX_LITERAL:
            case IDENT:
            case INTEGER:
            case LONG:
            case LPAR:
            case NULL:
            case OCTAL_LITERAL:
            case REAL:
            case STRING_LITERAL:
            case TRUE:
            case 111:
            case 131:
            case 132:
            case 133:
            case 134:
            case 135:
            case 136:
            case 137:
            case 166:
                {
                alt114=5;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 114, 0, input);

                throw nvae;

            }

            switch (alt114) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2922:9: '+' unary_expression
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal367=(Token)match(input,120,FOLLOW_120_in_unary_expression9761); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal367_tree = 
                    (Object)adaptor.create(char_literal367)
                    ;
                    adaptor.addChild(root_0, char_literal367_tree);
                    }

                    pushFollow(FOLLOW_unary_expression_in_unary_expression9763);
                    unary_expression368=unary_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression368.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2923:9: '-' unary_expression
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal369=(Token)match(input,123,FOLLOW_123_in_unary_expression9773); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal369_tree = 
                    (Object)adaptor.create(char_literal369)
                    ;
                    adaptor.addChild(root_0, char_literal369_tree);
                    }

                    pushFollow(FOLLOW_unary_expression_in_unary_expression9775);
                    unary_expression370=unary_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression370.getTree());

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2924:9: '++' unary_expression
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal371=(Token)match(input,121,FOLLOW_121_in_unary_expression9785); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal371_tree = 
                    (Object)adaptor.create(string_literal371)
                    ;
                    adaptor.addChild(root_0, string_literal371_tree);
                    }

                    pushFollow(FOLLOW_unary_expression_in_unary_expression9787);
                    unary_expression372=unary_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression372.getTree());

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:2925:9: '--' unary_expression
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal373=(Token)match(input,124,FOLLOW_124_in_unary_expression9797); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal373_tree = 
                    (Object)adaptor.create(string_literal373)
                    ;
                    adaptor.addChild(root_0, string_literal373_tree);
                    }

                    pushFollow(FOLLOW_unary_expression_in_unary_expression9799);
                    unary_expression374=unary_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression374.getTree());

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:2926:9: unary_expression_not_plus_minus
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_unary_expression_not_plus_minus_in_unary_expression9809);
                    unary_expression_not_plus_minus375=unary_expression_not_plus_minus();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression_not_plus_minus375.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 106, unary_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "unary_expression"


    public static class unary_expression_not_plus_minus_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "unary_expression_not_plus_minus"
    // com/senseidb/bql/parsers/BQL.g:2929:1: unary_expression_not_plus_minus : ( '~' unary_expression | '!' unary_expression | cast_expression | primary ( selector )* ( '++' | '--' )? );
    public final BQLParser.unary_expression_not_plus_minus_return unary_expression_not_plus_minus() throws RecognitionException {
        BQLParser.unary_expression_not_plus_minus_return retval = new BQLParser.unary_expression_not_plus_minus_return();
        retval.start = input.LT(1);

        int unary_expression_not_plus_minus_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal376=null;
        Token char_literal378=null;
        Token set383=null;
        BQLParser.unary_expression_return unary_expression377 =null;

        BQLParser.unary_expression_return unary_expression379 =null;

        BQLParser.cast_expression_return cast_expression380 =null;

        BQLParser.primary_return primary381 =null;

        BQLParser.selector_return selector382 =null;


        Object char_literal376_tree=null;
        Object char_literal378_tree=null;
        Object set383_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 107) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2930:5: ( '~' unary_expression | '!' unary_expression | cast_expression | primary ( selector )* ( '++' | '--' )? )
            int alt117=4;
            switch ( input.LA(1) ) {
            case 166:
                {
                alt117=1;
                }
                break;
            case 111:
                {
                alt117=2;
                }
                break;
            case LPAR:
                {
                int LA117_3 = input.LA(2);

                if ( (synpred197_BQL()) ) {
                    alt117=3;
                }
                else if ( (true) ) {
                    alt117=4;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 117, 3, input);

                    throw nvae;

                }
                }
                break;
            case BOOLEAN:
            case BYTE:
            case CHARACTER_LITERAL:
            case DOUBLE:
            case FALSE:
            case FLOATING_POINT_LITERAL:
            case HEX_LITERAL:
            case IDENT:
            case INTEGER:
            case LONG:
            case NULL:
            case OCTAL_LITERAL:
            case REAL:
            case STRING_LITERAL:
            case TRUE:
            case 131:
            case 132:
            case 133:
            case 134:
            case 135:
            case 136:
            case 137:
                {
                alt117=4;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 117, 0, input);

                throw nvae;

            }

            switch (alt117) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2930:9: '~' unary_expression
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal376=(Token)match(input,166,FOLLOW_166_in_unary_expression_not_plus_minus9828); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal376_tree = 
                    (Object)adaptor.create(char_literal376)
                    ;
                    adaptor.addChild(root_0, char_literal376_tree);
                    }

                    pushFollow(FOLLOW_unary_expression_in_unary_expression_not_plus_minus9830);
                    unary_expression377=unary_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression377.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2931:9: '!' unary_expression
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal378=(Token)match(input,111,FOLLOW_111_in_unary_expression_not_plus_minus9840); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal378_tree = 
                    (Object)adaptor.create(char_literal378)
                    ;
                    adaptor.addChild(root_0, char_literal378_tree);
                    }

                    pushFollow(FOLLOW_unary_expression_in_unary_expression_not_plus_minus9842);
                    unary_expression379=unary_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression379.getTree());

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2932:9: cast_expression
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_cast_expression_in_unary_expression_not_plus_minus9852);
                    cast_expression380=cast_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, cast_expression380.getTree());

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:2933:9: primary ( selector )* ( '++' | '--' )?
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_primary_in_unary_expression_not_plus_minus9862);
                    primary381=primary();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, primary381.getTree());

                    // com/senseidb/bql/parsers/BQL.g:2933:17: ( selector )*
                    loop115:
                    do {
                        int alt115=2;
                        switch ( input.LA(1) ) {
                        case 126:
                        case 138:
                            {
                            alt115=1;
                            }
                            break;

                        }

                        switch (alt115) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:2933:17: selector
                    	    {
                    	    pushFollow(FOLLOW_selector_in_unary_expression_not_plus_minus9864);
                    	    selector382=selector();

                    	    state._fsp--;
                    	    if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) adaptor.addChild(root_0, selector382.getTree());

                    	    }
                    	    break;

                    	default :
                    	    break loop115;
                        }
                    } while (true);


                    // com/senseidb/bql/parsers/BQL.g:2933:27: ( '++' | '--' )?
                    int alt116=2;
                    switch ( input.LA(1) ) {
                        case 121:
                        case 124:
                            {
                            alt116=1;
                            }
                            break;
                    }

                    switch (alt116) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:
                            {
                            set383=(Token)input.LT(1);

                            if ( input.LA(1)==121||input.LA(1)==124 ) {
                                input.consume();
                                if ( state.backtracking==0 ) adaptor.addChild(root_0, 
                                (Object)adaptor.create(set383)
                                );
                                state.errorRecovery=false;
                                state.failed=false;
                            }
                            else {
                                if (state.backtracking>0) {state.failed=true; return retval;}
                                MismatchedSetException mse = new MismatchedSetException(null,input);
                                throw mse;
                            }


                            }
                            break;

                    }


                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 107, unary_expression_not_plus_minus_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "unary_expression_not_plus_minus"


    public static class cast_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "cast_expression"
    // com/senseidb/bql/parsers/BQL.g:2936:1: cast_expression : ( '(' primitive_type ')' unary_expression | '(' ( type | expression ) ')' unary_expression_not_plus_minus );
    public final BQLParser.cast_expression_return cast_expression() throws RecognitionException {
        BQLParser.cast_expression_return retval = new BQLParser.cast_expression_return();
        retval.start = input.LT(1);

        int cast_expression_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal384=null;
        Token char_literal386=null;
        Token char_literal388=null;
        Token char_literal391=null;
        BQLParser.primitive_type_return primitive_type385 =null;

        BQLParser.unary_expression_return unary_expression387 =null;

        BQLParser.type_return type389 =null;

        BQLParser.expression_return expression390 =null;

        BQLParser.unary_expression_not_plus_minus_return unary_expression_not_plus_minus392 =null;


        Object char_literal384_tree=null;
        Object char_literal386_tree=null;
        Object char_literal388_tree=null;
        Object char_literal391_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 108) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2937:5: ( '(' primitive_type ')' unary_expression | '(' ( type | expression ) ')' unary_expression_not_plus_minus )
            int alt119=2;
            switch ( input.LA(1) ) {
            case LPAR:
                {
                int LA119_1 = input.LA(2);

                if ( (synpred201_BQL()) ) {
                    alt119=1;
                }
                else if ( (true) ) {
                    alt119=2;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 119, 1, input);

                    throw nvae;

                }
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 119, 0, input);

                throw nvae;

            }

            switch (alt119) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2937:8: '(' primitive_type ')' unary_expression
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal384=(Token)match(input,LPAR,FOLLOW_LPAR_in_cast_expression9890); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal384_tree = 
                    (Object)adaptor.create(char_literal384)
                    ;
                    adaptor.addChild(root_0, char_literal384_tree);
                    }

                    pushFollow(FOLLOW_primitive_type_in_cast_expression9892);
                    primitive_type385=primitive_type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, primitive_type385.getTree());

                    char_literal386=(Token)match(input,RPAR,FOLLOW_RPAR_in_cast_expression9894); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal386_tree = 
                    (Object)adaptor.create(char_literal386)
                    ;
                    adaptor.addChild(root_0, char_literal386_tree);
                    }

                    pushFollow(FOLLOW_unary_expression_in_cast_expression9896);
                    unary_expression387=unary_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression387.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2938:8: '(' ( type | expression ) ')' unary_expression_not_plus_minus
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal388=(Token)match(input,LPAR,FOLLOW_LPAR_in_cast_expression9905); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal388_tree = 
                    (Object)adaptor.create(char_literal388)
                    ;
                    adaptor.addChild(root_0, char_literal388_tree);
                    }

                    // com/senseidb/bql/parsers/BQL.g:2938:12: ( type | expression )
                    int alt118=2;
                    switch ( input.LA(1) ) {
                    case FAST_UTIL_DATA_TYPE:
                    case INT:
                    case 144:
                    case 151:
                    case 156:
                        {
                        alt118=1;
                        }
                        break;
                    case BOOLEAN:
                        {
                        int LA118_2 = input.LA(2);

                        if ( (((synpred202_BQL()&&synpred202_BQL())&&(( "Boolean".equals(input.LT(1).getText()) )||( "boolean".equals(input.LT(1).getText()) )))) ) {
                            alt118=1;
                        }
                        else if ( (( "Boolean".equals(input.LT(1).getText()) )) ) {
                            alt118=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 118, 2, input);

                            throw nvae;

                        }
                        }
                        break;
                    case BYTE:
                        {
                        int LA118_3 = input.LA(2);

                        if ( (((synpred202_BQL()&&synpred202_BQL())&&(( "Byte".equals(input.LT(1).getText()) )||( "byte".equals(input.LT(1).getText()) )))) ) {
                            alt118=1;
                        }
                        else if ( (( "Byte".equals(input.LT(1).getText()) )) ) {
                            alt118=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 118, 3, input);

                            throw nvae;

                        }
                        }
                        break;
                    case LONG:
                        {
                        int LA118_4 = input.LA(2);

                        if ( ((((( "long".equals(input.LT(1).getText()) )||( "Long".equals(input.LT(1).getText()) ))&&(( "long".equals(input.LT(1).getText()) )||( "Long".equals(input.LT(1).getText()) )))&&synpred202_BQL())) ) {
                            alt118=1;
                        }
                        else if ( (( "Long".equals(input.LT(1).getText()) )) ) {
                            alt118=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 118, 4, input);

                            throw nvae;

                        }
                        }
                        break;
                    case DOUBLE:
                        {
                        int LA118_5 = input.LA(2);

                        if ( ((((( "double".equals(input.LT(1).getText()) )||( "Double".equals(input.LT(1).getText()) ))&&(( "double".equals(input.LT(1).getText()) )||( "Double".equals(input.LT(1).getText()) )))&&synpred202_BQL())) ) {
                            alt118=1;
                        }
                        else if ( (( "Double".equals(input.LT(1).getText()) )) ) {
                            alt118=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 118, 5, input);

                            throw nvae;

                        }
                        }
                        break;
                    case 131:
                        {
                        int LA118_6 = input.LA(2);

                        if ( (synpred202_BQL()) ) {
                            alt118=1;
                        }
                        else if ( (true) ) {
                            alt118=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 118, 6, input);

                            throw nvae;

                        }
                        }
                        break;
                    case 135:
                        {
                        int LA118_7 = input.LA(2);

                        if ( (synpred202_BQL()) ) {
                            alt118=1;
                        }
                        else if ( (true) ) {
                            alt118=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 118, 7, input);

                            throw nvae;

                        }
                        }
                        break;
                    case 133:
                        {
                        int LA118_8 = input.LA(2);

                        if ( (synpred202_BQL()) ) {
                            alt118=1;
                        }
                        else if ( (true) ) {
                            alt118=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 118, 8, input);

                            throw nvae;

                        }
                        }
                        break;
                    case 132:
                        {
                        int LA118_9 = input.LA(2);

                        if ( (synpred202_BQL()) ) {
                            alt118=1;
                        }
                        else if ( (true) ) {
                            alt118=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 118, 9, input);

                            throw nvae;

                        }
                        }
                        break;
                    case 134:
                    case 136:
                    case 137:
                        {
                        int LA118_10 = input.LA(2);

                        if ( (synpred202_BQL()) ) {
                            alt118=1;
                        }
                        else if ( (true) ) {
                            alt118=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 118, 10, input);

                            throw nvae;

                        }
                        }
                        break;
                    case CHARACTER_LITERAL:
                    case FALSE:
                    case FLOATING_POINT_LITERAL:
                    case HEX_LITERAL:
                    case IDENT:
                    case INTEGER:
                    case LPAR:
                    case NULL:
                    case OCTAL_LITERAL:
                    case REAL:
                    case STRING_LITERAL:
                    case TRUE:
                    case 111:
                    case 120:
                    case 121:
                    case 123:
                    case 124:
                    case 166:
                        {
                        alt118=2;
                        }
                        break;
                    default:
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        NoViableAltException nvae =
                            new NoViableAltException("", 118, 0, input);

                        throw nvae;

                    }

                    switch (alt118) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2938:13: type
                            {
                            pushFollow(FOLLOW_type_in_cast_expression9908);
                            type389=type();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) adaptor.addChild(root_0, type389.getTree());

                            }
                            break;
                        case 2 :
                            // com/senseidb/bql/parsers/BQL.g:2938:20: expression
                            {
                            pushFollow(FOLLOW_expression_in_cast_expression9912);
                            expression390=expression();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression390.getTree());

                            }
                            break;

                    }


                    char_literal391=(Token)match(input,RPAR,FOLLOW_RPAR_in_cast_expression9915); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal391_tree = 
                    (Object)adaptor.create(char_literal391)
                    ;
                    adaptor.addChild(root_0, char_literal391_tree);
                    }

                    pushFollow(FOLLOW_unary_expression_not_plus_minus_in_cast_expression9917);
                    unary_expression_not_plus_minus392=unary_expression_not_plus_minus();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression_not_plus_minus392.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 108, cast_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "cast_expression"


    public static class primary_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "primary"
    // com/senseidb/bql/parsers/BQL.g:2941:1: primary : ( par_expression | literal | java_ident ( '.' java_method )* ( identifier_suffix )? );
    public final BQLParser.primary_return primary() throws RecognitionException {
        BQLParser.primary_return retval = new BQLParser.primary_return();
        retval.start = input.LT(1);

        int primary_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal396=null;
        BQLParser.par_expression_return par_expression393 =null;

        BQLParser.literal_return literal394 =null;

        BQLParser.java_ident_return java_ident395 =null;

        BQLParser.java_method_return java_method397 =null;

        BQLParser.identifier_suffix_return identifier_suffix398 =null;


        Object char_literal396_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 109) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2942:5: ( par_expression | literal | java_ident ( '.' java_method )* ( identifier_suffix )? )
            int alt122=3;
            switch ( input.LA(1) ) {
            case LPAR:
                {
                alt122=1;
                }
                break;
            case CHARACTER_LITERAL:
            case FALSE:
            case FLOATING_POINT_LITERAL:
            case HEX_LITERAL:
            case INTEGER:
            case NULL:
            case OCTAL_LITERAL:
            case REAL:
            case STRING_LITERAL:
            case TRUE:
                {
                alt122=2;
                }
                break;
            case BOOLEAN:
            case BYTE:
            case DOUBLE:
            case IDENT:
            case LONG:
            case 131:
            case 132:
            case 133:
            case 134:
            case 135:
            case 136:
            case 137:
                {
                alt122=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 122, 0, input);

                throw nvae;

            }

            switch (alt122) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2942:9: par_expression
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_par_expression_in_primary9936);
                    par_expression393=par_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, par_expression393.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2943:9: literal
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_literal_in_primary9946);
                    literal394=literal();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, literal394.getTree());

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2944:9: java_ident ( '.' java_method )* ( identifier_suffix )?
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_java_ident_in_primary9956);
                    java_ident395=java_ident();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, java_ident395.getTree());

                    // com/senseidb/bql/parsers/BQL.g:2944:20: ( '.' java_method )*
                    loop120:
                    do {
                        int alt120=2;
                        switch ( input.LA(1) ) {
                        case 126:
                            {
                            switch ( input.LA(2) ) {
                            case IDENT:
                                {
                                int LA120_3 = input.LA(3);

                                if ( (synpred205_BQL()) ) {
                                    alt120=1;
                                }


                                }
                                break;
                            case CONTAINS:
                                {
                                alt120=1;
                                }
                                break;

                            }

                            }
                            break;

                        }

                        switch (alt120) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:2944:21: '.' java_method
                    	    {
                    	    char_literal396=(Token)match(input,126,FOLLOW_126_in_primary9959); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal396_tree = 
                    	    (Object)adaptor.create(char_literal396)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal396_tree);
                    	    }

                    	    pushFollow(FOLLOW_java_method_in_primary9961);
                    	    java_method397=java_method();

                    	    state._fsp--;
                    	    if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) adaptor.addChild(root_0, java_method397.getTree());

                    	    }
                    	    break;

                    	default :
                    	    break loop120;
                        }
                    } while (true);


                    // com/senseidb/bql/parsers/BQL.g:2944:39: ( identifier_suffix )?
                    int alt121=2;
                    switch ( input.LA(1) ) {
                        case 138:
                            {
                            switch ( input.LA(2) ) {
                                case 139:
                                    {
                                    alt121=1;
                                    }
                                    break;
                            }

                            }
                            break;
                        case LPAR:
                            {
                            alt121=1;
                            }
                            break;
                        case 126:
                            {
                            switch ( input.LA(2) ) {
                                case 145:
                                case 157:
                                    {
                                    alt121=1;
                                    }
                                    break;
                                case 159:
                                    {
                                    int LA121_5 = input.LA(3);

                                    if ( (synpred206_BQL()) ) {
                                        alt121=1;
                                    }
                                    }
                                    break;
                            }

                            }
                            break;
                    }

                    switch (alt121) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2944:39: identifier_suffix
                            {
                            pushFollow(FOLLOW_identifier_suffix_in_primary9965);
                            identifier_suffix398=identifier_suffix();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) adaptor.addChild(root_0, identifier_suffix398.getTree());

                            }
                            break;

                    }


                    if ( state.backtracking==0 ) {
                                String var = (java_ident395!=null?input.toString(java_ident395.start,java_ident395.stop):null);
                                if (_facetInfoMap.containsKey(var)) {
                                    _usedFacets.add(var);
                                }
                                else if (_internalVarMap.containsKey(var)) {
                                    _usedInternalVars.add(var);
                                }
                                else if (!_supportedClasses.contains(var) && !verifyVariable(var)) {
                                    throw new FailedPredicateException(input,
                                                                       "primary",
                                                                       "Variable or class \"" + var + "\" is not defined.");
                                }
                            }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 109, primary_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "primary"


    public static class java_ident_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "java_ident"
    // com/senseidb/bql/parsers/BQL.g:2961:1: java_ident : ( boxed_type | limited_type | IDENT );
    public final BQLParser.java_ident_return java_ident() throws RecognitionException {
        BQLParser.java_ident_return retval = new BQLParser.java_ident_return();
        retval.start = input.LT(1);

        int java_ident_StartIndex = input.index();

        Object root_0 = null;

        Token IDENT401=null;
        BQLParser.boxed_type_return boxed_type399 =null;

        BQLParser.limited_type_return limited_type400 =null;


        Object IDENT401_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 110) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2962:5: ( boxed_type | limited_type | IDENT )
            int alt123=3;
            switch ( input.LA(1) ) {
            case BOOLEAN:
            case BYTE:
            case DOUBLE:
            case LONG:
            case 131:
            case 132:
            case 133:
            case 135:
                {
                alt123=1;
                }
                break;
            case 134:
            case 136:
            case 137:
                {
                alt123=2;
                }
                break;
            case IDENT:
                {
                alt123=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 123, 0, input);

                throw nvae;

            }

            switch (alt123) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2962:9: boxed_type
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_boxed_type_in_java_ident9995);
                    boxed_type399=boxed_type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, boxed_type399.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2963:9: limited_type
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_limited_type_in_java_ident10005);
                    limited_type400=limited_type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, limited_type400.getTree());

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2964:9: IDENT
                    {
                    root_0 = (Object)adaptor.nil();


                    IDENT401=(Token)match(input,IDENT,FOLLOW_IDENT_in_java_ident10015); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    IDENT401_tree = 
                    (Object)adaptor.create(IDENT401)
                    ;
                    adaptor.addChild(root_0, IDENT401_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 110, java_ident_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "java_ident"


    public static class java_method_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "java_method"
    // com/senseidb/bql/parsers/BQL.g:2969:1: java_method : ({...}? CONTAINS | IDENT );
    public final BQLParser.java_method_return java_method() throws RecognitionException {
        BQLParser.java_method_return retval = new BQLParser.java_method_return();
        retval.start = input.LT(1);

        int java_method_StartIndex = input.index();

        Object root_0 = null;

        Token CONTAINS402=null;
        Token IDENT403=null;

        Object CONTAINS402_tree=null;
        Object IDENT403_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 111) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2970:5: ({...}? CONTAINS | IDENT )
            int alt124=2;
            switch ( input.LA(1) ) {
            case CONTAINS:
                {
                alt124=1;
                }
                break;
            case IDENT:
                {
                alt124=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 124, 0, input);

                throw nvae;

            }

            switch (alt124) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2970:9: {...}? CONTAINS
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "contains".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "java_method", " \"contains\".equals(input.LT(1).getText()) ");
                    }

                    CONTAINS402=(Token)match(input,CONTAINS,FOLLOW_CONTAINS_in_java_method10038); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    CONTAINS402_tree = 
                    (Object)adaptor.create(CONTAINS402)
                    ;
                    adaptor.addChild(root_0, CONTAINS402_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2971:9: IDENT
                    {
                    root_0 = (Object)adaptor.nil();


                    IDENT403=(Token)match(input,IDENT,FOLLOW_IDENT_in_java_method10048); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    IDENT403_tree = 
                    (Object)adaptor.create(IDENT403)
                    ;
                    adaptor.addChild(root_0, IDENT403_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 111, java_method_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "java_method"


    public static class identifier_suffix_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "identifier_suffix"
    // com/senseidb/bql/parsers/BQL.g:2974:1: identifier_suffix : ( ( '[' ']' )+ '.' 'class' | arguments | '.' 'class' | '.' 'this' | '.' 'super' arguments );
    public final BQLParser.identifier_suffix_return identifier_suffix() throws RecognitionException {
        BQLParser.identifier_suffix_return retval = new BQLParser.identifier_suffix_return();
        retval.start = input.LT(1);

        int identifier_suffix_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal404=null;
        Token char_literal405=null;
        Token char_literal406=null;
        Token string_literal407=null;
        Token char_literal409=null;
        Token string_literal410=null;
        Token char_literal411=null;
        Token string_literal412=null;
        Token char_literal413=null;
        Token string_literal414=null;
        BQLParser.arguments_return arguments408 =null;

        BQLParser.arguments_return arguments415 =null;


        Object char_literal404_tree=null;
        Object char_literal405_tree=null;
        Object char_literal406_tree=null;
        Object string_literal407_tree=null;
        Object char_literal409_tree=null;
        Object string_literal410_tree=null;
        Object char_literal411_tree=null;
        Object string_literal412_tree=null;
        Object char_literal413_tree=null;
        Object string_literal414_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 112) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2975:5: ( ( '[' ']' )+ '.' 'class' | arguments | '.' 'class' | '.' 'this' | '.' 'super' arguments )
            int alt126=5;
            switch ( input.LA(1) ) {
            case 138:
                {
                alt126=1;
                }
                break;
            case LPAR:
                {
                alt126=2;
                }
                break;
            case 126:
                {
                switch ( input.LA(2) ) {
                case 145:
                    {
                    alt126=3;
                    }
                    break;
                case 159:
                    {
                    alt126=4;
                    }
                    break;
                case 157:
                    {
                    alt126=5;
                    }
                    break;
                default:
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 126, 3, input);

                    throw nvae;

                }

                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 126, 0, input);

                throw nvae;

            }

            switch (alt126) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2975:9: ( '[' ']' )+ '.' 'class'
                    {
                    root_0 = (Object)adaptor.nil();


                    // com/senseidb/bql/parsers/BQL.g:2975:9: ( '[' ']' )+
                    int cnt125=0;
                    loop125:
                    do {
                        int alt125=2;
                        switch ( input.LA(1) ) {
                        case 138:
                            {
                            alt125=1;
                            }
                            break;

                        }

                        switch (alt125) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:2975:10: '[' ']'
                    	    {
                    	    char_literal404=(Token)match(input,138,FOLLOW_138_in_identifier_suffix10068); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal404_tree = 
                    	    (Object)adaptor.create(char_literal404)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal404_tree);
                    	    }

                    	    char_literal405=(Token)match(input,139,FOLLOW_139_in_identifier_suffix10070); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal405_tree = 
                    	    (Object)adaptor.create(char_literal405)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal405_tree);
                    	    }

                    	    }
                    	    break;

                    	default :
                    	    if ( cnt125 >= 1 ) break loop125;
                    	    if (state.backtracking>0) {state.failed=true; return retval;}
                                EarlyExitException eee =
                                    new EarlyExitException(125, input);
                                throw eee;
                        }
                        cnt125++;
                    } while (true);


                    char_literal406=(Token)match(input,126,FOLLOW_126_in_identifier_suffix10074); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal406_tree = 
                    (Object)adaptor.create(char_literal406)
                    ;
                    adaptor.addChild(root_0, char_literal406_tree);
                    }

                    string_literal407=(Token)match(input,145,FOLLOW_145_in_identifier_suffix10076); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal407_tree = 
                    (Object)adaptor.create(string_literal407)
                    ;
                    adaptor.addChild(root_0, string_literal407_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2976:9: arguments
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_arguments_in_identifier_suffix10086);
                    arguments408=arguments();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, arguments408.getTree());

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2977:9: '.' 'class'
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal409=(Token)match(input,126,FOLLOW_126_in_identifier_suffix10096); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal409_tree = 
                    (Object)adaptor.create(char_literal409)
                    ;
                    adaptor.addChild(root_0, char_literal409_tree);
                    }

                    string_literal410=(Token)match(input,145,FOLLOW_145_in_identifier_suffix10098); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal410_tree = 
                    (Object)adaptor.create(string_literal410)
                    ;
                    adaptor.addChild(root_0, string_literal410_tree);
                    }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:2978:9: '.' 'this'
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal411=(Token)match(input,126,FOLLOW_126_in_identifier_suffix10108); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal411_tree = 
                    (Object)adaptor.create(char_literal411)
                    ;
                    adaptor.addChild(root_0, char_literal411_tree);
                    }

                    string_literal412=(Token)match(input,159,FOLLOW_159_in_identifier_suffix10110); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal412_tree = 
                    (Object)adaptor.create(string_literal412)
                    ;
                    adaptor.addChild(root_0, string_literal412_tree);
                    }

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:2979:9: '.' 'super' arguments
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal413=(Token)match(input,126,FOLLOW_126_in_identifier_suffix10120); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal413_tree = 
                    (Object)adaptor.create(char_literal413)
                    ;
                    adaptor.addChild(root_0, char_literal413_tree);
                    }

                    string_literal414=(Token)match(input,157,FOLLOW_157_in_identifier_suffix10122); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal414_tree = 
                    (Object)adaptor.create(string_literal414)
                    ;
                    adaptor.addChild(root_0, string_literal414_tree);
                    }

                    pushFollow(FOLLOW_arguments_in_identifier_suffix10124);
                    arguments415=arguments();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, arguments415.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 112, identifier_suffix_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "identifier_suffix"


    public static class literal_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "literal"
    // com/senseidb/bql/parsers/BQL.g:2982:1: literal : ( integer_literal | REAL | FLOATING_POINT_LITERAL | CHARACTER_LITERAL | STRING_LITERAL | boolean_literal |{...}? NULL );
    public final BQLParser.literal_return literal() throws RecognitionException {
        BQLParser.literal_return retval = new BQLParser.literal_return();
        retval.start = input.LT(1);

        int literal_StartIndex = input.index();

        Object root_0 = null;

        Token REAL417=null;
        Token FLOATING_POINT_LITERAL418=null;
        Token CHARACTER_LITERAL419=null;
        Token STRING_LITERAL420=null;
        Token NULL422=null;
        BQLParser.integer_literal_return integer_literal416 =null;

        BQLParser.boolean_literal_return boolean_literal421 =null;


        Object REAL417_tree=null;
        Object FLOATING_POINT_LITERAL418_tree=null;
        Object CHARACTER_LITERAL419_tree=null;
        Object STRING_LITERAL420_tree=null;
        Object NULL422_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 113) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2983:5: ( integer_literal | REAL | FLOATING_POINT_LITERAL | CHARACTER_LITERAL | STRING_LITERAL | boolean_literal |{...}? NULL )
            int alt127=7;
            switch ( input.LA(1) ) {
            case HEX_LITERAL:
            case INTEGER:
            case OCTAL_LITERAL:
                {
                alt127=1;
                }
                break;
            case REAL:
                {
                alt127=2;
                }
                break;
            case FLOATING_POINT_LITERAL:
                {
                alt127=3;
                }
                break;
            case CHARACTER_LITERAL:
                {
                alt127=4;
                }
                break;
            case STRING_LITERAL:
                {
                alt127=5;
                }
                break;
            case FALSE:
            case TRUE:
                {
                alt127=6;
                }
                break;
            case NULL:
                {
                alt127=7;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 127, 0, input);

                throw nvae;

            }

            switch (alt127) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2983:9: integer_literal
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_integer_literal_in_literal10144);
                    integer_literal416=integer_literal();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, integer_literal416.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2984:9: REAL
                    {
                    root_0 = (Object)adaptor.nil();


                    REAL417=(Token)match(input,REAL,FOLLOW_REAL_in_literal10154); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    REAL417_tree = 
                    (Object)adaptor.create(REAL417)
                    ;
                    adaptor.addChild(root_0, REAL417_tree);
                    }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2985:9: FLOATING_POINT_LITERAL
                    {
                    root_0 = (Object)adaptor.nil();


                    FLOATING_POINT_LITERAL418=(Token)match(input,FLOATING_POINT_LITERAL,FOLLOW_FLOATING_POINT_LITERAL_in_literal10164); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    FLOATING_POINT_LITERAL418_tree = 
                    (Object)adaptor.create(FLOATING_POINT_LITERAL418)
                    ;
                    adaptor.addChild(root_0, FLOATING_POINT_LITERAL418_tree);
                    }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:2986:9: CHARACTER_LITERAL
                    {
                    root_0 = (Object)adaptor.nil();


                    CHARACTER_LITERAL419=(Token)match(input,CHARACTER_LITERAL,FOLLOW_CHARACTER_LITERAL_in_literal10174); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    CHARACTER_LITERAL419_tree = 
                    (Object)adaptor.create(CHARACTER_LITERAL419)
                    ;
                    adaptor.addChild(root_0, CHARACTER_LITERAL419_tree);
                    }

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:2987:9: STRING_LITERAL
                    {
                    root_0 = (Object)adaptor.nil();


                    STRING_LITERAL420=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_literal10184); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    STRING_LITERAL420_tree = 
                    (Object)adaptor.create(STRING_LITERAL420)
                    ;
                    adaptor.addChild(root_0, STRING_LITERAL420_tree);
                    }

                    }
                    break;
                case 6 :
                    // com/senseidb/bql/parsers/BQL.g:2988:9: boolean_literal
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_boolean_literal_in_literal10194);
                    boolean_literal421=boolean_literal();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, boolean_literal421.getTree());

                    }
                    break;
                case 7 :
                    // com/senseidb/bql/parsers/BQL.g:2989:9: {...}? NULL
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "null".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "literal", " \"null\".equals(input.LT(1).getText()) ");
                    }

                    NULL422=(Token)match(input,NULL,FOLLOW_NULL_in_literal10206); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    NULL422_tree = 
                    (Object)adaptor.create(NULL422)
                    ;
                    adaptor.addChild(root_0, NULL422_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 113, literal_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "literal"


    public static class integer_literal_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "integer_literal"
    // com/senseidb/bql/parsers/BQL.g:2992:1: integer_literal : ( HEX_LITERAL | OCTAL_LITERAL | INTEGER );
    public final BQLParser.integer_literal_return integer_literal() throws RecognitionException {
        BQLParser.integer_literal_return retval = new BQLParser.integer_literal_return();
        retval.start = input.LT(1);

        int integer_literal_StartIndex = input.index();

        Object root_0 = null;

        Token set423=null;

        Object set423_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 114) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2993:5: ( HEX_LITERAL | OCTAL_LITERAL | INTEGER )
            // com/senseidb/bql/parsers/BQL.g:
            {
            root_0 = (Object)adaptor.nil();


            set423=(Token)input.LT(1);

            if ( input.LA(1)==HEX_LITERAL||input.LA(1)==INTEGER||input.LA(1)==OCTAL_LITERAL ) {
                input.consume();
                if ( state.backtracking==0 ) adaptor.addChild(root_0, 
                (Object)adaptor.create(set423)
                );
                state.errorRecovery=false;
                state.failed=false;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                MismatchedSetException mse = new MismatchedSetException(null,input);
                throw mse;
            }


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 114, integer_literal_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "integer_literal"


    public static class boolean_literal_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "boolean_literal"
    // com/senseidb/bql/parsers/BQL.g:2998:1: boolean_literal : ({...}? TRUE |{...}? FALSE );
    public final BQLParser.boolean_literal_return boolean_literal() throws RecognitionException {
        BQLParser.boolean_literal_return retval = new BQLParser.boolean_literal_return();
        retval.start = input.LT(1);

        int boolean_literal_StartIndex = input.index();

        Object root_0 = null;

        Token TRUE424=null;
        Token FALSE425=null;

        Object TRUE424_tree=null;
        Object FALSE425_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 115) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2999:5: ({...}? TRUE |{...}? FALSE )
            int alt128=2;
            switch ( input.LA(1) ) {
            case TRUE:
                {
                alt128=1;
                }
                break;
            case FALSE:
                {
                alt128=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 128, 0, input);

                throw nvae;

            }

            switch (alt128) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2999:9: {...}? TRUE
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "true".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "boolean_literal", " \"true\".equals(input.LT(1).getText()) ");
                    }

                    TRUE424=(Token)match(input,TRUE,FOLLOW_TRUE_in_boolean_literal10266); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    TRUE424_tree = 
                    (Object)adaptor.create(TRUE424)
                    ;
                    adaptor.addChild(root_0, TRUE424_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3000:9: {...}? FALSE
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "false".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "boolean_literal", " \"false\".equals(input.LT(1).getText()) ");
                    }

                    FALSE425=(Token)match(input,FALSE,FOLLOW_FALSE_in_boolean_literal10278); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    FALSE425_tree = 
                    (Object)adaptor.create(FALSE425)
                    ;
                    adaptor.addChild(root_0, FALSE425_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 115, boolean_literal_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "boolean_literal"


    public static class selector_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "selector"
    // com/senseidb/bql/parsers/BQL.g:3003:1: selector : ( '.' IDENT ( arguments )? | '.' 'this' | '[' expression ']' );
    public final BQLParser.selector_return selector() throws RecognitionException {
        BQLParser.selector_return retval = new BQLParser.selector_return();
        retval.start = input.LT(1);

        int selector_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal426=null;
        Token IDENT427=null;
        Token char_literal429=null;
        Token string_literal430=null;
        Token char_literal431=null;
        Token char_literal433=null;
        BQLParser.arguments_return arguments428 =null;

        BQLParser.expression_return expression432 =null;


        Object char_literal426_tree=null;
        Object IDENT427_tree=null;
        Object char_literal429_tree=null;
        Object string_literal430_tree=null;
        Object char_literal431_tree=null;
        Object char_literal433_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 116) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3004:5: ( '.' IDENT ( arguments )? | '.' 'this' | '[' expression ']' )
            int alt130=3;
            switch ( input.LA(1) ) {
            case 126:
                {
                switch ( input.LA(2) ) {
                case IDENT:
                    {
                    alt130=1;
                    }
                    break;
                case 159:
                    {
                    alt130=2;
                    }
                    break;
                default:
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 130, 1, input);

                    throw nvae;

                }

                }
                break;
            case 138:
                {
                alt130=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 130, 0, input);

                throw nvae;

            }

            switch (alt130) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3004:9: '.' IDENT ( arguments )?
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal426=(Token)match(input,126,FOLLOW_126_in_selector10297); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal426_tree = 
                    (Object)adaptor.create(char_literal426)
                    ;
                    adaptor.addChild(root_0, char_literal426_tree);
                    }

                    IDENT427=(Token)match(input,IDENT,FOLLOW_IDENT_in_selector10299); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    IDENT427_tree = 
                    (Object)adaptor.create(IDENT427)
                    ;
                    adaptor.addChild(root_0, IDENT427_tree);
                    }

                    // com/senseidb/bql/parsers/BQL.g:3004:19: ( arguments )?
                    int alt129=2;
                    switch ( input.LA(1) ) {
                        case LPAR:
                            {
                            alt129=1;
                            }
                            break;
                    }

                    switch (alt129) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:3004:19: arguments
                            {
                            pushFollow(FOLLOW_arguments_in_selector10301);
                            arguments428=arguments();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) adaptor.addChild(root_0, arguments428.getTree());

                            }
                            break;

                    }


                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3005:9: '.' 'this'
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal429=(Token)match(input,126,FOLLOW_126_in_selector10312); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal429_tree = 
                    (Object)adaptor.create(char_literal429)
                    ;
                    adaptor.addChild(root_0, char_literal429_tree);
                    }

                    string_literal430=(Token)match(input,159,FOLLOW_159_in_selector10314); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal430_tree = 
                    (Object)adaptor.create(string_literal430)
                    ;
                    adaptor.addChild(root_0, string_literal430_tree);
                    }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:3006:9: '[' expression ']'
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal431=(Token)match(input,138,FOLLOW_138_in_selector10324); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal431_tree = 
                    (Object)adaptor.create(char_literal431)
                    ;
                    adaptor.addChild(root_0, char_literal431_tree);
                    }

                    pushFollow(FOLLOW_expression_in_selector10326);
                    expression432=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression432.getTree());

                    char_literal433=(Token)match(input,139,FOLLOW_139_in_selector10328); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal433_tree = 
                    (Object)adaptor.create(char_literal433)
                    ;
                    adaptor.addChild(root_0, char_literal433_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 116, selector_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "selector"


    public static class arguments_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "arguments"
    // com/senseidb/bql/parsers/BQL.g:3009:1: arguments : '(' ( expression_list )? ')' ;
    public final BQLParser.arguments_return arguments() throws RecognitionException {
        BQLParser.arguments_return retval = new BQLParser.arguments_return();
        retval.start = input.LT(1);

        int arguments_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal434=null;
        Token char_literal436=null;
        BQLParser.expression_list_return expression_list435 =null;


        Object char_literal434_tree=null;
        Object char_literal436_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 117) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3010:5: ( '(' ( expression_list )? ')' )
            // com/senseidb/bql/parsers/BQL.g:3010:9: '(' ( expression_list )? ')'
            {
            root_0 = (Object)adaptor.nil();


            char_literal434=(Token)match(input,LPAR,FOLLOW_LPAR_in_arguments10347); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal434_tree = 
            (Object)adaptor.create(char_literal434)
            ;
            adaptor.addChild(root_0, char_literal434_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:3010:13: ( expression_list )?
            int alt131=2;
            switch ( input.LA(1) ) {
                case BOOLEAN:
                case BYTE:
                case CHARACTER_LITERAL:
                case DOUBLE:
                case FALSE:
                case FLOATING_POINT_LITERAL:
                case HEX_LITERAL:
                case IDENT:
                case INTEGER:
                case LONG:
                case LPAR:
                case NULL:
                case OCTAL_LITERAL:
                case REAL:
                case STRING_LITERAL:
                case TRUE:
                case 111:
                case 120:
                case 121:
                case 123:
                case 124:
                case 131:
                case 132:
                case 133:
                case 134:
                case 135:
                case 136:
                case 137:
                case 166:
                    {
                    alt131=1;
                    }
                    break;
            }

            switch (alt131) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3010:13: expression_list
                    {
                    pushFollow(FOLLOW_expression_list_in_arguments10349);
                    expression_list435=expression_list();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression_list435.getTree());

                    }
                    break;

            }


            char_literal436=(Token)match(input,RPAR,FOLLOW_RPAR_in_arguments10352); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal436_tree = 
            (Object)adaptor.create(char_literal436)
            ;
            adaptor.addChild(root_0, char_literal436_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 117, arguments_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "arguments"


    public static class relevance_model_clause_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "relevance_model_clause"
    // com/senseidb/bql/parsers/BQL.g:3013:1: relevance_model_clause returns [JSONObject json] : USING RELEVANCE MODEL IDENT prop_list[false] (model= relevance_model )? ;
    public final BQLParser.relevance_model_clause_return relevance_model_clause() throws RecognitionException {
        BQLParser.relevance_model_clause_return retval = new BQLParser.relevance_model_clause_return();
        retval.start = input.LT(1);

        int relevance_model_clause_StartIndex = input.index();

        Object root_0 = null;

        Token USING437=null;
        Token RELEVANCE438=null;
        Token MODEL439=null;
        Token IDENT440=null;
        BQLParser.relevance_model_return model =null;

        BQLParser.prop_list_return prop_list441 =null;


        Object USING437_tree=null;
        Object RELEVANCE438_tree=null;
        Object MODEL439_tree=null;
        Object IDENT440_tree=null;


            retval.json = new JSONObject();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 118) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3017:5: ( USING RELEVANCE MODEL IDENT prop_list[false] (model= relevance_model )? )
            // com/senseidb/bql/parsers/BQL.g:3017:9: USING RELEVANCE MODEL IDENT prop_list[false] (model= relevance_model )?
            {
            root_0 = (Object)adaptor.nil();


            USING437=(Token)match(input,USING,FOLLOW_USING_in_relevance_model_clause10384); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            USING437_tree = 
            (Object)adaptor.create(USING437)
            ;
            adaptor.addChild(root_0, USING437_tree);
            }

            RELEVANCE438=(Token)match(input,RELEVANCE,FOLLOW_RELEVANCE_in_relevance_model_clause10386); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            RELEVANCE438_tree = 
            (Object)adaptor.create(RELEVANCE438)
            ;
            adaptor.addChild(root_0, RELEVANCE438_tree);
            }

            MODEL439=(Token)match(input,MODEL,FOLLOW_MODEL_in_relevance_model_clause10388); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            MODEL439_tree = 
            (Object)adaptor.create(MODEL439)
            ;
            adaptor.addChild(root_0, MODEL439_tree);
            }

            IDENT440=(Token)match(input,IDENT,FOLLOW_IDENT_in_relevance_model_clause10390); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            IDENT440_tree = 
            (Object)adaptor.create(IDENT440)
            ;
            adaptor.addChild(root_0, IDENT440_tree);
            }

            pushFollow(FOLLOW_prop_list_in_relevance_model_clause10392);
            prop_list441=prop_list(false);

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, prop_list441.getTree());

            // com/senseidb/bql/parsers/BQL.g:3017:59: (model= relevance_model )?
            int alt132=2;
            switch ( input.LA(1) ) {
                case DEFINED:
                    {
                    alt132=1;
                    }
                    break;
            }

            switch (alt132) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3017:59: model= relevance_model
                    {
                    pushFollow(FOLLOW_relevance_model_in_relevance_model_clause10397);
                    model=relevance_model();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, model.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        try {
                            if (model == null) {
                                retval.json.put("predefined_model", (IDENT440!=null?IDENT440.getText():null));
                                retval.json.put("values", (prop_list441!=null?prop_list441.json:null));
                            }
                            else {
                                JSONObject modelInfo = (model!=null?model.json:null);
                                JSONObject modelJson = new JSONObject();
                                modelJson.put("function", (model!=null?model.functionBody:null));

                                JSONArray funcParams = modelInfo.optJSONArray("function_params");
                                if (funcParams != null) {
                                    modelJson.put("function_params", funcParams);
                                }

                                JSONObject facets = modelInfo.optJSONObject("facets");
                                if (facets != null) {
                                    modelJson.put("facets", facets);
                                }

                                JSONObject variables = modelInfo.optJSONObject("variables");
                                if (variables != null) {
                                    modelJson.put("variables", variables);
                                }

                                retval.json.put("model", modelJson);
                                retval.json.put("values", (prop_list441!=null?prop_list441.json:null));
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "relevance_model_clause",
                                                               "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 118, relevance_model_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "relevance_model_clause"


    public static class facet_param_list_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "facet_param_list"
    // com/senseidb/bql/parsers/BQL.g:3055:1: facet_param_list returns [JSONObject json] : p= facet_param ( COMMA p= facet_param )* ;
    public final BQLParser.facet_param_list_return facet_param_list() throws RecognitionException {
        BQLParser.facet_param_list_return retval = new BQLParser.facet_param_list_return();
        retval.start = input.LT(1);

        int facet_param_list_StartIndex = input.index();

        Object root_0 = null;

        Token COMMA442=null;
        BQLParser.facet_param_return p =null;


        Object COMMA442_tree=null;


            retval.json = new JSONObject();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 119) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3059:5: (p= facet_param ( COMMA p= facet_param )* )
            // com/senseidb/bql/parsers/BQL.g:3059:9: p= facet_param ( COMMA p= facet_param )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_facet_param_in_facet_param_list10438);
            p=facet_param();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, p.getTree());

            if ( state.backtracking==0 ) {
                        try {
                            if (!retval.json.has((p!=null?p.facet:null))) {
                                retval.json.put((p!=null?p.facet:null), (p!=null?p.param:null));
                            }
                            else {
                                JSONObject currentParam = (JSONObject) retval.json.get((p!=null?p.facet:null));
                                String paramName = (String) (p!=null?p.param:null).keys().next();
                                currentParam.put(paramName, (p!=null?p.param:null).get(paramName));
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "facet_param_list", "JSONException: " + err.getMessage());
                        }
                    }

            // com/senseidb/bql/parsers/BQL.g:3075:9: ( COMMA p= facet_param )*
            loop133:
            do {
                int alt133=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt133=1;
                    }
                    break;

                }

                switch (alt133) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:3075:10: COMMA p= facet_param
            	    {
            	    COMMA442=(Token)match(input,COMMA,FOLLOW_COMMA_in_facet_param_list10459); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA442_tree = 
            	    (Object)adaptor.create(COMMA442)
            	    ;
            	    adaptor.addChild(root_0, COMMA442_tree);
            	    }

            	    pushFollow(FOLLOW_facet_param_in_facet_param_list10463);
            	    p=facet_param();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, p.getTree());

            	    if ( state.backtracking==0 ) {
            	                    try {
            	                        if (!retval.json.has((p!=null?p.facet:null))) {
            	                            retval.json.put((p!=null?p.facet:null), (p!=null?p.param:null));
            	                        }
            	                        else {
            	                            JSONObject currentParam = (JSONObject) retval.json.get((p!=null?p.facet:null));
            	                            String paramName = (String) (p!=null?p.param:null).keys().next();
            	                            currentParam.put(paramName, (p!=null?p.param:null).get(paramName));
            	                        }
            	                    }
            	                    catch (JSONException err) {
            	                        throw new FailedPredicateException(input, "facet_param_list", "JSONException: " + err.getMessage());
            	                    }
            	                }

            	    }
            	    break;

            	default :
            	    break loop133;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 119, facet_param_list_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "facet_param_list"


    public static class facet_param_return extends ParserRuleReturnScope {
        public String facet;
        public JSONObject param;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "facet_param"
    // com/senseidb/bql/parsers/BQL.g:3094:1: facet_param returns [String facet, JSONObject param] : LPAR column_name COMMA STRING_LITERAL COMMA facet_param_type COMMA (val= value |valList= non_variable_value_list ) RPAR ;
    public final BQLParser.facet_param_return facet_param() throws RecognitionException {
        BQLParser.facet_param_return retval = new BQLParser.facet_param_return();
        retval.start = input.LT(1);

        int facet_param_StartIndex = input.index();

        Object root_0 = null;

        Token LPAR443=null;
        Token COMMA445=null;
        Token STRING_LITERAL446=null;
        Token COMMA447=null;
        Token COMMA449=null;
        Token RPAR450=null;
        BQLParser.value_return val =null;

        BQLParser.non_variable_value_list_return valList =null;

        BQLParser.column_name_return column_name444 =null;

        BQLParser.facet_param_type_return facet_param_type448 =null;


        Object LPAR443_tree=null;
        Object COMMA445_tree=null;
        Object STRING_LITERAL446_tree=null;
        Object COMMA447_tree=null;
        Object COMMA449_tree=null;
        Object RPAR450_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 120) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3095:5: ( LPAR column_name COMMA STRING_LITERAL COMMA facet_param_type COMMA (val= value |valList= non_variable_value_list ) RPAR )
            // com/senseidb/bql/parsers/BQL.g:3095:9: LPAR column_name COMMA STRING_LITERAL COMMA facet_param_type COMMA (val= value |valList= non_variable_value_list ) RPAR
            {
            root_0 = (Object)adaptor.nil();


            LPAR443=(Token)match(input,LPAR,FOLLOW_LPAR_in_facet_param10511); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LPAR443_tree = 
            (Object)adaptor.create(LPAR443)
            ;
            adaptor.addChild(root_0, LPAR443_tree);
            }

            pushFollow(FOLLOW_column_name_in_facet_param10513);
            column_name444=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, column_name444.getTree());

            COMMA445=(Token)match(input,COMMA,FOLLOW_COMMA_in_facet_param10515); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            COMMA445_tree = 
            (Object)adaptor.create(COMMA445)
            ;
            adaptor.addChild(root_0, COMMA445_tree);
            }

            STRING_LITERAL446=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_facet_param10517); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            STRING_LITERAL446_tree = 
            (Object)adaptor.create(STRING_LITERAL446)
            ;
            adaptor.addChild(root_0, STRING_LITERAL446_tree);
            }

            COMMA447=(Token)match(input,COMMA,FOLLOW_COMMA_in_facet_param10519); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            COMMA447_tree = 
            (Object)adaptor.create(COMMA447)
            ;
            adaptor.addChild(root_0, COMMA447_tree);
            }

            pushFollow(FOLLOW_facet_param_type_in_facet_param10521);
            facet_param_type448=facet_param_type();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, facet_param_type448.getTree());

            COMMA449=(Token)match(input,COMMA,FOLLOW_COMMA_in_facet_param10523); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            COMMA449_tree = 
            (Object)adaptor.create(COMMA449)
            ;
            adaptor.addChild(root_0, COMMA449_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:3095:76: (val= value |valList= non_variable_value_list )
            int alt134=2;
            switch ( input.LA(1) ) {
            case EOF:
            case AGO:
            case AND:
            case BROWSE:
            case DATE:
            case FALSE:
            case FETCHING:
            case GIVEN:
            case GROUP:
            case INTEGER:
            case LIMIT:
            case NOW:
            case OR:
            case ORDER:
            case REAL:
            case ROUTE:
            case RPAR:
            case SEMI:
            case STRING_LITERAL:
            case TRUE:
            case USING:
            case VARIABLE:
                {
                alt134=1;
                }
                break;
            case LPAR:
                {
                alt134=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 134, 0, input);

                throw nvae;

            }

            switch (alt134) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3095:77: val= value
                    {
                    pushFollow(FOLLOW_value_in_facet_param10528);
                    val=value();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, val.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3095:89: valList= non_variable_value_list
                    {
                    pushFollow(FOLLOW_non_variable_value_list_in_facet_param10534);
                    valList=non_variable_value_list();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, valList.getTree());

                    }
                    break;

            }


            RPAR450=(Token)match(input,RPAR,FOLLOW_RPAR_in_facet_param10537); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            RPAR450_tree = 
            (Object)adaptor.create(RPAR450)
            ;
            adaptor.addChild(root_0, RPAR450_tree);
            }

            if ( state.backtracking==0 ) {
                        retval.facet = (column_name444!=null?column_name444.text:null); // XXX Check error here?
                        try {
                            Object valArray = null;
                            if (val != null) {
                                String varName = (val!=null?input.toString(val.start,val.stop):null);
                                if (varName.matches("\\$[^$].*")) {
                                    // Here "value" is a variable.  In this case, it
                                    // is REQUIRED that the variable should be
                                    // replaced by a list, NOT a scalar value.
                                    valArray = varName;
                                }
                                else {
                                    valArray = new JSONArray().put(val.val);
                                }
                            }
                            else {
                                valArray = (valList!=null?valList.json:null);
                            }

                            String orig = (STRING_LITERAL446!=null?STRING_LITERAL446.getText():null);
                            orig = orig.substring(1, orig.length() - 1);
                            retval.param = new JSONObject().put(orig,
                                                          new JSONObject().put("type", (facet_param_type448!=null?facet_param_type448.paramType:null))
                                                                          .put("values", valArray));
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "facet_param", "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 120, facet_param_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "facet_param"


    public static class facet_param_type_return extends ParserRuleReturnScope {
        public String paramType;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "facet_param_type"
    // com/senseidb/bql/parsers/BQL.g:3128:1: facet_param_type returns [String paramType] : (t= BOOLEAN |t= INT |t= LONG |t= STRING |t= BYTEARRAY |t= DOUBLE ) ;
    public final BQLParser.facet_param_type_return facet_param_type() throws RecognitionException {
        BQLParser.facet_param_type_return retval = new BQLParser.facet_param_type_return();
        retval.start = input.LT(1);

        int facet_param_type_StartIndex = input.index();

        Object root_0 = null;

        Token t=null;

        Object t_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 121) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3129:5: ( (t= BOOLEAN |t= INT |t= LONG |t= STRING |t= BYTEARRAY |t= DOUBLE ) )
            // com/senseidb/bql/parsers/BQL.g:3129:9: (t= BOOLEAN |t= INT |t= LONG |t= STRING |t= BYTEARRAY |t= DOUBLE )
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:3129:9: (t= BOOLEAN |t= INT |t= LONG |t= STRING |t= BYTEARRAY |t= DOUBLE )
            int alt135=6;
            switch ( input.LA(1) ) {
            case BOOLEAN:
                {
                alt135=1;
                }
                break;
            case INT:
                {
                alt135=2;
                }
                break;
            case LONG:
                {
                alt135=3;
                }
                break;
            case STRING:
                {
                alt135=4;
                }
                break;
            case BYTEARRAY:
                {
                alt135=5;
                }
                break;
            case DOUBLE:
                {
                alt135=6;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 135, 0, input);

                throw nvae;

            }

            switch (alt135) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3129:10: t= BOOLEAN
                    {
                    t=(Token)match(input,BOOLEAN,FOLLOW_BOOLEAN_in_facet_param_type10606); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t_tree = 
                    (Object)adaptor.create(t)
                    ;
                    adaptor.addChild(root_0, t_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3129:22: t= INT
                    {
                    t=(Token)match(input,INT,FOLLOW_INT_in_facet_param_type10612); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t_tree = 
                    (Object)adaptor.create(t)
                    ;
                    adaptor.addChild(root_0, t_tree);
                    }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:3129:30: t= LONG
                    {
                    t=(Token)match(input,LONG,FOLLOW_LONG_in_facet_param_type10618); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t_tree = 
                    (Object)adaptor.create(t)
                    ;
                    adaptor.addChild(root_0, t_tree);
                    }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:3129:39: t= STRING
                    {
                    t=(Token)match(input,STRING,FOLLOW_STRING_in_facet_param_type10624); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t_tree = 
                    (Object)adaptor.create(t)
                    ;
                    adaptor.addChild(root_0, t_tree);
                    }

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:3129:50: t= BYTEARRAY
                    {
                    t=(Token)match(input,BYTEARRAY,FOLLOW_BYTEARRAY_in_facet_param_type10630); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t_tree = 
                    (Object)adaptor.create(t)
                    ;
                    adaptor.addChild(root_0, t_tree);
                    }

                    }
                    break;
                case 6 :
                    // com/senseidb/bql/parsers/BQL.g:3129:64: t= DOUBLE
                    {
                    t=(Token)match(input,DOUBLE,FOLLOW_DOUBLE_in_facet_param_type10636); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t_tree = 
                    (Object)adaptor.create(t)
                    ;
                    adaptor.addChild(root_0, t_tree);
                    }

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        retval.paramType = (t!=null?t.getText():null);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 121, facet_param_type_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "facet_param_type"

    // $ANTLR start synpred90_BQL
    public final void synpred90_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2418:11: ({...}? => STRING_LITERAL )
        // com/senseidb/bql/parsers/BQL.g:2418:11: {...}? => STRING_LITERAL
        {
        if ( !((((key_value_pair_scope)key_value_pair_stack.peek()).needString)) ) {
            if (state.backtracking>0) {state.failed=true; return ;}
            throw new FailedPredicateException(input, "synpred90_BQL", "$key_value_pair::needString");
        }

        match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_synpred90_BQL7075); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred90_BQL

    // $ANTLR start synpred103_BQL
    public final void synpred103_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2495:9: ( primitive_type ( '[' ']' )* )
        // com/senseidb/bql/parsers/BQL.g:2495:9: primitive_type ( '[' ']' )*
        {
        pushFollow(FOLLOW_primitive_type_in_synpred103_BQL7459);
        primitive_type();

        state._fsp--;
        if (state.failed) return ;

        // com/senseidb/bql/parsers/BQL.g:2495:24: ( '[' ']' )*
        loop141:
        do {
            int alt141=2;
            switch ( input.LA(1) ) {
            case 138:
                {
                alt141=1;
                }
                break;

            }

            switch (alt141) {
        	case 1 :
        	    // com/senseidb/bql/parsers/BQL.g:2495:25: '[' ']'
        	    {
        	    match(input,138,FOLLOW_138_in_synpred103_BQL7462); if (state.failed) return ;

        	    match(input,139,FOLLOW_139_in_synpred103_BQL7464); if (state.failed) return ;

        	    }
        	    break;

        	default :
        	    break loop141;
            }
        } while (true);


        }

    }
    // $ANTLR end synpred103_BQL

    // $ANTLR start synpred105_BQL
    public final void synpred105_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2497:9: ( boxed_type ( '[' ']' )* )
        // com/senseidb/bql/parsers/BQL.g:2497:9: boxed_type ( '[' ']' )*
        {
        pushFollow(FOLLOW_boxed_type_in_synpred105_BQL7486);
        boxed_type();

        state._fsp--;
        if (state.failed) return ;

        // com/senseidb/bql/parsers/BQL.g:2497:20: ( '[' ']' )*
        loop142:
        do {
            int alt142=2;
            switch ( input.LA(1) ) {
            case 138:
                {
                alt142=1;
                }
                break;

            }

            switch (alt142) {
        	case 1 :
        	    // com/senseidb/bql/parsers/BQL.g:2497:21: '[' ']'
        	    {
        	    match(input,138,FOLLOW_138_in_synpred105_BQL7489); if (state.failed) return ;

        	    match(input,139,FOLLOW_139_in_synpred105_BQL7491); if (state.failed) return ;

        	    }
        	    break;

        	default :
        	    break loop142;
            }
        } while (true);


        }

    }
    // $ANTLR end synpred105_BQL

    // $ANTLR start synpred133_BQL
    public final void synpred133_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2737:45: ( else_statement )
        // com/senseidb/bql/parsers/BQL.g:2737:45: else_statement
        {
        pushFollow(FOLLOW_else_statement_in_synpred133_BQL8408);
        else_statement();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred133_BQL

    // $ANTLR start synpred146_BQL
    public final void synpred146_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2767:9: ( switch_label )
        // com/senseidb/bql/parsers/BQL.g:2767:9: switch_label
        {
        pushFollow(FOLLOW_switch_label_in_synpred146_BQL8636);
        switch_label();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred146_BQL

    // $ANTLR start synpred148_BQL
    public final void synpred148_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2771:9: ( 'case' constant_expression COLON )
        // com/senseidb/bql/parsers/BQL.g:2771:9: 'case' constant_expression COLON
        {
        match(input,143,FOLLOW_143_in_synpred148_BQL8659); if (state.failed) return ;

        pushFollow(FOLLOW_constant_expression_in_synpred148_BQL8661);
        constant_expression();

        state._fsp--;
        if (state.failed) return ;

        match(input,COLON,FOLLOW_COLON_in_synpred148_BQL8663); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred148_BQL

    // $ANTLR start synpred149_BQL
    public final void synpred149_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2772:9: ( 'case' enum_constant_name COLON )
        // com/senseidb/bql/parsers/BQL.g:2772:9: 'case' enum_constant_name COLON
        {
        match(input,143,FOLLOW_143_in_synpred149_BQL8673); if (state.failed) return ;

        pushFollow(FOLLOW_enum_constant_name_in_synpred149_BQL8675);
        enum_constant_name();

        state._fsp--;
        if (state.failed) return ;

        match(input,COLON,FOLLOW_COLON_in_synpred149_BQL8677); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred149_BQL

    // $ANTLR start synpred150_BQL
    public final void synpred150_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2778:9: ( enhanced_for_control )
        // com/senseidb/bql/parsers/BQL.g:2778:9: enhanced_for_control
        {
        pushFollow(FOLLOW_enhanced_for_control_in_synpred150_BQL8715);
        enhanced_for_control();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred150_BQL

    // $ANTLR start synpred156_BQL
    public final void synpred156_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2816:33: ( assignment_operator expression )
        // com/senseidb/bql/parsers/BQL.g:2816:33: assignment_operator expression
        {
        pushFollow(FOLLOW_assignment_operator_in_synpred156_BQL8939);
        assignment_operator();

        state._fsp--;
        if (state.failed) return ;

        pushFollow(FOLLOW_expression_in_synpred156_BQL8941);
        expression();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred156_BQL

    // $ANTLR start synpred166_BQL
    public final void synpred166_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2829:9: ( '<' '<' '=' )
        // com/senseidb/bql/parsers/BQL.g:2829:10: '<' '<' '='
        {
        match(input,LT,FOLLOW_LT_in_synpred166_BQL9053); if (state.failed) return ;

        match(input,LT,FOLLOW_LT_in_synpred166_BQL9055); if (state.failed) return ;

        match(input,EQUAL,FOLLOW_EQUAL_in_synpred166_BQL9057); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred166_BQL

    // $ANTLR start synpred167_BQL
    public final void synpred167_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2834:9: ( '>' '>' '>' '=' )
        // com/senseidb/bql/parsers/BQL.g:2834:10: '>' '>' '>' '='
        {
        match(input,GT,FOLLOW_GT_in_synpred167_BQL9093); if (state.failed) return ;

        match(input,GT,FOLLOW_GT_in_synpred167_BQL9095); if (state.failed) return ;

        match(input,GT,FOLLOW_GT_in_synpred167_BQL9097); if (state.failed) return ;

        match(input,EQUAL,FOLLOW_EQUAL_in_synpred167_BQL9099); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred167_BQL

    // $ANTLR start synpred168_BQL
    public final void synpred168_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2841:9: ( '>' '>' '=' )
        // com/senseidb/bql/parsers/BQL.g:2841:10: '>' '>' '='
        {
        match(input,GT,FOLLOW_GT_in_synpred168_BQL9138); if (state.failed) return ;

        match(input,GT,FOLLOW_GT_in_synpred168_BQL9140); if (state.failed) return ;

        match(input,EQUAL,FOLLOW_EQUAL_in_synpred168_BQL9142); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred168_BQL

    // $ANTLR start synpred179_BQL
    public final void synpred179_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2885:9: ( '<' '=' )
        // com/senseidb/bql/parsers/BQL.g:2885:10: '<' '='
        {
        match(input,LT,FOLLOW_LT_in_synpred179_BQL9443); if (state.failed) return ;

        match(input,EQUAL,FOLLOW_EQUAL_in_synpred179_BQL9445); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred179_BQL

    // $ANTLR start synpred180_BQL
    public final void synpred180_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2888:9: ( '>' '=' )
        // com/senseidb/bql/parsers/BQL.g:2888:10: '>' '='
        {
        match(input,GT,FOLLOW_GT_in_synpred180_BQL9477); if (state.failed) return ;

        match(input,EQUAL,FOLLOW_EQUAL_in_synpred180_BQL9479); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred180_BQL

    // $ANTLR start synpred183_BQL
    public final void synpred183_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2900:9: ( '<' '<' )
        // com/senseidb/bql/parsers/BQL.g:2900:10: '<' '<'
        {
        match(input,LT,FOLLOW_LT_in_synpred183_BQL9568); if (state.failed) return ;

        match(input,LT,FOLLOW_LT_in_synpred183_BQL9570); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred183_BQL

    // $ANTLR start synpred184_BQL
    public final void synpred184_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2903:9: ( '>' '>' '>' )
        // com/senseidb/bql/parsers/BQL.g:2903:10: '>' '>' '>'
        {
        match(input,GT,FOLLOW_GT_in_synpred184_BQL9602); if (state.failed) return ;

        match(input,GT,FOLLOW_GT_in_synpred184_BQL9604); if (state.failed) return ;

        match(input,GT,FOLLOW_GT_in_synpred184_BQL9606); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred184_BQL

    // $ANTLR start synpred185_BQL
    public final void synpred185_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2908:9: ( '>' '>' )
        // com/senseidb/bql/parsers/BQL.g:2908:10: '>' '>'
        {
        match(input,GT,FOLLOW_GT_in_synpred185_BQL9642); if (state.failed) return ;

        match(input,GT,FOLLOW_GT_in_synpred185_BQL9644); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred185_BQL

    // $ANTLR start synpred197_BQL
    public final void synpred197_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2932:9: ( cast_expression )
        // com/senseidb/bql/parsers/BQL.g:2932:9: cast_expression
        {
        pushFollow(FOLLOW_cast_expression_in_synpred197_BQL9852);
        cast_expression();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred197_BQL

    // $ANTLR start synpred201_BQL
    public final void synpred201_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2937:8: ( '(' primitive_type ')' unary_expression )
        // com/senseidb/bql/parsers/BQL.g:2937:8: '(' primitive_type ')' unary_expression
        {
        match(input,LPAR,FOLLOW_LPAR_in_synpred201_BQL9890); if (state.failed) return ;

        pushFollow(FOLLOW_primitive_type_in_synpred201_BQL9892);
        primitive_type();

        state._fsp--;
        if (state.failed) return ;

        match(input,RPAR,FOLLOW_RPAR_in_synpred201_BQL9894); if (state.failed) return ;

        pushFollow(FOLLOW_unary_expression_in_synpred201_BQL9896);
        unary_expression();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred201_BQL

    // $ANTLR start synpred202_BQL
    public final void synpred202_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2938:13: ( type )
        // com/senseidb/bql/parsers/BQL.g:2938:13: type
        {
        pushFollow(FOLLOW_type_in_synpred202_BQL9908);
        type();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred202_BQL

    // $ANTLR start synpred205_BQL
    public final void synpred205_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2944:21: ( '.' java_method )
        // com/senseidb/bql/parsers/BQL.g:2944:21: '.' java_method
        {
        match(input,126,FOLLOW_126_in_synpred205_BQL9959); if (state.failed) return ;

        pushFollow(FOLLOW_java_method_in_synpred205_BQL9961);
        java_method();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred205_BQL

    // $ANTLR start synpred206_BQL
    public final void synpred206_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2944:39: ( identifier_suffix )
        // com/senseidb/bql/parsers/BQL.g:2944:39: identifier_suffix
        {
        pushFollow(FOLLOW_identifier_suffix_in_synpred206_BQL9965);
        identifier_suffix();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred206_BQL

    // Delegated rules

    public final boolean synpred133_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred133_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred166_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred166_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred179_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred179_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred180_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred180_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred185_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred185_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred205_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred205_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred202_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred202_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred197_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred197_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred90_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred90_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred184_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred184_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred146_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred146_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred103_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred103_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred156_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred156_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred167_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred167_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred105_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred105_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred168_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred168_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred183_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred183_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred201_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred201_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred148_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred148_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred150_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred150_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred206_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred206_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred149_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred149_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }


    protected DFA27 dfa27 = new DFA27(this);
    protected DFA40 dfa40 = new DFA40(this);
    protected DFA82 dfa82 = new DFA82(this);
    protected DFA95 dfa95 = new DFA95(this);
    protected DFA96 dfa96 = new DFA96(this);
    static final String DFA27_eotS =
        "\23\uffff";
    static final String DFA27_eofS =
        "\23\uffff";
    static final String DFA27_minS =
        "\1\70\2\4\2\uffff\1\70\1\4\1\76\10\uffff\2\4\1\uffff";
    static final String DFA27_maxS =
        "\1\143\2\176\2\uffff\1\143\1\140\1\152\10\uffff\2\176\1\uffff";
    static final String DFA27_acceptS =
        "\3\uffff\1\5\1\11\3\uffff\1\2\1\3\1\4\1\6\1\7\1\10\1\12\1\13\2\uffff"+
        "\1\1";
    static final String DFA27_specialS =
        "\23\uffff}>";
    static final String[] DFA27_transitionS = {
            "\1\1\15\uffff\1\4\5\uffff\1\4\11\uffff\1\3\14\uffff\1\2",
            "\1\15\10\uffff\1\15\1\uffff\1\13\12\uffff\1\10\11\uffff\1\11"+
            "\15\uffff\2\14\5\uffff\1\7\3\uffff\1\17\1\uffff\1\16\4\uffff"+
            "\2\14\6\uffff\1\6\1\12\22\uffff\1\15\35\uffff\1\5",
            "\1\15\10\uffff\1\15\1\uffff\1\13\12\uffff\1\10\11\uffff\1\11"+
            "\15\uffff\2\14\5\uffff\1\7\3\uffff\1\17\1\uffff\1\16\4\uffff"+
            "\2\14\6\uffff\1\6\1\12\22\uffff\1\15\35\uffff\1\5",
            "",
            "",
            "\1\20\52\uffff\1\21",
            "\1\15\10\uffff\1\15\1\uffff\1\13\51\uffff\1\7\5\uffff\1\16"+
            "\40\uffff\1\15",
            "\1\15\4\uffff\1\22\46\uffff\1\22",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\15\10\uffff\1\15\1\uffff\1\13\12\uffff\1\10\11\uffff\1\11"+
            "\15\uffff\2\14\5\uffff\1\7\3\uffff\1\17\1\uffff\1\16\4\uffff"+
            "\2\14\6\uffff\1\6\1\12\22\uffff\1\15\35\uffff\1\5",
            "\1\15\10\uffff\1\15\1\uffff\1\13\12\uffff\1\10\11\uffff\1\11"+
            "\15\uffff\2\14\5\uffff\1\7\3\uffff\1\17\1\uffff\1\16\4\uffff"+
            "\2\14\6\uffff\1\6\1\12\22\uffff\1\15\35\uffff\1\5",
            ""
    };

    static final short[] DFA27_eot = DFA.unpackEncodedString(DFA27_eotS);
    static final short[] DFA27_eof = DFA.unpackEncodedString(DFA27_eofS);
    static final char[] DFA27_min = DFA.unpackEncodedStringToUnsignedChars(DFA27_minS);
    static final char[] DFA27_max = DFA.unpackEncodedStringToUnsignedChars(DFA27_maxS);
    static final short[] DFA27_accept = DFA.unpackEncodedString(DFA27_acceptS);
    static final short[] DFA27_special = DFA.unpackEncodedString(DFA27_specialS);
    static final short[][] DFA27_transition;

    static {
        int numStates = DFA27_transitionS.length;
        DFA27_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA27_transition[i] = DFA.unpackEncodedString(DFA27_transitionS[i]);
        }
    }

    class DFA27 extends DFA {

        public DFA27(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 27;
            this.eot = DFA27_eot;
            this.eof = DFA27_eof;
            this.min = DFA27_min;
            this.max = DFA27_max;
            this.accept = DFA27_accept;
            this.special = DFA27_special;
            this.transition = DFA27_transition;
        }
        public String getDescription() {
            return "1693:1: predicate returns [JSONObject json] : ( in_predicate | contains_all_predicate | equal_predicate | not_equal_predicate | query_predicate | between_predicate | range_predicate | time_predicate | match_predicate | like_predicate | null_predicate );";
        }
    }
    static final String DFA40_eotS =
        "\11\uffff";
    static final String DFA40_eofS =
        "\11\uffff";
    static final String DFA40_minS =
        "\1\70\2\4\1\70\1\4\2\uffff\2\4";
    static final String DFA40_maxS =
        "\1\143\2\176\1\143\1\140\2\uffff\2\176";
    static final String DFA40_acceptS =
        "\5\uffff\1\1\1\2\2\uffff";
    static final String DFA40_specialS =
        "\11\uffff}>";
    static final String[] DFA40_transitionS = {
            "\1\1\52\uffff\1\2",
            "\1\6\10\uffff\1\6\53\uffff\1\5\22\uffff\1\4\23\uffff\1\6\35"+
            "\uffff\1\3",
            "\1\6\10\uffff\1\6\53\uffff\1\5\22\uffff\1\4\23\uffff\1\6\35"+
            "\uffff\1\3",
            "\1\7\52\uffff\1\10",
            "\1\6\10\uffff\1\6\53\uffff\1\5\46\uffff\1\6",
            "",
            "",
            "\1\6\10\uffff\1\6\53\uffff\1\5\22\uffff\1\4\23\uffff\1\6\35"+
            "\uffff\1\3",
            "\1\6\10\uffff\1\6\53\uffff\1\5\22\uffff\1\4\23\uffff\1\6\35"+
            "\uffff\1\3"
    };

    static final short[] DFA40_eot = DFA.unpackEncodedString(DFA40_eotS);
    static final short[] DFA40_eof = DFA.unpackEncodedString(DFA40_eofS);
    static final char[] DFA40_min = DFA.unpackEncodedStringToUnsignedChars(DFA40_minS);
    static final char[] DFA40_max = DFA.unpackEncodedStringToUnsignedChars(DFA40_maxS);
    static final short[] DFA40_accept = DFA.unpackEncodedString(DFA40_acceptS);
    static final short[] DFA40_special = DFA.unpackEncodedString(DFA40_specialS);
    static final short[][] DFA40_transition;

    static {
        int numStates = DFA40_transitionS.length;
        DFA40_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA40_transition[i] = DFA.unpackEncodedString(DFA40_transitionS[i]);
        }
    }

    class DFA40 extends DFA {

        public DFA40(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 40;
            this.eot = DFA40_eot;
            this.eof = DFA40_eof;
            this.min = DFA40_min;
            this.max = DFA40_max;
            this.accept = DFA40_accept;
            this.special = DFA40_special;
            this.transition = DFA40_transition;
        }
        public String getDescription() {
            return "2011:1: time_predicate returns [JSONObject json] : ( column_name ( NOT )? IN LAST time_span | column_name ( NOT )? (since= SINCE |since= AFTER |before= BEFORE ) time_expr );";
        }
    }
    static final String DFA82_eotS =
        "\36\uffff";
    static final String DFA82_eofS =
        "\36\uffff";
    static final String DFA82_minS =
        "\1\20\1\uffff\11\44\1\uffff\6\20\6\70\6\u008b";
    static final String DFA82_maxS =
        "\1\u00a6\1\uffff\11\u00a4\1\uffff\6\u00a6\6\u008a\6\u008b";
    static final String DFA82_acceptS =
        "\1\uffff\1\1\11\uffff\1\2\22\uffff";
    static final String DFA82_specialS =
        "\36\uffff}>";
    static final String[] DFA82_transitionS = {
            "\1\2\2\uffff\1\3\1\uffff\1\13\13\uffff\1\5\10\uffff\1\13\1\1"+
            "\1\uffff\1\13\7\uffff\1\13\2\uffff\1\13\1\uffff\1\1\1\13\6\uffff"+
            "\1\4\1\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\6\uffff\1"+
            "\13\3\uffff\1\13\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1"+
            "\uffff\2\13\6\uffff\1\6\1\11\1\10\1\12\1\7\2\12\4\uffff\1\13"+
            "\1\uffff\1\1\1\uffff\1\13\1\uffff\1\13\1\uffff\2\1\2\13\1\uffff"+
            "\1\13\1\1\1\uffff\1\13\1\uffff\2\13\4\uffff\1\13",
            "",
            "\1\13\15\uffff\1\13\5\uffff\1\1\12\uffff\2\13\32\uffff\1\13"+
            "\20\uffff\23\13\7\uffff\1\14\1\uffff\2\13\14\uffff\1\13\7\uffff"+
            "\3\13",
            "\1\13\15\uffff\1\13\5\uffff\1\1\12\uffff\2\13\32\uffff\1\13"+
            "\20\uffff\23\13\7\uffff\1\15\1\uffff\2\13\14\uffff\1\13\7\uffff"+
            "\3\13",
            "\1\13\15\uffff\1\13\5\uffff\1\1\12\uffff\2\13\32\uffff\1\13"+
            "\20\uffff\23\13\7\uffff\1\16\1\uffff\2\13\14\uffff\1\13\7\uffff"+
            "\3\13",
            "\1\13\15\uffff\1\13\5\uffff\1\1\12\uffff\2\13\32\uffff\1\13"+
            "\20\uffff\23\13\7\uffff\1\17\1\uffff\2\13\14\uffff\1\13\7\uffff"+
            "\3\13",
            "\1\13\15\uffff\1\13\5\uffff\1\1\12\uffff\2\13\32\uffff\1\13"+
            "\20\uffff\23\13\7\uffff\1\20\1\uffff\2\13\14\uffff\1\13\7\uffff"+
            "\3\13",
            "\1\13\15\uffff\1\13\5\uffff\1\1\12\uffff\2\13\32\uffff\1\13"+
            "\20\uffff\23\13\7\uffff\1\20\1\uffff\2\13\14\uffff\1\13\7\uffff"+
            "\3\13",
            "\1\13\15\uffff\1\13\5\uffff\1\1\12\uffff\2\13\32\uffff\1\13"+
            "\20\uffff\23\13\7\uffff\1\20\1\uffff\2\13\14\uffff\1\13\7\uffff"+
            "\3\13",
            "\1\13\15\uffff\1\13\5\uffff\1\1\12\uffff\2\13\32\uffff\1\13"+
            "\20\uffff\23\13\7\uffff\1\20\1\uffff\2\13\14\uffff\1\13\7\uffff"+
            "\3\13",
            "\1\13\15\uffff\1\13\5\uffff\1\1\12\uffff\2\13\32\uffff\1\13"+
            "\20\uffff\23\13\7\uffff\1\21\1\uffff\2\13\14\uffff\1\13\7\uffff"+
            "\3\13",
            "",
            "\1\13\2\uffff\1\13\1\uffff\1\13\13\uffff\1\13\10\uffff\1\13"+
            "\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1\13\6\uffff"+
            "\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12\uffff\1\13"+
            "\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff\2\13\6\uffff"+
            "\7\13\1\uffff\1\22\32\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\13\uffff\1\13\10\uffff\1\13"+
            "\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1\13\6\uffff"+
            "\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12\uffff\1\13"+
            "\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff\2\13\6\uffff"+
            "\7\13\1\uffff\1\23\32\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\13\uffff\1\13\10\uffff\1\13"+
            "\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1\13\6\uffff"+
            "\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12\uffff\1\13"+
            "\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff\2\13\6\uffff"+
            "\7\13\1\uffff\1\24\32\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\13\uffff\1\13\10\uffff\1\13"+
            "\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1\13\6\uffff"+
            "\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12\uffff\1\13"+
            "\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff\2\13\6\uffff"+
            "\7\13\1\uffff\1\25\32\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\13\uffff\1\13\10\uffff\1\13"+
            "\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1\13\6\uffff"+
            "\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12\uffff\1\13"+
            "\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff\2\13\6\uffff"+
            "\7\13\1\uffff\1\26\32\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\13\uffff\1\13\10\uffff\1\13"+
            "\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1\13\6\uffff"+
            "\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12\uffff\1\13"+
            "\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff\2\13\6\uffff"+
            "\7\13\1\uffff\1\27\32\uffff\1\13",
            "\1\1\105\uffff\1\13\13\uffff\1\30",
            "\1\1\105\uffff\1\13\13\uffff\1\31",
            "\1\1\105\uffff\1\13\13\uffff\1\32",
            "\1\1\105\uffff\1\13\13\uffff\1\33",
            "\1\1\105\uffff\1\13\13\uffff\1\34",
            "\1\1\105\uffff\1\13\13\uffff\1\35",
            "\1\22",
            "\1\23",
            "\1\24",
            "\1\25",
            "\1\26",
            "\1\27"
    };

    static final short[] DFA82_eot = DFA.unpackEncodedString(DFA82_eotS);
    static final short[] DFA82_eof = DFA.unpackEncodedString(DFA82_eofS);
    static final char[] DFA82_min = DFA.unpackEncodedStringToUnsignedChars(DFA82_minS);
    static final char[] DFA82_max = DFA.unpackEncodedStringToUnsignedChars(DFA82_maxS);
    static final short[] DFA82_accept = DFA.unpackEncodedString(DFA82_acceptS);
    static final short[] DFA82_special = DFA.unpackEncodedString(DFA82_specialS);
    static final short[][] DFA82_transition;

    static {
        int numStates = DFA82_transitionS.length;
        DFA82_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA82_transition[i] = DFA.unpackEncodedString(DFA82_transitionS[i]);
        }
    }

    class DFA82 extends DFA {

        public DFA82(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 82;
            this.eot = DFA82_eot;
            this.eof = DFA82_eof;
            this.min = DFA82_min;
            this.max = DFA82_max;
            this.accept = DFA82_accept;
            this.special = DFA82_special;
            this.transition = DFA82_transition;
        }
        public String getDescription() {
            return "2689:1: block_statement : ( local_variable_declaration_stmt | java_statement );";
        }
    }
    static final String DFA95_eotS =
        "\u0296\uffff";
    static final String DFA95_eofS =
        "\u0296\uffff";
    static final String DFA95_minS =
        "\2\20\1\70\1\30\1\70\1\30\2\70\1\30\1\70\6\30\21\uffff\16\70\1\20"+
        "\1\u008b\1\26\1\20\1\26\32\uffff\1\u008b\1\26\1\20\1\26\32\uffff"+
        "\1\u008b\1\26\1\u008b\1\26\1\20\1\26\32\uffff\1\u008b\1\26\1\20"+
        "\1\26\32\uffff\1\20\1\26\32\uffff\1\20\1\26\32\uffff\1\20\1\26\32"+
        "\uffff\1\20\1\26\32\uffff\1\20\1\26\32\uffff\54\0\5\uffff\1\0\36"+
        "\uffff\1\0\5\uffff\1\0\36\uffff\1\0\5\uffff\1\0\5\uffff\1\0\36\uffff"+
        "\1\0\5\uffff\1\0\36\uffff\1\0\36\uffff\1\0\36\uffff\1\0\36\uffff"+
        "\1\0\36\uffff\1\0\36\uffff";
    static final String DFA95_maxS =
        "\1\u00a6\1\u009c\1\u008a\1\u00a4\1\u008a\1\u00a4\2\u008a\1\u00a4"+
        "\1\u008a\6\u00a4\21\uffff\16\u008a\1\u009c\1\u008b\1\u008a\1\u00a6"+
        "\1\u008a\32\uffff\1\u008b\1\u008a\1\u00a6\1\u008a\32\uffff\1\u008b"+
        "\1\u008a\1\u008b\1\u008a\1\u00a6\1\u008a\32\uffff\1\u008b\1\u008a"+
        "\1\u00a6\1\u008a\32\uffff\1\u00a6\1\u008a\32\uffff\1\u00a6\1\u008a"+
        "\32\uffff\1\u00a6\1\u008a\32\uffff\1\u00a6\1\u008a\32\uffff\1\u00a6"+
        "\1\u008a\32\uffff\54\0\5\uffff\1\0\36\uffff\1\0\5\uffff\1\0\36\uffff"+
        "\1\0\5\uffff\1\0\5\uffff\1\0\36\uffff\1\0\5\uffff\1\0\36\uffff\1"+
        "\0\36\uffff\1\0\36\uffff\1\0\36\uffff\1\0\36\uffff\1\0\36\uffff";
    static final String DFA95_acceptS =
        "\20\uffff\1\2\u0151\uffff\1\1\u0133\uffff";
    static final String DFA95_specialS =
        "\u0136\uffff\1\0\1\1\1\2\1\3\1\4\1\5\1\6\1\7\1\10\1\11\1\12\1\13"+
        "\1\14\1\15\1\16\1\17\1\20\1\21\1\22\1\23\1\24\1\25\1\26\1\27\1\30"+
        "\1\31\1\32\1\33\1\34\1\35\1\36\1\37\1\40\1\41\1\42\1\43\1\44\1\45"+
        "\1\46\1\47\1\50\1\51\1\52\1\53\5\uffff\1\54\36\uffff\1\55\5\uffff"+
        "\1\56\36\uffff\1\57\5\uffff\1\60\5\uffff\1\61\36\uffff\1\62\5\uffff"+
        "\1\63\36\uffff\1\64\36\uffff\1\65\36\uffff\1\66\36\uffff\1\67\36"+
        "\uffff\1\70\36\uffff}>";
    static final String[] DFA95_transitionS = {
            "\1\3\2\uffff\1\5\1\uffff\1\20\13\uffff\1\12\10\uffff\1\20\1"+
            "\2\1\uffff\1\20\7\uffff\1\20\2\uffff\1\20\1\uffff\1\7\1\20\6"+
            "\uffff\1\10\1\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\6\uffff"+
            "\1\20\3\uffff\1\20\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20"+
            "\1\uffff\2\20\6\uffff\1\13\1\16\1\15\1\17\1\14\2\17\6\uffff"+
            "\1\4\5\uffff\1\1\1\11\4\uffff\1\6\11\uffff\1\20",
            "\1\42\2\uffff\1\44\15\uffff\1\51\11\uffff\1\41\16\uffff\1\46"+
            "\7\uffff\1\47\100\uffff\1\52\1\55\1\54\1\56\1\53\2\56\6\uffff"+
            "\1\43\5\uffff\1\57\1\50\4\uffff\1\45",
            "\1\61\121\uffff\1\60",
            "\1\20\13\uffff\1\20\15\uffff\1\20\5\uffff\1\63\12\uffff\2\20"+
            "\32\uffff\1\20\20\uffff\23\20\7\uffff\1\62\1\uffff\2\20\14\uffff"+
            "\1\20\7\uffff\3\20",
            "\1\117\121\uffff\1\116",
            "\1\20\13\uffff\1\20\15\uffff\1\20\5\uffff\1\121\12\uffff\2"+
            "\20\32\uffff\1\20\20\uffff\23\20\7\uffff\1\120\1\uffff\2\20"+
            "\14\uffff\1\20\7\uffff\3\20",
            "\1\155\121\uffff\1\154",
            "\1\157\121\uffff\1\156",
            "\1\20\13\uffff\1\20\15\uffff\1\20\5\uffff\1\161\12\uffff\2"+
            "\20\32\uffff\1\20\20\uffff\23\20\7\uffff\1\160\1\uffff\2\20"+
            "\14\uffff\1\20\7\uffff\3\20",
            "\1\u008d\121\uffff\1\u008c",
            "\1\20\13\uffff\1\20\15\uffff\1\20\5\uffff\1\u008f\12\uffff"+
            "\2\20\32\uffff\1\20\20\uffff\23\20\7\uffff\1\u008e\1\uffff\2"+
            "\20\14\uffff\1\20\7\uffff\3\20",
            "\1\20\13\uffff\1\20\15\uffff\1\20\5\uffff\1\u00ab\12\uffff"+
            "\2\20\32\uffff\1\20\20\uffff\23\20\7\uffff\1\u00aa\1\uffff\2"+
            "\20\14\uffff\1\20\7\uffff\3\20",
            "\1\20\13\uffff\1\20\15\uffff\1\20\5\uffff\1\u00c7\12\uffff"+
            "\2\20\32\uffff\1\20\20\uffff\23\20\7\uffff\1\u00c6\1\uffff\2"+
            "\20\14\uffff\1\20\7\uffff\3\20",
            "\1\20\13\uffff\1\20\15\uffff\1\20\5\uffff\1\u00e3\12\uffff"+
            "\2\20\32\uffff\1\20\20\uffff\23\20\7\uffff\1\u00e2\1\uffff\2"+
            "\20\14\uffff\1\20\7\uffff\3\20",
            "\1\20\13\uffff\1\20\15\uffff\1\20\5\uffff\1\u00ff\12\uffff"+
            "\2\20\32\uffff\1\20\20\uffff\23\20\7\uffff\1\u00fe\1\uffff\2"+
            "\20\14\uffff\1\20\7\uffff\3\20",
            "\1\20\13\uffff\1\20\15\uffff\1\20\5\uffff\1\u011b\12\uffff"+
            "\2\20\32\uffff\1\20\20\uffff\23\20\7\uffff\1\u011a\1\uffff\2"+
            "\20\14\uffff\1\20\7\uffff\3\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\u0137\121\uffff\1\u0136",
            "\1\u0139\121\uffff\1\u0138",
            "\1\u013b\121\uffff\1\u013a",
            "\1\u013d\121\uffff\1\u013c",
            "\1\u013f\121\uffff\1\u013e",
            "\1\u0141\121\uffff\1\u0140",
            "\1\u0143\121\uffff\1\u0142",
            "\1\u0145\121\uffff\1\u0144",
            "\1\u0147\121\uffff\1\u0146",
            "\1\u0149\121\uffff\1\u0148",
            "\1\u014b\121\uffff\1\u014a",
            "\1\u014d\121\uffff\1\u014c",
            "\1\u014f\121\uffff\1\u014e",
            "\1\u0151\121\uffff\1\u0150",
            "\1\u0153\2\uffff\1\u0155\15\uffff\1\u015a\11\uffff\1\u0152"+
            "\16\uffff\1\u0157\7\uffff\1\u0158\100\uffff\1\u015b\1\u015e"+
            "\1\u015d\1\u015f\1\u015c\2\u015f\6\uffff\1\u0154\5\uffff\1\u0160"+
            "\1\u0159\4\uffff\1\u0156",
            "\1\u0161",
            "\1\u0162\1\uffff\1\20\13\uffff\1\20\72\uffff\1\20\52\uffff"+
            "\1\20",
            "\1\20\2\uffff\1\20\1\uffff\1\20\13\uffff\1\20\10\uffff\1\20"+
            "\2\uffff\1\20\7\uffff\1\20\2\uffff\1\20\2\uffff\1\20\6\uffff"+
            "\2\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\12\uffff\1\20"+
            "\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20\1\uffff\2\20\6\uffff"+
            "\7\20\1\uffff\1\u0167\32\uffff\1\20",
            "\1\u0162\1\uffff\1\20\13\uffff\1\20\72\uffff\1\20\52\uffff"+
            "\1\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\u0186",
            "\1\u0162\1\uffff\1\20\13\uffff\1\20\72\uffff\1\20\52\uffff"+
            "\1\20",
            "\1\20\2\uffff\1\20\1\uffff\1\20\13\uffff\1\20\10\uffff\1\20"+
            "\2\uffff\1\20\7\uffff\1\20\2\uffff\1\20\2\uffff\1\20\6\uffff"+
            "\2\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\12\uffff\1\20"+
            "\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20\1\uffff\2\20\6\uffff"+
            "\7\20\1\uffff\1\u018c\32\uffff\1\20",
            "\1\u0162\1\uffff\1\20\13\uffff\1\20\72\uffff\1\20\52\uffff"+
            "\1\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\u01ab",
            "\1\u0162\1\uffff\1\20\13\uffff\1\20\72\uffff\1\20\52\uffff"+
            "\1\20",
            "\1\u01b1",
            "\1\u0162\1\uffff\1\20\13\uffff\1\20\72\uffff\1\20\52\uffff"+
            "\1\20",
            "\1\20\2\uffff\1\20\1\uffff\1\20\13\uffff\1\20\10\uffff\1\20"+
            "\2\uffff\1\20\7\uffff\1\20\2\uffff\1\20\2\uffff\1\20\6\uffff"+
            "\2\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\12\uffff\1\20"+
            "\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20\1\uffff\2\20\6\uffff"+
            "\7\20\1\uffff\1\u01b7\32\uffff\1\20",
            "\1\u0162\1\uffff\1\20\13\uffff\1\20\72\uffff\1\20\52\uffff"+
            "\1\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\u01d6",
            "\1\u0162\1\uffff\1\20\13\uffff\1\20\72\uffff\1\20\52\uffff"+
            "\1\20",
            "\1\20\2\uffff\1\20\1\uffff\1\20\13\uffff\1\20\10\uffff\1\20"+
            "\2\uffff\1\20\7\uffff\1\20\2\uffff\1\20\2\uffff\1\20\6\uffff"+
            "\2\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\12\uffff\1\20"+
            "\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20\1\uffff\2\20\6\uffff"+
            "\7\20\1\uffff\1\u01dc\32\uffff\1\20",
            "\1\u0162\1\uffff\1\20\13\uffff\1\20\72\uffff\1\20\52\uffff"+
            "\1\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\20\2\uffff\1\20\1\uffff\1\20\13\uffff\1\20\10\uffff\1\20"+
            "\2\uffff\1\20\7\uffff\1\20\2\uffff\1\20\2\uffff\1\20\6\uffff"+
            "\2\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\12\uffff\1\20"+
            "\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20\1\uffff\2\20\6\uffff"+
            "\7\20\1\uffff\1\u01fb\32\uffff\1\20",
            "\1\u0162\1\uffff\1\20\13\uffff\1\20\72\uffff\1\20\52\uffff"+
            "\1\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\20\2\uffff\1\20\1\uffff\1\20\13\uffff\1\20\10\uffff\1\20"+
            "\2\uffff\1\20\7\uffff\1\20\2\uffff\1\20\2\uffff\1\20\6\uffff"+
            "\2\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\12\uffff\1\20"+
            "\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20\1\uffff\2\20\6\uffff"+
            "\7\20\1\uffff\1\u021a\32\uffff\1\20",
            "\1\u0162\1\uffff\1\20\13\uffff\1\20\72\uffff\1\20\52\uffff"+
            "\1\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\20\2\uffff\1\20\1\uffff\1\20\13\uffff\1\20\10\uffff\1\20"+
            "\2\uffff\1\20\7\uffff\1\20\2\uffff\1\20\2\uffff\1\20\6\uffff"+
            "\2\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\12\uffff\1\20"+
            "\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20\1\uffff\2\20\6\uffff"+
            "\7\20\1\uffff\1\u0239\32\uffff\1\20",
            "\1\u0162\1\uffff\1\20\13\uffff\1\20\72\uffff\1\20\52\uffff"+
            "\1\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\20\2\uffff\1\20\1\uffff\1\20\13\uffff\1\20\10\uffff\1\20"+
            "\2\uffff\1\20\7\uffff\1\20\2\uffff\1\20\2\uffff\1\20\6\uffff"+
            "\2\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\12\uffff\1\20"+
            "\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20\1\uffff\2\20\6\uffff"+
            "\7\20\1\uffff\1\u0258\32\uffff\1\20",
            "\1\u0162\1\uffff\1\20\13\uffff\1\20\72\uffff\1\20\52\uffff"+
            "\1\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\20\2\uffff\1\20\1\uffff\1\20\13\uffff\1\20\10\uffff\1\20"+
            "\2\uffff\1\20\7\uffff\1\20\2\uffff\1\20\2\uffff\1\20\6\uffff"+
            "\2\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\12\uffff\1\20"+
            "\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20\1\uffff\2\20\6\uffff"+
            "\7\20\1\uffff\1\u0277\32\uffff\1\20",
            "\1\u0162\1\uffff\1\20\13\uffff\1\20\72\uffff\1\20\52\uffff"+
            "\1\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            ""
    };

    static final short[] DFA95_eot = DFA.unpackEncodedString(DFA95_eotS);
    static final short[] DFA95_eof = DFA.unpackEncodedString(DFA95_eofS);
    static final char[] DFA95_min = DFA.unpackEncodedStringToUnsignedChars(DFA95_minS);
    static final char[] DFA95_max = DFA.unpackEncodedStringToUnsignedChars(DFA95_maxS);
    static final short[] DFA95_accept = DFA.unpackEncodedString(DFA95_acceptS);
    static final short[] DFA95_special = DFA.unpackEncodedString(DFA95_specialS);
    static final short[][] DFA95_transition;

    static {
        int numStates = DFA95_transitionS.length;
        DFA95_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA95_transition[i] = DFA.unpackEncodedString(DFA95_transitionS[i]);
        }
    }

    class DFA95 extends DFA {

        public DFA95(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 95;
            this.eot = DFA95_eot;
            this.eof = DFA95_eof;
            this.min = DFA95_min;
            this.max = DFA95_max;
            this.accept = DFA95_accept;
            this.special = DFA95_special;
            this.transition = DFA95_transition;
        }
        public String getDescription() {
            return "2776:1: for_control options {k=3; } : ( enhanced_for_control | ( for_init )? SEMI ( expression )? SEMI ( for_update )? );";
        }
        public int specialStateTransition(int s, IntStream _input) throws NoViableAltException {
            TokenStream input = (TokenStream)_input;
        	int _s = s;
            switch ( s ) {
                    case 0 : 
                        int LA95_310 = input.LA(1);

                         
                        int index95_310 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_310);

                        if ( s>=0 ) return s;
                        break;
                    case 1 : 
                        int LA95_311 = input.LA(1);

                         
                        int index95_311 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_311);

                        if ( s>=0 ) return s;
                        break;
                    case 2 : 
                        int LA95_312 = input.LA(1);

                         
                        int index95_312 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_312);

                        if ( s>=0 ) return s;
                        break;
                    case 3 : 
                        int LA95_313 = input.LA(1);

                         
                        int index95_313 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_313);

                        if ( s>=0 ) return s;
                        break;
                    case 4 : 
                        int LA95_314 = input.LA(1);

                         
                        int index95_314 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_314);

                        if ( s>=0 ) return s;
                        break;
                    case 5 : 
                        int LA95_315 = input.LA(1);

                         
                        int index95_315 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_315);

                        if ( s>=0 ) return s;
                        break;
                    case 6 : 
                        int LA95_316 = input.LA(1);

                         
                        int index95_316 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_316);

                        if ( s>=0 ) return s;
                        break;
                    case 7 : 
                        int LA95_317 = input.LA(1);

                         
                        int index95_317 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_317);

                        if ( s>=0 ) return s;
                        break;
                    case 8 : 
                        int LA95_318 = input.LA(1);

                         
                        int index95_318 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_318);

                        if ( s>=0 ) return s;
                        break;
                    case 9 : 
                        int LA95_319 = input.LA(1);

                         
                        int index95_319 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_319);

                        if ( s>=0 ) return s;
                        break;
                    case 10 : 
                        int LA95_320 = input.LA(1);

                         
                        int index95_320 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_320);

                        if ( s>=0 ) return s;
                        break;
                    case 11 : 
                        int LA95_321 = input.LA(1);

                         
                        int index95_321 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_321);

                        if ( s>=0 ) return s;
                        break;
                    case 12 : 
                        int LA95_322 = input.LA(1);

                         
                        int index95_322 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_322);

                        if ( s>=0 ) return s;
                        break;
                    case 13 : 
                        int LA95_323 = input.LA(1);

                         
                        int index95_323 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_323);

                        if ( s>=0 ) return s;
                        break;
                    case 14 : 
                        int LA95_324 = input.LA(1);

                         
                        int index95_324 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_324);

                        if ( s>=0 ) return s;
                        break;
                    case 15 : 
                        int LA95_325 = input.LA(1);

                         
                        int index95_325 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_325);

                        if ( s>=0 ) return s;
                        break;
                    case 16 : 
                        int LA95_326 = input.LA(1);

                         
                        int index95_326 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_326);

                        if ( s>=0 ) return s;
                        break;
                    case 17 : 
                        int LA95_327 = input.LA(1);

                         
                        int index95_327 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_327);

                        if ( s>=0 ) return s;
                        break;
                    case 18 : 
                        int LA95_328 = input.LA(1);

                         
                        int index95_328 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_328);

                        if ( s>=0 ) return s;
                        break;
                    case 19 : 
                        int LA95_329 = input.LA(1);

                         
                        int index95_329 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_329);

                        if ( s>=0 ) return s;
                        break;
                    case 20 : 
                        int LA95_330 = input.LA(1);

                         
                        int index95_330 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_330);

                        if ( s>=0 ) return s;
                        break;
                    case 21 : 
                        int LA95_331 = input.LA(1);

                         
                        int index95_331 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_331);

                        if ( s>=0 ) return s;
                        break;
                    case 22 : 
                        int LA95_332 = input.LA(1);

                         
                        int index95_332 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_332);

                        if ( s>=0 ) return s;
                        break;
                    case 23 : 
                        int LA95_333 = input.LA(1);

                         
                        int index95_333 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_333);

                        if ( s>=0 ) return s;
                        break;
                    case 24 : 
                        int LA95_334 = input.LA(1);

                         
                        int index95_334 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_334);

                        if ( s>=0 ) return s;
                        break;
                    case 25 : 
                        int LA95_335 = input.LA(1);

                         
                        int index95_335 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_335);

                        if ( s>=0 ) return s;
                        break;
                    case 26 : 
                        int LA95_336 = input.LA(1);

                         
                        int index95_336 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_336);

                        if ( s>=0 ) return s;
                        break;
                    case 27 : 
                        int LA95_337 = input.LA(1);

                         
                        int index95_337 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_337);

                        if ( s>=0 ) return s;
                        break;
                    case 28 : 
                        int LA95_338 = input.LA(1);

                         
                        int index95_338 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_338);

                        if ( s>=0 ) return s;
                        break;
                    case 29 : 
                        int LA95_339 = input.LA(1);

                         
                        int index95_339 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_339);

                        if ( s>=0 ) return s;
                        break;
                    case 30 : 
                        int LA95_340 = input.LA(1);

                         
                        int index95_340 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_340);

                        if ( s>=0 ) return s;
                        break;
                    case 31 : 
                        int LA95_341 = input.LA(1);

                         
                        int index95_341 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_341);

                        if ( s>=0 ) return s;
                        break;
                    case 32 : 
                        int LA95_342 = input.LA(1);

                         
                        int index95_342 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_342);

                        if ( s>=0 ) return s;
                        break;
                    case 33 : 
                        int LA95_343 = input.LA(1);

                         
                        int index95_343 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_343);

                        if ( s>=0 ) return s;
                        break;
                    case 34 : 
                        int LA95_344 = input.LA(1);

                         
                        int index95_344 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_344);

                        if ( s>=0 ) return s;
                        break;
                    case 35 : 
                        int LA95_345 = input.LA(1);

                         
                        int index95_345 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_345);

                        if ( s>=0 ) return s;
                        break;
                    case 36 : 
                        int LA95_346 = input.LA(1);

                         
                        int index95_346 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_346);

                        if ( s>=0 ) return s;
                        break;
                    case 37 : 
                        int LA95_347 = input.LA(1);

                         
                        int index95_347 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_347);

                        if ( s>=0 ) return s;
                        break;
                    case 38 : 
                        int LA95_348 = input.LA(1);

                         
                        int index95_348 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_348);

                        if ( s>=0 ) return s;
                        break;
                    case 39 : 
                        int LA95_349 = input.LA(1);

                         
                        int index95_349 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_349);

                        if ( s>=0 ) return s;
                        break;
                    case 40 : 
                        int LA95_350 = input.LA(1);

                         
                        int index95_350 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_350);

                        if ( s>=0 ) return s;
                        break;
                    case 41 : 
                        int LA95_351 = input.LA(1);

                         
                        int index95_351 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_351);

                        if ( s>=0 ) return s;
                        break;
                    case 42 : 
                        int LA95_352 = input.LA(1);

                         
                        int index95_352 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_352);

                        if ( s>=0 ) return s;
                        break;
                    case 43 : 
                        int LA95_353 = input.LA(1);

                         
                        int index95_353 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_353);

                        if ( s>=0 ) return s;
                        break;
                    case 44 : 
                        int LA95_359 = input.LA(1);

                         
                        int index95_359 = input.index();
                        input.rewind();

                        s = -1;
                        if ( ((((( "Boolean".equals(input.LT(1).getText()) )||( "boolean".equals(input.LT(1).getText()) ))&&(( "Boolean".equals(input.LT(1).getText()) )||( "boolean".equals(input.LT(1).getText()) )))&&synpred150_BQL())) ) {s = 354;}

                        else if ( ((( "Boolean".equals(input.LT(1).getText()) )||( "boolean".equals(input.LT(1).getText()) ))) ) {s = 16;}

                         
                        input.seek(index95_359);

                        if ( s>=0 ) return s;
                        break;
                    case 45 : 
                        int LA95_390 = input.LA(1);

                         
                        int index95_390 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_390);

                        if ( s>=0 ) return s;
                        break;
                    case 46 : 
                        int LA95_396 = input.LA(1);

                         
                        int index95_396 = input.index();
                        input.rewind();

                        s = -1;
                        if ( ((((( "Byte".equals(input.LT(1).getText()) )||( "byte".equals(input.LT(1).getText()) ))&&(( "Byte".equals(input.LT(1).getText()) )||( "byte".equals(input.LT(1).getText()) )))&&synpred150_BQL())) ) {s = 354;}

                        else if ( ((( "Byte".equals(input.LT(1).getText()) )||( "byte".equals(input.LT(1).getText()) ))) ) {s = 16;}

                         
                        input.seek(index95_396);

                        if ( s>=0 ) return s;
                        break;
                    case 47 : 
                        int LA95_427 = input.LA(1);

                         
                        int index95_427 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_427);

                        if ( s>=0 ) return s;
                        break;
                    case 48 : 
                        int LA95_433 = input.LA(1);

                         
                        int index95_433 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (((( "int".equals(input.LT(1).getText()) )&&( "int".equals(input.LT(1).getText()) ))&&synpred150_BQL())) ) {s = 354;}

                        else if ( (( "int".equals(input.LT(1).getText()) )) ) {s = 16;}

                         
                        input.seek(index95_433);

                        if ( s>=0 ) return s;
                        break;
                    case 49 : 
                        int LA95_439 = input.LA(1);

                         
                        int index95_439 = input.index();
                        input.rewind();

                        s = -1;
                        if ( ((((( "long".equals(input.LT(1).getText()) )||( "Long".equals(input.LT(1).getText()) ))&&(( "long".equals(input.LT(1).getText()) )||( "Long".equals(input.LT(1).getText()) )))&&synpred150_BQL())) ) {s = 354;}

                        else if ( ((( "long".equals(input.LT(1).getText()) )||( "Long".equals(input.LT(1).getText()) ))) ) {s = 16;}

                         
                        input.seek(index95_439);

                        if ( s>=0 ) return s;
                        break;
                    case 50 : 
                        int LA95_470 = input.LA(1);

                         
                        int index95_470 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_470);

                        if ( s>=0 ) return s;
                        break;
                    case 51 : 
                        int LA95_476 = input.LA(1);

                         
                        int index95_476 = input.index();
                        input.rewind();

                        s = -1;
                        if ( ((((( "double".equals(input.LT(1).getText()) )||( "Double".equals(input.LT(1).getText()) ))&&(( "double".equals(input.LT(1).getText()) )||( "Double".equals(input.LT(1).getText()) )))&&synpred150_BQL())) ) {s = 354;}

                        else if ( ((( "double".equals(input.LT(1).getText()) )||( "Double".equals(input.LT(1).getText()) ))) ) {s = 16;}

                         
                        input.seek(index95_476);

                        if ( s>=0 ) return s;
                        break;
                    case 52 : 
                        int LA95_507 = input.LA(1);

                         
                        int index95_507 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_507);

                        if ( s>=0 ) return s;
                        break;
                    case 53 : 
                        int LA95_538 = input.LA(1);

                         
                        int index95_538 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_538);

                        if ( s>=0 ) return s;
                        break;
                    case 54 : 
                        int LA95_569 = input.LA(1);

                         
                        int index95_569 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_569);

                        if ( s>=0 ) return s;
                        break;
                    case 55 : 
                        int LA95_600 = input.LA(1);

                         
                        int index95_600 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_600);

                        if ( s>=0 ) return s;
                        break;
                    case 56 : 
                        int LA95_631 = input.LA(1);

                         
                        int index95_631 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred150_BQL()) ) {s = 354;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index95_631);

                        if ( s>=0 ) return s;
                        break;
            }
            if (state.backtracking>0) {state.failed=true; return -1;}

            NoViableAltException nvae =
                new NoViableAltException(getDescription(), 95, _s, input);
            error(nvae);
            throw nvae;
        }

    }
    static final String DFA96_eotS =
        "\36\uffff";
    static final String DFA96_eofS =
        "\2\uffff\11\13\23\uffff";
    static final String DFA96_minS =
        "\1\20\1\uffff\11\30\1\uffff\6\20\6\70\6\u008b";
    static final String DFA96_maxS =
        "\1\u00a6\1\uffff\11\u00a4\1\uffff\6\u00a6\6\u008a\6\u008b";
    static final String DFA96_acceptS =
        "\1\uffff\1\1\11\uffff\1\2\22\uffff";
    static final String DFA96_specialS =
        "\36\uffff}>";
    static final String[] DFA96_transitionS = {
            "\1\2\2\uffff\1\3\1\uffff\1\13\13\uffff\1\5\10\uffff\1\13\1\1"+
            "\1\uffff\1\13\7\uffff\1\13\2\uffff\1\13\1\uffff\1\1\1\13\6\uffff"+
            "\1\4\1\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12\uffff\1"+
            "\13\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff\2\13\6"+
            "\uffff\1\6\1\11\1\10\1\12\1\7\2\12\6\uffff\1\1\5\uffff\2\1\4"+
            "\uffff\1\1\11\uffff\1\13",
            "",
            "\1\13\13\uffff\1\13\15\uffff\1\13\5\uffff\1\1\12\uffff\2\13"+
            "\32\uffff\1\13\20\uffff\23\13\7\uffff\1\14\1\uffff\2\13\14\uffff"+
            "\1\13\7\uffff\3\13",
            "\1\13\13\uffff\1\13\15\uffff\1\13\5\uffff\1\1\12\uffff\2\13"+
            "\32\uffff\1\13\20\uffff\23\13\7\uffff\1\15\1\uffff\2\13\14\uffff"+
            "\1\13\7\uffff\3\13",
            "\1\13\13\uffff\1\13\15\uffff\1\13\5\uffff\1\1\12\uffff\2\13"+
            "\32\uffff\1\13\20\uffff\23\13\7\uffff\1\16\1\uffff\2\13\14\uffff"+
            "\1\13\7\uffff\3\13",
            "\1\13\13\uffff\1\13\15\uffff\1\13\5\uffff\1\1\12\uffff\2\13"+
            "\32\uffff\1\13\20\uffff\23\13\7\uffff\1\17\1\uffff\2\13\14\uffff"+
            "\1\13\7\uffff\3\13",
            "\1\13\13\uffff\1\13\15\uffff\1\13\5\uffff\1\1\12\uffff\2\13"+
            "\32\uffff\1\13\20\uffff\23\13\7\uffff\1\20\1\uffff\2\13\14\uffff"+
            "\1\13\7\uffff\3\13",
            "\1\13\13\uffff\1\13\15\uffff\1\13\5\uffff\1\1\12\uffff\2\13"+
            "\32\uffff\1\13\20\uffff\23\13\7\uffff\1\20\1\uffff\2\13\14\uffff"+
            "\1\13\7\uffff\3\13",
            "\1\13\13\uffff\1\13\15\uffff\1\13\5\uffff\1\1\12\uffff\2\13"+
            "\32\uffff\1\13\20\uffff\23\13\7\uffff\1\20\1\uffff\2\13\14\uffff"+
            "\1\13\7\uffff\3\13",
            "\1\13\13\uffff\1\13\15\uffff\1\13\5\uffff\1\1\12\uffff\2\13"+
            "\32\uffff\1\13\20\uffff\23\13\7\uffff\1\20\1\uffff\2\13\14\uffff"+
            "\1\13\7\uffff\3\13",
            "\1\13\13\uffff\1\13\15\uffff\1\13\5\uffff\1\1\12\uffff\2\13"+
            "\32\uffff\1\13\20\uffff\23\13\7\uffff\1\21\1\uffff\2\13\14\uffff"+
            "\1\13\7\uffff\3\13",
            "",
            "\1\13\2\uffff\1\13\1\uffff\1\13\13\uffff\1\13\10\uffff\1\13"+
            "\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1\13\6\uffff"+
            "\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12\uffff\1\13"+
            "\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff\2\13\6\uffff"+
            "\7\13\1\uffff\1\22\32\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\13\uffff\1\13\10\uffff\1\13"+
            "\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1\13\6\uffff"+
            "\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12\uffff\1\13"+
            "\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff\2\13\6\uffff"+
            "\7\13\1\uffff\1\23\32\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\13\uffff\1\13\10\uffff\1\13"+
            "\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1\13\6\uffff"+
            "\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12\uffff\1\13"+
            "\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff\2\13\6\uffff"+
            "\7\13\1\uffff\1\24\32\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\13\uffff\1\13\10\uffff\1\13"+
            "\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1\13\6\uffff"+
            "\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12\uffff\1\13"+
            "\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff\2\13\6\uffff"+
            "\7\13\1\uffff\1\25\32\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\13\uffff\1\13\10\uffff\1\13"+
            "\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1\13\6\uffff"+
            "\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12\uffff\1\13"+
            "\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff\2\13\6\uffff"+
            "\7\13\1\uffff\1\26\32\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\13\uffff\1\13\10\uffff\1\13"+
            "\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1\13\6\uffff"+
            "\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12\uffff\1\13"+
            "\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff\2\13\6\uffff"+
            "\7\13\1\uffff\1\27\32\uffff\1\13",
            "\1\1\105\uffff\1\13\13\uffff\1\30",
            "\1\1\105\uffff\1\13\13\uffff\1\31",
            "\1\1\105\uffff\1\13\13\uffff\1\32",
            "\1\1\105\uffff\1\13\13\uffff\1\33",
            "\1\1\105\uffff\1\13\13\uffff\1\34",
            "\1\1\105\uffff\1\13\13\uffff\1\35",
            "\1\22",
            "\1\23",
            "\1\24",
            "\1\25",
            "\1\26",
            "\1\27"
    };

    static final short[] DFA96_eot = DFA.unpackEncodedString(DFA96_eotS);
    static final short[] DFA96_eof = DFA.unpackEncodedString(DFA96_eofS);
    static final char[] DFA96_min = DFA.unpackEncodedStringToUnsignedChars(DFA96_minS);
    static final char[] DFA96_max = DFA.unpackEncodedStringToUnsignedChars(DFA96_maxS);
    static final short[] DFA96_accept = DFA.unpackEncodedString(DFA96_acceptS);
    static final short[] DFA96_special = DFA.unpackEncodedString(DFA96_specialS);
    static final short[][] DFA96_transition;

    static {
        int numStates = DFA96_transitionS.length;
        DFA96_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA96_transition[i] = DFA.unpackEncodedString(DFA96_transitionS[i]);
        }
    }

    class DFA96 extends DFA {

        public DFA96(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 96;
            this.eot = DFA96_eot;
            this.eof = DFA96_eof;
            this.min = DFA96_min;
            this.max = DFA96_max;
            this.accept = DFA96_accept;
            this.special = DFA96_special;
            this.transition = DFA96_transition;
        }
        public String getDescription() {
            return "2782:1: for_init : ( local_variable_declaration | expression_list );";
        }
    }
 

    public static final BitSet FOLLOW_select_stmt_in_statement3529 = new BitSet(new long[]{0x0000000000000000L,0x0000000080000000L});
    public static final BitSet FOLLOW_describe_stmt_in_statement3545 = new BitSet(new long[]{0x0000000000000000L,0x0000000080000000L});
    public static final BitSet FOLLOW_SEMI_in_statement3559 = new BitSet(new long[]{0x0000000000000000L});
    public static final BitSet FOLLOW_EOF_in_statement3562 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_SELECT_in_select_stmt3590 = new BitSet(new long[]{0x0100000000000000L,0x0040000800000000L});
    public static final BitSet FOLLOW_118_in_select_stmt3593 = new BitSet(new long[]{0x0003900000020002L,0x0000110004080001L});
    public static final BitSet FOLLOW_column_name_list_in_select_stmt3599 = new BitSet(new long[]{0x0003900000020002L,0x0000110004080001L});
    public static final BitSet FOLLOW_FROM_in_select_stmt3611 = new BitSet(new long[]{0x0100000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_set_in_select_stmt3613 = new BitSet(new long[]{0x0003100000020002L,0x0000110004080001L});
    public static final BitSet FOLLOW_where_in_select_stmt3633 = new BitSet(new long[]{0x0003100000020002L,0x0000010004080001L});
    public static final BitSet FOLLOW_given_clause_in_select_stmt3646 = new BitSet(new long[]{0x0002100000020002L,0x0000010004080001L});
    public static final BitSet FOLLOW_order_by_clause_in_select_stmt3665 = new BitSet(new long[]{0x0002100000020002L,0x0000010004080001L});
    public static final BitSet FOLLOW_limit_clause_in_select_stmt3698 = new BitSet(new long[]{0x0002100000020002L,0x0000010004080001L});
    public static final BitSet FOLLOW_group_by_clause_in_select_stmt3730 = new BitSet(new long[]{0x0002100000020002L,0x0000010004080001L});
    public static final BitSet FOLLOW_browse_by_clause_in_select_stmt3762 = new BitSet(new long[]{0x0002100000020002L,0x0000010004080001L});
    public static final BitSet FOLLOW_fetching_stored_clause_in_select_stmt3794 = new BitSet(new long[]{0x0002100000020002L,0x0000010004080001L});
    public static final BitSet FOLLOW_route_by_clause_in_select_stmt3826 = new BitSet(new long[]{0x0002100000020002L,0x0000010004080001L});
    public static final BitSet FOLLOW_relevance_model_clause_in_select_stmt3858 = new BitSet(new long[]{0x0002100000020002L,0x0000010004080001L});
    public static final BitSet FOLLOW_DESCRIBE_in_describe_stmt3922 = new BitSet(new long[]{0x0100000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_set_in_describe_stmt3924 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_column_name_list3960 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_COMMA_in_column_name_list3981 = new BitSet(new long[]{0x0100000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_column_name_in_column_name_list3985 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_IDENT_in_column_name4058 = new BitSet(new long[]{0x0000000000000002L,0x4000000000000000L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_column_name4064 = new BitSet(new long[]{0x0000000000000002L,0x4000000000000000L});
    public static final BitSet FOLLOW_126_in_column_name4086 = new BitSet(new long[]{0x0100000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_IDENT_in_column_name4091 = new BitSet(new long[]{0x0000000000000002L,0x4000000000000000L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_column_name4097 = new BitSet(new long[]{0x0000000000000002L,0x4000000000000000L});
    public static final BitSet FOLLOW_WHERE_in_where4152 = new BitSet(new long[]{0x0100000000000000L,0x0000000800401048L});
    public static final BitSet FOLLOW_search_expr_in_where4155 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_ORDER_in_order_by_clause4188 = new BitSet(new long[]{0x0000000000040000L});
    public static final BitSet FOLLOW_BY_in_order_by_clause4190 = new BitSet(new long[]{0x0100000000000000L,0x0000000802000000L});
    public static final BitSet FOLLOW_RELEVANCE_in_order_by_clause4193 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_sort_specs_in_order_by_clause4197 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_sort_spec_in_sort_specs4238 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_COMMA_in_sort_specs4259 = new BitSet(new long[]{0x0100000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_sort_spec_in_sort_specs4263 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_column_name_in_sort_spec4324 = new BitSet(new long[]{0x0000000040001002L});
    public static final BitSet FOLLOW_ASC_in_sort_spec4329 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_DESC_in_sort_spec4335 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LIMIT_in_limit_clause4370 = new BitSet(new long[]{0x0800000000000000L});
    public static final BitSet FOLLOW_INTEGER_in_limit_clause4375 = new BitSet(new long[]{0x0000000001000000L});
    public static final BitSet FOLLOW_COMMA_in_limit_clause4377 = new BitSet(new long[]{0x0800000000000000L});
    public static final BitSet FOLLOW_INTEGER_in_limit_clause4383 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_or_column_name_list4423 = new BitSet(new long[]{0x0000000000000002L,0x0000000000040000L});
    public static final BitSet FOLLOW_OR_in_or_column_name_list4444 = new BitSet(new long[]{0x0100000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_column_name_in_or_column_name_list4448 = new BitSet(new long[]{0x0000000000000002L,0x0000000000040000L});
    public static final BitSet FOLLOW_GROUP_in_group_by_clause4496 = new BitSet(new long[]{0x0000000000040000L});
    public static final BitSet FOLLOW_BY_in_group_by_clause4498 = new BitSet(new long[]{0x0100000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_or_column_name_list_in_group_by_clause4500 = new BitSet(new long[]{0x0000000000000002L,0x0000002000000000L});
    public static final BitSet FOLLOW_TOP_in_group_by_clause4503 = new BitSet(new long[]{0x0800000000000000L});
    public static final BitSet FOLLOW_INTEGER_in_group_by_clause4507 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_BROWSE_in_browse_by_clause4547 = new BitSet(new long[]{0x0000000000040000L});
    public static final BitSet FOLLOW_BY_in_browse_by_clause4549 = new BitSet(new long[]{0x0100000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_facet_spec_in_browse_by_clause4553 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_COMMA_in_browse_by_clause4574 = new BitSet(new long[]{0x0100000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_facet_spec_in_browse_by_clause4578 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_column_name_in_facet_spec4631 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000008L});
    public static final BitSet FOLLOW_LPAR_in_facet_spec4655 = new BitSet(new long[]{0x0000040000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_TRUE_in_facet_spec4671 = new BitSet(new long[]{0x0000000001000000L});
    public static final BitSet FOLLOW_FALSE_in_facet_spec4677 = new BitSet(new long[]{0x0000000001000000L});
    public static final BitSet FOLLOW_COMMA_in_facet_spec4680 = new BitSet(new long[]{0x0800000000000000L});
    public static final BitSet FOLLOW_INTEGER_in_facet_spec4696 = new BitSet(new long[]{0x0000000001000000L});
    public static final BitSet FOLLOW_COMMA_in_facet_spec4698 = new BitSet(new long[]{0x0800000000000000L});
    public static final BitSet FOLLOW_INTEGER_in_facet_spec4714 = new BitSet(new long[]{0x0000000001000000L});
    public static final BitSet FOLLOW_COMMA_in_facet_spec4716 = new BitSet(new long[]{0x0040000000000000L,0x0000020000000000L});
    public static final BitSet FOLLOW_HITS_in_facet_spec4731 = new BitSet(new long[]{0x0000000000000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_VALUE_in_facet_spec4735 = new BitSet(new long[]{0x0000000000000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_RPAR_in_facet_spec4752 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000008L});
    public static final BitSet FOLLOW_FETCHING_in_fetching_stored_clause4801 = new BitSet(new long[]{0x0000000000000000L,0x0000000200000000L});
    public static final BitSet FOLLOW_STORED_in_fetching_stored_clause4803 = new BitSet(new long[]{0x0000040000000002L,0x0000004000000000L});
    public static final BitSet FOLLOW_TRUE_in_fetching_stored_clause4815 = new BitSet(new long[]{0x0000040000000002L,0x0000004000000000L});
    public static final BitSet FOLLOW_FALSE_in_fetching_stored_clause4819 = new BitSet(new long[]{0x0000040000000002L,0x0000004000000000L});
    public static final BitSet FOLLOW_ROUTE_in_route_by_clause4856 = new BitSet(new long[]{0x0000000000040000L});
    public static final BitSet FOLLOW_BY_in_route_by_clause4858 = new BitSet(new long[]{0x0000000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_route_by_clause4860 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_term_expr_in_search_expr4901 = new BitSet(new long[]{0x0000000000000002L,0x0000000000040000L});
    public static final BitSet FOLLOW_OR_in_search_expr4914 = new BitSet(new long[]{0x0100000000000000L,0x0000000800401048L});
    public static final BitSet FOLLOW_term_expr_in_search_expr4918 = new BitSet(new long[]{0x0000000000000002L,0x0000000000040000L});
    public static final BitSet FOLLOW_factor_expr_in_term_expr4994 = new BitSet(new long[]{0x0000000000000202L});
    public static final BitSet FOLLOW_AND_in_term_expr5007 = new BitSet(new long[]{0x0100000000000000L,0x0000000800401048L});
    public static final BitSet FOLLOW_factor_expr_in_term_expr5011 = new BitSet(new long[]{0x0000000000000202L});
    public static final BitSet FOLLOW_predicate_in_factor_expr5080 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LPAR_in_factor_expr5092 = new BitSet(new long[]{0x0100000000000000L,0x0000000800401048L});
    public static final BitSet FOLLOW_search_expr_in_factor_expr5094 = new BitSet(new long[]{0x0000000000000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_RPAR_in_factor_expr5096 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_in_predicate_in_predicate5141 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_contains_all_predicate_in_predicate5153 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_equal_predicate_in_predicate5165 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_not_equal_predicate_in_predicate5177 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_query_predicate_in_predicate5189 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_between_predicate_in_predicate5201 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_range_predicate_in_predicate5213 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_time_predicate_in_predicate5225 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_match_predicate_in_predicate5237 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_like_predicate_in_predicate5249 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_null_predicate_in_predicate5261 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_in_predicate5286 = new BitSet(new long[]{0x0200000000000000L,0x0000000000001000L});
    public static final BitSet FOLLOW_NOT_in_in_predicate5290 = new BitSet(new long[]{0x0200000000000000L});
    public static final BitSet FOLLOW_IN_in_in_predicate5293 = new BitSet(new long[]{0x0000000000000000L,0x0000040000000008L});
    public static final BitSet FOLLOW_value_list_in_in_predicate5295 = new BitSet(new long[]{0x0000008000000002L,0x0000200000000000L});
    public static final BitSet FOLLOW_except_clause_in_in_predicate5299 = new BitSet(new long[]{0x0000000000000002L,0x0000200000000000L});
    public static final BitSet FOLLOW_predicate_props_in_in_predicate5302 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_contains_all_predicate5365 = new BitSet(new long[]{0x0000000004000000L});
    public static final BitSet FOLLOW_CONTAINS_in_contains_all_predicate5367 = new BitSet(new long[]{0x0000000000000080L});
    public static final BitSet FOLLOW_ALL_in_contains_all_predicate5369 = new BitSet(new long[]{0x0000000000000000L,0x0000040000000008L});
    public static final BitSet FOLLOW_value_list_in_contains_all_predicate5371 = new BitSet(new long[]{0x0000008000000002L,0x0000200000000000L});
    public static final BitSet FOLLOW_except_clause_in_contains_all_predicate5375 = new BitSet(new long[]{0x0000000000000002L,0x0000200000000000L});
    public static final BitSet FOLLOW_predicate_props_in_contains_all_predicate5378 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_equal_predicate5439 = new BitSet(new long[]{0x0000001000000000L});
    public static final BitSet FOLLOW_EQUAL_in_equal_predicate5441 = new BitSet(new long[]{0x0800040008000040L,0x0000044801004000L});
    public static final BitSet FOLLOW_value_in_equal_predicate5443 = new BitSet(new long[]{0x0000000000000002L,0x0000200000000000L});
    public static final BitSet FOLLOW_predicate_props_in_equal_predicate5447 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_not_equal_predicate5502 = new BitSet(new long[]{0x0000000000000000L,0x0000000000002000L});
    public static final BitSet FOLLOW_NOT_EQUAL_in_not_equal_predicate5504 = new BitSet(new long[]{0x0800040008000040L,0x0000044801004000L});
    public static final BitSet FOLLOW_value_in_not_equal_predicate5506 = new BitSet(new long[]{0x0000000000000002L,0x0000200000000000L});
    public static final BitSet FOLLOW_predicate_props_in_not_equal_predicate5508 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_QUERY_in_query_predicate5563 = new BitSet(new long[]{0x2000000000000000L});
    public static final BitSet FOLLOW_IS_in_query_predicate5565 = new BitSet(new long[]{0x0000000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_query_predicate5567 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_between_predicate5616 = new BitSet(new long[]{0x0000000000008000L,0x0000000000001000L});
    public static final BitSet FOLLOW_NOT_in_between_predicate5620 = new BitSet(new long[]{0x0000000000008000L});
    public static final BitSet FOLLOW_BETWEEN_in_between_predicate5623 = new BitSet(new long[]{0x0800040008000040L,0x0000044801004000L});
    public static final BitSet FOLLOW_value_in_between_predicate5627 = new BitSet(new long[]{0x0000000000000200L});
    public static final BitSet FOLLOW_AND_in_between_predicate5629 = new BitSet(new long[]{0x0800040008000040L,0x0000044801004000L});
    public static final BitSet FOLLOW_value_in_between_predicate5633 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_range_predicate5689 = new BitSet(new long[]{0x000C000000000000L,0x0000000000000030L});
    public static final BitSet FOLLOW_GT_in_range_predicate5694 = new BitSet(new long[]{0x0800040008000040L,0x0000044801004000L});
    public static final BitSet FOLLOW_GTE_in_range_predicate5700 = new BitSet(new long[]{0x0800040008000040L,0x0000044801004000L});
    public static final BitSet FOLLOW_LT_in_range_predicate5706 = new BitSet(new long[]{0x0800040008000040L,0x0000044801004000L});
    public static final BitSet FOLLOW_LTE_in_range_predicate5712 = new BitSet(new long[]{0x0800040008000040L,0x0000044801004000L});
    public static final BitSet FOLLOW_value_in_range_predicate5717 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_time_predicate5769 = new BitSet(new long[]{0x0200000000000000L,0x0000000000001000L});
    public static final BitSet FOLLOW_NOT_in_time_predicate5772 = new BitSet(new long[]{0x0200000000000000L});
    public static final BitSet FOLLOW_IN_in_time_predicate5776 = new BitSet(new long[]{0x4000000000000000L});
    public static final BitSet FOLLOW_LAST_in_time_predicate5778 = new BitSet(new long[]{0x0800000000000000L});
    public static final BitSet FOLLOW_time_span_in_time_predicate5780 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_time_predicate5800 = new BitSet(new long[]{0x0000000000002010L,0x0000000100001000L});
    public static final BitSet FOLLOW_NOT_in_time_predicate5803 = new BitSet(new long[]{0x0000000000002010L,0x0000000100000000L});
    public static final BitSet FOLLOW_SINCE_in_time_predicate5810 = new BitSet(new long[]{0x0800000008000040L,0x0000000000004000L});
    public static final BitSet FOLLOW_AFTER_in_time_predicate5816 = new BitSet(new long[]{0x0800000008000040L,0x0000000000004000L});
    public static final BitSet FOLLOW_BEFORE_in_time_predicate5822 = new BitSet(new long[]{0x0800000008000040L,0x0000000000004000L});
    public static final BitSet FOLLOW_time_expr_in_time_predicate5825 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_time_week_part_in_time_span5860 = new BitSet(new long[]{0x0800000000000002L});
    public static final BitSet FOLLOW_time_day_part_in_time_span5865 = new BitSet(new long[]{0x0800000000000002L});
    public static final BitSet FOLLOW_time_hour_part_in_time_span5870 = new BitSet(new long[]{0x0800000000000002L});
    public static final BitSet FOLLOW_time_minute_part_in_time_span5884 = new BitSet(new long[]{0x0800000000000002L});
    public static final BitSet FOLLOW_time_second_part_in_time_span5889 = new BitSet(new long[]{0x0800000000000002L});
    public static final BitSet FOLLOW_time_millisecond_part_in_time_span5894 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_INTEGER_in_time_week_part5928 = new BitSet(new long[]{0x0000000000000000L,0x0000080000000000L});
    public static final BitSet FOLLOW_WEEKS_in_time_week_part5930 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_INTEGER_in_time_day_part5963 = new BitSet(new long[]{0x0000000010000000L});
    public static final BitSet FOLLOW_DAYS_in_time_day_part5965 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_INTEGER_in_time_hour_part5998 = new BitSet(new long[]{0x0080000000000000L});
    public static final BitSet FOLLOW_HOURS_in_time_hour_part6000 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_INTEGER_in_time_minute_part6033 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000300L});
    public static final BitSet FOLLOW_set_in_time_minute_part6035 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_INTEGER_in_time_second_part6074 = new BitSet(new long[]{0x0000000000000000L,0x0000000030000000L});
    public static final BitSet FOLLOW_set_in_time_second_part6076 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_INTEGER_in_time_millisecond_part6116 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000880L});
    public static final BitSet FOLLOW_set_in_time_millisecond_part6118 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_time_span_in_time_expr6157 = new BitSet(new long[]{0x0000000000000040L});
    public static final BitSet FOLLOW_AGO_in_time_expr6159 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_date_time_string_in_time_expr6179 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_NOW_in_time_expr6199 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_DATE_in_date_time_string6232 = new BitSet(new long[]{0x0000000000000002L,0x0000001000000000L});
    public static final BitSet FOLLOW_TIME_in_date_time_string6234 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_NOT_in_match_predicate6269 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000040L});
    public static final BitSet FOLLOW_MATCH_in_match_predicate6273 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000008L});
    public static final BitSet FOLLOW_LPAR_in_match_predicate6275 = new BitSet(new long[]{0x0100000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_column_name_list_in_match_predicate6277 = new BitSet(new long[]{0x0000000000000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_RPAR_in_match_predicate6279 = new BitSet(new long[]{0x0000000000000020L});
    public static final BitSet FOLLOW_AGAINST_in_match_predicate6281 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000008L});
    public static final BitSet FOLLOW_LPAR_in_match_predicate6283 = new BitSet(new long[]{0x0000000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_match_predicate6285 = new BitSet(new long[]{0x0000000000000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_RPAR_in_match_predicate6287 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_like_predicate6320 = new BitSet(new long[]{0x8000000000000000L,0x0000000000001000L});
    public static final BitSet FOLLOW_NOT_in_like_predicate6323 = new BitSet(new long[]{0x8000000000000000L});
    public static final BitSet FOLLOW_LIKE_in_like_predicate6327 = new BitSet(new long[]{0x0000000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_like_predicate6329 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_null_predicate6362 = new BitSet(new long[]{0x2000000000000000L});
    public static final BitSet FOLLOW_IS_in_null_predicate6364 = new BitSet(new long[]{0x0000000000000000L,0x0000000000009000L});
    public static final BitSet FOLLOW_NOT_in_null_predicate6367 = new BitSet(new long[]{0x0000000000000000L,0x0000000000008000L});
    public static final BitSet FOLLOW_NULL_in_null_predicate6371 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LPAR_in_non_variable_value_list6409 = new BitSet(new long[]{0x0800040008000040L,0x0000044801004000L});
    public static final BitSet FOLLOW_value_in_non_variable_value_list6413 = new BitSet(new long[]{0x0000000001000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_COMMA_in_non_variable_value_list6434 = new BitSet(new long[]{0x0800040008000040L,0x0000044801004000L});
    public static final BitSet FOLLOW_value_in_non_variable_value_list6438 = new BitSet(new long[]{0x0000000001000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_RPAR_in_non_variable_value_list6465 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_138_in_python_style_list6493 = new BitSet(new long[]{0x0800040008000040L,0x0000044801004000L,0x0000000200000400L});
    public static final BitSet FOLLOW_python_style_value_in_python_style_list6497 = new BitSet(new long[]{0x0000000001000000L,0x0000000000000000L,0x0000000000000800L});
    public static final BitSet FOLLOW_COMMA_in_python_style_list6518 = new BitSet(new long[]{0x0800040008000040L,0x0000044801004000L,0x0000000200000400L});
    public static final BitSet FOLLOW_python_style_value_in_python_style_list6522 = new BitSet(new long[]{0x0000000001000000L,0x0000000000000000L,0x0000000000000800L});
    public static final BitSet FOLLOW_139_in_python_style_list6549 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_161_in_python_style_dict6577 = new BitSet(new long[]{0x0100000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_key_value_pair_in_python_style_dict6581 = new BitSet(new long[]{0x0000000001000000L,0x0000000000000000L,0x0000002000000000L});
    public static final BitSet FOLLOW_COMMA_in_python_style_dict6603 = new BitSet(new long[]{0x0100000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_key_value_pair_in_python_style_dict6607 = new BitSet(new long[]{0x0000000001000000L,0x0000000000000000L,0x0000002000000000L});
    public static final BitSet FOLLOW_165_in_python_style_dict6635 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_value_in_python_style_value6658 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_python_style_list_in_python_style_value6670 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_python_style_dict_in_python_style_value6682 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_non_variable_value_list_in_value_list6707 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_VARIABLE_in_value_list6727 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_numeric_in_value6760 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_value6772 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_TRUE_in_value6792 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_FALSE_in_value6804 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_VARIABLE_in_value6816 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_time_expr_in_numeric6849 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_INTEGER_in_numeric6861 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_REAL_in_numeric6873 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_EXCEPT_in_except_clause6898 = new BitSet(new long[]{0x0000000000000000L,0x0000040000000008L});
    public static final BitSet FOLLOW_value_list_in_except_clause6901 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_WITH_in_predicate_props6936 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000008L});
    public static final BitSet FOLLOW_prop_list_in_predicate_props6939 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LPAR_in_prop_list6979 = new BitSet(new long[]{0x0100000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_key_value_pair_in_prop_list6983 = new BitSet(new long[]{0x0000000001000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_COMMA_in_prop_list7005 = new BitSet(new long[]{0x0100000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_key_value_pair_in_prop_list7009 = new BitSet(new long[]{0x0000000001000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_RPAR_in_prop_list7037 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_key_value_pair7075 = new BitSet(new long[]{0x0000000000400000L});
    public static final BitSet FOLLOW_IDENT_in_key_value_pair7090 = new BitSet(new long[]{0x0000000000400000L});
    public static final BitSet FOLLOW_COLON_in_key_value_pair7110 = new BitSet(new long[]{0x0800040008000040L,0x0000044801004000L,0x0000000200000400L});
    public static final BitSet FOLLOW_value_in_key_value_pair7115 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_python_style_list_in_key_value_pair7121 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_python_style_dict_in_key_value_pair7127 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GIVEN_in_given_clause7161 = new BitSet(new long[]{0x0000020000000000L});
    public static final BitSet FOLLOW_FACET_in_given_clause7163 = new BitSet(new long[]{0x0000000000000000L,0x0000000000200000L});
    public static final BitSet FOLLOW_PARAM_in_given_clause7165 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000008L});
    public static final BitSet FOLLOW_facet_param_list_in_given_clause7167 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_variable_declarator_in_variable_declarators7212 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_COMMA_in_variable_declarators7233 = new BitSet(new long[]{0x0100000000000000L});
    public static final BitSet FOLLOW_variable_declarator_in_variable_declarators7237 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_variable_declarator_id_in_variable_declarator7285 = new BitSet(new long[]{0x0000001000000002L});
    public static final BitSet FOLLOW_EQUAL_in_variable_declarator7288 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000042000003F8L});
    public static final BitSet FOLLOW_variable_initializer_in_variable_declarator7290 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_IDENT_in_variable_declarator_id7325 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000400L});
    public static final BitSet FOLLOW_138_in_variable_declarator_id7328 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000000800L});
    public static final BitSet FOLLOW_139_in_variable_declarator_id7330 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000400L});
    public static final BitSet FOLLOW_array_initializer_in_variable_initializer7361 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_expression_in_variable_initializer7371 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_161_in_array_initializer7390 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000062000003F8L});
    public static final BitSet FOLLOW_variable_initializer_in_array_initializer7393 = new BitSet(new long[]{0x0000000001000000L,0x0000000000000000L,0x0000002000000000L});
    public static final BitSet FOLLOW_COMMA_in_array_initializer7396 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000042000003F8L});
    public static final BitSet FOLLOW_variable_initializer_in_array_initializer7398 = new BitSet(new long[]{0x0000000001000000L,0x0000000000000000L,0x0000002000000000L});
    public static final BitSet FOLLOW_COMMA_in_array_initializer7403 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000002000000000L});
    public static final BitSet FOLLOW_165_in_array_initializer7409 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_class_or_interface_type_in_type7432 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000400L});
    public static final BitSet FOLLOW_138_in_type7435 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000000800L});
    public static final BitSet FOLLOW_139_in_type7437 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000400L});
    public static final BitSet FOLLOW_primitive_type_in_type7459 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000400L});
    public static final BitSet FOLLOW_138_in_type7462 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000000800L});
    public static final BitSet FOLLOW_139_in_type7464 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000400L});
    public static final BitSet FOLLOW_boxed_type_in_type7486 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000400L});
    public static final BitSet FOLLOW_138_in_type7489 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000000800L});
    public static final BitSet FOLLOW_139_in_type7491 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000400L});
    public static final BitSet FOLLOW_limited_type_in_type7513 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000400L});
    public static final BitSet FOLLOW_138_in_type7516 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000000800L});
    public static final BitSet FOLLOW_139_in_type7518 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000400L});
    public static final BitSet FOLLOW_FAST_UTIL_DATA_TYPE_in_class_or_interface_type7553 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LT_in_type_arguments7591 = new BitSet(new long[]{0x0400080200090000L,0x0000000000000004L,0x00000000108103FCL});
    public static final BitSet FOLLOW_type_argument_in_type_arguments7603 = new BitSet(new long[]{0x0004000001000000L});
    public static final BitSet FOLLOW_COMMA_in_type_arguments7624 = new BitSet(new long[]{0x0400080200090000L,0x0000000000000004L,0x00000000108103FCL});
    public static final BitSet FOLLOW_type_argument_in_type_arguments7628 = new BitSet(new long[]{0x0004000001000000L});
    public static final BitSet FOLLOW_GT_in_type_arguments7655 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_type_in_type_argument7684 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_130_in_type_argument7694 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000020200000L});
    public static final BitSet FOLLOW_set_in_type_argument7697 = new BitSet(new long[]{0x0400080200090000L,0x0000000000000004L,0x00000000108103F8L});
    public static final BitSet FOLLOW_type_in_type_argument7705 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LPAR_in_formal_parameters7730 = new BitSet(new long[]{0x0400080200090000L,0x0000000000000004L,0x0000000010C103F8L});
    public static final BitSet FOLLOW_formal_parameter_decls_in_formal_parameters7732 = new BitSet(new long[]{0x0000000000000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_RPAR_in_formal_parameters7734 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_formal_parameter_decl_in_formal_parameter_decls7774 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_COMMA_in_formal_parameter_decls7795 = new BitSet(new long[]{0x0400080200090000L,0x0000000000000004L,0x0000000010C103F8L});
    public static final BitSet FOLLOW_formal_parameter_decl_in_formal_parameter_decls7799 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_variable_modifiers_in_formal_parameter_decl7851 = new BitSet(new long[]{0x0400080200090000L,0x0000000000000004L,0x00000000108103F8L});
    public static final BitSet FOLLOW_type_in_formal_parameter_decl7853 = new BitSet(new long[]{0x0100000000000000L});
    public static final BitSet FOLLOW_variable_declarator_id_in_formal_parameter_decl7855 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_BOOLEAN_in_primitive_type7886 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_144_in_primitive_type7896 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_BYTE_in_primitive_type7908 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_156_in_primitive_type7918 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_INT_in_primitive_type7930 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LONG_in_primitive_type7942 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_151_in_primitive_type7952 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_DOUBLE_in_primitive_type7964 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_BOOLEAN_in_boxed_type7985 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_131_in_boxed_type7995 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_BYTE_in_boxed_type8007 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_135_in_boxed_type8017 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_133_in_boxed_type8027 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LONG_in_boxed_type8039 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_132_in_boxed_type8049 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_DOUBLE_in_boxed_type8061 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_150_in_variable_modifier8119 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_DEFINED_in_relevance_model8147 = new BitSet(new long[]{0x0000000000000800L});
    public static final BitSet FOLLOW_AS_in_relevance_model8149 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000008L});
    public static final BitSet FOLLOW_formal_parameters_in_relevance_model8153 = new BitSet(new long[]{0x0000000000004000L});
    public static final BitSet FOLLOW_BEGIN_in_relevance_model8173 = new BitSet(new long[]{0x0D202C0200290000L,0x1B0080488102800CL,0x000000435BD543F8L});
    public static final BitSet FOLLOW_model_block_in_relevance_model8175 = new BitSet(new long[]{0x0000000800000000L});
    public static final BitSet FOLLOW_END_in_relevance_model8177 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_block_statement_in_model_block8206 = new BitSet(new long[]{0x0D202C0200290002L,0x1B0080488102800CL,0x000000435BD543F8L});
    public static final BitSet FOLLOW_161_in_block8226 = new BitSet(new long[]{0x0D202C0200290000L,0x1B0080488102800CL,0x000000635BD543F8L});
    public static final BitSet FOLLOW_block_statement_in_block8247 = new BitSet(new long[]{0x0D202C0200290000L,0x1B0080488102800CL,0x000000635BD543F8L});
    public static final BitSet FOLLOW_165_in_block8269 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_local_variable_declaration_stmt_in_block_statement8288 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_java_statement_in_block_statement8298 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_local_variable_declaration_in_local_variable_declaration_stmt8317 = new BitSet(new long[]{0x0000000000000000L,0x0000000080000000L});
    public static final BitSet FOLLOW_SEMI_in_local_variable_declaration_stmt8319 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_variable_modifiers_in_local_variable_declaration8338 = new BitSet(new long[]{0x0400080200090000L,0x0000000000000004L,0x00000000108103F8L});
    public static final BitSet FOLLOW_type_in_local_variable_declaration8340 = new BitSet(new long[]{0x0100000000000000L});
    public static final BitSet FOLLOW_variable_declarators_in_local_variable_declaration8342 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_variable_modifier_in_variable_modifiers8371 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000400000L});
    public static final BitSet FOLLOW_block_in_java_statement8391 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_153_in_java_statement8401 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000008L});
    public static final BitSet FOLLOW_par_expression_in_java_statement8403 = new BitSet(new long[]{0x0920240200290000L,0x1B0080488102800CL,0x000000434B1443F8L});
    public static final BitSet FOLLOW_java_statement_in_java_statement8405 = new BitSet(new long[]{0x0000000400000002L});
    public static final BitSet FOLLOW_else_statement_in_java_statement8408 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_152_in_java_statement8420 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000008L});
    public static final BitSet FOLLOW_LPAR_in_java_statement8422 = new BitSet(new long[]{0x0D202C0200290000L,0x1B0080488102800CL,0x0000004010C103F8L});
    public static final BitSet FOLLOW_for_control_in_java_statement8442 = new BitSet(new long[]{0x0000000000000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_RPAR_in_java_statement8444 = new BitSet(new long[]{0x0920240200290000L,0x1B0080488102800CL,0x000000434B1443F8L});
    public static final BitSet FOLLOW_java_statement_in_java_statement8446 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_160_in_java_statement8466 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000008L});
    public static final BitSet FOLLOW_par_expression_in_java_statement8468 = new BitSet(new long[]{0x0920240200290000L,0x1B0080488102800CL,0x000000434B1443F8L});
    public static final BitSet FOLLOW_java_statement_in_java_statement8470 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_148_in_java_statement8480 = new BitSet(new long[]{0x0920240200290000L,0x1B0080488102800CL,0x000000434B1443F8L});
    public static final BitSet FOLLOW_java_statement_in_java_statement8482 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000100000000L});
    public static final BitSet FOLLOW_160_in_java_statement8484 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000008L});
    public static final BitSet FOLLOW_par_expression_in_java_statement8486 = new BitSet(new long[]{0x0000000000000000L,0x0000000080000000L});
    public static final BitSet FOLLOW_SEMI_in_java_statement8488 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_158_in_java_statement8498 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000008L});
    public static final BitSet FOLLOW_par_expression_in_java_statement8500 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000200000000L});
    public static final BitSet FOLLOW_161_in_java_statement8502 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000002000088000L});
    public static final BitSet FOLLOW_switch_block_statement_groups_in_java_statement8504 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000002000000000L});
    public static final BitSet FOLLOW_165_in_java_statement8506 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_155_in_java_statement8516 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_expression_in_java_statement8518 = new BitSet(new long[]{0x0000000000000000L,0x0000000080000000L});
    public static final BitSet FOLLOW_SEMI_in_java_statement8520 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_142_in_java_statement8530 = new BitSet(new long[]{0x0100000000000000L,0x0000000080000000L});
    public static final BitSet FOLLOW_IDENT_in_java_statement8532 = new BitSet(new long[]{0x0000000000000000L,0x0000000080000000L});
    public static final BitSet FOLLOW_SEMI_in_java_statement8535 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_146_in_java_statement8545 = new BitSet(new long[]{0x0100000000000000L,0x0000000080000000L});
    public static final BitSet FOLLOW_IDENT_in_java_statement8547 = new BitSet(new long[]{0x0000000000000000L,0x0000000080000000L});
    public static final BitSet FOLLOW_SEMI_in_java_statement8550 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_SEMI_in_java_statement8560 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_statement_expression_in_java_statement8570 = new BitSet(new long[]{0x0000000000000000L,0x0000000080000000L});
    public static final BitSet FOLLOW_SEMI_in_java_statement8572 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_ELSE_in_else_statement8593 = new BitSet(new long[]{0x0920240200290000L,0x1B0080488102800CL,0x000000434B1443F8L});
    public static final BitSet FOLLOW_java_statement_in_else_statement8595 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_switch_block_statement_group_in_switch_block_statement_groups8615 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000088000L});
    public static final BitSet FOLLOW_switch_label_in_switch_block_statement_group8636 = new BitSet(new long[]{0x0D202C0200290002L,0x1B0080488102800CL,0x000000435BDDC3F8L});
    public static final BitSet FOLLOW_block_statement_in_switch_block_statement_group8639 = new BitSet(new long[]{0x0D202C0200290002L,0x1B0080488102800CL,0x000000435BD543F8L});
    public static final BitSet FOLLOW_143_in_switch_label8659 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_constant_expression_in_switch_label8661 = new BitSet(new long[]{0x0000000000400000L});
    public static final BitSet FOLLOW_COLON_in_switch_label8663 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_143_in_switch_label8673 = new BitSet(new long[]{0x0100000000000000L});
    public static final BitSet FOLLOW_enum_constant_name_in_switch_label8675 = new BitSet(new long[]{0x0000000000400000L});
    public static final BitSet FOLLOW_COLON_in_switch_label8677 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_147_in_switch_label8687 = new BitSet(new long[]{0x0000000000400000L});
    public static final BitSet FOLLOW_COLON_in_switch_label8689 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_enhanced_for_control_in_for_control8715 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_for_init_in_for_control8725 = new BitSet(new long[]{0x0000000000000000L,0x0000000080000000L});
    public static final BitSet FOLLOW_SEMI_in_for_control8728 = new BitSet(new long[]{0x0920240200290000L,0x1B0080488102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_expression_in_for_control8730 = new BitSet(new long[]{0x0000000000000000L,0x0000000080000000L});
    public static final BitSet FOLLOW_SEMI_in_for_control8733 = new BitSet(new long[]{0x0920240200290002L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_for_update_in_for_control8735 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_local_variable_declaration_in_for_init8755 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_expression_list_in_for_init8765 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_variable_modifiers_in_enhanced_for_control8784 = new BitSet(new long[]{0x0400080200090000L,0x0000000000000004L,0x00000000108103F8L});
    public static final BitSet FOLLOW_type_in_enhanced_for_control8786 = new BitSet(new long[]{0x0100000000000000L});
    public static final BitSet FOLLOW_IDENT_in_enhanced_for_control8788 = new BitSet(new long[]{0x0000000000400000L});
    public static final BitSet FOLLOW_COLON_in_enhanced_for_control8790 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_expression_in_enhanced_for_control8792 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_expression_list_in_for_update8811 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LPAR_in_par_expression8830 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_expression_in_par_expression8832 = new BitSet(new long[]{0x0000000000000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_RPAR_in_par_expression8834 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_expression_in_expression_list8853 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_COMMA_in_expression_list8856 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_expression_in_expression_list8858 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_expression_in_statement_expression8879 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_expression_in_constant_expression8898 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_IDENT_in_enum_constant_name8917 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_conditional_expression_in_expression8936 = new BitSet(new long[]{0x0004001000000002L,0x24A4000000000010L,0x0000000800002001L});
    public static final BitSet FOLLOW_assignment_operator_in_expression8939 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_expression_in_expression8941 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_EQUAL_in_assignment_operator8962 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_122_in_assignment_operator8972 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_125_in_assignment_operator8982 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_119_in_assignment_operator8992 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_128_in_assignment_operator9002 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_117_in_assignment_operator9012 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_163_in_assignment_operator9022 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_141_in_assignment_operator9032 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_114_in_assignment_operator9042 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LT_in_assignment_operator9063 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000010L});
    public static final BitSet FOLLOW_LT_in_assignment_operator9067 = new BitSet(new long[]{0x0000001000000000L});
    public static final BitSet FOLLOW_EQUAL_in_assignment_operator9071 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_assignment_operator9105 = new BitSet(new long[]{0x0004000000000000L});
    public static final BitSet FOLLOW_GT_in_assignment_operator9109 = new BitSet(new long[]{0x0004000000000000L});
    public static final BitSet FOLLOW_GT_in_assignment_operator9113 = new BitSet(new long[]{0x0000001000000000L});
    public static final BitSet FOLLOW_EQUAL_in_assignment_operator9117 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_assignment_operator9148 = new BitSet(new long[]{0x0004000000000000L});
    public static final BitSet FOLLOW_GT_in_assignment_operator9152 = new BitSet(new long[]{0x0000001000000000L});
    public static final BitSet FOLLOW_EQUAL_in_assignment_operator9156 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_conditional_or_expression_in_conditional_expression9185 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000004L});
    public static final BitSet FOLLOW_130_in_conditional_expression9189 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_expression_in_conditional_expression9191 = new BitSet(new long[]{0x0000000000400000L});
    public static final BitSet FOLLOW_COLON_in_conditional_expression9193 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_expression_in_conditional_expression9195 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_conditional_and_expression_in_conditional_or_expression9217 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000001000000000L});
    public static final BitSet FOLLOW_164_in_conditional_or_expression9221 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_conditional_and_expression_in_conditional_or_expression9223 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000001000000000L});
    public static final BitSet FOLLOW_inclusive_or_expression_in_conditional_and_expression9245 = new BitSet(new long[]{0x0000000000000002L,0x0008000000000000L});
    public static final BitSet FOLLOW_115_in_conditional_and_expression9248 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_inclusive_or_expression_in_conditional_and_expression9250 = new BitSet(new long[]{0x0000000000000002L,0x0008000000000000L});
    public static final BitSet FOLLOW_exclusive_or_expression_in_inclusive_or_expression9272 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000400000000L});
    public static final BitSet FOLLOW_162_in_inclusive_or_expression9275 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_exclusive_or_expression_in_inclusive_or_expression9277 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000400000000L});
    public static final BitSet FOLLOW_and_expression_in_exclusive_or_expression9299 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000001000L});
    public static final BitSet FOLLOW_140_in_exclusive_or_expression9302 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_and_expression_in_exclusive_or_expression9304 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000001000L});
    public static final BitSet FOLLOW_equality_expression_in_and_expression9326 = new BitSet(new long[]{0x0000000000000002L,0x0010000000000000L});
    public static final BitSet FOLLOW_116_in_and_expression9330 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_equality_expression_in_and_expression9332 = new BitSet(new long[]{0x0000000000000002L,0x0010000000000000L});
    public static final BitSet FOLLOW_instanceof_expression_in_equality_expression9354 = new BitSet(new long[]{0x0000000000000002L,0x0001000000000000L,0x0000000000000002L});
    public static final BitSet FOLLOW_set_in_equality_expression9358 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_instanceof_expression_in_equality_expression9366 = new BitSet(new long[]{0x0000000000000002L,0x0001000000000000L,0x0000000000000002L});
    public static final BitSet FOLLOW_relational_expression_in_instanceof_expression9388 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000004000000L});
    public static final BitSet FOLLOW_154_in_instanceof_expression9391 = new BitSet(new long[]{0x0400080200090000L,0x0000000000000004L,0x00000000108103F8L});
    public static final BitSet FOLLOW_type_in_instanceof_expression9393 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_shift_expression_in_relational_expression9414 = new BitSet(new long[]{0x0004000000000002L,0x0000000000000010L});
    public static final BitSet FOLLOW_relational_op_in_relational_expression9418 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_shift_expression_in_relational_expression9420 = new BitSet(new long[]{0x0004000000000002L,0x0000000000000010L});
    public static final BitSet FOLLOW_LT_in_relational_op9451 = new BitSet(new long[]{0x0000001000000000L});
    public static final BitSet FOLLOW_EQUAL_in_relational_op9455 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_relational_op9485 = new BitSet(new long[]{0x0000001000000000L});
    public static final BitSet FOLLOW_EQUAL_in_relational_op9489 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LT_in_relational_op9510 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_relational_op9520 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_additive_expression_in_shift_expression9539 = new BitSet(new long[]{0x0004000000000002L,0x0000000000000010L});
    public static final BitSet FOLLOW_shift_op_in_shift_expression9543 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_additive_expression_in_shift_expression9545 = new BitSet(new long[]{0x0004000000000002L,0x0000000000000010L});
    public static final BitSet FOLLOW_LT_in_shift_op9576 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000010L});
    public static final BitSet FOLLOW_LT_in_shift_op9580 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_shift_op9612 = new BitSet(new long[]{0x0004000000000000L});
    public static final BitSet FOLLOW_GT_in_shift_op9616 = new BitSet(new long[]{0x0004000000000000L});
    public static final BitSet FOLLOW_GT_in_shift_op9620 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_shift_op9650 = new BitSet(new long[]{0x0004000000000000L});
    public static final BitSet FOLLOW_GT_in_shift_op9654 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_multiplicative_expression_in_additive_expression9683 = new BitSet(new long[]{0x0000000000000002L,0x0900000000000000L});
    public static final BitSet FOLLOW_set_in_additive_expression9687 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_multiplicative_expression_in_additive_expression9695 = new BitSet(new long[]{0x0000000000000002L,0x0900000000000000L});
    public static final BitSet FOLLOW_unary_expression_in_multiplicative_expression9717 = new BitSet(new long[]{0x0000000000000002L,0x8042000000000000L});
    public static final BitSet FOLLOW_set_in_multiplicative_expression9721 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_unary_expression_in_multiplicative_expression9735 = new BitSet(new long[]{0x0000000000000002L,0x8042000000000000L});
    public static final BitSet FOLLOW_120_in_unary_expression9761 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_unary_expression_in_unary_expression9763 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_123_in_unary_expression9773 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_unary_expression_in_unary_expression9775 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_121_in_unary_expression9785 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_unary_expression_in_unary_expression9787 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_124_in_unary_expression9797 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_unary_expression_in_unary_expression9799 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_unary_expression_not_plus_minus_in_unary_expression9809 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_166_in_unary_expression_not_plus_minus9828 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_unary_expression_in_unary_expression_not_plus_minus9830 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_111_in_unary_expression_not_plus_minus9840 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_unary_expression_in_unary_expression_not_plus_minus9842 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_cast_expression_in_unary_expression_not_plus_minus9852 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_primary_in_unary_expression_not_plus_minus9862 = new BitSet(new long[]{0x0000000000000002L,0x5200000000000000L,0x0000000000000400L});
    public static final BitSet FOLLOW_selector_in_unary_expression_not_plus_minus9864 = new BitSet(new long[]{0x0000000000000002L,0x5200000000000000L,0x0000000000000400L});
    public static final BitSet FOLLOW_LPAR_in_cast_expression9890 = new BitSet(new long[]{0x0400000200090000L,0x0000000000000004L,0x0000000010810000L});
    public static final BitSet FOLLOW_primitive_type_in_cast_expression9892 = new BitSet(new long[]{0x0000000000000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_RPAR_in_cast_expression9894 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_unary_expression_in_cast_expression9896 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LPAR_in_cast_expression9905 = new BitSet(new long[]{0x0D202C0200290000L,0x1B0080480102800CL,0x00000040108103F8L});
    public static final BitSet FOLLOW_type_in_cast_expression9908 = new BitSet(new long[]{0x0000000000000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_expression_in_cast_expression9912 = new BitSet(new long[]{0x0000000000000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_RPAR_in_cast_expression9915 = new BitSet(new long[]{0x0920240200290000L,0x000080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_unary_expression_not_plus_minus_in_cast_expression9917 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_par_expression_in_primary9936 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_literal_in_primary9946 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_java_ident_in_primary9956 = new BitSet(new long[]{0x0000000000000002L,0x4000000000000008L,0x0000000000000400L});
    public static final BitSet FOLLOW_126_in_primary9959 = new BitSet(new long[]{0x0100000004000000L});
    public static final BitSet FOLLOW_java_method_in_primary9961 = new BitSet(new long[]{0x0000000000000002L,0x4000000000000008L,0x0000000000000400L});
    public static final BitSet FOLLOW_identifier_suffix_in_primary9965 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_boxed_type_in_java_ident9995 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_limited_type_in_java_ident10005 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_IDENT_in_java_ident10015 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_CONTAINS_in_java_method10038 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_IDENT_in_java_method10048 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_138_in_identifier_suffix10068 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000000800L});
    public static final BitSet FOLLOW_139_in_identifier_suffix10070 = new BitSet(new long[]{0x0000000000000000L,0x4000000000000000L,0x0000000000000400L});
    public static final BitSet FOLLOW_126_in_identifier_suffix10074 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000020000L});
    public static final BitSet FOLLOW_145_in_identifier_suffix10076 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_arguments_in_identifier_suffix10086 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_126_in_identifier_suffix10096 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000020000L});
    public static final BitSet FOLLOW_145_in_identifier_suffix10098 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_126_in_identifier_suffix10108 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000080000000L});
    public static final BitSet FOLLOW_159_in_identifier_suffix10110 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_126_in_identifier_suffix10120 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000020000000L});
    public static final BitSet FOLLOW_157_in_identifier_suffix10122 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000008L});
    public static final BitSet FOLLOW_arguments_in_identifier_suffix10124 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_integer_literal_in_literal10144 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_REAL_in_literal10154 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_FLOATING_POINT_LITERAL_in_literal10164 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_CHARACTER_LITERAL_in_literal10174 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_literal10184 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_boolean_literal_in_literal10194 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_NULL_in_literal10206 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_TRUE_in_boolean_literal10266 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_FALSE_in_boolean_literal10278 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_126_in_selector10297 = new BitSet(new long[]{0x0100000000000000L});
    public static final BitSet FOLLOW_IDENT_in_selector10299 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000008L});
    public static final BitSet FOLLOW_arguments_in_selector10301 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_126_in_selector10312 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000080000000L});
    public static final BitSet FOLLOW_159_in_selector10314 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_138_in_selector10324 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_expression_in_selector10326 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000000800L});
    public static final BitSet FOLLOW_139_in_selector10328 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LPAR_in_arguments10347 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480902800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_expression_list_in_arguments10349 = new BitSet(new long[]{0x0000000000000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_RPAR_in_arguments10352 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_USING_in_relevance_model_clause10384 = new BitSet(new long[]{0x0000000000000000L,0x0000000002000000L});
    public static final BitSet FOLLOW_RELEVANCE_in_relevance_model_clause10386 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000400L});
    public static final BitSet FOLLOW_MODEL_in_relevance_model_clause10388 = new BitSet(new long[]{0x0100000000000000L});
    public static final BitSet FOLLOW_IDENT_in_relevance_model_clause10390 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000008L});
    public static final BitSet FOLLOW_prop_list_in_relevance_model_clause10392 = new BitSet(new long[]{0x0000000020000002L});
    public static final BitSet FOLLOW_relevance_model_in_relevance_model_clause10397 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_facet_param_in_facet_param_list10438 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_COMMA_in_facet_param_list10459 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000008L});
    public static final BitSet FOLLOW_facet_param_in_facet_param_list10463 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_LPAR_in_facet_param10511 = new BitSet(new long[]{0x0100000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_column_name_in_facet_param10513 = new BitSet(new long[]{0x0000000001000000L});
    public static final BitSet FOLLOW_COMMA_in_facet_param10515 = new BitSet(new long[]{0x0000000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_facet_param10517 = new BitSet(new long[]{0x0000000001000000L});
    public static final BitSet FOLLOW_COMMA_in_facet_param10519 = new BitSet(new long[]{0x0400000200110000L,0x0000000400000004L});
    public static final BitSet FOLLOW_facet_param_type_in_facet_param10521 = new BitSet(new long[]{0x0000000001000000L});
    public static final BitSet FOLLOW_COMMA_in_facet_param10523 = new BitSet(new long[]{0x0800040008000040L,0x0000044801004008L});
    public static final BitSet FOLLOW_value_in_facet_param10528 = new BitSet(new long[]{0x0000000000000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_non_variable_value_list_in_facet_param10534 = new BitSet(new long[]{0x0000000000000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_RPAR_in_facet_param10537 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_BOOLEAN_in_facet_param_type10606 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_INT_in_facet_param_type10612 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LONG_in_facet_param_type10618 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_STRING_in_facet_param_type10624 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_BYTEARRAY_in_facet_param_type10630 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_DOUBLE_in_facet_param_type10636 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_synpred90_BQL7075 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_primitive_type_in_synpred103_BQL7459 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000400L});
    public static final BitSet FOLLOW_138_in_synpred103_BQL7462 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000000800L});
    public static final BitSet FOLLOW_139_in_synpred103_BQL7464 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000400L});
    public static final BitSet FOLLOW_boxed_type_in_synpred105_BQL7486 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000400L});
    public static final BitSet FOLLOW_138_in_synpred105_BQL7489 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000000800L});
    public static final BitSet FOLLOW_139_in_synpred105_BQL7491 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000400L});
    public static final BitSet FOLLOW_else_statement_in_synpred133_BQL8408 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_switch_label_in_synpred146_BQL8636 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_143_in_synpred148_BQL8659 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_constant_expression_in_synpred148_BQL8661 = new BitSet(new long[]{0x0000000000400000L});
    public static final BitSet FOLLOW_COLON_in_synpred148_BQL8663 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_143_in_synpred149_BQL8673 = new BitSet(new long[]{0x0100000000000000L});
    public static final BitSet FOLLOW_enum_constant_name_in_synpred149_BQL8675 = new BitSet(new long[]{0x0000000000400000L});
    public static final BitSet FOLLOW_COLON_in_synpred149_BQL8677 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_enhanced_for_control_in_synpred150_BQL8715 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_assignment_operator_in_synpred156_BQL8939 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_expression_in_synpred156_BQL8941 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LT_in_synpred166_BQL9053 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000010L});
    public static final BitSet FOLLOW_LT_in_synpred166_BQL9055 = new BitSet(new long[]{0x0000001000000000L});
    public static final BitSet FOLLOW_EQUAL_in_synpred166_BQL9057 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_synpred167_BQL9093 = new BitSet(new long[]{0x0004000000000000L});
    public static final BitSet FOLLOW_GT_in_synpred167_BQL9095 = new BitSet(new long[]{0x0004000000000000L});
    public static final BitSet FOLLOW_GT_in_synpred167_BQL9097 = new BitSet(new long[]{0x0000001000000000L});
    public static final BitSet FOLLOW_EQUAL_in_synpred167_BQL9099 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_synpred168_BQL9138 = new BitSet(new long[]{0x0004000000000000L});
    public static final BitSet FOLLOW_GT_in_synpred168_BQL9140 = new BitSet(new long[]{0x0000001000000000L});
    public static final BitSet FOLLOW_EQUAL_in_synpred168_BQL9142 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LT_in_synpred179_BQL9443 = new BitSet(new long[]{0x0000001000000000L});
    public static final BitSet FOLLOW_EQUAL_in_synpred179_BQL9445 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_synpred180_BQL9477 = new BitSet(new long[]{0x0000001000000000L});
    public static final BitSet FOLLOW_EQUAL_in_synpred180_BQL9479 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LT_in_synpred183_BQL9568 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000010L});
    public static final BitSet FOLLOW_LT_in_synpred183_BQL9570 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_synpred184_BQL9602 = new BitSet(new long[]{0x0004000000000000L});
    public static final BitSet FOLLOW_GT_in_synpred184_BQL9604 = new BitSet(new long[]{0x0004000000000000L});
    public static final BitSet FOLLOW_GT_in_synpred184_BQL9606 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_synpred185_BQL9642 = new BitSet(new long[]{0x0004000000000000L});
    public static final BitSet FOLLOW_GT_in_synpred185_BQL9644 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_cast_expression_in_synpred197_BQL9852 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LPAR_in_synpred201_BQL9890 = new BitSet(new long[]{0x0400000200090000L,0x0000000000000004L,0x0000000010810000L});
    public static final BitSet FOLLOW_primitive_type_in_synpred201_BQL9892 = new BitSet(new long[]{0x0000000000000000L,0x0000000008000000L});
    public static final BitSet FOLLOW_RPAR_in_synpred201_BQL9894 = new BitSet(new long[]{0x0920240200290000L,0x1B0080480102800CL,0x00000040000003F8L});
    public static final BitSet FOLLOW_unary_expression_in_synpred201_BQL9896 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_type_in_synpred202_BQL9908 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_126_in_synpred205_BQL9959 = new BitSet(new long[]{0x0100000004000000L});
    public static final BitSet FOLLOW_java_method_in_synpred205_BQL9961 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_identifier_suffix_in_synpred206_BQL9965 = new BitSet(new long[]{0x0000000000000002L});

}